*CMZ :  2.00/03 26/05/2000  17.50.25  by  Klaus Rabbertz
*CMZ :  1.02/00 02/10/98  12.12.08  by  Tancredi Carli
*-- Author :    Klaus Rabbertz   22/11/96
      SUBROUTINE DEVSHP(QPART,NPART,Q2,EVSHP,NCURR,ECQ)
************************************************************************
*                                                                      *
*   DEVSHP                                                             *
*   purpose: calculate event shapes in the current hemisphere          *
*            (y for both hemispheres) of the Breit frame               *
*                                                                      *
*   called by: user                                                    *
*                                                                      *
*   input : - Breit frame input 4 vectors: QPART (E,p)                 *
*           - # of vectors               : NPART                       *
*           - Q^2                        : Q2                          *
*                                                                      *
*   output: - event shape array EVSHP containing:                      *
*              1: thrust like in e+e-, but only 1 hemisphere!          *
*              2: sum_i(|p_z|)/(Q/2) => thrust axis = (0,0,-1) (Webber)*
*              3: sum_i(|p_z|)/sum_i(|p|) => thrust axis = (0,0,-1)    *
*              4: jet broadening sum_i(p_t)/(Q/2)/2                    *
*              5: jet broadening sum_i(p_t)/sum_i(|p|)/2               *
*              6: jet mass (sum_i(E)^2-sum_i(p_x)^2-...)/(2Q/2)^2      *
*              7: jet mass (sum_i(E)^2-sum_i(p_x)^2-...)/(2E)^2        *
*              8: C parameter 3*(l1*l2+l2*l3+l3*l1) with l1,l2,l3      *
*                 eigen values of sum_i(p_i^j*p_i^k/|p_i|)/(Q/2)       *
*              9: C parameter 3*(l1*l2+l2*l3+l3*l1) with l1,l2,l3      *
*                 eigen values of sum_i(p_i^j*p_i^k/|p_i|)/sum_i(|p|)  *
*             10: y_kt for Q^2 scale (KTCLUS package)                  *
*             11: y_kt for fixed scale of 49.GeV^2 (KTCLUS package)    *
*             12: y_fJade for Q^2 scale                                *
*             13: y_fJade for fixed scale of 49.GeV^2                  *
*           - # of vectors in the curr. hemisph.: NCURR                *
*           - sum_i(E)/Q: ECQ                                          *
*                                                                      *
*   written by: Klaus Rabbertz, 22/11/96 15.20.00                      *
************************************************************************
      IMPLICIT NONE

C-----------------------------------------------------------------------
#include "qjmacs.inc"
C-----------------------------------------------------------------------
      DOUBLE PRECISION DSMALL
      PARAMETER(DSMALL = 1.D-15)
      INTEGER NEVSHP
      PARAMETER (NEVSHP=13)
      DOUBLE PRECISION FS
      PARAMETER (FS=7.D0)

      INTEGER NPART,NCURR
      DOUBLE PRECISION QPART(4,2000),Q2,EVSHP(NEVSHP),ECQ
* !!!!!!!!!!!!!! changes in line above !!!!!!!!!!!!!
      CHARACTER*80 JTEXT
      INTEGER I,J,IERR
      DOUBLE PRECISION QS,ES,PXS,PYS,PZS,PS,PTS,MS
      DOUBLE PRECISION QJI(4,JTBIMX)
      REAL YC(JTBIMX)

*---initialization
      DO I=1,NEVSHP
         EVSHP(I) = -1.D0
      ENDDO
      NCURR = 0
      ECQ = 0.D0

      JTBI = 0
      ES  = 0.D0
      PXS = 0.D0
      PYS = 0.D0
      PZS = 0.D0
      PS  = 0.D0
      PTS = 0.D0

*---check input
      IF (NPART.LE.0.OR.NPART.GT.JTBIMX) THEN
         WRITE(JTEXT,*)'S: DEVSHP: Illegal nr. of vectors:',NPART
         WRITE(*,*)JTEXT
ckr         CALL ERRLOG(9701,JTEXT)
ckr         RETURN
         STOP
      ENDIF
      IF (Q2.LT.1.D-3) THEN
         WRITE(JTEXT,*)'S: DEVSHP: Q^2 value too low:',Q2
         WRITE(*,*)JTEXT
ckr         CALL ERRLOG(9702,JTEXT)
ckr         RETURN
         STOP
      ENDIF

*---copy qpart to qpi if in current hemisphere, count E,pt,...
      DO I=1,NPART
         IF (QPART(4,I).LT.0.D0) THEN
            JTBI = JTBI + 1
            ES   = ES   + QPART(1,I)
            PXS  = PXS  + QPART(2,I)
            PYS  = PYS  + QPART(3,I)
            PZS  = PZS  + QPART(4,I)
            PS   = PS   + SQRT(QPART(2,I)**2+QPART(3,I)**2+
     &           QPART(4,I)**2)
            PTS  = PTS  + SQRT(QPART(2,I)**2+QPART(3,I)**2)
            DO J=1,3
               QPI(J,JTBI) = QPART(J+1,I)
            ENDDO
            QPI(4,JTBI) = QPART(1,I)
         ENDIF
      ENDDO
      NCURR = JTBI

*---calculate event shapes
      QS = SQRT(Q2)
      IF (NCURR.GT.0) THEN
         ECQ   = ES/QS
*---(1-thrust)_C (a la e+e-)
         CALL DGTHRU
         EVSHP(1) = 1.D0 - QTBOR(1,1)
*---(1-thrust)_Q (a la Webber)
         EVSHP(2) = 1.D0 - 2.D0*ABS(PZS)/QS
*---(1-thrust)_P (a la KR)
         IF (PS.GT.DSMALL) THEN
            EVSHP(3) = 1.D0 - ABS(PZS)/PS
         ELSE
            EVSHP(3) = 0.D0
            WRITE(JTEXT,*)'W: DEVSHP: sum_p value too low:',PS
            WRITE(*,*)JTEXT
ckr            CALL ERRLOG(9703,JTEXT)
         ENDIF
*---jet broadening_Q and _P
         EVSHP(4) = PTS/QS
         IF (PS.GT.DSMALL) THEN
            EVSHP(5) = PTS/PS/2.D0
         ELSE
            EVSHP(5) = 0.D0
            WRITE(JTEXT,*)'W: DEVSHP: sum_p value too low:',PS
            WRITE(*,*)JTEXT
ckr            CALL ERRLOG(9703,JTEXT)
         ENDIF
*---jet mass_Q and _E
         MS = (ES**2-PXS**2-PYS**2-PZS**2)
         EVSHP(6) = MS/Q2
         IF (ES.GT.DSMALL) THEN
            EVSHP(7) = MS/ES**2/4.D0
         ELSE
            EVSHP(7) = 0.D0
            WRITE(JTEXT,*)'W: DEVSHP: sum_E value too low:',ES
            WRITE(*,*)JTEXT
ckr            CALL ERRLOG(9704,JTEXT)
         ENDIF
*---C parameter_Q and _P
         CALL DGEIG(2,IERR)
         IF (IERR.EQ.0) THEN
            EVSHP(9) = 3.D0*(QTBOR(1,1)*QTBOR(1,2) +
     &                       QTBOR(1,2)*QTBOR(1,3) +
     &                       QTBOR(1,3)*QTBOR(1,1))
            EVSHP(8) = EVSHP(9)*4.D0*PS**2/Q2
         ELSE
            EVSHP(8) = 0.D0
            EVSHP(9) = 0.D0
            WRITE(JTEXT,*)'W: DEVSHP: error in DGEIG:',IERR
            WRITE(*,*)JTEXT
ckr            CALL ERRLOG(9705,JTEXT)
         ENDIF
      ENDIF
*---y_kt
*---copy complete qpart to qji (KTCLUS needs QJI(4,.))
      EVSHP(10) = 0.D0
      EVSHP(11) = 0.D0
      IF (NPART.GT.1) THEN
         DO I=1,NPART
            DO J=1,3
               QJI(J,I) = QPART(J+1,I)
            ENDDO
            QJI(4,I) = QPART(1,I)
         ENDDO
*---Q2 scale
         CALL KTCLUS(3,QJI,NPART,REAL(QS),YC,*999)
         EVSHP(10) = DBLE(YC(2))
*---fixed scale
         CALL KTCLUS(3,QJI,NPART,REAL(FS),YC,*999)
         EVSHP(11) = DBLE(YC(2))
      ENDIF

*---y_fJade
*---copy complete qpart to qpi, qpi(i,1)="remnant"
      QPI(1,1) = 0.D0
      QPI(2,1) = 0.D0
      QPI(3,1) = QS/2.D0
      QPI(4,1) = QS/2.D0
      JTBI = 1
      DO I=1,NPART
         JTBI = JTBI + 1
         DO J=1,3
            QPI(J,JTBI) = QPART(J+1,I)
         ENDDO
         QPI(4,JTBI) = QPART(1,I)
      ENDDO
*---Q2 scale
      CALL DGFJAD(1,QS)
      EVSHP(12) = QTBOR(1,2)
*---fixed scale
      CALL DGFJAD(1,FS)
      EVSHP(13) = QTBOR(1,2)

      RETURN

 999  WRITE(JTEXT,*)'S: DEVSHP: Error in KTCLUS!'
      WRITE(*,*)JTEXT
ckr      CALL ERRLOG(9706,JTEXT)
      STOP

      END
