*CMZ :  2.00/03 06/06/2000  16.41.49  by  Tancredi Carli
*CMZ :  0.03/00 15/05/91  21.34.12  by  Unknown
*-- Author :    C.Jacobsson   15/05/91
***********************************************************************
      SUBROUTINE DGEIG(NOPT,IERR)
C----------------------------------------------------------------------
C   Author      : E. Blucher            27-FEB-1989 (QGSPHE)
C   Changed by  : C. Jacobsson, H1KCJA, 13-APR-1991
C   Changed by  : K. Rabbertz           21-JUL-1997
C
C   Libraries required: CERNLIB (DEISRS1,DTRED2,DTQL2)
C
C   Description
C   ===========
C  Calculate eigenvalues and eigenvectors
C  from momentum vectors stored in QCTBUF.
C  Two different normalizations can be used.
C  For NOPT = 1,2 sqrt(sum_i |p(i)^2|) resp. (sum_i |p(i)|) > 10^-6
C  has to be fulfilled.
C
C  Input:  NOPT = 1: TT(j,k)=1/P**2*SUMi:p(j,i)*p(k,i)
C               = 2: TT(j,k)=(sum_i p(j,i)*p(k,i)/|p(i)|)/(sum_i |p(i)|)
C
C  Output: QPO(1,1),(2,1),(3,1)   = major axis
C          QPO(1,2),(2,2),(3,2)   = semi-major axis
C          QPO(1,3),(2,3),(3,3)   = minor axis
C          QTBOR(1,1),(1,2),(1,3) = eigenvalues in descending order
C          Note: For equal eigenvalues the eigenvectors are not
C                uniquely determined.
C  Errors: IERR = 1: sqrt(sum_i |p(i)^2|) <= 10^-6
C          IERR = 2: (sum_i |p(i)|)       <= 10^-6
C          IERR = 3: error in DEIGRS1
C----------------------------------------------------------------------

ckr
      IMPLICIT NONE
ckr

#include "qjmacs.inc"
C----------------------------------------------------------------------
ckr add. declaration
      INTEGER NOPT,IERR
      CHARACTER*80 JTEXT
      INTEGER I,J,K
      INTEGER IND(3)
      DOUBLE PRECISION PA,SUMP,SUMT
      DOUBLE PRECISION EVAL(3),EVEC(3,3),P(3),TT(3,3),WK(14)
ckr ende

*---initialization
      IERR = 0
      JTBO = 3
      DO I=1,JTBO
         QPO(1,I) = 0.D0
         QPO(2,I) = 0.D0
         QPO(3,I) = 0.D0
         QPO(4,I) = 0.D0
         QTBOR(1,I) = 0.D0
      ENDDO

*---compute momentum tensor TT
      SUMP = 0.D0
      DO J=1,3
         DO I=1,3
            TT(I,J) = 0.D0
         ENDDO
      ENDDO
      DO I=1,JTBI
         P(1) = QPI(1,I)
         P(2) = QPI(2,I)
         P(3) = QPI(3,I)
         PA   = SQRT(P(1)**2+P(2)**2+P(3)**2)
         SUMP = SUMP + PA
         DO J=1,3
            DO K=1,J
               IF (NOPT.EQ.1) THEN
                  TT(J,K) = TT(J,K) + P(J)*P(K)
               ELSE
                  IF (PA.GT.1.D-10) THEN
                     TT(J,K) = TT(J,K) + P(J)*P(K)/PA
                  ENDIF
               ENDIF
            ENDDO
         ENDDO
      ENDDO
      TT(1,2) = TT(2,1)
      TT(1,3) = TT(3,1)
      TT(2,3) = TT(3,2)

*---do the normalization
*---minimal total momentum required
      IF (NOPT.EQ.1) THEN
         SUMT = TT(1,1) + TT(2,2) + TT(3,3)
         IF (SQRT(SUMT).GT.1.D-10) THEN
            DO J=1,3
               DO I=1,3
                  TT(I,J) = 1.D0/SUMT*TT(I,J)
               ENDDO
            ENDDO
         ELSE
            IERR = 1
            WRITE(JTEXT,*)'W: DGEIG: sqrt(sum_i |p(i)^2|) too low:',SUMT
            WRITE(*,*)JTEXT
ckr            CALL ERRLOG(9707,JTEXT)
            RETURN
         ENDIF
      ELSE
         IF (SUMP.GT.1.D-10) THEN
            DO J=1,3
               DO I=1,3
                  TT(I,J) = 1.D0/SUMP*TT(I,J)
               ENDDO
            ENDDO
         ELSE
            IERR = 2
            WRITE(JTEXT,*)'W: DGEIG: (sum_i |p(i)|) too low:',SUMP
            WRITE(*,*)JTEXT
ckr            CALL ERRLOG(9708,JTEXT)
            RETURN
         ENDIF
      ENDIF

*---calculate the eigenvectors and -values
*---(attention: EISRS1 may cause underflow exceptions!)
      CALL DEISRS1(3,3,TT,EVAL,EVEC,IERR,WK)
      IF (IERR.NE.0) THEN
         WRITE(JTEXT,*)'W: DGEIG: calc. of eigen val./vect. failed',
     &        IERR
         WRITE(*,*)JTEXT
ckr         CALL ERRLOG(9709,JTEXT)
         IERR = 3
         RETURN
      ENDIF

*---put them in descending order
      IF (EVAL(1).GE.MAX(EVAL(2),EVAL(3))) THEN
         IND(1) = 1
         IF (EVAL(2).GE.EVAL(3)) THEN
            IND(2) = 2
            IND(3) = 3
         ELSE
            IND(2) = 3
            IND(3) = 2
         ENDIF
      ELSEIF (MIN(EVAL(2),EVAL(3)).GT.EVAL(1)) THEN
         IND(3) = 1
         IF (EVAL(2).GE.EVAL(3)) THEN
            IND(1) = 2
            IND(2) = 3
         ELSE
            IND(1) = 3
            IND(2) = 2
         ENDIF
      ELSE
         IND(2) = 1
         IF (EVAL(2).GE.EVAL(3)) THEN
            IND(1) = 2
            IND(3) = 3
         ELSE
            IND(1) = 3
            IND(3) = 2
         ENDIF
      ENDIF

*---store major axis, semi-major axis, minor axis,
*---and eigenvalues in descending order
      DO I=1,JTBO
         QPO(1,I) = EVEC(1,IND(I))
         QPO(2,I) = EVEC(2,IND(I))
         QPO(3,I) = EVEC(3,IND(I))
         QPO(4,I) = 1.D0
         QTBOR(1,I) = EVAL(IND(I))
      ENDDO

      RETURN
      END
