*****************************************************************************
	Subroutine HZ00017(iflag)
*****************************************************************************
*
* For Monte Carlo with both direct and resolved interactions call hz00017
* with iflag +1000 for the DIRECT component run and iflag +2000 for 
* the RESOLVED component run for all three phases (iflag=1,2,3). 
* For a direct only / DIS Monte Carlo just use iflag=1,2,3.
*
* Relevant Variables:
*
*       Cuts :      0.2 < y_bj < 0.55
*       Jets:
*           Symmetric:  ET1 > 5.5 GeV, ET2 > 5.5 GeV, -1.125 < eta < 2.2
*           Asymmetric: ET1 > 7.5 GeV, ET2 > 6.5 GeV, -1.125 < eta < 1.875
*       Bins: 
*           Q2:    (0-1, 0.1-0.55, 1.5-3.0, 3.0-4.5)
*           XGamma (0.0625-0.25, 0.25-0.5, 0.5-0.75, 0.75-1.0)
*
* Histograms in the routine:
*
* For symmetric jets  = 10 Low to high xGamma cross section ratio vs Q2
*                     = 11 xGamma cross section of Q2 < 1
*                     = 12 xGamma cross section of 0.1 < Q2 < 0.55
*                     = 13 xGamma cross section of 1.5 < Q2 < 4.5
*
* For asymmetric jets = 20 Low to high xGamma cross section ratio vs Q2
*                     = 21 xGamma cross section of Q2 < 1
*                     = 22 xGamma cross section of 0.1 < Q2 < 0.55
*                     = 23 xGamma cross section of 1.5 < Q2 < 4.5
* 
* Author : Ben West (westbj@mail.desy.de)
*          Based on hz98085.fpp by Matt Wing
*
* Date   : 03/05/01
*
* Reference : DESY-00-017
*
*****************************************************************************
 
	Implicit None
 
#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
#include "hzjetcmn.inc"

	CHARACTER*6 jetf
	
        INTEGER NJET
c        DOUBLE PRECISION Jets(50,8)
        DOUBLE PRECISION CONER
	SAVE CONER

	Logical dismode
	DOUBLE PRECISION Ee
	INTEGER myil, myip
	INTEGER i,j,k
	REAL eta(2),et(2), y,q2, xg
	REAL xgtemp(4), xgtemperr(4)

C --- choose same jet finder as in the paper --- KTCLUS
	INTEGER initflag, chjet, iproc, iflag
	SAVE chjet
	INTEGER hzflag 
	Data chjet/3/

c --- Q2 range safety check

	Real MaxQ2(2), MinQ2(2)
	Common / HZ00017C / MinQ2, MaxQ2

C --- Results from paper:

c --- Figure 2

	Real SymXGamPHP(4),    SymXGamIQS(4),    SymXGamDIS(4)
	Real SymXGamPHPErr(4), SymXGamIQSErr(4), SymXGamDISErr(4)

*	Data SymXGamPHP    / /
*	Data SymXGamPHPErr / /

*	Data SymXGamIQS    / /
*	Data SymXGamIQSErr / /

*	Data SymXGamDIS    / /
*	Data SymXGamDISErr / /

c --- Figure 3

	Real AsyXGamPHP(4),    AsyXGamIQS(4),    AsyXGamDIS(4)
	Real AsyXGamPHPErr(4), AsyXGamIQSErr(4), AsyXGamDISErr(4)

*	Data AsyXGamPHP    / /
*	Data AsyXGamPHPErr / /

*	Data AsyXGamIQS    / /
*	Data AsyXGamIQSErr / /

*	Data AsyXGamDIS    / /
*	Data AsyXGamDISErr / /


C --- Figure 4

	Real AsyRatio(4), AsyRatioErr(4)
	Real SymRatio(4), SymRatioErr(4)
c --- a)
*	Data SymRatio    / /
*	Data SymRatioErr / /
c --- b)
*	Data AsyRatio    / /
*	Data AsyRatioErr / /
 
C --- Kinematic cuts:

	Real ycut(2), Q2Min(4), Q2Max(4)
	Real SymEtCut(2), AsyEtCut(2), SymEtaCut(2), AsyEtaCut(2)
	Real XGamBins(5)
	save Q2Min, Q2Max

	Data ycut      / 0.2, 0.55 /
	Data SymEtCut  / 5.5, 5.5 /
	Data SymEtaCut / -1.125, 2.2  /
	Data AsyEtCut  / 7.5, 6.5 /
	Data AsyEtaCut / -1.125, 1.875 /
	Data Q2Min     / 0.0, 0.1,  1.5, 3.0 /
	Data Q2Max     / 1.0, 0.55, 3.0, 4.5 /
	Data XGamBins  / 0.0625, 0.25, 0.5, 0.75, 1.0 /

c -- Interpret iflag

	If (iflag.gt.1000) Then
	   iproc=INT(iflag/1000)
	   initflag=MOD(iflag,1000)
	   dismode = .false.
	Else
	   dismode = .true.
	   iproc=1
	   initflag = iflag
	EndIf
 	
	If (initflag.eq.1) then

	   q2min(iproc) = 100000.
	   q2max(iproc) = 0.

	   If (iproc.eq.1.or.dismode) then
	      Call hcdir('//HISTO',' ')
	      Call hmdir('00017','S')
	      Call hcdir('//PAWC',' ')
	      Call hmdir('00017','S')
	      Call HCDIR('//HISTO/00017',' ')
	      Call HCDIR('//PAWC/00017',' ')

** Book paper result histograms

	      Call hbook1(-10,
     &     'Zeus [s](x?[g]!^obs! "L# 0.75)/[s](x?[g]!^obs! "G# 0.75) '//
     &     'vs bin number see DESY-00-017', 
     &        4, 0.5, 4.5, 0.)
	      Call hbookb(-11, 
     &     'Zeus d[s]/dx?[g]!^obs! (nb) (Q^2! "L# 1.0 GeV^2!)', 
     &        4, XGamBins, 0.)
	      Call hbookb(-12, 
     &     'Zeus d[s]/dx?[g]!^obs! (nb) (0.1 "L# Q^2! "L# 0.55 GeV^2!)', 
     &        4, XGamBins, 0.)
	      Call hbookb(-13, 
     &     'Zeus d[s]/dx?[g]!^obs! (nb) (1.5 "L# Q^2! "L# 4.5 GeV^2!)', 
     &        4, XGamBins, 0.)
	      Call hpak (-10, SymRatio)
	      Call hpake(-10, SymRatioErr)
	      Call hpak (-11, SymXGamPHP)
	      Call hpake(-11, SymXGamPHPErr)
	      Call hpak (-12, SymXGamIQS)
	      Call hpake(-12, SymXGamIQSErr)
	      Call hpak (-13, SymXGamDIS)
	      Call hpake(-13, SymXGamDISErr)

	      Call hbook1(-20,
     &     'Zeus [s](x?[g]!^obs! "L# 0.75)/[s](x?[g]!^obs! "G# 0.75) '//
     &     'vs bin number see DESY-00-017', 
     &        4, 0.5, 4.5, 0.)
	      Call hbookb(-21, 
     &     'Zeus d[s]/dx?[g]!^obs! (nb) (Q^2! "L# 1.0 GeV^2!)', 
     &        4, XGamBins, 0.)
	      Call hbookb(-22, 
     &     'Zeus d[s]/dx?[g]!^obs! (nb) (0.1 "L# Q^2! "L# 0.55 GeV^2!)', 
     &        4, XGamBins, 0.)
	      Call hbookb(-23, 
     &     'Zeus d[s]/dx?[g]!^obs! (nb) (1.5 "L# Q^2! "L# 4.5 GeV^2!)', 
     &        4, XGamBins, 0.)
	      Call hpak (-20, AsyRatio)
	      Call hpake(-20, AsyRatioErr)
	      Call hpak (-21, AsyXGamPHP)
	      Call hpake(-21, AsyXGamPHPErr)
	      Call hpak (-22, AsyXGamIQS)
	      Call hpake(-22, AsyXGamIQSErr)
	      Call hpak (-23, AsyXGamDIS)
	      Call hpake(-23, AsyXGamDISErr)

** Book MC result histograms

	      Call hbook1(10,
     &     'MC [s](x?[g]!^obs! "L# 0.75)/[s](x?[g]!^obs! "G# 0.75) '//
     &     'vs bin number see DESY-00-017', 
     &        4, 0.5, 4.5, 0.)
	      Call hbookb(11, 
     &     'MC d[s]/dx?[g]!^obs! (nb) (Q^2! "L# 1.0 GeV^2!)', 
     &        4, XGamBins, 0.)
	      Call hbookb(12, 
     &     'MC d[s]/dx?[g]!^obs! (nb) (0.1 "L# Q^2! "L# 0.55 GeV^2!)', 
     &        4, XGamBins, 0.)
	      Call hbookb(13, 
     &     'MC d[s]/dx?[g]!^obs! (nb) (1.5 "L# Q^2! "L# 4.5 GeV^2!)', 
     &        4, XGamBins, 0.)
	      Call hbook1(20,
     &     'MC [s](x?[g]!^obs! "L# 0.75)/[s](x?[g]!^obs! "G# 0.75) '//
     &     'vs bin number see DESY-00-017', 
     &        4, 0.5, 4.5, 0.)
	      Call hbookb(21, 
     &     'MC d[s]/dx?[g]!^obs! (nb) (Q^2! "L# 1.0 GeV^2!)', 
     &        4, XGamBins, 0.)
	      Call hbookb(22, 
     &     'MC d[s]/dx?[g]!^obs! (nb) (0.1 "L# Q^2! "L# 0.55 GeV^2!)', 
     &        4, XGamBins, 0.)
	      Call hbookb(23, 
     &     'MC d[s]/dx?[g]!^obs! (nb) (1.5 "L# Q^2! "L# 4.5 GeV^2!)', 
     &        4, XGamBins, 0.)
	   EndIf
	   Call HCDIR('//HISTO/00017',' ')
	   Call HCDIR('//PAWC/00017',' ')

** Book intermediate histograms

	   Do i = 1, 2
	      Do j = 1, 3
		 Call hbookb(iproc*100+10*i+j, ' ', 4, XGamBins, 0.)
	      EndDo
	      Do j = 4, 5
		 Call hbook1(iproc*100+10*i+j, ' ', 4, 0.5, 4.5, 0.)
	      EndDo
	   EndDo

	   Call HIDOPT(0,'stat')
	   Call HBARX(0)

	   Call HZJETRAD(2,CONER)
	   If (CONER.lt.0.0) Then
	      CONER=1.0
	   EndIf
	   Call HZJTNAME(chjet,jetf)
 
	   WRITE(6,*)'**********************************************'
	   WRITE(6,*)'* HZ00017 called, histograms will be output. *'
	   WRITE(6,*)'*     Jet finder selected : ',jetf,'           *'
	   WRITE(6,1001)'*        with cone radius : ',CONER,'   *'
	   WRITE(6,*)'* Plots cover the range 0 < Q2 < 4.5 GeV2.   *'
	   WRITE(6,*)'* Be careful to check for Q2 cuts in your    *'
	   WRITE(6,*)'* generator, especially on the resolved      *'
	   WRITE(6,*)'* component.                                 *'
	   WRITE(6,*)'**********************************************'
 
 1001	  FORMAT(A29,F5.2,A13)
 
	Else If (initflag.eq.2) Then

	   Call HCDIR('//PAWC/00017',' ')
 
	   q2=hzphokin(1)
	   If (q2.eq.-1) Then
	      PRINT*,'HZ00017: HZPHOKIN returns error for Q2'
	   ElseIf (q2.lt.0) Then
	      q2=abs(q2)
	   EndIf
	   y=hzphokin(3)
	   If (y.eq.-1) Then
	      PRINT*,'HZ00017: HZPHOKIN returns error for y'
	   EndIf
	   hzflag=HZIBEAM(myip,myil)
	   If (hzflag.ge.0) Then
	      ee=abs(PHEP(4,myil))
	   Else
	      PRINT*,'HZ00017: Electron not found. E=0.001'
	      PRINT*,'         event will be rejected'
	      ee=0.001
	   EndIf

	   If (q2.gt.MaxQ2(iproc)) MaxQ2(iproc) = q2
	   If (q2.lt.MinQ2(iproc)) MinQ2(iproc) = q2
	   
	   Call hzjtfind(chjet,CONER,NJET,Jets)

	   If (y.ge.ycut(1).and.y.le.ycut(2).and.njet.ge.2) Then

c --- Symmetric Jet Selection
	      j = 0
	      xg = 0.
	      Do i = 1, njet
		 If (jets(i,1).gt.SymEtaCut(1).and.
     &               jets(i,1).lt.SymEtaCut(2).and.j.lt.2) Then
		    j = j+1
		    et(j)  = Jets(i,3)
		    eta(j) = Jets(i,1)
		    xg = xg + et(j)*exp(-eta(j))/(2*y*Ee)
		 endif
	      EndDo
	      If (j.eq.2.and.xg.gt.0.0625.and.xg.lt.1.0) then
		 If (et(1).gt.SymEtCut(1).and.et(2).gt.SymEtCut(2)) Then
		    Do i = 1, 4
		       If (q2.gt.q2min(i).and.q2.lt.q2max(i)) then
			  Call hf1(iproc*100+10+min(i,3), xg, wtx)
			  If (xg.lt.0.75) then
			     Call hf1(iproc*100+14, real(i), wtx) 
			  else
			     Call hf1(iproc*100+15, real(i), wtx) 
			  endif
		       endif
		    EndDo
		 EndIf
	      EndIf

c --- Asymmetric Jet Selection
	      j = 0
	      xg = 0.
	      Do i = 1, njet
		 If (jets(i,1).gt.AsyEtaCut(1).and.
     &               jets(i,1).lt.AsyEtaCut(2).and.j.lt.2) Then
		    j = j+1
		    et(j)  = Jets(i,3)
		    eta(j) = Jets(i,1)
		    xg = xg + et(j)*exp(-eta(j))/(2*y*Ee)
		 endif
	      EndDo
	      If (j.eq.2.and.xg.gt.0.0625.and.xg.lt.1.0) then
		 If (et(1).gt.AsyEtCut(1).and.et(2).gt.AsyEtCut(2)) Then
		    Do i = 1, 4
		       If (q2.gt.q2min(i).and.q2.lt.q2max(i)) then
			  Call hf1(iproc*100+20+min(i,3), xg, wtx)
			  If (xg.lt.0.75) then
			     Call hf1(iproc*100+24, real(i), wtx) 
			  else
			     Call hf1(iproc*100+25, real(i), wtx)
			  endif
		       endif
		    EndDo
		 EndIf
	      EndIf

	   EndIf

	Else If (initflag.eq.3) Then

	   Call HCDIR('//PAWC/00017',' ')
 
	   If (Xsec.eq.0) then
	      write(*,*) 'HZ00017: termination called with 
     &	      zero cross section'
	      write(*,*) '          cross section graph meaningless'
	      write(*,*) '	      in process :',iproc*1000
	      Xsec=1
	   endif
	   If (Ntot.eq.0) then
	      write(*,*) 'HZ00017: termination called with 
     &        no total events'
	      write(*,*) '          cross section graph meaningless'
	      write(*,*) '	      in process :',iproc*1000
	      Ntot=1
	   endif
	   
	   write(*,*) 'HZ00017: termination called for process:',iproc*1000
	   write(*,*) '         Minimum Q2 was:', MinQ2(iproc)
	   write(*,*) '         Maximum Q2 was:', MaxQ2(iproc)
	   If (MaxQ2(iproc).lt.4.5) then
	      write(*,*) 'HZ00017: WARNING! Maximum Q2 < 4.5, 
     &        comparison to data'
	      write(*,*) '         may be meaningless.'
	   endif
	   If (MinQ2(iproc).gt.0.001) then
	      write(*,*) 'HZ00017: WARNING! Minimum Q2 > 0.0, 
     &        comparison to data'
	      write(*,*) '         may be meaningless.'
	   endif
 
* Scale plots to cross sections and add to combined plot

	   Do i = 1, 2
	      Do j = 1, 5
		 Call HOPERA(100*iproc+10*i+j, '+e',100*iproc+10*i+j,10*i+j,
     &                       real(Xsec/Ntot), 0.)
	      EndDo
	   EndDo

	   If (iproc.eq.2.or.dismode) Then

*** divide by bin widths
	      Do i = 1, 2
		 Do j = 1, 3
		    Call hunpak(10*i+j, xgtemp, ' ', 0)
		    Call hunpke(10*i+j, xgtemperr, ' ', 0)
		    Do k = 1, 4
		       xgtemp(k)    = xgtemp(k)   /(XGamBins(k+1)-XGamBins(k))
		       xgtemperr(k) = xgtemperr(k)/(XGamBins(k+1)-XGamBins(k))
		    EndDo
		    Call hpak (10*i+j, xgtemp)
		    Call hpake(10*i+j, xgtemperr)
		 EndDo
	      EndDo

*** Form ratio

	      Do i = 1, 2
		 Call hopera(10*i+4, '/e', 10*i+5, 10*i, 1., 1.)
	      EndDo

*** Do Chi2 calculations

	      Do i = 1, 2
		 Do j = 1,3
		    Call HZCHISQ(-(10*i+j),10*i+j)
		 EndDo
	      EndDo

	   EndIf
 
	Else

	   write(*,*) 'HZ00017: Please run routine with 
     &     iflag set to 1,2 or 3'
	   write(*,*) '         for DIS Monte Carlo or with 
     &     iflag +1000 (2000)'
	   write(*,*) '         for Direct (Resolved) events.'

	EndIf
 
	Return

	End
