*CMZ :          31/07/2001  14.18.36  by   H. Jung
*-- Author : F.-P.Schilling

      SUBROUTINE HZ00174 (IFLAG)

*******************************************************************************
* H1 Collaboration
* Diffractive Jet Production in Deep-Inelastic e+p Collisions at HERA
* DESY 00-174, hep-ex/0012051, accepted by Eur.Phys.J. C
*
* Analysis by Frank-Peter Schilling, Univ. Heidelberg (Germany)
* This code (C) 2001 by Frank-Peter Schilling (fpschill@mail.desy.de)
*
*******************************************************************************
*
* Event selection: Q2 = 4 ... 80 GeV2 , y = 0.1 ... 0.7
*                  xpom < 0.05 , My < 1.6 GeV , |t| < 1.0 GeV2
*                  N>=2 CDFCONE Jets in HCMS with pt > 4 GeV
*                  eta-jet= -3 ... 0 (in HCMS)
*
* Arguments: iflag=1 initialisation
*            iflag=1 filling
*            iflag=3 termination
*
*******************************************************************************

      IMPLICIT NONE

#include "hzfunc.inc"
#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzjetcmn.inc"

* *** Variable declarations ***************************************************

      integer nentry,iflag
      data nentry /0/

c     name of HZTOOL routine
      Character*5 xxxx
      save xxxx
      Data xxxx/'00174'/

c     Print debug flag
      logical lp
      Data lp/.false./

*     common that gives the position of particles in X and Y system (hzhadgap)
      integer nxobj,nyobj,nxgag,nygag
      double precision pom
      common /diffhad/pom(5),nxobj(5000),nyobj(5000),nxgag,nygag

*     Jets
      Integer numjet,nj
      Parameter (numjet=200)
      Double precision pj(numjet,8)
      integer ipjet2(numjet)

*     all the rest

      real xw,nall,nall2,nall3,n2,lum,n3
      integer i,ij,idum,ierr,iy,ihep,ierr1,ierr2
      integer ibeam,ielec,iel,nsel2,hid

      Double precision pelec(4),pscat(4),pbeam(4),pgam(4)
      Double precision ph(5),plab(5,NMXHEP),pcm(4)
      Double precision hjet1(4),hjet2(4),ljet1(4),ljet2(4)

      real q2,x,yy,x_pom,xmas,ymas,t,mx2,ww,beta
      real ptj2,pj1,pj2,pt_had,p_sh(4),shat,xgluon,q2pt2
      real th1,th2,rap1,rap2,eta_lab,eta1,eta2
      real pj3,m3,shat3,z3,eta3
      real etot,pztot,empztot,empz1,empz2,xgam,egamma,ptpomeron

      double precision pbeam2(4),pgamma(4),ppom(4)

      logical disjet,difjet

* *** Cross section definitions ***********************************************

*     q**2
      integer nbi01
      data    nbi01 /9/
      real    bin01(10)
      data    bin01 / 4., 6., 10., 15., 20., 30., 40., 50., 60., 80./
*     pt
      integer nbi02
      data    nbi02 /6/
      real    bin02(7)
      data    bin02 / 4., 5., 6., 7.5, 9., 11., 14./
*     eta-lab
      integer nbi03
      data    nbi03 /7/
      real    bin03(8)
      data    bin03 / -1., -0.66, -0.33,  0., 0.33, 0.66, 1., 1.5/
*     mx
      integer nbi04
      data    nbi04 /5/
      real    bin04(6)
      data    bin04 / 8., 14., 20., 30., 40., 60./
*     W
      integer nbi05
      data    nbi05 /7/
      real    bin05(8)
      data    bin05 / 90., 115., 140., 165., 190., 215., 240., 260./
*     log(xpom)
      integer nbi06
      data    nbi06 /6/
      real    bin06(7)
      data    bin06 / -2.5, -2.3, -2.1, -1.9, -1.7, -1.5, -1.3/
*     log(beta)
      integer nbi07
      data    nbi07 /8/
      real    bin07(9)
      data    bin07 / -2.8, -2.5,-2.2,-1.9,-1.6,-1.3,-1.1,-0.8,-0.5/
*     zpom
      integer nbi08
      data    nbi08 /8/
      real    bin08(9)
      data    bin08 / 0., 0.125, 0.25, 0.375, 0.5, 0.625, 0.75,0.875,1./
*     xgamma
      integer nbi09
      data    nbi09 /5/
      real    bin09(6)
      data    bin09 / 0.0, 0.2, 0.4, 0.6, 0.8, 1.0/
*     e gamma-hem.
      integer nbi10
      data    nbi10 /4/
      real    bin10(5)
      data    bin10 / 0., 4., 8., 12.,  20./

*     z in xpom bins ----------------------------------------------------------

      real zxpsplit(5)
      data zxpsplit / -1.3, -1.5, -1.75, -2.0, -3.0/
*     z1 xpom=-1.3...-1.5
      integer nbi11
      data    nbi11 /5/
      real    bin11(6)
      data    bin11 / 0., .15, .3, .5, .7, 1. /
*     z2 xpom=-1.5...-1.75
      integer nbi12
      data    nbi12 /5/
      real    bin12(6)
      data    bin12 / 0., .2, .4, .6, .8, 1./
*     z3 xpom=-1.75...-2.00
      integer nbi13
      data    nbi13 /5/
      real    bin13(6)
      data    bin13 / 0., .3, .45, .6, .8, 1./
*     z4 xpom=-2...-3
      integer nbi14
      data    nbi14 /5/
      real    bin14(6)
      data    bin14 / 0., .3, .45, .6, .8, 1./

*     z in q2pt2 bins ---------------------------------------------------------

      real zqpsplit(3)
      data zqpsplit / 35., 45., 60./
*     q2+pt2 = 20 .. 35
      integer nbi15
      data    nbi15 /5/
      real    bin15(6)
      data    bin15 / 0., .2, .4, .6,  .8, 1./
*     q2+pt2 = 35 .. 45
      integer nbi16
      data    nbi16 /5/
      real    bin16(6)
      data    bin16 / 0., .2, .4, .6,  .8, 1./
*     q2+pt2 = 45 .. 60
      integer nbi17
      data    nbi17 /5/
      real    bin17(6)
      data    bin17 / 0., .2, .4, .6,  .8, 1./
*     q2+pt2 = 60 ..
      integer nbi18
      data    nbi18 /5/
      real    bin18(6)
      data    bin18 / 0., .2, .4, .6,  .8, 1./

*     xpom <0.01 --------------------------------------------------------------

*     q2
      integer nbi19
      data    nbi19 /4/
      real    bin19(5)
      data    bin19 / 4.,  10.,  20.,  40.,  80./
*     pt
      integer nbi20
      data    nbi20 /4/
      real    bin20(5)
      data    bin20 / 4., 5., 6., 7.5,  9./
*     zpom
      integer nbi21
      data    nbi21 /5/
      real    bin21(6)
      data    bin21 / 0., .3, .45, .6, .8, 1./
*     pt pomeron-hem.
      integer nbi22
      data    nbi22 /3/
      real    bin22(4)
      data    bin22 / 0., .5,  1.,  3./

*     3jet --------------------------------------------------------------------

*     m-123 (3 jets)
      integer nbi23
      data    nbi23 /3/
      real    bin23(4)
      data    bin23 / 12., 20., 30.,  40. /
*     z3 (3 jets)
      integer nbi24
      data    nbi24 /3/
      real    bin24(4)
      data    bin24 / .2, .4, .6, .8 /

* *** Data points *************************************************************

      real dat01(9),sta01(9),tot01(9)
      data dat01 /21.44,12.95, 6.25, 4.07, 2.33, 1.22, 0.75, 0.72, 0.39/
      data sta01 / 0.96, 0.52, 0.30, 0.25, 0.13, 0.10, 0.08, 0.09, 0.05/
      data tot01 / 3.64, 2.15, 1.10, 0.72, 0.41, 0.22, 0.17, 0.19, 0.13/

      real dat02(6),sta02(6),tot02(6)
      data dat02 / 74.87, 59.50, 28.76,  9.89,  3.35,  0.92/
      data sta02 /  3.00,  1.96,  1.13,  0.69,  0.37,  0.17/
      data tot02 / 13.69,  9.98,  5.10,  1.90,  0.70,  0.24/

      real dat03(7),sta03(7),tot03(7)
      data dat03 / 22.36, 68.85,112.78,131.61,127.86, 85.33, 16.38/
      data sta03 /  3.02,  4.31,  5.28,  5.53,  5.44,  4.38,  1.12/
      data tot03 /  8.24, 12.93, 18.52, 21.43, 23.00, 15.52,  4.35/

      real dat04(5),sta04(5),tot04(5)
      data dat04 /  1.90,  7.54,  7.29,  4.47,  1.21/
      data sta04 /  0.19,  0.33,  0.23,  0.18,  0.07/
      data tot04 /  0.42,  1.18,  1.25,  0.82,  0.34/

      real dat05(7),sta05(7),tot05(7)
      data dat05 /  1.08,  1.40,  1.68,  1.34,  1.13,  0.93,  0.46/
      data sta05 /  0.07,  0.07,  0.07,  0.06,  0.05,  0.05,  0.05/
      data tot05 /  0.23,  0.27,  0.31,  0.25,  0.21,  0.17,  0.14/

      real dat06(6),sta06(6),tot06(6)
      data dat06 /  7.32, 35.44, 88.23,171.24,269.26,440.73/
      data sta06 /  1.60,  3.84,  6.00,  8.07,  9.82, 14.08/
      data tot06 /  2.64,  9.71, 16.56, 29.11, 45.07, 83.01/

      real dat07(8),sta07(8),tot07(8)
      data dat07 /24.90,88.28,129.88,152.69,145.92, 85.03, 53.40, 13.54/
      data sta07 / 2.82, 4.94,  5.65,  6.03,  6.30,  5.95,  4.16,  2.39/
      data tot07 / 7.15,16.77, 22.40, 27.27, 25.30, 15.99, 10.16,  4.69/

      real dat08(8),sta08(8),tot08(8)
      data dat08 /269.40,493.90,331.28,233.17,174.15, 94.01,39.76,29.97/
      data sta08 / 15.74, 18.88, 14.01, 11.42, 10.23,  7.58, 4.65, 4.99/
      data tot08 / 65.82, 92.93, 63.24, 44.66, 29.88, 17.13, 7.97, 8.89/

      real dat09(5),sta09(5),tot09(5)
      data dat09 / 25.37,104.76,153.78,331.54,428.29/
      data sta09 /  3.62,  6.80,  7.64, 11.85, 13.34/
      data tot09 /  9.63, 19.79, 28.88, 60.71, 72.79/

      real dat10(4),sta10(4),tot10(4)
      data dat10 / 38.14,  9.02,  3.99,  2.02/
      data sta10 /  0.94,  0.42,  0.27,  0.16/
      data tot10 /  6.61,  1.64,  1.05,  0.80/

      real dat11(5),sta11(5),tot11(5)
      data dat11 /232.85,209.38, 85.36, 30.91,  3.36/
      data sta11 / 13.97, 11.13,  5.46,  3.22,  0.97/
      data tot11 / 70.53, 51.84, 18.58,  6.64,  1.85/

      real dat12(5),sta12(5),tot12(5)
      data dat12 / 97.08,134.30, 63.43, 21.77,  8.55/
      data sta12 /  6.21,  7.06,  4.47,  2.74,  2.21/
      data tot12 / 21.14, 26.46, 11.25,  4.54,  3.68/

      real dat13(5),sta13(5),tot13(5)
      data dat13 / 37.76, 59.74, 49.07, 33.84,  9.02/
      data sta13 /  3.15,  5.52,  5.49,  3.69,  1.97/
      data tot13 /  9.39, 12.20, 10.97,  7.43,  2.98/

      real dat14(5),sta14(5),tot14(5)
      data dat14 /  5.17, 25.78, 27.97, 24.80, 11.88/
      data sta14 /  1.56,  4.18,  3.88,  3.18,  2.07/
      data tot14 /  4.67,  9.14,  7.12,  5.67,  3.33/

      real dat15(5),sta15(5),tot15(5)
      data dat15 /150.55,109.03, 45.16, 18.72,  5.86/
      data sta15 / 10.04,  7.91,  4.87,  3.04,  1.85/
      data tot15 / 49.84, 30.10, 13.90,  6.65,  3.69/

      real dat16(5),sta16(5),tot16(5)
      data dat16 / 89.72, 71.75, 39.25, 16.94,  4.25/
      data sta16 /  6.36,  4.94,  3.52,  2.45,  1.18/
      data tot16 / 23.49, 16.40, 10.82,  5.07,  1.62/

      real dat17(5),sta17(5),tot17(5)
      data dat17 / 74.64, 78.00, 43.17, 14.74,  5.51/
      data sta17 /  5.78,  5.21,  3.72,  2.17,  1.30/
      data tot17 / 19.24, 19.83,  8.86,  3.68,  2.05/

      real dat18(5),sta18(5),tot18(5)
      data dat18 / 58.66,114.63, 73.39, 45.30, 14.44/
      data sta18 /  5.38,  6.69,  5.01,  4.24,  2.64/
      data tot18 / 15.76, 21.05, 12.47,  8.14,  4.16/

      real dat19(4),sta19(4),tot19(4)
      data dat19 /  1.5750,  0.4045,  0.1154,  0.0143/
      data sta19 /  0.1530,  0.0561,  0.0204,  0.0064/
      data tot19 /  0.3297,  0.0932,  0.0394,  0.0105/

      real dat20(4),sta20(4),tot20(4)
      data dat20 /  8.3167,  4.6510,  2.0343,  0.2935/
      data sta20 /  0.9940,  0.5136,  0.3257,  0.1468/
      data tot20 /  2.2257,  1.1162,  0.5070,  0.1939/

      real dat21(5),sta21(5),tot21(5)
      data dat21 /  5.1740, 25.7836, 27.9743, 24.7987, 11.8816/
      data sta21 /  1.5600,  4.1827,  3.8793,  3.1751,  2.0683/
      data tot21 /  4.6747,  9.1375,  7.1212,  5.6711,  3.3307/

      real dat22(3),sta22(3),tot22(3)
      data dat22 / 24.4935, 11.2122,  1.2875/
      data sta22 /  2.5263,  1.4239,  0.2062/
      data tot22 /  7.0444,  3.1236,  0.7763/

      real dat23(3),sta23(3),tot23(3)
      data dat23 /  0.4821,  0.4336,  0.0562/
      data sta23 /  0.0696,  0.0504,  0.0199/
      data tot23 /  0.1738,  0.1116,  0.0302/

      real dat24(3),sta24(3),tot24(3)
      data dat24 / 13.1448, 13.8502, 10.4796/
      data sta24 /  2.1324,  1.9207,  1.9460/
      data tot24 /  6.1423,  3.4237,  3.0623/

*******************************************************************************
*
*     Initialization
*
*******************************************************************************

      NENTRY=NENTRY+1

      IF (iflag.eq.1) then

        nall = 0.
        nall2= 0.
        nall3= 0.
        n2   = 0.
        n3   = 0.

*       Initialisation: The following MUST always be done
*       (i) make subdirectory in PAWC
*       - use the name as the xxxxxx in HZxxxxxx subroutine
*       (i) make subdirectory in o/p file

        call hcdir('//PAWC',' ')
        call hmdir(xxxx,'S')
        call hcdir('//HISTO',' ')
        call hmdir(xxxx,'S')

        write(6,*) 'HZ'//xxxx

c       book control histos for all events

        call hbook1(211,'y'        ,100,0.0,0.8,0.)
        call hbook1(212,'Q^2!'     ,100,0.,90.,0.)
        call hbook1(213,'log x?IP!',100,-5.,0.,0.)
        call hbook1(214,'"b#t"b#'  ,100,0.,2.,0.)
        call hbook1(215,'M?X!'     ,100,0.,80.,0.)
        call hbook1(216,'M?Y!'     ,100,0.,5.,0.)

c       book control histos for inclusive DIS

        call hbook1(221,'y'        ,100,0.0,0.8,0.)
        call hbook1(222,'Q^2!'     ,100,0.,90.,0.)
        call hbook1(223,'log x?IP!',100,-5.,0.,0.)
        call hbook1(224,'"b#t"b#'  ,100,0.,2.,0.)
        call hbook1(225,'M?X!'     ,100,0.,80.,0.)
        call hbook1(226,'M?Y!'     ,100,0.,5.,0.)

c       book control histos for incl. diffr. DIS

        call hbook1(231,'y'        ,100,0.0,0.8,0.)
        call hbook1(232,'Q^2!'     ,100,0.,90.,0.)
        call hbook1(233,'log x?IP!',100,-5.,0.,0.)
        call hbook1(234,'"b#t"b#'  ,100,0.,2.,0.)
        call hbook1(235,'M?X!'     ,100,0.,80.,0.)
        call hbook1(236,'M?Y!'     ,100,0.,5.,0.)

c       cross section histograms for mc

        call hbookb(1010,'q2  (MC)'               ,nbi01,bin01,0.)
        call hbookb(1020,'pt  (MC)'               ,nbi02,bin02,0.)
        call hbookb(1030,'eta-lab (MC)'           ,nbi03,bin03,0.)
        call hbookb(1040,'mx  (MC)'               ,nbi04,bin04,0.)
        call hbookb(1050,'W (MC)'                 ,nbi05,bin05,0.)
        call hbookb(1060,'log(xpom) (MC)'         ,nbi06,bin06,0.)
        call hbookb(1070,'log beta (MC)'          ,nbi07,bin07,0.)
        call hbookb(1080,'z (MC)'                 ,nbi08,bin08,0.)
        call hbookb(1090,'x-gamma (MC)'           ,nbi09,bin09,0.)
        call hbookb(1100,'e gamma-hem. (MC)'      ,nbi10,bin10,0.)
        call hbookb(1110,'z1 (MC)'                ,nbi11,bin11,0.)
        call hbookb(1120,'z2 (MC)'                ,nbi12,bin12,0.)
        call hbookb(1130,'z3 (MC)'                ,nbi13,bin13,0.)
        call hbookb(1140,'z4 (MC)'                ,nbi14,bin14,0.)
        call hbookb(1150,'z-q2pt2-1 (MC)'         ,nbi15,bin15,0.)
        call hbookb(1160,'z-q2pt2-2 (MC)'         ,nbi16,bin16,0.)
        call hbookb(1170,'z-q2pt2-3 (MC)'         ,nbi17,bin17,0.)
        call hbookb(1180,'z-q2pt2-4 (MC)'         ,nbi18,bin18,0.)
        call hbookb(1190,'q2 (xpom.lt.0.01) (MC)' ,nbi19,bin19,0.)
        call hbookb(1200,'pt (xpom.lt.0.01) (MC)' ,nbi20,bin20,0.)
        call hbookb(1210,'z (xpom.lt.0.01) (MC)'  ,nbi21,bin21,0.)
        call hbookb(1220,'pt pom.-hem. xplow (MC)',nbi22,bin22,0.)
        call hbookb(1230,'m3 (MC)'                ,nbi23,bin23,0.)
        call hbookb(1240,'z3 (MC)'                ,nbi24,bin24,0.)

c       cross section histograms for data

        call hbookb(1011,'Q2 (Data, sta)'  ,nbi01,bin01,0.)
        call hbookb(1012,'Q2 (Data, tot)'  ,nbi01,bin01,0.)
        call hbookb(1021,'pt (Data, sta)'  ,nbi02,bin02,0.)
        call hbookb(1022,'pt (Data, tot)'  ,nbi02,bin02,0.)
        call hbookb(1031,'eta (Data, sta)' ,nbi03,bin03,0.)
        call hbookb(1032,'eta (Data, tot)' ,nbi03,bin03,0.)
        call hbookb(1041,'mx (Data, sta)'  ,nbi04,bin04,0.)
        call hbookb(1042,'mx (Data, tot)'  ,nbi04,bin04,0.)
        call hbookb(1051,'W (Data, sta)'   ,nbi05,bin05,0.)
        call hbookb(1052,'W (Data, tot)'   ,nbi05,bin05,0.)
        call hbookb(1061,'xpom (Data, sta)',nbi06,bin06,0.)
        call hbookb(1062,'xpom (Data, tot)',nbi06,bin06,0.)
        call hbookb(1071,'beta (Data, sta)',nbi07,bin07,0.)
        call hbookb(1072,'beta (Data, tot)',nbi07,bin07,0.)
        call hbookb(1081,'z (Data, sta)'   ,nbi08,bin08,0.)
        call hbookb(1082,'z (Data, tot)'   ,nbi08,bin08,0.)
        call hbookb(1091,'xgam (Data, sta)',nbi09,bin09,0.)
        call hbookb(1092,'xgam (Data, tot)',nbi09,bin09,0.)
        call hbookb(1101,'egam (Data, sta)',nbi10,bin10,0.)
        call hbookb(1102,'egam (Data, tot)',nbi10,bin10,0.)
        call hbookb(1111,'z1 (Data, sta)'  ,nbi11,bin11,0.)
        call hbookb(1112,'z1 (Data, tot)'  ,nbi11,bin11,0.)
        call hbookb(1121,'z2 (Data, sta)'  ,nbi12,bin12,0.)
        call hbookb(1122,'z2 (Data, tot)'  ,nbi12,bin12,0.)
        call hbookb(1131,'z3 (Data, sta)'  ,nbi13,bin13,0.)
        call hbookb(1132,'z3 (Data, tot)'  ,nbi13,bin13,0.)
        call hbookb(1141,'z4 (Data, sta)'  ,nbi14,bin14,0.)
        call hbookb(1142,'z4 (Data, tot)'  ,nbi14,bin14,0.)
        call hbookb(1151,'z11 (Data, sta)' ,nbi15,bin15,0.)
        call hbookb(1152,'z11 (Data, tot)' ,nbi15,bin15,0.)
        call hbookb(1161,'z12 (Data, sta)' ,nbi16,bin16,0.)
        call hbookb(1162,'z12 (Data, tot)' ,nbi16,bin16,0.)
        call hbookb(1171,'z13 (Data, sta)' ,nbi17,bin17,0.)
        call hbookb(1172,'z13 (Data, tot)' ,nbi17,bin17,0.)
        call hbookb(1181,'z14 (Data, sta)' ,nbi18,bin18,0.)
        call hbookb(1182,'z14 (Data, tot)' ,nbi18,bin18,0.)
        call hbookb(1191,'q2 (xp.lt.0.01) (Data, sta)'  ,nbi19,bin19,0.)
        call hbookb(1192,'q2 (xp.lt.0.01) (Data, tot)'  ,nbi19,bin19,0.)
        call hbookb(1201,'pt (xp.lt.0.01) (Data, sta)'  ,nbi20,bin20,0.)
        call hbookb(1202,'pt (xp.lt.0.01) (Data, tot)'  ,nbi20,bin20,0.)
        call hbookb(1211,'z  (xp.lt.0.01)(Data, sta)'   ,nbi21,bin21,0.)
        call hbookb(1212,'z (xp.lt.0.01) (Data, tot)'   ,nbi21,bin21,0.)
        call hbookb(1221,'ptpr (xp.lt.0.01) (Data, sta)',nbi22,bin22,0.)
        call hbookb(1222,'ptpr (xp.lt.0.01)(Data, tot)' ,nbi22,bin22,0.)
        call hbookb(1231,'m-123 (3 jets) (Data, sta)'   ,nbi23,bin23,0.)
        call hbookb(1232,'m-123 (3 jets) (Data, tot)'   ,nbi23,bin23,0.)
        call hbookb(1241,'z-123 (3 jets) (Data, sta)'   ,nbi24,bin24,0.)
        call hbookb(1242,'z-123 (3 jets) (Data, tot)'   ,nbi24,bin24,0.)

*       pack data into histograms

        Call hpak (1011,dat01)
        Call hpake(1011,sta01)
        Call hpak (1012,dat01)
        Call hpake(1012,tot01)

        Call hpak (1021,dat02)
        Call hpake(1021,sta02)
        Call hpak (1022,dat02)
        Call hpake(1022,tot02)

        Call hpak (1031,dat03)
        Call hpake(1031,sta03)
        Call hpak (1032,dat03)
        Call hpake(1032,tot03)

        Call hpak (1041,dat04)
        Call hpake(1041,sta04)
        Call hpak (1042,dat04)
        Call hpake(1042,tot04)

        Call hpak (1051,dat05)
        Call hpake(1051,sta05)
        Call hpak (1052,dat05)
        Call hpake(1052,tot05)

        Call hpak (1061,dat06)
        Call hpake(1061,sta06)
        Call hpak (1062,dat06)
        Call hpake(1062,tot06)

        Call hpak (1071,dat07)
        Call hpake(1071,sta07)
        Call hpak (1072,dat07)
        Call hpake(1072,tot07)

        Call hpak (1081,dat08)
        Call hpake(1081,sta08)
        Call hpak (1082,dat08)
        Call hpake(1082,tot08)

        Call hpak (1091,dat09)
        Call hpake(1091,sta09)
        Call hpak (1092,dat09)
        Call hpake(1092,tot09)

        Call hpak (1101,dat10)
        Call hpake(1101,sta10)
        Call hpak (1102,dat10)
        Call hpake(1102,tot10)

        Call hpak (1111,dat11)
        Call hpake(1111,sta11)
        Call hpak (1112,dat11)
        Call hpake(1112,tot11)

        Call hpak (1121,dat12)
        Call hpake(1121,sta12)
        Call hpak (1122,dat12)
        Call hpake(1122,tot12)

        Call hpak (1131,dat13)
        Call hpake(1131,sta13)
        Call hpak (1132,dat13)
        Call hpake(1132,tot13)

        Call hpak (1141,dat14)
        Call hpake(1141,sta14)
        Call hpak (1142,dat14)
        Call hpake(1142,tot14)

        Call hpak (1151,dat15)
        Call hpake(1151,sta15)
        Call hpak (1152,dat15)
        Call hpake(1152,tot15)

        Call hpak (1161,dat16)
        Call hpake(1161,sta16)
        Call hpak (1162,dat16)
        Call hpake(1162,tot16)

        Call hpak (1171,dat17)
        Call hpake(1171,sta17)
        Call hpak (1172,dat17)
        Call hpake(1172,tot17)

        Call hpak (1181,dat18)
        Call hpake(1181,sta18)
        Call hpak (1182,dat18)
        Call hpake(1182,tot18)

        Call hpak (1191,dat19)
        Call hpake(1191,sta19)
        Call hpak (1192,dat19)
        Call hpake(1192,tot19)

        Call hpak (1201,dat20)
        Call hpake(1201,sta20)
        Call hpak (1202,dat20)
        Call hpake(1202,tot20)

        Call hpak (1211,dat21)
        Call hpake(1211,sta21)
        Call hpak (1212,dat21)
        Call hpake(1212,tot21)

        Call hpak (1221,dat22)
        Call hpake(1221,sta22)
        Call hpak (1222,dat22)
        Call hpake(1222,tot22)

        Call hpak (1231,dat23)
        Call hpake(1231,sta23)
        Call hpak (1232,dat23)
        Call hpake(1232,tot23)

        Call hpak (1241,dat24)
        Call hpake(1241,sta24)
        Call hpak (1242,dat24)
        Call hpake(1242,tot24)

*******************************************************************************
*
*     Event Processing
*
*******************************************************************************

      Else if (iflag.eq.2) then

*       Filling: The following MUST always be done
*       (i) move to the correct sub-directory in PAWC

        call hcdir('//PAWC/'//xxxx,' ')

c       count event (weighted)

        xw=wtx
        nall=nall+xw

*       get beams, boson and sc.electron vectors

        ierr=HZIBEAM(ibeam,ielec)
        if (.not.(ierr.eq.1)) then
          write(6,*) 'HZ'//xxxx,' beams not found ! '
          return
        else
          Do i=1,4
            pbeam(i)=PHEP(i,ibeam)
            pelec(i)=PHEP(i,ielec)
          enddo
        endif

        if (abs(abs(real(PHEP(4,ielec)))-27.5).gt.0.2) then
          write(6,*) 'HZ'//xxxx,' Electrons not at 27.5 GeV!'
          return
        endif
        if (abs(abs(real(PHEP(4,ibeam)))-820.0).gt.0.2) then
          write(6,*) 'HZ'//xxxx,' Protons not at 820.0 GeV!'
          return
        endif

        ierr=HZIPGAM(ph)
        if (ierr.eq.-1) then
          write(6,*) 'HZ'//xxxx,' Boson vector not found!'
          call VZERO(ph,5)
        else
          Do i=1,4
            pgam(i)=ph(i)
          enddo
        endif

        iel=HZIDELEC(idum)
        if (iel.eq.-1) then
           write(*,*) 'Hz'//xxxx,' Sc.electron not found!'
           return
        endif
        Do i=1,4
           pscat(i)=PHEP(i,iel)
        enddo

*       get DIS kinematics

        q2 = abs(real(HZDISKIN(1)))
        x  = real(HZDISKIN(2))
        yy = real(HZDISKIN(3))

        if (lp) write(*,*) 'Hz'//xxxx,' x y q2=',x,yy,q2

*       get diffractive kinematics

        call hzhadgap(xmas,ymas,x_pom,t)

        if (.not.(( x_pom.le.1.).and.(x_pom.ge.x))) then
          write (6,*)'Hz'//xxxx,' xpom out of range: xpom,x: ;',x_pom,x
          return
        endif

        if (lp) then
          write(6,*) 'Hz'//xxxx,' xpom= ',x_pom,' m_y= ',ymas,' t=',t
        endif

*       fill histos for inclusive events

        call hfill(211,yy,0.,xw)
        call hfill(212,q2,0.,xw)
        call hfill(213,log10(max(0.,x_pom)),0.,xw)
        call hfill(214,abs(t),0.,xw)
        call hfill(215,xmas,0.,xw)
        call hfill(216,ymas,0.,xw)

*       select diffractive DIS events

        disjet=.true.
        difjet=.true.

        if (yy.lt.0.1) disjet=.false.
        if (yy.gt.0.7) disjet=.false.
        if (q2.lt.4.)  disjet=.false.
        if (q2.gt.80.) disjet=.false.

        if (x_pom.gt.0.05) difjet=.false.
        if (ymas.gt.1.6)   difjet=.false.
        if (abs(t).gt.1.0) difjet=.false.

        if (disjet) then

          nall2=nall2+xw

*         fill histos for inclusive DIS events

          call hfill(221,yy,0.,xw)
          call hfill(222,q2,0.,xw)
          call hfill(223,log10(max(0.,x_pom)),0.,xw)
          call hfill(224,abs(t),0.,xw)
          call hfill(225,xmas,0.,xw)
          call hfill(226,ymas,0.,xw)

          if (difjet) then

            nall3=nall3+xw

*           fill histos for diffractive DIS events

            call hfill(231,yy,0.,xw)
            call hfill(232,q2,0.,xw)
            call hfill(233,log10(max(0.,x_pom)),0.,xw)
            call hfill(234,abs(t),0.,xw)
            call hfill(235,xmas,0.,xw)
            call hfill(236,ymas,0.,xw)

*           loop and boost

            etot=0.
            pztot=0.

            do ihep=1,nhep
              Do i=1,4
                Ph(i)=PHEP(i,ihep)
                Plab(i,ihep)=PHEP(i,ihep)
                phep(i,ihep)=0.0
              enddo
*             do not count decayed partices
              IF(isthep(ihep).ne.1) GOTO 20
*             do not count scattered electron
              if(ihep.eq.iel) goto 20
*             do not count particles assigned to Y system
              do iy=1,nygag
                if(ihep.eq.nyobj(iy)) goto 20
              enddo

              CALL HZHCMTOL(0,PELEC,PBEAM,PSCAT,ph,pcm,ierr)

              if (ierr.eq.1) then
                write(6,*) 'HZ'//xxxx,' problem with boost to cms ! '
                return
              endif

c             store boosted vectors in PHEP
              Do i=1,4
                PHEP(i,ihep)=Pcm(i)
              enddo

c             sum up e-pz
              etot=etot+real(pcm(4))
              pztot=pztot+real(pcm(3))

   20         continue
            enddo

*           get total E-Pz
            empztot=etot-pztot

*           run jet algorithm (8 = pxcone, pt mode)
            call hzjtfind(8,1.0d0,nj,pj)

            if (nj.gt.numjet) then
               write(6,*) 'Hz'//xxxx,' too many jets found!'
               return
            endif
            if (lp) write(6,*) 'number of jets: ',nj

*           select dijet events
            nsel2 = 0
            do ij=1,nj
               ptj2=real(pj(ij,3))
               if (ptj2.gt. 4.0 ) then
*                      pt > 4.0 GeV
                  nsel2 = nsel2 + 1
                  ipjet2(nsel2)=ij
               endif
            enddo

c           diffr. dijets

            if (nsel2.ge.2) then

             eta1=real(pj(ipjet2(1),1))
             eta2=real(pj(ipjet2(2),1))

             if ((eta1.gt.-3.0.and.eta1.lt.0.0).and.
     &           (eta2.gt.-3.0.and.eta2.lt.0.0)) then

              n2=n2+xw

              pj1=real(pj(ipjet2(1),3))
              pj2=real(pj(ipjet2(2),3))
              pt_had=0.5*(pj1+pj2)
              do i=1,4
                p_sh(i)=real(pj(ipjet2(1),3+i)+pj(ipjet2(2),3+i))
              enddo
              shat=p_sh(1)**2 - p_sh(2)**2 - p_sh(3)**2 - p_sh(4)**2
              mx2=xmas**2
              xgluon=max(0.0001,min(0.9999,(q2 + shat)/(q2 + mx2)))
              q2pt2=q2+pt_had**2
              ww=sqrt(max(0.000001,yy*90180.-q2))
              beta=q2/(q2+mx2)
              empz1=real(pj(ipjet2(1),4)-pj(ipjet2(1),4+3))
              empz2=real(pj(ipjet2(2),4)-pj(ipjet2(2),4+3))
              xgam=(empz1+empz2)/empztot

c             boost jets back to lab

              do i=1,3
                hjet1(i)=pj(ipjet2(1),4+i)
                hjet2(i)=pj(ipjet2(2),4+i)
              enddo
              hjet1(4)=pj(ipjet2(1),4)
              hjet2(4)=pj(ipjet2(2),4)
              CALL HZHCMTOL(1,PELEC,PBEAM,PSCAT,hjet1,ljet1,ierr1)
              CALL HZHCMTOL(1,PELEC,PBEAM,PSCAT,hjet2,ljet2,ierr2)

              if (ierr1.eq.1 .or. ierr2.eq.1) then
                write(6,*) 'HZ'//xxxx,' problem with boost to lab!'
              endif

c             calc. eta-lab

              th1=real(HZPHMANG(ljet1(3),
     &                 sqrt(ljet1(1)**2+ljet1(2)**2)))
              th2=real(HZPHMANG(ljet2(3),
     &                 sqrt(ljet2(1)**2+ljet2(2)**2)))
              rap1=-log(tan(th1/2.))
              rap2=-log(tan(th2/2.))
              eta_lab = 0.5*(rap1+rap2)

              if (lp) then
                write(6,*) ' '
                write(6,*) 'pt1*,pt2*: ',pj1,pj2
                write(6,*) 'eta1*,eta2*:',
     &                     real(pj(ipjet2(1),1)),real(pj(ipjet2(2),1))
                write(6,*) 'phi1*,phi2*:',
     &                     real(pj(ipjet2(1),2)),real(pj(ipjet2(2),2))
                write(6,*) 'mean pt: ',pt_had
                write(6,*) 'shat,q2,mx2,z: ',shat,q2,mx2,xgluon
                write(6,*) 'etalab1,etalab2,etalab: ',rap1,rap2,eta_lab
              endif

*             fill cross sections

              call hfill(1010,q2,0.,xw)
              call hfill(1020,pt_had,0.,xw)
              call hfill(1030,eta_lab,0.,xw)
              call hfill(1040,xmas,0.,xw)
              call hfill(1050,ww,0.,xw)
              call hfill(1060,log10(x_pom),0.,xw)
              call hfill(1070,log10(beta),0.,xw)
              call hfill(1080,xgluon,0.,xw)
              call hfill(1090,xgam,0.,xw)

c             z in xpom bins

              if ((log10(x_pom).lt.zxpsplit(1)).and.
     &            (log10(x_pom).ge.zxpsplit(2))) then
                call hfill(1110,xgluon,0.,xw)
              else if ((log10(x_pom).lt.zxpsplit(2)).and.
     &            (log10(x_pom).ge.zxpsplit(3))) then
                call hfill(1120,xgluon,0.,xw)
              else if ((log10(x_pom).lt.zxpsplit(3)).and.
     &            (log10(x_pom).ge.zxpsplit(4))) then
                call hfill(1130,xgluon,0.,xw)
              else if ((log10(x_pom).lt.zxpsplit(4)).and.
     &            (log10(x_pom).ge.zxpsplit(5))) then
                call hfill(1140,xgluon,0.,xw)
              else
                continue
              endif

c             z in q2pt2 bins

              if (q2pt2.lt.zqpsplit(1)) then
                call hfill(1150,xgluon,0.,xw)
              else if ((q2pt2.ge.zqpsplit(1)).and.
     &            (q2pt2.lt.zqpsplit(2))) then
                call hfill(1160,xgluon,0.,xw)
              else if ((q2pt2.ge.zqpsplit(2)).and.
     &            (q2pt2.lt.zqpsplit(3))) then
                call hfill(1170,xgluon,0.,xw)
              else if (q2pt2.ge.zqpsplit(3)) then
                call hfill(1180,xgluon,0.,xw)
              else
                continue
              endif

c             xpom .lt.0.01

              if (x_pom.lt.0.01) then
                call hfill(1190,q2,0.,xw)
                call hfill(1200,pt_had,0.,xw)
                call hfill(1210,xgluon,0.,xw)
              endif

c             three-jets

              if (nsel2.eq.3) then

               eta3=real(pj(ipjet2(3),1))

               if (eta3.gt.-3.0.and.eta3.lt.0.0) then

                n3=n3+xw

                pj3=real(pj(ipjet2(3),3))
                do i=1,4
                  p_sh(i)=real(pj(ipjet2(1),3+i)+pj(ipjet2(2),3+i)
     &                      +pj(ipjet2(3),3+i))
                enddo
                shat3=p_sh(1)**2 - p_sh(2)**2 - p_sh(3)**2 - p_sh(4)**2
                m3=sqrt(max(0.,shat3))
                z3=max(0.0001,min(0.9999,(q2 + shat3)/(q2 + mx2)))

                call hfill(1230,m3,0.,xw)
                call hfill(1240,z3,0.,xw)

               endif
              endif

c             end 3-jets ------------------------------------------------------
c             now the remnant stuff -------------------------------------------

c             xpom*p vector
              do i=1,4
                 pbeam2(i)=pbeam(i)*dble(x_pom)
                 pgamma(i)=0.d0
                 ppom(i)=0.d0
              enddo

              do ihep=1,nhep
                if ((ijetno(ihep).ne.1).and.(ijetno(ihep).ne.2)) then
                  Do i=1,4
                    Pcm(i)=PHEP(i,ihep)
                  enddo
c                 boost back to lab
                  CALL HZHCMTOL(1,PELEC,PBEAM,PSCAT,pcm,plab,ierr1)
c                 boost from lab to gamma-pom CMS
                  CALL HZHCMTOL(0,PELEC,PBEAM2,PSCAT,plab,pcm,ierr2)

                  if (ierr1.ne.0.or.ierr2.ne.0) then
                     print *,'HZ00174: Boost error!'
                  endif

c                 gamma-hemisphere
                  if (pcm(3).lt.0.d0) then
                    do i=1,4
                      pgamma(i)=pgamma(i)+pcm(i)
                    enddo
c                 pomeron-hemisphere
                  else
                    do i=1,4
                      ppom(i)=ppom(i)+pcm(i)
                    enddo
                  endif
                endif
              enddo

              egamma=real(pgamma(4))
              ptpomeron=real(sqrt(ppom(1)**2+ppom(2)**2))

              call hfill(1100,egamma,0.,xw)
              if(x_pom.lt.0.01) then
                call hfill(1220,ptpomeron,0.,xw)
              endif

c             end of remnant stuff

             endif
            endif

*           end diffr. dijets

*           restore hep common

            do ihep=1,nhep
              Do i=1,4
                PHEP(i,ihep)= Plab(i,ihep)
              enddo
            enddo

*           end incl. diffr. and dis inclusive

          endif
        endif

*******************************************************************************
*
*     Termination
*
*******************************************************************************

      Else if (iflag.eq.3) then

*       Termination: The following MUST always be done
*       (i) Move to the correct PAW subdirectory

        call hcdir('//PAWC/'//xxxx,' ')

c       calc lumi

        lum=999999999.
        if (xsec.ne.0.) then
          lum=real(nall)/real(xsec)
        else
          write(6,*) 'hz',xxxx,' xsec=0 ! '
        endif

        write(6,*) ' '
        write(6,*) '--------------------------------------------------'
        write(6,*) 'Hz'//xxxx,': Total Cross Section (pb): ',real(xsec)
        write(6,*) 'Hz'//xxxx,': Generated events:         ',nall
        write(6,*) 'Hz'//xxxx,': Lumi (1/pb):              ',real(lum)
        write(6,*) 'Hz'//xxxx,': Inclusive DIS events:     ',nall2
        write(6,*) 'Hz'//xxxx,': Diffr. DIS events:        ',nall3
        write(6,*) 'Hz'//xxxx,': Diffr. DIS Dijet events:  ',n2
        write(6,*) 'Hz'//xxxx,': Diffr. DIS 3-jet events:  ',n3
        write(6,*) '--------------------------------------------------'
        write(6,*) ' '

        if(n2.gt.1.) then

c         divide by lumi and binwidth

          do hid=1010,1240,10
            call hopera (hid,'+E',hid,hid,1./lum,0.)
            call hzhinrm(hid, 0, 1., 1 )
          enddo

        endif

*******************************************************************************

      endif

 999  RETURN
      END

* END OF CODE *****************************************************************
