	Subroutine HZ02023(iflag)
*****************************************************************************
* This routine produces plots from the H1 gaps between jets analysis :
* 
*     cross section in nano-barns
*
* Cuts : 	0.3 < y < 0.65
*		Et > 6.0,5.0 GeV
*               eta < 2.6
*               Q2 < 0.01 GeV^2
*               Delta eta > 2.5
* A gap event is defined as one in which there is a region of rapidity of at
* least 1.4 units of rapidity containing no object of energy > MeV between
* the two highest ET jet axis
*               
* Call the routine by:-
* 
* Call hz02023(1001) - initialisation (direct)
* Call hz02023(1002) - filling histos (direct)
* Call hz02023(1003) - termination (direct)
* Call hz02023(2001) - initialisation (resolved)
* Call hz02023(2002) - filling histos (resolved)
* Call hz02023(2003) - termination (resolved)
*
* 1st digit: Process: 1 = direct, 2 = resolved
* 4th digit: Phase: 1 = initialisation, 2 = filling histos, 
*                   3 = termination
*
* Value for Ptmin is recommended to be set at 2.5 GeV (set in MC set up)
*
* Author : Brian Cox (Brian.Cox@cern.ch)
* Date   : 15 Jan 2002
*
* Reference : 
*
*
*****************************************************************************
*
* HERA Tuning defined commons
 
	IMPLICIT NONE
 
#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
#include "hzjetcmn.inc"
 
c *** ETGAP (figure 3a)
        REAL etgap(5),etgap_stat(5),etgap_sys_stat(5)
        DATA etgap/.122,.089,.141,.124,.054/
        DATA etgap_stat/.013,.006,.005,.003,.001/
        DATA etgap_sys_stat/.0,.0,.0,.0,.0/
	REAL etgap_bin(6)
	DATA etgap_bin/0.00,0.5,1.50,3.50,7.00,12.00/


c *** DELTA ETA (figure 3b)
        REAL dleta(4),deta_stat(4),deta_sys_stat(4)
        DATA dleta/1.72,1.16,0.67,0.17/
        DATA deta_stat/0.04,0.03,0.02,0.01/
        DATA deta_sys_stat/.0,.0,.0,.0/
	REAL deta_bin(5)
	DATA deta_bin/2.5,2.8,3.1,3.5,4.0/


c *** XGAMMA (figure 3b)
        REAL xgam(4),xgam_stat(4),xgam_sys_stat(4)
        DATA xgam/1.29,2.27,2.53,0.68/
        DATA xgam_stat/0.03,0.06,0.07,0.05/
        DATA xgam_sys_stat/.0,.0,.0,.0/
	REAL xgam_bin(5)
	DATA xgam_bin/0.3,0.6,0.75,0.90,1.00/

c *** XPROTON (figure 3b)
        REAL xpro(4),xpro_stat(4),xpro_sys_stat(4)
        DATA xpro/22.3,23.2,8.6,2.8/
        DATA xpro_stat/0.5,0.6,0.3,0.2/
        DATA xpro_sys_stat/.0,.0,.0,.0/
	REAL xpro_bin(5)
	DATA xpro_bin/0.02,0.04,0.06,0.08,0.10/

c *** DELTA ETA GAP FRACTIONS (figure 4)
	REAL dletag1(4),deta_statg1(4),deta_sys_statg1(4)
	REAL dletag2(4),deta_statg2(4),deta_sys_statg2(4)
	REAL dletag3(4),deta_statg3(4),deta_sys_statg3(4)
	REAL dletag4(4),deta_statg4(4),deta_sys_statg4(4)


        DATA dletag1/.050,.056,.025,.100/
        DATA deta_statg1/.006,.011,.009,.034/
        DATA deta_sys_statg1/.0,.0,.0,.0/

        DATA dletag2/.085,.079,.069,.101/
        DATA deta_statg2/.008,.011,.014,.029/
        DATA deta_sys_statg2/.0,.0,.0,.0/

        DATA dletag3/.137,.136,.099,.114/
        DATA deta_statg3/.010,.014,.014,.027/
        DATA deta_sys_statg3/.0,.0,.0,.0/

        DATA dletag4/.178,.167,.137,.189/
        DATA deta_statg4/.010,.014,.016,.034/
        DATA deta_sys_statg4/.0,.0,.0,.0/

c *** XGAMMA GAP FRACTIONS (figure 5)
	REAL xgamg1(4),xgam_statg1(4),xgam_sys_statg1(4)
	REAL xgamg2(4),xgam_statg2(4),xgam_sys_statg2(4)
	REAL xgamg3(4),xgam_statg3(4),xgam_sys_statg3(4)
	REAL xgamg4(4),xgam_statg4(4),xgam_sys_statg4(4)


        DATA xgamg1/.024,.046,.045,.277/
        DATA xgam_statg1/.007,.011,.008,.035/
        DATA xgam_sys_statg1/.0,.0,.0,.0/

        DATA xgamg2/.034,.060,.077,.462/
        DATA xgam_statg2/.007,.011,.010,.034/
        DATA xgam_sys_statg2/.0,.0,.0,.0/

        DATA xgamg3/.062,.079,.147,.623/
        DATA xgam_statg3/.009,.011,.013,.028/
        DATA xgam_sys_statg3/.0,.0,.0,.0/

        DATA xgamg4/.086,.098,.205,.712/
        DATA xgam_statg4/.011,.012,.013,.024/
        DATA xgam_sys_statg4/.0,.0,.0,.0/

c *** XPROTON GAP FRACTIONS (figure 6)
	REAL xprog1(4),xpro_statg1(4),xpro_sys_statg1(4)
	REAL xprog2(4),xpro_statg2(4),xpro_sys_statg2(4)
	REAL xprog3(4),xpro_statg3(4),xpro_sys_statg3(4)
	REAL xprog4(4),xpro_statg4(4),xpro_sys_statg4(4)


        DATA xprog1/.046,.044,.040,.034/
        DATA xpro_statg1/.007,.009,.016,.024/
        DATA xpro_sys_statg1/.0,.0,.0,.0/

        DATA xprog2/.102,.052,.054,.057/
        DATA xpro_statg2/.010,.008,.015,.024/
        DATA xpro_sys_statg2/.0,.0,.0,.0/

        DATA xprog3/.165,.083,.063,.057/
        DATA xpro_statg3/.012,.010,.014,.024/
        DATA xpro_sys_statg3/.0,.0,.0,.0/

        DATA xprog4/.202,.125,.118,.101/
        DATA xpro_statg4/.012,.011,.020,.033/
        DATA xpro_sys_statg4/.0,.0,.0,.0/


 
C --- cuts arrays
	DOUBLE PRECISION ycut(2),q2cut,detamax
	DOUBLE PRECISION etacut,Etcut(2),detacut,gapcut

C --- photoproduction cuts
        DATA q2cut/1.D-2/
	DATA ycut/0.3,0.65/
	DATA Etcut/6.,5./
	DATA etacut/2.65/
	DATA detacut/2.5/
	DATA detamax/4.0/
        
        CHARACTER*6 jetf

C --- information returned by jet finder
        INTEGER NJET,NJET_GOOD
        DOUBLE PRECISION Jet(maxhzjets,8)
        DOUBLE PRECISION CONER
	SAVE CONER

c ---
        DOUBLE PRECISION ye,q2,elecE,pflux
	INTEGER myil,myip
	INTEGER iproc, iflag, phase, chjet,chjetfind
	INTEGER hzflag

c ---   Analysis variables
	REAL rp(NMXHEP),en(NMXHEP),eppz,empz,rapmin,rapmax,dum,deta
	INTEGER Q
	REAL QR,rp2(NMXHEP),xp,eppzs,empzs,eppzj,empzj,xg,xg1
	INTEGER iobj,iloop,jloop,iround,iobj2,a,b,qcount,kloop,floop
	LOGICAL FLAG
	REAL et(NMXHEP),theta(NMXHEP),QET
	REAL profwgt(0:4),lumnb

	Character*6 xxxx
	DATA xxxx /'d02023'/

	eppzs=0.
	empzs=0.
	eppzj=0.
	empzj=0.
	
C       --- choose direct/resolved 
C       --- recommend +1000 - DIRECT, +2000 - RESOLVED, +3000 - BFKL
C       --- iproc = 1 --> DIRECT, iproc = 2 --> RESOLVED iproc = 3 --> BFKL 

	iproc=(mod(iflag,10000)/1000)
	if ((iproc.lt.0).or.(iproc.gt.3)) then
	   write(HZUNIT,*) 'HZ',xxxx,': illegal IPROC:',iproc
	   return
	endif

 
C --- choose jet finder
c fixed to ktclus for now.
	chjetfind=9
        chjet=9

C --- 'phase' should be equal to 1,2,3, defines 
C --- initialisation, processing, termination phase respectively
        
        phase = mod(iflag,10)

	IF (phase.eq.1) THEN
	   
           If (iproc.le.1) then

C *** Initialisation run ***
C *** Create directories ***
	      CALL HCDIR('//HISTO',' ')
	      CALL HMDIR(xxxx,'S')
	      CALL HCDIR('//PAWC',' ')
	      CALL HMDIR(xxxx,'S')
	      
	      CALL HCDIR('//HISTO/'//xxxx,' ')
	      CALL HCDIR('//PAWC/'//xxxx,' ')     

C       ---Book Histograms
	      CALL jetprofile(1.D0,1.D0,100,101,phase)
	      
C       ---Cross sections as function of etgap
	      CALL HBOOKB(-1,'DATA ETGAP',5,etgap_bin,0.)  

C       ---Cross sections as function of delta eta
	      CALL HBOOKB(-2,'DATA DETA',4,deta_bin,0.)  

C       ---Cross sections as function of xg
	      CALL HBOOKB(-3,'DATA XGAM',4,xgam_bin,0.)  

C       ---Cross sections as function of xp
	      CALL HBOOKB(-4,'DATA XPRO',4,xpro_bin,0.)  

C       ---Gap cross sections as a function of delta eta
	      CALL HBOOKB(-25,'DATA DETA ETCUT 0.5',4,deta_bin,0.)

	      CALL HBOOKB(-26,'DATA DETA ETCUT 1.0',4,deta_bin,0.)

	      CALL HBOOKB(-27,'DATA DETA ETCUT 1.5',4,deta_bin,0.)

	      CALL HBOOKB(-28,'DATA DETA ETCUT 2.0',4,deta_bin,0.)

C       ---Gap cross sections as a function of xg
	      CALL HBOOKB(-35,'DATA XGAM ETCUT 0.5',4,xgam_bin,0.)

	      CALL HBOOKB(-36,'DATA XGAM ETCUT 1.0',4,xgam_bin,0.)

	      CALL HBOOKB(-37,'DATA XGAM ETCUT 1.5',4,xgam_bin,0.)

	      CALL HBOOKB(-38,'DATA XGAM ETCUT 2.0',4,xgam_bin,0.)

C       ---Gap cross sections as a function of xp
	      CALL HBOOKB(-45,'DATA XPRO ETCUT 0.5',4,xpro_bin,0.)

	      CALL HBOOKB(-46,'DATA XPRO ETCUT 1.0',4,xpro_bin,0.)

	      CALL HBOOKB(-47,'DATA XPRO ETCUT 1.5',4,xpro_bin,0.)

	      CALL HBOOKB(-48,'DATA XPRO ETCUT 2.0',4,xpro_bin,0.)
	      
	      CALL HIDOPT(0,'stat')
	      CALL HBARX(0)

	      CALL HPAK(-1, etgap)
	      CALL HPAKE(-1,etgap_stat)
	      CALL HPAK(-2, dleta)
	      CALL HPAKE(-2,deta_stat)
	      
	      CALL HPAK(-25, dletag1)
	      CALL HPAKE(-25,deta_statg1)
	      
	      CALL HPAK(-26, dletag2)
	      CALL HPAKE(-26,deta_statg2)
	      
	      CALL HPAK(-27, dletag3)
	      CALL HPAKE(-27,deta_statg3)
	      
	      CALL HPAK(-28, dletag4)
	      CALL HPAKE(-28,deta_statg4)
		    
	      CALL HPAK(-3, xgam)
	      CALL HPAKE(-3,xgam_stat)
	      
	      CALL HPAK(-35, xgamg1)
	      CALL HPAKE(-35,xgam_statg1)
	      
	      CALL HPAK(-36, xgamg2)
	      CALL HPAKE(-36,xgam_statg2)
	      	      
	      CALL HPAK(-37, xgamg3)
	      CALL HPAKE(-37,xgam_statg3)

	      CALL HPAK(-38, xgamg4)
	      CALL HPAKE(-38,xgam_statg4)
	      
	      CALL HPAK(-4, xpro)
	      CALL HPAKE(-4,xpro_stat)
	      
	      CALL HPAK(-45, xprog1)
	      CALL HPAKE(-45,xpro_statg1)
	      
	      CALL HPAK(-46, xprog2)
	      CALL HPAKE(-46,xpro_statg2)
	      	      
	      CALL HPAK(-47, xprog3)
	      CALL HPAKE(-47,xpro_statg3)
	      
	      CALL HPAK(-48, xprog4)
	      CALL HPAKE(-48,xpro_statg4)
	      
	   endif
           
           CALL HCDIR('//HISTO/'//xxxx,' ')
           CALL HCDIR('//PAWC/'//xxxx,' ')

	   CALL HBOOKB(1000*iproc+1,'ETGAP',5,etgap_bin,0.) 
	   CALL HBOOKB(1000*iproc+2,'DETA',4,deta_bin,0.) 
	   CALL HBOOKB(1000*iproc+3,'XGAM',4,xgam_bin,0.) 
	   CALL HBOOKB(1000*iproc+4,'XPRO',4,xpro_bin,0.) 

	   CALL HBOOKB(1000*iproc+21,'DETA ETCUT 0.5',4,deta_bin,0.)
	   CALL HBOOKB(1000*iproc+22,'DETA ETCUT 1.0',4,deta_bin,0.)
	   CALL HBOOKB(1000*iproc+23,'DETA ETCUT 1.5',4,deta_bin,0.)
	   CALL HBOOKB(1000*iproc+24,'DETA ETCUT 2.0',4,deta_bin,0.)
	   CALL HBOOKB(1000*iproc+31,'XGAM ETCUT 0.5',4,xgam_bin,0.)
	   CALL HBOOKB(1000*iproc+32,'XGAM ETCUT 1.0',4,xgam_bin,0.)
	   CALL HBOOKB(1000*iproc+33,'XGAM ETCUT 1.5',4,xgam_bin,0.)
	   CALL HBOOKB(1000*iproc+34,'XGAM ETCUT 2.0',4,xgam_bin,0.)
	   CALL HBOOKB(1000*iproc+41,'XPRO ETCUT 0.5',4,xpro_bin,0.)
	   CALL HBOOKB(1000*iproc+42,'XPRO ETCUT 1.0',4,xpro_bin,0.)
	   CALL HBOOKB(1000*iproc+43,'XPRO ETCUT 1.5',4,xpro_bin,0.)
	   CALL HBOOKB(1000*iproc+44,'XPRO ETCUT 2.0',4,xpro_bin,0.)

	   
C       --- standard cone radius is 1.0 (for EUCELL)
C       --- cone radius not used for KTCLUS
           CALL HZJETRAD(2,CONER)
           IF (CONER.lt.0.0) THEN
              CONER=1.0
           ENDIF
           CALL HZJTNAME(chjetfind,jetf)
	   
	   WRITE(HZUNIT,*)
     &	         '**********************************************'
	   WRITE(HZUNIT,*)
     &  	 '* HZ02023 called, histograms will be output. *'
	   WRITE(HZUNIT,*)'*     Jet finder selected : ',jetf
     &  	,'           *'
	   WRITE(HZUNIT,1001)'*        with cone radius : ',CONER,'   *'
	   WRITE(HZUNIT,*)
     &  	'**********************************************'
	   
 1001	   FORMAT(A29,F5.2,A13)

	ELSE IF (phase.eq.2) THEN

C       *** Filling runs ***
C       *** Change directory to our graphs ***
	   
           CALL HCDIR('//PAWC/'//xxxx,' ')
           
           q2=hzphokin(1)
           IF (q2.eq.-1) THEN
              PRINT*,'HZ02023: HZPHOKIN returns error for Q2'
           ELSEIF (q2.lt.0) THEN
              q2=abs(q2)
           ENDIF
           ye=hzphokin(3)
           IF (ye.eq.-1) THEN
              PRINT*,'HZ02023: HZPHOKIN returns error for y'
           ENDIF
	   
           IF ((ye.ge.ycut(1)).and.(ye.le.ycut(2))
     &          .and.(q2.lt.q2cut)) then 
              elecE=27.5
              hzflag=HZIBEAM(myip,myil)
              IF (hzflag.ge.0) then
                 elecE=abs(PHEP(4,myil))
              ELSE
                 PRINT*,'HZ02023: Electron not found. E=0.001'
                 PRINT*,'         event will be rejected'
                 elecE=0.001
              ENDIF
	      
C       --- Call Jetfinder 
	      
              CALL hzjtfind(chjetfind,CONER,NJET,Jet)
	      
              IF (NJET.ge.2) THEN
		 
C       --- Make jet cuts
C       --- Jets are ordered in Et by hzjtfind
                 If (Jet(1,3).ge.Etcut(1).and.Jet(2,3).ge.Etcut(2).and
     &	      .Jet(1,1).lt.etacut.and.Jet(2,1).lt.etacut.and
     &	      .abs(Jet(1,1)-Jet(2,1)).gt.detacut.and.abs(Jet(1,1
     &	      )-Jet(2,1)).lt.detamax) then
		    profwgt(iproc)=profwgt(iproc)+wtx
C		    CALL jetprofile(jet(1,1),jet(1,2),100,101,phase)

		    deta=abs(Jet(1,1)-Jet(2,1))
C       --- order jets, a most forward
		    a=1
		    b=2
		    if (Jet(2,1).gt.Jet(1,1)) then
		       a=2
		       b=1
		    endif
                    CALL jetprofile(jet(a,1),jet(a,2),100,101,phase)
C       --- sum eppz and empz jets
		    eppzj=REAL(Jet(1,4)+Jet(1,7)+Jet(2,4)+Jet(2,7))
		    empzj=REAL(Jet(1,4)-Jet(1,7)+Jet(2,4)-Jet(2,7))

		    QET=0.
c loop over jets and sum energy of all jets inbetween the 2 highest et jets
		    do iloop = 3,NJET,1
		       if (Jet(iloop,1).gt.Jet(b,1) .and.
     &  		    Jet(iloop,1).lt.Jet(a,1)) then
			  QET=QET+REAL(Jet(iloop,3))
c			  write(*,*) Jet(iloop,1),Jet(iloop,3)
		       endif
		    enddo

C Fill etgap histograms

		    CALL HFILL(iproc*1000+1, QET ,0.,wtx)

C Fill delta eta histograms

		    CALL HFILL(iproc*1000+2, deta ,0.,wtx)
	
		    xp=REAL((jet(1,3)*exp(1*jet(1,1))+jet(2,3)
     &  		 *exp(1*jet(2,1))))
     &  		 /(2*820.0)
		    xg=REAL((jet(1,3)*exp(-1*jet(1,1))+jet(2,3)
     &  		 *exp(-1*jet(2,1))))
     &  		 /(2*ye*elecE)

C Fill xgamma histograms

		    CALL HFILL(iproc*1000+3, xg ,0.,wtx)


C Fill xproton histograms

		    CALL HFILL(iproc*1000+4, xp ,0.,wtx)
		    
C Fill gap histograms for QET < 0.5, 1.0, 1.5, 2.0 GeV

		    Do iloop = 1,4,1
		       if (QET.lt.(0.5*float(iloop))) then
C			  write(*,*) 0.5*float(iloop),QET
			  CALL HFILL(iproc*1000+20+iloop,deta,0.,wtx)
			  CALL HFILL(iproc*1000+30+iloop,xg,0.,wtx)
			  CALL HFILL(iproc*1000+40+iloop,xp,0.,wtx)
		       endif
		    enddo
		 endif		! jet cuts
	      endif		! njet > 2
	   endif		! y and Q^2 cuts
	   
	   
	ELSE IF (phase.eq.3) THEN

	   if (iproc.ne.1) then
	      write(hzunit,*) 'HZ02023: finishing process ',iproc
	   endif
	   
C       *** Termination run ***
C       *** Change directory ***
	   
	   CALL HCDIR('//PAWC/'//xxxx,' ')
	   
C       *** Finish off histograms ***
	   
	   IF (Xsec.eq.0) THEN
	      PRINT*
     &  	   ,'HZ02023: termination called with zero cross section'
	      PRINT*,'          cross section graph meaningless'
	      PRINT*,'          in process :',iproc*1000
	      Xsec=1
	   ENDIF
	   IF (Ntot.eq.0.0) THEN
	      PRINT*,'HZ02023: termination called with no total events'
	      PRINT*,'          cross section graph meaningless'
	      PRINT*,'          in process :',iproc*1000
	      Ntot=1.0
	   ENDIF
	   
c	   Print*,"HZ02023: iproc = ",iproc
c	   Print*,"HZ02023: Xsec = ",Xsec
c	   Print*,"HZ02023: Ntot = ",Ntot
	   
	   lumnb=ntot/xsec
	   CALL HZHINRM(1+iproc*1000,0,lumnb,1)
	   CALL HZHINRM(2+iproc*1000,0,lumnb,1)
	   CALL HZHINRM(3+iproc*1000,0,lumnb,1)
	   CALL HZHINRM(4+iproc*1000,0,lumnb,1)

c       add resolved & direct
	   if (iproc.eq.2) then
	      CALL HOPERA(1001,'+e',2001,1,1.,1.)
	      CALL HOPERA(1002,'+e',2002,2,1.,1.)
	      CALL HOPERA(1003,'+e',2003,3,1.,1.)
	      CALL HOPERA(1004,'+e',2004,4,1.,1.)
	   endif
c       add diff to resolved+direct
	   if (iproc.eq.3) then
	      CALL HOPERA(3001,'+e',1,1,1.,1.)
	      CALL HOPERA(3002,'+e',2,2,1.,1.)
	      CALL HOPERA(3003,'+e',3,3,1.,1.)
	      CALL HOPERA(3004,'+e',4,4,1.,1.)
	   endif

	   Do iloop=1,4,1
	      CALL HZHINRM(iproc*1000+20+iloop,0,lumnb,1)
	      CALL HZHINRM(iproc*1000+30+iloop,0,lumnb,1)
	      CALL HZHINRM(iproc*1000+40+iloop,0,lumnb,1)
	      
c       add resolved and direct
	      if (iproc.eq.2) then
		 CALL HOPERA(1020+iloop,'+e',2020+iloop,20+iloop
     &  	      ,1.,1.)
		 CALL HOPERA(1030+iloop,'+e',2030+iloop,30+iloop
     &  	      ,1.,1.)
		 CALL HOPERA(1040+iloop,'+e',2040+iloop,40+iloop
     &  	      ,1.,1.)
	      endif

c       add diff to resolved+direct
	      if (iproc.eq.3) then
		 CALL HOPERA(3020+iloop,'+e',20+iloop,20+iloop
     &  	      ,1.,1.)
		 CALL HOPERA(3030+iloop,'+e',30+iloop,30+iloop
     &  	      ,1.,1.)
		 CALL HOPERA(3040+iloop,'+e',40+iloop,40+iloop
     &  	      ,1.,1.)
	      endif
	   enddo

C       --- Divide to get gap fractions (Doesn't matter that this is
c       done more than once.... the last one counts.)

	   if (iproc.ne.1) then
	      Do iloop = 1,4,1
		 CALL HOPERA(20+iloop,'/e',2,24+iloop,1.,1.)
		 CALL HOPERA(30+iloop,'/e',3,34+iloop,1.,1.)
		 CALL HOPERA(40+iloop,'/e',4,44+iloop,1.,1.)
	      Enddo
	   endif

C       --- Delete histograms
	      
c	      do iloop=1,4,1
c		 CALL HDELET(iloop)
c		 do jloop=1,2,1
c		    CALL HDELET(jloop*1000+iloop)
c		    CALL HDELET(jloop*1000+20+iloop)
c		    CALL HDELET(jloop*1000+30+iloop)
c		    CALL HDELET(jloop*1000+40+iloop)
c		 enddo
c	      enddo

C       *** End ***
	ENDIF
	
	RETURN
	END
	

