        Subroutine HZ95033(iflag)
*****************************************************************************
* This photoproduction routine has to be run twice with the following
* code additions:
*       +1000 for the DIRECT component run.
*       +2000 for the RESOLVED component run.
*       +0    for both together
* for all three phases (iflag=1,2,3)
*
*      +10000 to run on parton showers instead of final state particles.
*
* iflag = 1,2 or 3 depending on wether it is the initialization,
*               processing or termination phase (respectively)
*
* The default jetfinder is EUCELL
* To change jetfinder use these values to call the initialisation phase.
* 1 + 10 * (jetfinder number)
*
* This routine produces two Monte Carlo graphs.
* id 10 is the normalised cross section in nb for x_gamma.ge.0.75.
* id 20 is the normalised cross section in nb for x_gamma.lt.0.75.
*               These graph will be
*               meaningless unless Xsec and Ntot are set before
*               calling the termination routine.
*               (Xsec - total cross section returned by MC)
*               (Ntot - number of events passed to this routine)
*
*     Cuts :        Look at the 2 highest ET jets between -1.375 and 1
c     .875 in eta 
*               Q2 < 4
*               0.2< y_bj < 0.8
*               /delta/eta < 0.5
*               Depending on graph Et > 6
*                       and -1 < eta < or
*                        0 < eta < 2
*
* Recommended value for Ptmin is 3 GeV (to be set in MC set up)
*
* Author : Mark Hayes 
* Contact: hztool@cedar.ac.uk
*
* Reference : DESY-95-033
*             Phys. Lett. B348 (1995) 665
*
*
*****************************************************************************
*
* HERA Tuning defined commons

        IMPLICIT NONE

#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
#include "hzjetcmn.inc"

        CHARACTER*6 jetf

        CHARACTER*6 xxxx
        DATA xxxx /'d95033'/

        INTEGER njet, i
c        DOUBLE PRECISION Jets(50,8)
        DOUBLE PRECISION CONER
        SAVE CONER

        DOUBLE PRECISION elecE
        INTEGER myil,myip
        INTEGER loop,loop2,count
        DOUBLE PRECISION deltaeta,etabar,xgam
        DOUBLE PRECISION phi(2),px(2),py(2),pz(2),ptsq(2)
        DOUBLE PRECISION ppsq(2),energy(2),mass(2)
        DOUBLE PRECISION rap(2),Et(2)
        DOUBLE PRECISION ybj,q2

C --- cuts arrays
        DOUBLE PRECISION ybjcut(2),q2cut(2)
        DOUBLE PRECISION rapcut(2),Etcut(5)
        SAVE YBJCUT, Q2CUT, ETCUT, RAPCUT

C *** Data for x_gamma.ge.0.75
        REAL eta(8),etacr(8),cresta(8)
        REAL cresys(8),band(8)
C *** Actual data for x_gamma.ge.0.75
        DATA eta/-0.75, -0.50, -0.25, 0.00, 0.25, 0.50, 0.75, 1.00/
        DATA etacr/0.63, 1.31, 1.32, 1.37, 1.29, 0.88, 0.50, 0.23/
        DATA cresta/0.08, 0.13, 0.14, 0.11, 0.11, 0.10, 0.06, 0.04/
        DATA cresys/0.16, 0.10, 0.08, 0.06, 0.17, 0.17, 0.27, 0.17/
        DATA band/0.08, 0.19, 0.20, 0.24, 0.15, 0.16, 0.06, 0.05/
C *** Data for x_gamma.lt.0.75
        REAL eta2(7),eta2cr(7),cre2sta(7)
        REAL cre2sys(7),band2(7)
C *** Actual data for x_gamma.lt.0.75
        DATA eta2/0.00, 0.25, 0.50, 0.75, 1.00, 1.25, 1.50/
        DATA eta2cr/1.74, 2.20, 2.64, 3.51, 3.83, 3.88, 4.32/
        DATA cre2sta/0.10, 0.13, 0.13, 0.16, 0.17, 0.16, 0.16/
        DATA cre2sys/0.22, 0.23, 0.33, 0.51, 0.63, 0.51, 0.66/
        DATA band2/0.66, 0.54, 0.59, 0.71, 0.66, 0.91, 0.99/

        INTEGER chjet, iproc, iflag
        SAVE chjet
C --- choose same jet finder as in the paper --- EUCELL
        DATA chjet/1/
        INTEGER hzflag

        REAL pttemp(12)
        INTEGER ivec,jvec
        REAL sqrt

	LOGICAL parton_showers
        
	parton_showers=.FALSE.
	IF (iflag/10000.gt.0) THEN
	   parton_showers=.TRUE.           
	ENDIF

*     Determine the process type.
        iproc=(mod(iflag,10000)/1000)
        if ((iproc.lt.0).or.(iproc.gt.2)) then
           write(HZUNIT,*) 'HZ',xxxx,': illegal IPROC:',iproc
           return
        endif

C --- choose jet finder
        IF ((MOD(iflag,100)/10.gt.0).and.MOD(iflag,10).eq.1) THEN
          chjet=INT(MOD(iflag,100)/10)
        ENDIF

	IF (parton_showers) THEN
	   chjet=-1*chjet
	ENDIF

        IF (MOD(iflag,10).eq.1) THEN
C *** Initialisation run ***
C *** Create directories ***
          IF (iproc.le.1) THEN
             CALL HCDIR('//HISTO',' ')
	     IF (parton_showers) THEN
		CALL HMDIR('ps'//xxxx,'S')
		CALL HCDIR('//PAWC',' ')
		CALL HMDIR('ps'//xxxx,'S')
	     ELSE
		CALL HMDIR(xxxx,'S')
		CALL HCDIR('//PAWC',' ')
		CALL HMDIR(xxxx,'S')
	     ENDIF

          ENDIF
          IF (parton_showers) THEN
             CALL HCDIR('//HISTO/ps'//xxxx,' ')
             CALL HCDIR('//PAWC/ps'//xxxx,' ')
          ELSE
             CALL HCDIR('//HISTO/'//xxxx,' ')
             CALL HCDIR('//PAWC/'//xxxx,' ')
          ENDIF
          
          CALL HBOOK1(iproc*100+10,
     &         'Monte Carlo cross-section component x_gam ge 0.75'
     &         ,8 ,-0.875 ,1.125 ,0.)
          CALL HBOOK1(iproc*100+20,
     &         'Monte Carlo cross-section component x_gam lt 0.75'
     &         ,7 ,-0.125 ,1.625 ,0.)
          
	   CALL HIDOPT(0,'stat')
	   CALL HBARX(0)
           
C --- photoproduction cuts

          ybjcut(1)=0.2
          ybjcut(2)=0.8
          q2cut(1)=0.
          q2cut(2)=4.

C --- basic cuts for rapidity plots
          Etcut(1)=6.0
          rapcut(1)=-1.125
          rapcut(2)= 1.875

C --- standard cone radius is 1.0
          CALL HZJETRAD(2,CONER)
          IF (CONER.lt.0.0) THEN
            CONER=1.0
          ENDIF
          CALL HZJTNAME(chjet,jetf)

          WRITE(6,*)'**********************************************'
          WRITE(6,*)'* HZ95033 called, histograms will be output. *'
          WRITE(6,*)'*     Jet finder selected : ',jetf,'           *'
          WRITE(6,1001)'*        with cone radius : ',CONER,'   *'
          WRITE(6,*)'**********************************************'

 1001     FORMAT(A29,F5.2,A13)

        ELSE IF (MOD(iflag,10).eq.2) THEN
C *** Filling runs ***
        
C *** Change directory to our graphs ***
	   IF (parton_showers) THEN              
              CALL HCDIR('//PAWC/psd95033',' ')
           ELSE
              CALL HCDIR('//PAWC/d95033',' ')
           ENDIF

          q2=hzphokin(1)
          IF (q2.eq.-1) THEN
            PRINT*,'HZ95033: HZPHOKIN returns error for Q2'
          ELSEIF (q2.lt.0) THEN
            q2=abs(q2)
          ENDIF
          ybj=hzphokin(3)
          IF (ybj.eq.-1) THEN
            PRINT*,'HZ95033: HZPHOKIN returns error for ybj'
          ENDIF

C *** Fill our histogram ***

          IF ((ybj.ge.ybjcut(1)).and.(ybj.le.ybjcut(2))
     &          .and.(q2.lt.q2cut(2))) Then

          elecE=26.7
          hzflag=HZIBEAM(myip,myil)
          IF (hzflag.ge.0) Then
            elecE=abs(PHEP(4,myil))
          ELSE
            PRINT*,'HZ95033: Electron not found. E=0.001'
            PRINT*,'         event will be rejected'
            elecE=0.001
          ENDIF
        
            CALL hzjtfind(chjet,CONER,NJET,Jets)

            DO i=1,2
               et(i)=0.0
            ENDDO

c     Make sure we find the highest ET jets which are within the
c     rapidity interval.
            IF (NJET.ge.2) THEN
               i=0
               DO loop=1,njet
c     Search jets in the rapidity interval.
                  IF ( jets(loop,1).gt.rapcut(1).and.jets(loop,1).lt
     &                 .rapcut(2) ) THEN
                     i=i+1
                     IF (i.LE.2) THEN
                        Et(i)=Jets(loop,3)
                        rap(i)=Jets(loop,1)
                     ENDIF
                  ENDIF
               ENDDO

              deltaeta=abs(rap(1)-rap(2))
              etabar=(rap(1)+rap(2))/2
              xgam=(Et(1)*exp(-1*rap(1))+Et(2)*exp(-1*rap(2)))
     &                  /(2*ybj*elecE)

              IF ((Et(2).gt.Etcut(1)).and.(deltaeta.lt.0.5)) THEN
                IF (xgam.ge.0.75) THEN
                 CALL HFILL(INT(iproc*100+10), REAL(etabar) , 0. ,wtx)
                ELSE
                 CALL HFILL(INT(iproc*100+20), REAL(etabar) , 0. ,wtx)
                ENDIF
              ENDIF
            ENDIF

          ENDIF         ! Q2 & y cuts

        ELSE IF (MOD(iflag,10).eq.3) THEN
C *** Termination run ***
C *** Change directory ***
	   IF (parton_showers) THEN
              CALL HCDIR('//PAWC/ps'//xxxx,' ')
           ELSE
              CALL HCDIR('//PAWC/'//xxxx,' ')
           ENDIF

C *** Finish off histograms ***

          IF (Xsec.eq.0) THEN
            PRINT*,'HZ95033: termination called with zero cross section'
            PRINT*,'          cross section graph meaningless'
            PRINT*,'          in process :',iproc*1000
            Xsec=1
          ENDIF
          IF (Ntot.eq.0) THEN
            PRINT*,'HZ95033: termination called with no total events'
            PRINT*,'          cross section graph meaningless'
            PRINT*,'          in process :',iproc*1000
            Ntot=1
          ENDIF

C --- normalise resolved/direct component histograms
          CALL HZHINRM(100*iproc+10,0,REAL(ntot/xsec),1)
          CALL HZHINRM(100*iproc+20,0,REAL(ntot/xsec),1)

          IF (iproc.eq.2.or.iproc.eq.0) THEN
C --- only finally
C --- create proper cross-section data
            write (HZUNIT,*) 'HZ95033: finishing off'


            CALL HBOOK1(-10,'Data cross section (nb) x_gam ge 0.75'
     &          ,8 ,-0.875 ,1.125 ,0.)
            CALL HBOOK1(-20,'Data cross section (nb) x_gam lt 0.75'
     &          ,7 ,-0.125 ,1.625 ,0.)

            CALL HPAK(-10,etacr)
            CALL HPAKE(-10,cresta)
            CALL HPAK(-20,eta2cr)
            CALL HPAKE(-20,cre2sta)

            CALL HBOOK1(-11,'Data full errors (nb) x_gam ge 0.75'
     &          ,8 ,-0.875 ,1.125 ,0.)
            CALL HBOOK1(-21,'Data full errors (nb) x_gam lt 0.75'
     &          ,7 ,-0.125 ,1.625 ,0.)

            CALL HPAK(-11,etacr)
            DO ivec=1,8
              pttemp(ivec)=sqrt(cresta(ivec)**2+cresys(ivec)**2)
            ENDDO
            CALL HPAKE(-11,pttemp)
            CALL HPAK(-21,eta2cr)
            DO ivec=1,7
              pttemp(ivec)=sqrt(cre2sta(ivec)**2+cre2sys(ivec)**2)
            ENDDO
            CALL HPAKE(-21,pttemp)

            IF (iproc.eq.2) THEN
C     --- now calculate cross-section graphs 10,20
               CALL HOPERA(110,'+e',210,10,1.,1.)
               CALL HOPERA(120,'+e',220,20,1.,1.)
c               CALL HZCHISQ(-10,10)
c               CALL HZCHISQ(-20,20)
            ENDIF

         ENDIF


        ELSE
C *** End ***
          PRINT*,'HZ95033:Please run routine with iflag set to 1,2 or 3'
          PRINT*,'        with +1000 or +2000 for DIR or RES events.'
        ENDIF

        RETURN
        END
