*CMZ :  1.01/15 18/04/98  21.01.57  by  Tancredi Carli
*CMZ :  1.01/11 30/12/97  09.34.09  by  Tancredi Carli
*CMZ :  1.01/07 13/10/97  12.12.22  by  Tancredi Carli
*CMZ :  1.01/05 25/11/96  19.42.25  by  Tancredi Carli
*-- Author :    Tancredi Carli   03/09/96
*-- Author :
      Subroutine hz96122(iflag)
******************************************************************************
*                                                                            *
* Author: Birger Koblitz (koblitz@dice2.desy.de)                             *
*                                                                            *
* This routine plots angular and pT2 distributions of k0 and lambda          *
* particles. Additionaly the mean number of K0s and this number divided by   *
* the mean number of Tracks is plotted with respect to x in three Q2 bins:   *
* 10GeV < Q2 < 15GeV, 15GeV < Q2 < 35GeV, 35GeV <Q2< 70GeV                   *
* At last the seagull plot is drawn in the hadronic center of mass system    *
* with mean pT2 over x_F                                                     *
*                                                                            *
* Beams: 27.6 GeV electrons, 820 GeV protons   [1994 HERA running]           *
*                                                                            *
* Event selection:                                                           *
* 10GeV2 < Q2 < 70 GeV2                                                      *
* 1E-4 < x < 0.01                                                            *
* 0.05 < y < 0.6                                                             *
* Energie deposited in polar angular range 4.4deg < theta < 15deg > 0.5 GeV  *
*                                                                            *
* Reference:                                                                 *
* Modified:                                                                  *
******************************************************************************
      implicit none
      integer iflag
*
*        HERA tuning defined commons:
*
#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
#include "hzhbook.inc"
*
* Local variables
*
      Real                ee,eb
      Parameter           (ee=27.6)
      Character           xxxx*5
      Double Precision    w2,q2,eta,theta,pt,Xkin,Ykin,pt2,xF
      Double precision    pbeam(4), pgam(4),pgam4(4)
      Double Precision    plab(4),pcms(4)
      Integer             IErr
      Integer             ebid,pbid,lepid,ip
      Integer             HId,I,J,N
      Real                NAcc,NAcc2,NTotal
      Real                efwd,rd,pi,NormC
      Integer             EvInB(6)
      Real                a(6),e(6),s(6)

      Parameter           (pi=3.1415927,rd=180./pi)
* CUTS:
      Double Precision    q2cut(2), w2cut(2)
*
* Data points from plots:
*
*
      Real etak(8),etakt(8),etake(8)
      Real etal(4),etalt(4),etale(4)
      Real ptsl(4),ptslt(4),ptsle(4)
      Real seabin(5),logb1(5),logb2(5),logb3(4)
      Real seadat(4),seaerr(4)
      Real k0dat(8),k0err(8)

      Data xxxx/'96122'/

      Data etal/0.013405,0.0137718,0.0257133,0.028401/
      Data etalt/0.0046255,0.00363401,0.00593823,0.0056646/
      Data etale/ 0.00692702,0.00530095,0.00874126,0.00844715/

      Data ptsl/0.0334636,0.00766033,0.00394519,0.00572521/
      Data ptslt/0.0058265,0.00209549,0.000841117,0.00151743/
      Data ptsle/0.00876668,0.00304164,0.00129237,0.00226776/

      Data etak/0.0676159,0.0970571,0.102679,0.0999274,0.104132,
     &          0.117154,0.142281,0.12515/
      Data etakt/0.0062,0.0075,0.0078,0.0074,0.0077,0.0082,
     &          0.0093,0.0091/
      Data etake/0.0105,0.0141,0.0147,0.0140,0.0146,0.0163,
     &          0.0196,0.0182/


      Data seabin/0.05,0.1,0.15,0.2,0.4/

      Data logb1/2.7740E-4,5.1299E-4,9.4868E-04,1.7544E-3,3.2445E-3/
      Data logb2/5.1299E-4,9.4868E-04,1.7544E-3,3.2445E-3,6E-3/
      Data logb3/9.4868E-04,1.7544E-3,3.2445E-3,6E-3/
      Data seadat/0.47,0.84,0.85,1.70/
      Data seaerr/0.11,0.38,0.28,0.43/

      Data k0dat/0.05,0.11,0.11,0.16,0.14,0.14,0.18,0.11/
      Data k0err/0.01,0.02,0.02,0.02,0.02,0.02,0.04,0.02/

      save NTotal, NAcc, NAcc2, EvInB, q2cut

*
* INITIALISATION
*
      if (iflag.eq.1) then
*
        NTotal=0.
        NAcc = 0.
        NAcc2= 0.
        do I=1,6
          EvInB(I)=0
        end do
*
	call hcdir('//PAWC',' ')
        call hmdir(xxxx,'S')
        Call hcdir('//HISTO',' ')
        call hmdir(xxxx,'S')
*
* Book histos:
*
        call Hbook1(100, 'eta-spectrum of K0', 8, -1.3,1.3, 0.)
        call Hbook1(101, 'p_T2-spectrum of k0',8,0.25,4.5,0.)
        call Hbook1(110, 'eta-spectrum of lambda', 4, -1.3,1.3, 0.)
        call Hbook1(111, 'p_T2-spectrum of lambda',4,0.25,4.5,0.)
*
        call HbookB(120, '<K0> in 10<Q2<15',4,logb1,0.)
        call HbookB(121, '<K0> in 15<Q2<35',4,logb2,0.)
        call HbookB(122, '<K0> in 35<Q2<70',3,logb3,0.)
*
        call HbookB(130, '<K0>/<Tracks> in 10<Q2<15',4,logb1,0.)
        call HbookB(131, '<K0>/<Tracks> in 15<Q2<35',4,logb2,0.)
        call HbookB(132, '<K0>/<Tracks> in 35<Q2<70',3,logb3,0.)
*
        call HbookB(900, '<Tracks> in 10<Q2<15',4,logb1,0.)
        call HbookB(901, '<Tracks> in 15<Q2<35',4,logb2,0.)
        call HbookB(902, '<Tracks> in 35<Q2<70',3,logb3,0.)

        call HbookB(910, 'norm 10<Q2<15',4,logb1,0.)
        call HbookB(911, 'norm 15<Q2<35',4,logb2,0.)
        call HbookB(912, 'norm 35<Q2<70',3,logb3,0.)

        call HbookB(200, 'seagull',4,seabin,0.)
        call HbookB(10, 'seagull-norm',4,seabin,0.)

        call HBook1(300,'K0 eta spektrum (Instantons)',8, -1.3,1.3, 0.)
*
* set cuts:
*
        q2cut(1) = 10.
        q2cut(2) = 70.
      end if
*
* ANALYSIS:
*
      if(iflag.eq.2) then
*
        call hcdir('//PAWC',' ')
        call hcdir(xxxx,' ')
*
*  Check electron beam energy
*
        NTotal=NTotal+wtx
        ierr=HZIBEAM(pbid,ebid)
        if (.not.(ierr.eq.1)) then
          write(6,*) 'HZ'//xxxx,' Beams not found ! '
          return
        endif
        eb=real(PHEP(4,ebid))
        if (abs(abs(eb)-ee).ge.0.2) then
          if (NTotal.lt.10) then
            write(6,*) 'HZ'//xxxx,' Electron beam not at ',ee,'  ! '
          elseif (NTotal.eq.10.) then
            write(6,*) 'HZ'//xxxx,' Electron beam not at ',ee,
     &   ' ...last message ! '
          endif
C          return
        endif
*
* Event Selection:
*
*
         q2=hzdiskin(1)
         w2=hzdiskin(4)
         XKin=HZDISKIN(2)
         Ykin=HZDISKIN(3)
*
         if (q2.gt.q2cut(1).and.q2.lt.q2cut(2)
     &       .and.Xkin.gt.1.D-4.and.Xkin.lt.0.01
     &       .and.Ykin.gt.0.05.and.Ykin.lt.0.6) then
*
* INITIALISE BOOST:
*
           if (Hzipgam(pgam).lt.0) then
             write (6,*) 'Photon momentum not found'
             return
           endif
           lepid=hzidelec(0)
           if (lepid.lt.0) then
             write (6,*) 'Id of electron not found'
             return
           endif
           do J=1,4
             pbeam(J) = PHEP(J,pbid)
             pgam4(J) = pgam(J)
           end do
           call HZHCMINI(pbeam,pgam4,IErr)
           if(IErr.eq.1) then
             write (6,*) ' HZHCMINIT failed'
             return
           endif
*
* Fill histos:
*
*      Cut on forward energie
           efwd=0.
           do I=1,NHEP
             if (isthep(I).eq.1.and.I.ne.lepid) then
               theta=real(HZPHMANG(PHEP(3,I),
     &              sqrt(PHEP(1,I)**2+PHEP(2,I)**2)))*rd
               if(theta.gt.4.4.and.theta.lt.15.) then
                 efwd=real(PHEP(4,I))+efwd
               endif
             endif
           enddo
*      Cut on positron angle
           theta=real(HZPHMANG(PHEP(3,lepid),
     &              sqrt(PHEP(1,lepid)**2+PHEP(2,lepid)**2)))*rd
           if (efwd.ge.0.5) then
             Nacc = Nacc + wtx
*    Fill in normalization Histograms for <K0>
             if(q2.lt.15)then
               call HF1(910,sngl(Xkin),wtx)
             elseif(q2.lt.35)then
               call HF1(911,sngl(Xkin),wtx)
             else
               call HF1(912,sngl(Xkin),wtx)
             endif


             do I = 1,Nhep
*     Get particle kinematics
               do J=1,4
                 plab(J) = phep(J,I)
               end do
               pt2=plab(1)**2+plab(2)**2
               pt=sqrt(pt2)
               theta= hzphmang(plab(3),pt)
               if (theta.gt.1.e-4 .and. theta.lt.pi-1.e-4) then
                eta= -1.*log(tan(theta/2.))
               else
                eta=-999
               endif
*
*     Count tracks
               if(I.ne.lepid.and.ISTHEP(I).eq.1
     &            .and.IHCHRG(I).ne.0.and.
     &            sqrt(pt2).gt.0.15.and.
     &           eta.gt.-1.3.and.eta.lt.1.3)then
*
                 if(q2.lt.15.)then
                   call HF1(900,sngl(Xkin),wtx)
                 elseif(q2.lt.35.)then
                   call HF1(901,sngl(Xkin),wtx)
                 else
                   call HF1(902,sngl(Xkin),wtx)
                 endif
               endif
*
*     Deal with lambdas
               if(abs(idhep(I)).eq.3122) then
                 if (pt2.gt.0.25.and.pt2.lt.4.5)then
                   call HF1(110,sngl(eta),wtx)
                 endif
                 if (eta.gt.-1.3.and.eta.lt.1.3)then
                   call HF1(111,sngl(pt2),wtx)
                 endif
               endif
*
*     Deal with K0
               if(abs(idhep(I)).eq.130.or.
     &            abs(idhep(I)).eq.310) then
                 if (pt2.gt.0.25.and.pt2.lt.4.5)then
                   call HF1(100,sngl(eta),wtx)
                   if(eta.gt.-1.3.and.eta.lt.1.3)then
                     if(q2.lt.15.)then
                       call HF1(120,sngl(Xkin),wtx)
                     elseif(q2.lt.35.)then
                       call HF1(121,sngl(Xkin),wtx)
                     else
                       call HF1(122,sngl(Xkin),wtx)
                     endif
                   endif
                 endif
                 if (eta.gt.-1.3.and.eta.lt.1.3)then
                   call HF1(101,sngl(pt2),wtx)
                 endif
                 call HZHCM(plab, pcms, Ierr)
                 if(Ierr.eq.1)  then
                   write (6,*) ' HZHCM BOOST failed'
                   return
                 endif
                 pt2=pcms(1)**2+pcms(2)**2
                 xF=2.*pcms(3)/sqrt(W2)
                 call HF1(200,sngl(xF),sngl(pt2)*wtx)
                 call HF1(10,sngl(xF),wtx)
               endif
             end do
           endif
         endif
*
         if (q2.gt.q2cut(1).and.q2.lt.q2cut(2)
     &       .and.Xkin.gt.1.D-3.and.Xkin.lt.0.01
     &       .and.Ykin.gt.0.1.and.Ykin.lt.0.6) then
*
*      Cut on forward energie
           efwd=0.
           do I=1,NHEP
             if (isthep(I).eq.1.and.I.ne.lepid) then
               theta=real(HZPHMANG(PHEP(3,I),
     &              sqrt(PHEP(1,I)**2+PHEP(2,I)**2)))*rd
               if(theta.gt.4.4.and.theta.lt.15.) then
                 efwd=real(PHEP(4,I))+efwd
               endif
             endif
           enddo
*
           if (efwd.ge.0.5) then
             Nacc2 = Nacc2 + wtx
             do I = 1,Nhep
*     Get particle kinematics
               do J=1,4
                 plab(J) = phep(J,I)
               end do
               pt2=plab(1)**2+plab(2)**2
               pt=sqrt(pt2)
               theta= hzphmang(plab(3),pt)
               if (theta.gt.1.e-4 .and. theta.lt.pi-1.e-4) then
                eta= -1.*log(tan(theta/2.))
               else
                eta=-999.
               endif
               if(abs(idhep(I)).eq.130.or.
     &            abs(idhep(I)).eq.310) then
                 if (pt2.gt.0.25.and.pt2.lt.4.5)then
                   call HF1(300,sngl(eta),wtx)
                 endif
               endif
             enddo
           endif
         endif
       endif
*
* TERMINATION:
*
      if(iflag.eq.3) then
*
        call hcdir('//PAWC',' ')
        call hcdir(xxxx,' ')
*
C        write (6,*) 'Hz',xxxx,': number accepted = ',Nacc
*
*
* Deal with data plots
*
        call Hbook1(-100, 'eta-spectrum of K0', 8, -1.3,1.3, 0.)
        call HPAK(-100,etak)
        call HPAKE(-100,etake)
        call Hbook1(-110, 'eta-spectrum of lambda', 4, -1.3,1.3, 0.)
        call Hbook1(-111, 'p_T^2-spectrum of lambda',4,0.25,4.5,0.)
        call HPAK(-110,etal)
        call HPAKE(-110,etale)
        call HPAK(-111,ptsl)
        call HPAKE(-111,ptsle)
        call HbookB(-200, 'seagull',4,seabin,0.)
        call HPAK(-200,seadat)
        call HPAKE(-200,seaerr)
        call HBook1(-300,'K0 eta spektrum (Instantons)',8,-1.3,1.3,0.)
        call HPAK(-300,k0dat)
        call HPAKE(-300,k0err)
*
*
        if(NAcc.eq.0..or.NAcc2.eq.0.)then
          write(6,*)'Error : Number of accepted events is 0!'
        else
* Normalize eta-spectrums
          call HOpera(100,'+E',100,100,1.0/Nacc/.325,0.0)
          call HOpera(110,'+E',110,110,1.0/Nacc/0.65,0.0)
          call HOpera(300,'+E',300,300,1.0/Nacc2/.325,0.0)
          call hzhinfo(100,nacc)
          call hzhinfo(110,nacc)
          call hzhinfo(300,nacc2)
*
* Normalize p_T^2-spectrums
          call HOpera(101,'+E',101,101,1.0/Nacc/.53125,0.0)
          call HOpera(111,'+E',111,111,1.0/Nacc/1.0625,0.0)
          call hzhinfo(101,nacc)
          call hzhinfo(111,nacc)
*
* Normalize <K0>
          call HOpera(120,'/E',910,120,1.0,1.0)
          call HOpera(121,'/E',911,121,1.0,1.0)
          call HOpera(122,'/E',912,122,1.0,1.0)

*
* Make histograms <K0>/<N_Tracks>
*
          do j=0,2
            call HOpera(900+j,'/E',910+j,900+j,1.0,1.0)
            call HOpera(120+j,'/E',900+j,130+j,1.0,1.0)
          enddo
*
* Normalize seagull
*
          call HOpera(200,'/E',10,200,1.0,1.0)
*
* Calculate Chi2
*
          call HZChiSQ(100,-100)
          call HZChiSQ(110,-110)
          call HZChiSQ(111,-111)
          call HZChiSQ(200,-200)
          call HZChiSQ(300,-300)
        Endif
*
      endif
*
      Return
      End
