      Subroutine HZ97196(iflag)
*****************************************************************************
* This photoproduction routine has to be run twice with the following
* code additions:
*       +1000 for the DIRECT component run.
*       +2000 for the RESOLVED component run.
*       +0 for both together.
* for all three phases (iflag=1,2,3)
*
*      +10000 to run on parton showers instead of final state particles.
*
* iflag = 1,2 or 3 depending on wether it is the initialization,
*               processing or termination phase (respectively)
*
* The default jetfinder is KTCLUS
* To change jetfinder use these values to call the initialisation phase.
* 1 + 10 * (jetfinder number)
*
* This routine produces eight Monte Carlo graphs:
*               graphs 10,11,12,13 are for xg<0.75 for ET>6,8,11,15 resp.
*               graphs 20,21,22,23 are for xg<0.75 for ET>6,8,11,15 resp.
* It also outputs the corresponding data points (stat+sys errors)
*               graphs -10,-11,-12,-13,-20,-21,-22,-23
*
* Cuts :        Q2 < 4 GeV^2
*               0.2 < y_bj < 0.8
*               Et > 6,8,11,15 GeV
*               |delta_eta| < 0.5
*
* Recommended value for Ptmin is 2.5 GeV (to be set in MC set up)
*
* Author : Mark Hayes (mhayes@zow.desy.de)
*          JMB 2/6/99 Added the all xgamma plot.
*
* Reference : DESY-97-196
*             Eur. Phys. J. C 1 (1998) 1/2, 109-122 hep-ex/9710018
*
*****************************************************************************
*
* HERA Tuning defined commons

      IMPLICIT NONE

#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
#include "hzjetcmn.inc"
      
      Character*6 xxxx
      DATA xxxx /'d97196'/


c *** a is >6, b>8, c>11, d>15
c *** Data for DIRECT xg>0.75
      REAL cs1a(9),sta1a(9)
      DATA cs1a/0.22,0.66,1.12,1.32,1.48,1.46,1.05,0.49,0.22/
      DATA sta1a/0.03,0.05,0.07,0.07,0.07,0.07,0.06,0.04,0.03/
      REAL sys1a(9)
      DATA sys1a/0.12,0.10,0.22,0.15,0.22,0.13,0.11,0.09,0.05/
      REAL cs1b(8),sta1b(8)
      DATA cs1b/0.06,0.36,0.55,0.56,0.60,0.60,0.42,0.22/
      DATA sta1b/0.01,0.04,0.05,0.04,0.04,0.04,0.04,0.03/
      REAL sys1b(8)
      DATA sys1b/0.06,0.03,0.09,0.08,0.05,0.05,0.08,0.04/
      REAL cs1c(4),sta1c(4)
      DATA cs1c/0.19,0.22,0.20,0.04/
      DATA sta1c/0.02,0.02,0.02,0.01/
      REAL sys1c(4)
      DATA sys1c/0.04,0.03,0.03,0.01/
      REAL cs1d(4),sta1d(4)
      DATA cs1d/0.033,0.082,0.078,0.035/
      DATA sta1d/0.008,0.012,0.011,0.007/
      REAL sys1d(4)
      DATA sys1d/0.006,0.014,0.014,0.009/
c     *** Data for RESOLVED 0.3<xg<0.75
      REAL cs2a(8),sta2a(8)
      DATA cs2a/0.56,0.89,1.35,1.47,1.49,1.16,0.57,0.26/
      DATA sta2a/0.05,0.06,0.07,0.07,0.08,0.07,0.05,0.03/
      REAL sys2a(8)
      DATA sys2a/0.20,0.24,0.33,0.19,0.36,0.15,0.14,0.07/
      REAL cs2b(8),sta2b(8)
      DATA cs2b/0.09,0.25,0.37,0.45,0.59,0.55,0.31,0.23/
      DATA sta2b/0.02,0.03,0.03,0.04,0.05,0.05,0.03,0.03/
      REAL sys2b(8)
      DATA sys2b/0.08,0.04,0.06,0.09,0.25,0.07,0.09,0.05/
      REAL cs2c(3),sta2c(3)
      DATA cs2c/0.08,0.16,0.11/
      DATA sta2c/0.01,0.02,0.01/
      REAL sys2c(3)
      DATA sys2c/0.03,0.06,0.03/
      REAL cs2d(3),sta2d(3)
      DATA cs2d/0.013,0.046,0.033/
      DATA sta2d/0.004,0.008,0.006/
      REAL sys2d(3)
      DATA sys2d/0.009,0.026,0.006/
c     *** Data for ALL xg
      REAL cs3a(11),sta3a(11)
      DATA cs3a/0.22,0.75,1.44,1.86,2.29,2.90,2.92,2.91,2.80,2.87,2.74/
      DATA sta3a/0.02,0.05,0.08,0.08,0.09,0.10,0.10,0.11,0.11,0.12,0.11/
      REAL sys3a(11)
      DATA sys3a/0.12,0.16,0.17,0.32,0.38,0.66,0.55,0.55,0.47,0.49,0.98/

      REAL cs3b(10),sta3b(10)
      DATA cs3b/0.06,0.38,0.65,0.81,0.97,1.07,1.16,0.99,0.82,0.71/
      DATA sta3b/0.01,0.04,0.05,0.05,0.06,0.06,0.06,0.06,0.06,0.05/
      REAL sys3b(10)
      DATA sys3b/0.06,0.02,0.08,0.09,0.11,0.17,0.20,0.12,0.12,0.08/

      REAL cs3c(4),sta3c(4)
      DATA cs3c/0.21,0.29,0.37,0.23/
      DATA sta3c/0.02,0.02,0.02,0.02/
      REAL sys3c(4)
      DATA sys3c/0.02,0.02,0.06,0.02/

      REAL cs3d(4),sta3d(4)
      DATA cs3d/0.033,0.093,0.126,0.079/
      DATA sta3d/0.008,0.012,0.014,0.010/
      REAL sys3d(4)
      DATA sys3d/0.005,0.036,0.027,0.016/
      Integer chjet
      SAVE chjet
C     --- choose same jet finder as in the paper --- KTCLUS
      DATA chjet/3/
      CHARACTER*6 jetf
      
C     --- information returned by jet finder
      INTEGER NJET
c      DOUBLE PRECISION Jets(50,8)
      DOUBLE PRECISION CONER
      SAVE CONER
      
C     --- cuts arrays
      DOUBLE PRECISION ybjcut(2),q2cut(2)
      DOUBLE PRECISION rapcut(2),Etcut(4)
C     --- photoproduction cuts
      DATA ybjcut/0.2,0.8/
      DATA q2cut/0.0,4/
      DATA Etcut/6.0,8.0,11.0,15.0/
      DATA rapcut/-1.375,1.875/
      
C     --- private copies of Xsec and Ntot from HERACMN
      DOUBLE PRECISION mhxsec(2),mhntot(2)
      
c     ---
      DOUBLE PRECISION elecE
      INTEGER myil,myip,loop,graph
      DOUBLE PRECISION deltaeta,etabar,xgam
      DOUBLE PRECISION rap(2),Et(2),phi(2)
      DOUBLE PRECISION ybj,q2
      INTEGER iproc, iflag
      INTEGER hzflag
      
      REAL pttemp(12), lumnb
      INTEGER ivec,jvec
      REAL sqrt
      
      LOGICAL parton_showers
      
      parton_showers=.FALSE.

*     Determine the process type.
      iproc=(mod(iflag,10000)/1000)
      if ((iproc.lt.0).or.(iproc.gt.2)) then
         write(HZUNIT,*) 'HZ',xxxx,': illegal IPROC:',iproc
         return
      endif

*     --- choose jet finder
      IF ((MOD(iflag,100)/10.gt.0).and.MOD(iflag,10).eq.1) THEN
         chjet=INT(MOD(iflag,100)/10)
      ENDIF

*     Decide whether we are running on parton showers or final state particles.
      IF (MOD(iflag,100000)/10000.gt.0) THEN
         parton_showers=.TRUE.
*     use a different paw directory for parton showers.
         chjet=-1*ABS(chjet)
      ENDIF

      IF (MOD(iflag,10).eq.1) THEN
C     *** Initialisation run ***
C     *** Create directories ***
         IF (iproc.le.1) THEN
            CALL HCDIR('//HISTO',' ')
            IF (parton_showers) THEN
               CALL HMDIR('ps'//xxxx,'S')
               CALL HCDIR('//PAWC',' ')
               CALL HMDIR('ps'//xxxx,'S')
            ELSE
               CALL HMDIR(xxxx,'S')
               CALL HCDIR('//PAWC',' ')
               CALL HMDIR(xxxx,'S')
            ENDIF

C     --- create proper cross-section data
            CALL HBOOK1(-10,'DATA etabar DIR ET.gt.6'
     &          ,9, -1.125, 1.125, 0.)
            CALL HBOOK1(-11,'DATA etabar DIR ET.gt.8'
     &          ,8, -0.875, 1.125, 0.)
            CALL HBOOK1(-12,'DATA etabar DIR ET.gt.11'
     &          ,4, -0.375, 1.625, 0.)
            CALL HBOOK1(-13,'DATA etabar DIR ET.gt.15'
     &          ,4, -0.375, 1.625, 0.)
            CALL HBOOK1(-20,'DATA etabar RES ET.gt.6'
     &          ,8, -0.375, 1.625, 0.)
            CALL HBOOK1(-21,'DATA etabar RES ET.gt.8'
     &          ,8, -0.375, 1.625, 0.)
            CALL HBOOK1(-22,'DATA etabar RES ET.gt.11'
     &          ,3, 0.125, 1.625, 0.)
            CALL HBOOK1(-23,'DATA etabar RES ET.gt.15'
     &          ,3, 0.125, 1.625, 0.)
            CALL HBOOK1(-30,'DATA etabar ALL ET.gt.6'
     &          ,11, -1.125, 1.625, 0.)
            CALL HBOOK1(-31,'DATA etabar ALL ET.gt.8'
     &          ,10, -0.875, 1.625, 0.)
            CALL HBOOK1(-32,'DATA etabar ALL ET.gt.11'
     &          ,4, -0.5, 1.5, 0.)
            CALL HBOOK1(-33,'DATA etabar ALL ET.gt.15'
     &          ,4, -0.5, 1.5, 0.)

            CALL HPAK(-10,cs1a)
            DO ivec=1,9
               pttemp(ivec)=sqrt(sta1a(ivec)**2+sys1a(ivec)**2)
            ENDDO
            CALL HPAKE(-10,pttemp)
            CALL HPAK(-11,cs1b)
            DO ivec=1,8
               pttemp(ivec)=sqrt(sta1b(ivec)**2+sys1b(ivec)**2)
            ENDDO
            CALL HPAKE(-11,pttemp)
            CALL HPAK(-12,cs1c)
            DO ivec=1,4
               pttemp(ivec)=sqrt(sta1c(ivec)**2+sys1c(ivec)**2)
            ENDDO
            CALL HPAKE(-12,pttemp)
            CALL HPAK(-13,cs1d)
            DO ivec=1,4
               pttemp(ivec)=sqrt(sta1d(ivec)**2+sys1d(ivec)**2)
            ENDDO
            CALL HPAKE(-13,pttemp)

            CALL HPAK(-20,cs2a)
            DO ivec=1,8
               pttemp(ivec)=sqrt(sta2a(ivec)**2+sys2a(ivec)**2)
            ENDDO
            CALL HPAKE(-20,pttemp)
            CALL HPAK(-21,cs2b)
            DO ivec=1,8
               pttemp(ivec)=sqrt(sta2b(ivec)**2+sys2b(ivec)**2)
            ENDDO
            CALL HPAKE(-21,pttemp)
            CALL HPAK(-22,cs2c)
            DO ivec=1,3
               pttemp(ivec)=sqrt(sta2c(ivec)**2+sys2c(ivec)**2)
            ENDDO
            CALL HPAKE(-22,pttemp)
            CALL HPAK(-23,cs2d)
            DO ivec=1,3
               pttemp(ivec)=sqrt(sta2d(ivec)**2+sys2d(ivec)**2)
            ENDDO
            CALL HPAKE(-23,pttemp)
            
            CALL HPAK(-30,cs3a)
            DO ivec=1,11
               pttemp(ivec)=sqrt(sta3a(ivec)**2+sys3a(ivec)**2)
            ENDDO
            CALL HPAKE(-30,pttemp)
            CALL HPAK(-31,cs3b)
            DO ivec=1,10
               pttemp(ivec)=sqrt(sta3b(ivec)**2+sys3b(ivec)**2)
            ENDDO
            CALL HPAKE(-31,pttemp)
            CALL HPAK(-32,cs3c)
            DO ivec=1,4
               pttemp(ivec)=sqrt(sta3c(ivec)**2+sys3c(ivec)**2)
            ENDDO
            CALL HPAKE(-32,pttemp)
            CALL HPAK(-33,cs3d)
            DO ivec=1,4
               pttemp(ivec)=sqrt(sta3d(ivec)**2+sys3d(ivec)**2)
            ENDDO
            CALL HPAKE(-33,pttemp)


         ENDIF

         IF (parton_showers) THEN
            CALL HCDIR('//HISTO/ps'//xxxx,' ')
            CALL HCDIR('//PAWC/ps'//xxxx,' ')
         ELSE
            CALL HCDIR('//HISTO/'//xxxx,' ')
            CALL HCDIR('//PAWC/'//xxxx,' ')
         ENDIF
         CALL HBOOK1(iproc*100+10,'MC etabar DIR ET.gt.6'
     &        ,9, -1.125, 1.125, 0.)
         CALL HBOOK1(iproc*100+11,'MC etabar DIR ET.gt.8'
     &        ,8, -0.875, 1.125, 0.)
         CALL HBOOK1(iproc*100+12,'MC etabar DIR ET.gt.11'
     &        ,4, -0.375, 1.625, 0.)
         CALL HBOOK1(iproc*100+13,'MC etabar DIR ET.gt.15'
     &        ,4, -0.375, 1.625, 0.)
         CALL HBOOK1(iproc*100+20,'MC etabar RES ET.gt.6'
     &        ,8, -0.375, 1.625, 0.)
         CALL HBOOK1(iproc*100+21,'MC etabar RES ET.gt.8'
     &        ,8, -0.375, 1.625, 0.)
         CALL HBOOK1(iproc*100+22,'MC etabar RES ET.gt.11'
     &        ,3, 0.125, 1.625, 0.)
         CALL HBOOK1(iproc*100+23,'MC etabar RES ET.gt.15'
     &        ,3, 0.125, 1.625, 0.)
         CALL HBOOK1(iproc*100+30,'MC etabar ALL ET.gt.6'
     &        ,11, -1.125, 1.625, 0.)
         CALL HBOOK1(iproc*100+31,'MC etabar ALL ET.gt.8'
     &        ,10, -0.875, 1.625, 0.)
         CALL HBOOK1(iproc*100+32,'MC etabar ALL ET.gt.11'
     &        ,4, -0.5, 1.5, 0.)
         CALL HBOOK1(iproc*100+33,'MC etabar ALL ET.gt.15'
     &        ,4, -0.5, 1.5, 0.)
         
         CALL HIDOPT(0,'stat')
         CALL HBARX(0)
	   
C     --- standard cone radius is 1.0
         CALL HZJETRAD(2,CONER)
         IF (CONER.lt.0.0) THEN
            CONER=1.0
         ENDIF
         CALL HZJTNAME(chjet,jetf)
         
         WRITE(HZUNIT,*)'**********************************************'
         WRITE(HZUNIT,*)'* HZ97196 called, histograms will be output. *'
         WRITE(HZUNIT,*)'*     Jet finder selected : ',jetf
     $        ,'           *'
         WRITE(HZUNIT,1001)'*        with cone radius : ',CONER,'   *'
         WRITE(HZUNIT,*)'**********************************************'
         
 1001    FORMAT(A29,F5.2,A13)
         
      ELSE IF (MOD(iflag,10).eq.2) THEN
C     *** Filling runs ***
         
C     *** Change directory to our graphs ***
         IF (parton_showers) THEN              
            CALL HCDIR('//PAWC/ps'//xxxx,' ')
         ELSE
            CALL HCDIR('//PAWC/'//xxxx,' ')
         ENDIF
         
         q2=hzphokin(1)
         IF (q2.eq.-1.0d0) THEN
C     PRINT*,'HZ97196: HZPHOKIN returns error for Q2'
         ENDIF
         q2=abs(q2)
         ybj=hzphokin(3)
         IF (ybj.eq.-1.d0) THEN
C     PRINT*,'HZ97196: HZPHOKIN returns error for ybj'
         ENDIF
         
C     *** Fill our histogram ***
         
         IF ((ybj.ge.ybjcut(1)).and.(ybj.le.ybjcut(2))
     &        .and.(q2.lt.q2cut(2))) Then
            
            elecE=27.5d0
            hzflag=HZIBEAM(myip,myil)
            IF (hzflag.ge.0) Then
               elecE=abs(PHEP(4,myil))
            ELSE
               PRINT*,'HZ97196: Electron not found. E=0.001'
               PRINT*,'         event will be rejected'
               elecE=0.001d0
            ENDIF
            
            CALL hzjtfind(chjet,CONER,NJET,Jets)
            
            IF (NJET.ge.2) THEN
               DO loop=1,2
                  Et(loop)=Jets(loop,3)
                  phi(loop)=Jets(loop,2)
                  rap(loop)=Jets(loop,1)
               ENDDO
               IF (rap(1).ge.rapcut(1).and.rap(2).ge.rapcut(1)
     +              .and.rap(1).le.rapcut(2).and.rap(2).le.rapcut(2))
     +              THEN
                  deltaeta=abs(rap(1)-rap(2))
                  etabar=(rap(1)+rap(2))/2
                  xgam=(Et(1)*exp(-1*rap(1))+Et(2)*exp(-1*rap(2)))
     +                 /(2*ybj*elecE)
                  
                  IF (abs(deltaeta).lt.0.5) THEN
                     graph=0
                     IF (xgam.gt.0.75) graph=9
                     IF (xgam.le.0.75.and.xgam.gt.0.3) graph=19
                     DO loop=1,4
                        IF (Et(2).gt.Etcut(loop)) THEN
                           IF (graph.ne.0) THEN
                              CALL HFILL(iproc*100+graph+loop
     &                             ,REAL(etabar),0.,wtx)
                           ENDIF
                           CALL HFILL(iproc*100+29
     &                          +loop,REAL(etabar),0.,wtx)
                        ENDIF
                     ENDDO
                  ENDIF         ! deltaeta.lt.0.5
               ENDIF            ! rapcut
            ENDIF               ! NJET.ge.2
            
         ENDIF                  ! Q2 & y cuts
         
      ELSE IF (MOD(iflag,10).eq.3) THEN
C     *** Termination run ***
C     *** Change directory ***
         IF (iproc.eq.0.or.iproc.eq.2) THEN
            WRITE(HZUNIT,*) 'HZ97196: finishing off.'
         ENDIF

         IF (parton_showers) THEN
            CALL HCDIR('//PAWC/ps'//xxxx,' ')
         ELSE
            CALL HCDIR('//PAWC/'//xxxx,' ')
         ENDIF
C     *** Finish off histograms ***
         
         IF (Xsec.eq.0) THEN
            PRINT*,'HZ97196: termination called with zero cross section'
            PRINT*,'          cross section graph meaningless'
            PRINT*,'          in process :',iproc*1000
            Xsec=1
         ENDIF
         IF (Ntot.eq.0) THEN
            PRINT*,'HZ97196: termination called with no total events'
            PRINT*,'          cross section graph meaningless'
            PRINT*,'          in process :',iproc*1000
            Ntot=1
         ENDIF

         lumnb = ntot/xsec
         
C     --- now calculate cross-section graphs 10,20
         DO loop=0,3
            
            CALL HZHINRM(100*iproc+10+loop,0,lumnb,1)
            CALL HZHINRM(100*iproc+20+loop,0,lumnb,1)
            CALL HZHINRM(100*iproc+30+loop,0,lumnb,1)
            
         ENDDO
                  
         IF (iproc.eq.2) THEN
C     --- only finally
            
            DO loop=0,3

               CALL HOPERA(110+loop,'+e',210+loop,10+loop,1.,1.)
               CALL HOPERA(120+loop,'+e',220+loop,20+loop,1.,1.)
               CALL HOPERA(130+loop,'+e',230+loop,30+loop,1.,1.)

            ENDDO

         ENDIF
         
      ELSE
C     *** End ***
         PRINT*,'HZ97196:Please run routine with iflag set to 1,2 or 3'
         PRINT*,'        with +1000 or +2000 for DIR or RES events.'
      ENDIF
      
      RETURN
      END
