*CMZ :  2.00/05 31/07/2000  21.21.54  by  Tancredi Carli
*CMZ :  2.00/04 31/07/2000  21.08.05  by  Tancredi Carli
*CMZ :  1.02/09 03/08/99  15.53.54  by  Tancredi Carli
*-- Author :
*-- Author :    Alice Valkarova  1999
c-----------------------------------------------------------------------

      SUBROUTINE HZ97210(iflag)

c-----------------------------------------------------------------------
c---- Description

c Purpose: Study topological structure of the hadronic final state in
c          diffractive deep inelastic scattering
c
c Event selection: 10. < Q2 < 100 GeV2
c                  y < 0.5
c                  xpom < 0.05
c                  4 < Mx < 36 GeV
c                  |t| < 1 GeV2
c                  My < 1.6 GeV
c
c Reference: DESY 97-210
c            Eur. Phys. J. C (1998)
c
c Arguments: iflag=1 initialisation
c            iflag=1 filling
c            iflag=3 termination
c
c Written by: Alice Valkarova and Gerhard Knies

c-----------------------------------------------------------------------
c---- Declarations

      IMPLICIT NONE

c.... HzTool stuff

#include "hzfunc.inc"
#include "hzhbook.inc"
#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzjetcmn.inc"
*

c.... input variable
      INTEGER iflag

c.... local variables
      LOGICAL          iacc(NMXHEP)
      INTEGER          nacc, ihep, i, j, ierr, ibp, ibl, isl,
     &                 id,
     &                 index(NMXHEP),id1,id2,ii
      REAL             p, pz, gap, tmp, w2, mp,
     &                 xvec(4), yvec(4), rap(NMXHEP)


      REAL             evpt2(7),thrmc,mxmc,pt2,rndm,ptsqr(8)
      PARAMETER        (mp = 0.938)
      DOUBLE PRECISION pbeam(4), gamma(4), p_lab(4), p_hcms(4)
      CHARACTER*4      low, high
      Character*6 xxxx
      Data xxxx/'97210'/

C****************** QUANTITIES FOR THRUST ANALYSIS *********************
      DOUBLE PRECISION
     & p_xcms(4), prot_xcms(4), zerovec(4), onevec(4),p_xcms5
     &, decoj(5,15),pj2, pjprot, pprot2,  cosi2, psum, pv2
      LOGICAL evacc	
      integer nt,ik,ij
C****************** QUANTITIES FOR THRUST ANALYSIS *********************

c.... Mx bin limits
      REAL mxbins(8), ptbins(9),mxbins1(7)
      DATA mxbins / 4., 6., 8., 11., 15., 19., 24., 36./
      DATA mxbins1 / 6., 8., 11., 15., 19., 24., 36./
      DATA ptbins / 0., 1., 3., 6., 12., 25., 43., 79., 151./

c.... kinematical selection criteria
      REAL q2, q2_low, q2_high
      DATA q2_low  / 10.  /, q2_high / 100. /
      REAL y,  y_low,  y_high
      DATA y_low   / 0.0 /, y_high  / 0.5  /
      REAL xp, xp_high
      DATA xp_high / 0.05 /
      REAL mx_low, mx_high
      DATA mx_low  / 4.   /, mx_high / 36.  /
      REAL t,  t_high
      DATA t_high  / 1.   /
      REAL my, my_high
      DATA my_high / 1.6  /
      real e_el,pt_el
      integer jj
c-----------------------------------------------------------------------
c---- Corrected data from H1
C Thrust data
      REAL mxx(7),thr(7),ethr1(7),ethr2(7)
      REAL mx_lt1(7),mx_gt1(7),thr_lt1(7),thr_gt1(7)
      REAL ethr1_lt1(7),ethr2_lt1(7),ethr1_gt1(7),ethr2_gt1(7)

C Pt data
*      REAL pt2_int(48),mx_int(48),dsig(48),dsig_est(48),dsig_esy(48)
      REAL pt2_int(8,6),mx_int(8,6),dsig(8,6),dsig_est(8,6),
     &  dsig_esy(8,6)

C fraction data
      REAL frac_1(6),mxf_1(6),efr1_1(6),efr2_1(6)
      REAL frac_3(6),mxf_3(6),efr1_3(6),efr2_3(6)
c--------------------------
C Thrust MOCA
      REAL mx_mc(7),thr_mc(7),ethr1_mc(7)
      REAL mx_lt1_mc(7),mx_gt1_mc(7),thr_lt1_mc(7),thr_gt1_mc(7)
      REAL ethr1_lt1_mc(7),ethr1_gt1_mc(7)
      REAL nev_mc(7),nev_lt1_mc(7),nev_gt1_mc(7)

C Pt MOCA
      REAL pt2_int_mc(8,6),dsig_mc(8,6),dsig_esy_mc(8,6)
C auxilliary

      REAL pn_tmp(8),pne_tmp(8),pns_tmp(8)

      DATA mxx    /
*   exp. mean mass of X
     &              4.97,  6.98,  9.40, 12.81, 16.82, 21.20, 28.64/


      DATA thr   /
*   exp. mean thrust
     &              0.777, 0.781, 0.802, 0.822, 0.847, 0.865, 0.875/

      DATA ethr1    / 0.004, 0.004, 0.004, 0.004, 0.005, 0.005, 0.006/

      DATA ethr2    / 0.008, 0.005, 0.006, 0.005, 0.006, 0.008, 0.009/

      DATA thr_lt1/ 0.787, 0.810, 0.828, 0.850, 0.890, 0.909, 0.910/
      DATA thr_gt1/ 0.762, 0.758, 0.785, 0.806, 0.831, 0.844, 0.864/

      DATA mx_lt1 / 4.95,  6.96,  9.44, 12.75, 16.90, 21.17, 28.08/
      DATA mx_gt1 / 4.99,  6.99,  9.38, 12.85, 16.80, 21.21, 28.82/

      DATA ethr1_lt1/ 0.005, 0.005, 0.005, 0.005, 0.007, 0.007, 0.008/
      DATA ethr1_gt1/ 0.006, 0.005, 0.005, 0.005, 0.006, 0.007, 0.007/

      DATA ethr2_lt1/ 0.006, 0.002, 0.003, 0.002, 0.002, 0.002, 0.009/
      DATA ethr2_gt1/ 0.004, 0.006, 0.003, 0.002, 0.002, 0.002, 0.002/


      DATA pt2_int / 0.37, 1.88, 4.21, 0.0,
     &               0.,   0.,   0.,   0.,
     &               0.38, 1.84, 4.30, 8.38,
     &               0.,   0.,   0.,   0.,
     &               0.40, 1.83, 4.22, 8.54,
     &              15.97, 0.,  0.,   0.,
     &               0.41, 1.79, 4.32, 8.47,
     &              17.1, 30.75, 0.,  0.,
     &               0.37, 1.84, 4.22, 8.08,
     &              16.19, 33.18, 51.7, 0.,
     &               0.41, 1.83, 4.26, 8.42,
     &              17.38, 32.09, 58.28, 113.2/

       DATA mx_int / 6.96,   6.96,  7.10,  0.,
     &               0.,     0.,    0.,    0.,
     &               9.39,   9.36,  9.31,  9.54,
     &               0.,     0.,    0.,    0.,
     &              12.72, 12.74, 12.77, 12.78,
     &              13.10,  0.,    0.,    0.,
     &              16.90, 16.78, 16.70, 16.98,
     &              16.89, 16.85,  0.,    0.,
     &              21.01, 21.10, 21.40, 20.91,
     &              21.07, 20.79, 22.15,  0.,
     &              27.79, 28.05, 27.94, 28.26,
     &              28.65, 29.35, 28.18, 30.45/

       DATA dsig   / 0.527,   0.144,   0.042,    0.,
     &               0.,      0.,      0.,       0.,
     &               0.470,   0.129,   0.052,    0.016,
     &               0.,      0.,      0.,       0.,
     &               0.451,   0.117,   0.045,    0.019,
     &               0.0051,  0.,      0.,       0.,
     &               0.404,   0.133,   0.053,    0.016,
     &               0.005,   0.0007,  0.,       0.,
     &               0.444,   0.111,   0.045,    0.020,
     &               0.0038,  0.0008,  0.00039,  0.,
     &               0.361,   0.141,   0.048,    0.017,
     &               0.0049,  0.0014,  0.00054,  0.00011/

       DATA dsig_est / 0.018,   0.009,   0.004,   0.,
     &                 0.,      0.,      0.,      0.,
     &                 0.017,   0.008,   0.004,   0.002,
     &                 0.,      0.,      0.,      0.,
     &                 0.019,   0.009,   0.004,   0.002,
     &                 0.0007,  0.,      0.,      0.,
     &                 0.024,   0.012,   0.006,   0.002,
     &                 0.001,   0.0002,  0.,      0.,
     &                 0.030,   0.013,   0.006,   0.003,
     &                 0.0008,  0.0003,  0.00014, 0.,
     &                 0.032,   0.018,   0.008,   0.003,
     &                 0.0010,  0.0004,  0.00018, 0.00006/
       DATA dsig_esy / 0.038,   0.022,   0.007,   0.,
     &                 0.,      0.,      0.,      0.,
     &                 0.039,   0.022,   0.007,   0.003,
     &                 0.,      0.,      0.,      0.,
     &                 0.033,   0.013,   0.005,   0.003,
     &                 0.0013,  0.,      0.,      0.,
     &                 0.038,   0.016,   0.008,   0.004,
     &                 0.001,   0.0003,  0.,      0.,
     &                 0.044,   0.016,   0.009,   0.005,
     &                 0.001,   0.0004,  0.00022, 0.,
     &                 0.039,   0.028,   0.013,   0.004,
     &                 0.0011,  0.0006,  0.00025, 0.00011/


       DATA mxf_1  /  7.1,  9.6, 12.9, 17.,  21.2, 28.3 /
       DATA frac_1 / 47.3, 53.,  54.9, 59.6, 55.6, 63.9 /
       DATA efr1_1 /  1.7,  1.6,  1.8,  2.3,  2.9,  3.1 /
       DATA efr2_1 /  3.4,  3.6,  2.8,  3.0,  2.6,  2.3 /

       DATA mxf_3  /  7.0,  9.6, 12.9, 17.,  21.2, 28.3 /
       DATA frac_3 / 18.6, 27.2, 31.5, 33.,  33.4, 35.7 /
       DATA efr1_3 /  1.3,  1.4,  1.6,  2.,   2.4,  2.9 /
       DATA efr2_3 /  2.4,  3.4,  3.8,  3.8,  2.6,  3.2 /

       save nacc


C****************** commons used with DECO *****************************
C                                                                      *
      integer jptmax,jfroff
      real econf
#include "qjmacs.inc"
#include "decoma.inc"
C								       *
C***********************************************************************

c-----------------------------------------------------------------------
c---- Initialisation

      IF (iflag .EQ. 1) THEN

c....   set number of accepted events to zero
        nacc= 0

c....   create subdirectories for use with HBOOK/PAW
        CALL HCDIR ('//PAWC', ' ')
        CALL HMDIR ('97210', 'S')
        CALL HCDIR ('//HISTO', ' ')
        CALL HMDIR ('97210', 'S')

	 WRITE(6,*)'***************************************************'
	 WRITE(6,*)'HZ'//xxxx//' called'
     	 WRITE(6,*)'***************************************************'
c....   book moca histograms

        DO i = 1, 7
          WRITE (low, '(F4.0)') mxbins(i)
          WRITE (high,'(F4.0)') mxbins(i+1)
**  thrust and mass histograms
          id = 10+i
          CALL HBOOK1 (id,
     &      'Moca TD - '//low//' .lt. Mx .lt. '//high//' GeV, total',
     &      20, 0.5, 1.0, 0.)
          id = 60+i
          CALL HBOOK1 (id,
     &      'Moca MD - '//low//' .lt. Mx .lt. '//high//' GeV, total',
     &      64, 4. , 36. , 0.)
          id = 20+i
          CALL HBOOK1 (id,
     &      'Moca TD - '//low//' .lt. Mx .lt. '//high//' GeV, ptgt1',
     &      20, 0.5, 1.0, 0.)
          id = 70+i
          CALL HBOOK1 (id,
     &      'Moca MD - '//low//' .lt. Mx .lt. '//high//' GeV, total',
     &     64, 4. , 36.  , 0.)
          id = 30+i
          CALL HBOOK1 (id,
     &      'Moca TD - '//low//' .lt. Mx .lt. '//high//' GeV, ptlt1',
     &      20, 0.5, 1.0, 0.)
          id = 80+i
          CALL HBOOK1 (id,
     &      'Moca MD - '//low//' .lt. Mx .lt. '//high//' GeV, total',
     &      64, 4. , 36. , 0.)

           ENDDO
            DO i = 1,6
          WRITE (low, '(F4.0)') mxbins1(i)
          WRITE (high,'(F4.0)') mxbins1(i+1)
** pt2 histograms
          id = 90 + i
          CALL HBOOKB (id,
     &      'Moca Pt2D - '//low//' .lt. Mx .lt. '//high//' GeV, total',
     &      8, ptbins , 0.)
          id = 300 +i
          CALL HBOOKB (id,
     &      'Moca bins Pt2D - '//low//' .lt. Mx .lt. '//high//' GeV, ',
     &      8, ptbins , 0.)
         DO j = 1,8
            id = 300 +i*10 +j
          CALL HBOOKB (id,
     &      'Moca bins Pt2D - '//low//' .lt. Mx .lt. '//high//' GeV, ',
     &      8, ptbins , 0.)
         ENDDO
        ENDDO


        CALL HBOOKB (510, 'Moca thrust vs mass',
     &               7, mxbins, 0.)

          CALL HBOOKB (560, 'Moca mass vs mass',
     &               7, mxbins, 0.)

        CALL HBOOKB (520, 'Moca thrust ptgt1 vs mass',
     &               7, mxbins, 0.)

          CALL HBOOKB (570, 'Moca mass ptgt1 vs mass',
     &               7, mxbins, 0.)

        CALL HBOOKB (530, 'Moca thrust ptlt1 vs mass',
     &               7, mxbins, 0.)

          CALL HBOOKB (580, 'Moca mass ptlt1 vs mass',
     &               7, mxbins, 0.)

********* data histograms
** pt2 histograms
        DO i=1,6
          WRITE (low, '(F4.0)') mxbins1(i)
          WRITE (high,'(F4.0)') mxbins1(i+1)

          id = -90 -i
          CALL HBOOKB (id,
     &      'Data Pt2D - '//low//' .lt. Mx .lt. '//high//' GeV, total',
     &      8, ptbins , 0.)
          id = -150 -i
          CALL HBOOKB (id,
     &      'Data Pt2D - '//low//' .lt. Mx .lt. '//high//' GeV, total',
     &      8, ptbins , 0.)
          id = -300 -i
          CALL HBOOKB (id,
     &      'Data bins Pt2D - '//low//' .lt. Mx .lt. '//high//' GeV, ',
     &      8, ptbins , 0.)

        ENDDO

********* thrust
         DO   i=1,2
             id = -410-i*100
          CALL HBOOKB (id,
     &      'Data Thrust - 4 .lt. Mx .lt. 36 GeV, total',
     &      7, mxbins , 0.)

             id = -420-i*100
          CALL HBOOKB (id,
     &      'Data Thr pt lt 1 - 4 .lt. Mx .lt. 36 GeV, total',
     &      7, mxbins , 0.)

             id = -430-i*100
          CALL HBOOKB (id,
     &      'Data Thr pt gt 1 - 4 .lt. Mx .lt. 36 GeV, total',
     &      7, mxbins , 0.)

            ENDDO

          CALL HBOOKB (-560,
     &      'Data mass all pt - 4 .lt. Mx .lt. 36 GeV, total',
     &      7, mxbins , 0.)

          CALL HBOOKB (-570,
     &      'Data mass pt lt 1 - 4 .lt. Mx .lt. 36 GeV, total',
     &      7, mxbins , 0.)

          CALL HBOOKB (-580,
     &      'Data mass pt gt 1 - 4 .lt. Mx .lt. 36 GeV, total',
     &      7, mxbins , 0.)

CC--------fraction of events with pt**2>1 and >3

             DO i=1,2
                id = -360 -i*50

          CALL HBOOKB (id,
     &      'Data fraction  pt gt 1- 4 .lt. Mx .lt. 36 GeV, total',
     &      6, mxbins1 , 0.)

          CALL HBOOKB (id-10,
     &      'Data fraction  pt gt 3- 4 .lt. Mx .lt. 36 GeV, total',
     &      6, mxbins1 , 0.)

            ENDDO

          CALL HBOOKB (-430,
     &      'Data fraction mass pt gt 1- 4 .lt. Mx .lt. 36 GeV, total',
     &      6, mxbins1 , 0.)

          CALL HBOOKB (-440,
     &      'Data fraction mass pt gt 3- 4 .lt. Mx .lt. 36 GeV, total',
     &      6, mxbins1 , 0.)


c-----------------------------------------------------------------------
c---- Event analysis

      ELSEIF (iflag .EQ. 2) THEN

c....   move to the correct subdirectory in PAWC
        CALL HCDIR ('//PAWC/97210', ' ')

c....   get vectors for boost to hCMS
        ierr = HZIBEAM (ibp, ibl)
        isl = HZIDELEC(id)
c         print *,'ibp,ibl',ibp,ibl
c         print *,'isl',isl
c         do j=1,nhep
c         print *,'phep(i,j)',(PHEP(i,j),i=1,4)
c         enddo
            e_el=PHEP(4,isl)
            pt_el=sqrt(PHEP(1,isl)**2+PHEP(2,isl)**2)
c           print *,'*************************************'
c           print *,'scattered elec',(PHEP(i,isl),i=1,4)
        IF (ierr .EQ. 1 .AND. isl .GT. 0) THEN
          DO i = 1, 4
            pbeam(i) = PHEP (i, ibp)
            gamma(i) = PHEP (i, ibl) - PHEP (i, isl)
          ENDDO
        ELSEIF (ierr .NE. 1 .AND. isl .GT. 0) THEN
          PRINT *, 'HZ97210-- Warning: No beam(s) found,'
          PRINT *, '        -- proton beam energy set to 820 GeV,'
          PRINT *, '        -- positron beam energy set to 27.55 GeV.'
          IF (ierr .EQ.  0) ibp = 0
          IF (ierr .EQ. -1) ibl = 0
          pbeam(1) = 0.
          pbeam(2) = 0.
          pbeam(3) = +820.
          pbeam(4) = +820.
          gamma(1) = - PHEP (1, isl)
          gamma(2) = - PHEP (2, isl)
          gamma(3) = -27.55 - PHEP (3, isl)
          gamma(4) = +27.55 - PHEP (4, isl)
        ELSEIF (isl .LE. 0) THEN
          PRINT *, 'HZ97210-- Warning: No scattered positron found,'
          PRINT *, '        -- event skipped !'
          GOTO 999
        ENDIF

c....   initialise boost to hCMS
        CALL HZHCMINI (pbeam, gamma, ierr)
	NPRI=0
         IF(NPRI.GE.2)
     1     PRINT *, 'HZ97210: boost to hadronic CMS defined'
        IF (ierr .EQ. 1) THEN
          PRINT *, 'HZ97210-- Warning: boost to hadronic CMS failed,'
          PRINT *, '        -- event skipped !'
          GOTO 999
        ENDIF
c control print out: 0=none, 1=medium, 2=detailed
            CALL HZHCM(pbeam, p_hcms, ierr)

C	NPRI=2
         IF(NPRI.GE.2)
     1    PRINT *,'HZ97210, pbeam,ierr',(pbeam(i),i=1,4),ierr
         IF(NPRI.GE.2)
     1    PRINT *,'HZ97210, prot_hcms ',(p_hcms(i),i=1,4)

c....   select final state particles and store pseudorapidity in hCMS
        DO ihep = 1, NHEP
          iacc(ihep) = .FALSE.
          rap(ihep) = 9999.
          IF (
     &        ISTHEP(ihep) .EQ. 1 .AND.
     &        ihep .NE. ibp       .AND.
     &        ihep .NE. ibl       .AND.
     &        ihep .NE. isl       .AND.
     &        .TRUE.) THEN
            iacc(ihep) = .TRUE.
            p_lab(1) = PHEP (1, ihep)
            p_lab(2) = PHEP (2, ihep)
            p_lab(3) = PHEP (3, ihep)
            p_lab(4) = PHEP (4, ihep)
c            print *,'p_lab',ihep,(p_lab(jj),jj=1,4)
            CALL HZHCM(p_lab, p_hcms, ierr)
c            print *,'p_hcms',ihep,(p_hcms(jj),jj=1,4)
            IF (ierr .EQ. 1) THEN
              PRINT *,
     &          'HZ97210-- Warning: boost to hadronic CMS failed,'
              PRINT *, '        -- event skipped !'
              GOTO 999
            ENDIF
c           rap is pseudorapidity !
            p = SQRT(p_hcms(1)**2+p_hcms(2)**2+p_hcms(3)**2)
            pz = p_hcms(3)
            IF (p .GT. ABS(pz)) rap(ihep) = 0.5*(LOG(p+pz)-LOG(p-pz))
          ENDIF
        ENDDO

c....   sort particles with increasing pseudorapidity
        CALL SORTZV (rap, index, NHEP, 1, 0, 0)

c....   look for largest gap
        gap =  0.
        DO ihep = 2, NHEP
          tmp = rap(index(ihep)) - rap(index(ihep-1))
          IF (tmp .GT. gap .AND. iacc(index(ihep))) THEN
            gap = tmp
            id = ihep
          ENDIF
        ENDDO
c       store boundary between X and Y in variable gap
        gap = rap(index(id-1)) + gap/2

c....   get DIS kinematics
        q2 = HZDISKIN (1)
        y  = HZDISKIN (3)
        w2 = HZDISKIN (4)

c....   get diffractive DIS kinematics and select particles of system X
        DO i = 1, 4
          xvec(i) = 0.
          yvec(i) = 0.
        ENDDO
        DO ihep = 1, NHEP
          IF (iacc(ihep)) THEN
            IF (rap(ihep) .GT. gap) THEN
              xvec(1) = xvec(1) + PHEP (1, ihep)
              xvec(2) = xvec(2) + PHEP (2, ihep)
              xvec(3) = xvec(3) + PHEP (3, ihep)
              xvec(4) = xvec(4) + PHEP (4, ihep)
            ELSE
              iacc(ihep) = .FALSE.
              yvec(1) = yvec(1) + PHEP (1, ihep)
              yvec(2) = yvec(2) + PHEP (2, ihep)
              yvec(3) = yvec(3) + PHEP (3, ihep)
              yvec(4) = yvec(4) + PHEP (4, ihep)
            ENDIF
          ENDIF
        ENDDO
        mxmc = xvec(4)**2 - xvec(1)**2 - xvec(2)**2 - xvec(3)**2
        IF (mxmc .GT. 0) THEN
          mxmc = SQRT(mxmc)
        ELSE
          mxmc = 0.
        ENDIF
        my = yvec(4)**2 - yvec(1)**2 - yvec(2)**2 - yvec(3)**2
        IF (my .GT. 0) THEN
          my = SQRT(my)
        ELSE
          my = 0.
        ENDIF
        t = (pbeam(4)-yvec(4))**2 - (pbeam(1)-yvec(1))**2
     &    - (pbeam(2)-yvec(2))**2 - (pbeam(3)-yvec(3))**2
        xp = (q2 + mxmc**2 - t)/(q2 + w2 - mp**2)
c        print *,'mxmc,xp',mxmc,xp
c....   select event

	evacc = .FALSE.
        IF (
     &      q2 .GT. q2_low      .AND.  q2 .LT. q2_high  .AND.
     &      y  .GT. y_low       .AND.  y  .LT. y_high   .AND.
     &      mxmc .GT. mx_low      .AND.  mxmc .LT. mx_high  .AND.
     &      my .LT. my_high     .AND.
     &      ABS(t) .LT. t_high  .AND.
     &      xp .LT. xp_high     .AND.
     &      .TRUE.) THEN
	  evacc = .TRUE.
          IF(.NOT.EVACC) GOTO 998	
          nacc = nacc + 1
C...  initialise DECO
	  CALL HEDECI(NHEP,1, IFLAGG)
c	 IF(IFLAGG.NE.0) GOTO 998
C...   event to large for DECO buffer

c....     initialise boost to X_CMS
          zerovec(1) = 0.
          zerovec(2) = 0.
          zerovec(3) = 1.
          zerovec(4) = 1.
          onevec(1)=xvec(1)-zerovec(1)
          onevec(2)=xvec(2)-zerovec(2)
          onevec(3)=xvec(3)-zerovec(3)
          onevec(4)=xvec(4)-zerovec(4)
          prot_xcms(1)=0.
          prot_xcms(2)=0.
          prot_xcms(3)=0.
          prot_xcms(4)=0.

          CALL HZHCMINI (onevec, zerovec, ierr)
         IF(NPRI.GE.2)
     1    PRINT *,'HZ97210, pbeam,ierr',(pbeam(i),i=1,4),ierr
         IF(NPRI.GE.2)
     1    PRINT *,'HZ97210, xvec,ierr',(xvec(i),i=1,4),ierr
c... proton into X_cms
               CALL HZHCM (pbeam,prot_xcms, ierr)
c control print out: 0=none, 1=medium, 2=detailed
c	NPRI=2
          IF(NPRI.GE.2)
     1print *,'NEVHEP, M_X' , nevhep, mxmc
         IF(NPRI.GE.2)
     1    PRINT *,'HZ97210, prot_xcms ',(prot_xcms(i),i=1,4)

          nt = 0
	  psum = 0.
          DO ihep = 1, NHEP
          IF(NPRI.GE.2)
     1PRINT *,'hz97210, PHEP',ihep,(PHEP(i,ihep),i=1,4)
       IF (iacc(ihep) .AND. rap(ihep) .GT. gap) THEN
c....     get number of particles in gamma-pom CMS
		nt = nt + 1
              p_lab(1) = PHEP(1, ihep)
              p_lab(2) = PHEP(2, ihep)
              p_lab(3) = PHEP(3, ihep)
              p_lab(4) = PHEP(4, ihep)
             CALL HZHCM (p_lab, p_xcms, ierr)
C... mom sum for  thrust normalisation
	      pv2 = p_xcms(1)**2 + p_xcms(2)**2 + p_xcms(3)**2
              psum = psum + sqrt(pv2)
C... fill qctbuf with input vectors for DECO
		DO ik = 1,4
		 qpi(ik,nt) = p_xcms(ik)
		ENDDO
	        p_xcms5=p_xcms(4)**2 - pv2
	        if(p_xcms5.ge.0.) p_xcms5 = sqrt(p_xcms5)
	        if(p_xcms5.lt.0.) p_xcms5 = -sqrt(-p_xcms5)
                qpi(5,nt)=p_xcms5
         IF(NPRI.GE.2)
     1    PRINT *,'HZ97210, qpi ***** ',nt,(qpi(i,nt),i=1,5)

C... remeember ihep index
		jtbit(nt) = ihep	
            ENDIF
          ENDDO
c                print *,'number of tracks HZ97210',nt
C...  initialise DECO
	  CALL HEDECI(nt,2, IFLAGG)
	 IF(IFLAGG.NE.0) GOTO 998
C...   event to large for DECO buffer



         IF(NPRI.GE.2)
     1 PRINT *,'HZ97210, qpi of X in C_cms **'
     1,(qpi(i,iptmax+1),i=1,5)

Call DECO
	JETNAM(1)='MOCA'
	JETNA ='MOCARLO'
	JETFU=1
	MAXMUL=2
	NCON=0
       
	CALL HGDECO(JETNA,jetfu,maxmul,ncon,npri,iflagg) 	

*Write jet vector from qctbuf to decoj(5,15)
          DO ik = 1,5
		DO ij = 1,15
		decoj(ik,ij) = qpo(ik,ij)
		ENDDO
	  ENDDO
c       print *,'decoj(ik,ij)',decoj

C...     THRUST:thrmc,  thrust-jet Pt**2: pt2, from decoj(i,2)
	pj2 = (decoj(1,2)**2 + decoj(2,2)**2 + decoj(3,2)**2)
	thrmc=2.*sqrt(pj2)/psum
c         print *,'thrmc',thrmc
         pprot2=prot_xcms(1)**2+prot_xcms(2)**2+prot_xcms(3)**2
	pjprot=decoj(1,2)*prot_xcms(1)
     &         +decoj(2,2)*prot_xcms(2)
     &         +decoj(3,2)*prot_xcms(3)
	cosi2=pjprot**2/(pj2*pprot2)
	pt2=pj2*(1.-cosi2)


********************************************************
              wtx = 1.

c....     fill histograms
*          DO j=1,3
          DO i = 1,7
            id1 = 10+i
            id2 = 50+10+i
            IF (mxmc .GT. mxbins(i) .AND. mxmc .LT. mxbins(i+1)) THEN
              CALL HFILL (id1, thrmc, 0., wtx)
              CALL HFILL (id2, mxmc, 0., wtx)
            ENDIF
          ENDDO

          if(pt2.gt.1.) j=2
          if(pt2.le.1.) j=3
          DO i = 1,7
            id1 = j*10+i
            id2 = 50+j*10+i
            IF (mxmc .GT. mxbins(i) .AND. mxmc .LT. mxbins(i+1)) THEN
              CALL HFILL (id1, thrmc, 0., wtx)
              CALL HFILL (id2, mxmc, 0., wtx)
            ENDIF
          ENDDO

          DO i = 1,7
             id=90+i
            IF (mxmc .GT. mxbins(i) .AND. mxmc .LT. mxbins(i+1)) THEN
              CALL HFILL (id, pt2, 0., wtx)
          DO j = 1,8
            If (pt2. gt. ptbins(j) .AND. pt2 .lt. ptbins(j+1)) THEN
             id = 300 +i*10+j
              CALL HFILL (id, pt2, 0., wtx)
            ENDIF
          ENDDO

            ENDIF
          ENDDO

  998   CONTINUE
        ENDIF

c-----------------------------------------------------------------------
c---- Termination

      ELSEIF (iflag .EQ. 3) THEN

c....   move to the correct PAW subdirectory
        call hcdir('//PAWC/97210',' ')

c....   normalise pt2 MOCA distributions

           do i = 1,6
            id = 90 + i
            evpt2(i) = HSTATI (id,   3, ' ', 1)
            print *,'events',i, evpt2(i)
            CALL HZHINRM (id,   0, evpt2(i), 1)
        ENDDO

c....  calculates the center of bins of pt2 distr. for each mx

           do i= 1,8
             do j = 1,6
             id = 300 +i*10+j
             write(6,*) i,j,id
             pt2_int_mc(i,j)= HSTATI (id,   1, ' ', 1)
             ENDDO
           ENDDO

c....   extract mean values for thrust and mass (centers of bin)
        DO i = 1, 7
          id = 10 +i
          thr_mc(i) = HSTATI (id,   1, ' ', 1)
          ethr1_mc(i) = HSTATI (id,   2, ' ', 1)
          nev_mc(i) = HSTATI (id,   3, ' ', 1)
          print *,'mean and error',i,thr_mc(i),ethr1_mc(i), nev_mc(i)
          ethr1_mc(i)= ethr1_mc(i)/sqrt(nev_mc(i))
          print *,'mean and error',i,thr_mc(i),ethr1_mc(i), nev_mc(i)
          id = 60 +i
          mx_mc(i) = HSTATI (id,   1, ' ', 1)
          id = 20 +i
          thr_lt1_mc(i) = HSTATI (id,   1, ' ', 1)
          ethr1_lt1_mc(i) = HSTATI (id,   2, ' ', 1)
          nev_lt1_mc(i) = HSTATI (id,   3, ' ', 1)
          ethr1_lt1_mc(i)= ethr1_lt1_mc(i)/sqrt(nev_lt1_mc(i))
          id = 70 +i
          mx_lt1_mc(i) = HSTATI (id,   1, ' ', 1)
          id = 30 +i
          thr_gt1_mc(i) = HSTATI (id,   1, ' ', 1)
          ethr1_gt1_mc(i) = HSTATI (id,   2, ' ', 1)
          nev_gt1_mc(i) = HSTATI (id,   3, ' ', 1)
          ethr1_gt1_mc(i)= ethr1_gt1_mc(i)/sqrt(nev_gt1_mc(i))
          id = 80 +i
          mx_gt1_mc(i) = HSTATI (id,   1, ' ', 1)


        ENDDO

       print *,'thr,ethr,mx',(thr_mc(i),ethr1_mc(i),mx_mc(i),i=1,7)

        CALL HPAK (510, thr_mc)
        CALL HPAKE(510, ethr1_mc)
        CALL HPAK (560, mx_mc)
        CALL HPAK (520, thr_gt1_mc)
        CALL HPAKE(520, ethr1_gt1_mc)
        CALl HPAK (570, mx_gt1_mc)
        CALL HPAK (530, thr_lt1_mc)
        CALL HPAKE(530, ethr1_lt1_mc)
        CALL HPAK (580, mx_lt1_mc)

          DO j = 1, 6
             DO i = 1, 8
               pn_tmp (i) = pt2_int_mc(i,j)
          ENDDO
           id= 300 + j
            CALL HPAK  (id, pn_tmp)
          ENDDO

c....   store H1 data in histograms
c....   pt2 distributions

****** data pt**2 distributions - cross section
          DO j = 1, 6
            DO i = 1, 8

              pn_tmp (i) = dsig(i,j)
              pne_tmp (i) = dsig_est(i,j)
              pns_tmp (i) = dsig_esy(i,j)
            ENDDO
              print *,'pn_tmp',(pn_tmp(i),i=1,8)
            id = -90 - j
            CALL HPAK  (id, pn_tmp)
            CALL HPAKE (id, pne_tmp)
            CALL HPAK  (id-60, pn_tmp)
            CALL HPAKE (id-60, pns_tmp)
          CALL HUNPAK (id, ptsqr, ' ', 1)
              print *,'ptsqr',(ptsqr(i),i=1,8)
          ENDDO
****** data pt**2 distributions centers of bins in pt**2
          DO j = 1, 6
             DO i = 1, 8
               pn_tmp (i) = pt2_int(i,j)
          ENDDO
           id= -300 - j
            CALL HPAK  (id, pn_tmp)
          ENDDO


        CALL HPAK (-510, thr)
        CALL HPAKE (-510, ethr1)
        CALL HPAK (-610, thr)
        CALL HPAKE (-610, ethr2)
        CALL HPAK (-560, mxx)

        CALL HPAK (-520, thr_lt1)
        CALL HPAKE (-520, ethr1_lt1)
        CALL HPAK (-620, thr_lt1)
        CALL HPAKE (-620, ethr2_lt1)
        CALl HPAK (-570, mx_lt1)

        CALL HPAK (-530, thr_gt1)
        CALL HPAKE (-530, ethr1_gt1)
        CALL HPAK (-630, thr_gt1)
        CALL HPAKE (-630, ethr2_gt1)
        CALL HPAK (-580, mx_gt1)

        CALL HPAK (-410, frac_1)
        CALL HPAKE (-410, efr1_1)
        CALL HPAK (-460, frac_1)
        CALL HPAKE (-460, efr2_1)
        CALL HPAK (-430, mxf_1)

        CALL HPAK (-420, frac_3)
        CALL HPAKE (-420, efr1_3)
        CALL HPAK (-470, frac_3)
        CALL HPAKE (-470, efr2_3)
        CALL HPAK (-440, mxf_3)

c....   print number of accepted events
        PRINT *, 'HZ97210-- Number of accepted moca events: ', nacc

      ENDIF

c----------------------------------------------------------------------
c---- Return

999   RETURN
      END
