	Subroutine HZ98085(iflag)
*****************************************************************************
* This photoproduction routine has to be run twice with the following
* code additions:
* 	+1000 for the DIRECT component run.
*	+2000 for the RESOLVED component run.
*       +0 for both together
* for all three phases (iflag=1,2,3)
*
* iflag = 1,2 or 3 depending on wether it is the initialization,
*		processing or termination phase (respectively)
*
* The default jetfinder is KTCLUS
* To change jetfinder use these values to call the initialisation phase.
* 1 + 10 * (jetfinder number)
*
*       These graph will be
*       meaningless unless Xsec and Ntot are set before
*       calling the termination routine.
*       (Xsec - total cross section returned by MC)
*       (Ntot - number of events passed to this routine)
*
* D* + jets measurement
*
* Cuts : 	Q2 < 1 GeV^2
*		0.19 < y_bj < 0.87
*     Jets:  ET1 > 7 GeV, ET2 > 6 GeV, |eta| < 2.4
*     Dstar: pt > 3 GeV , |eta| < 1.5
*
* Distribution:   id = 10, xgamma cross-section
* 
* Inclusive D* measurements.
*
* Cuts:       Q2 < 1 GeV^2
*           0.19 < y_bj < 0.87
*      Dstar: pt > 2 GeV , |eta| < 1.5
*
* Distributions:  id = 10, xgamma cross-section
*                      20, pT(D*)  cross-section
*                      30, eta(D*) cross-section for pT(D*) > 2 GeV
*                      40, eta(D*) cross-section for pT(D*) > 3 GeV
*                      50, eta(D*) cross-section for pT(D*) > 4 GeV
*                      60, eta(D*) cross-section for pT(D*) > 6 GeV
*
* Author : Matthew Wing (wing@mail.desy.de)
*
* (based on routine from Mark Hayes)
*
* Date   : 14/02/99
*
* Reference : DESY-98-085
*
*****************************************************************************
*
* HERA Tuning defined commons
 
	IMPLICIT NONE
 
#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
#include "hzjetcmn.inc"
 
	CHARACTER*6 jetf
	Character*6 xxxx
	DATA xxxx /'d98085'/

 
        INTEGER NJET
c        DOUBLE PRECISION Jets(50,8)
        DOUBLE PRECISION CONER
	SAVE CONER
 
	DOUBLE PRECISION elecE
	INTEGER myil,myip
	INTEGER i,j,loop
	DOUBLE PRECISION xgam
	DOUBLE PRECISION px(2),py(2),pz(2),ptsq(2)
	DOUBLE PRECISION ppsq(2),partrap,rap(2),Et(2)
	DOUBLE PRECISION ybj,q2
 
C --- cuts arrays
	DOUBLE PRECISION ybjcut(2),q2cut(2)
	DOUBLE PRECISION rapcut,Etcut(2)
	DATA ybjcut / 0.19D0, 0.87D0 /
	DATA q2cut / 0.0D0, 1.0D0 /
	DATA etcut / 7.0D0, 6.0D0 /
	DATA rapcut / 2.4D0 /

	INTEGER chjet, iproc, iflag
	SAVE chjet
	INTEGER hzflag
 
	real lumnb

	real ptdsbins(7)

C --- choose same jet finder as in the paper --- KTCLUS
	DATA chjet/3/
c --  binning for pT(D*) distribution
	data ptdsbins / 2., 3., 4., 5., 6., 8., 12. /
C *** Inclusive D* data
c
C *** pT cross-section
	real ptcr(6), ptsta(6), ptsysa(6), ptsysb(6)
	data ptcr   /  9.68,  4.94,  2.22,  1.076,  0.328,  0.067 /
        data ptsta  /  1.16,  0.31,  0.13,  0.073,  0.024,  0.008 /
        data ptsysa /  1.55,  0.30,  0.10,  0.071,  0.020,  0.004 /
	data ptsysb / -0.56, -0.31, -0.11, -0.043, -0.013, -0.006 /
C *** eta cross-section pT(D*) > 2GeV
	real eta2cr(6), eta2sta(6), eta2sysa(6), eta2sysb(6)
	data eta2cr   /  8.89,  8.16,  7.61,  5.23,  3.21,  4.65 /
        data eta2sta  /  0.81,  0.80,  0.79,  0.99,  1.02,  1.40 /
        data eta2sysa /  0.89,  0.70,  0.97,  1.09,  0.86,  1.49 /
	data eta2sysb / -0.33, -0.48, -0.48, -0.55, -0.66, -1.06 /
C *** eta cross-section pT(D*) > 3GeV
	real eta3cr(6), eta3sta(6), eta3sysa(6), eta3sysb(6)
	data eta3cr   /  2.96,  4.17,  3.88,  2.93,  2.11,  2.32 /
        data eta3sta  /  0.23,  0.29,  0.28,  0.28,  0.28,  0.34 /
        data eta3sysa /  0.24,  0.15,  0.25,  0.18,  0.19,  0.46 /
	data eta3sysb / -0.26, -0.26, -0.18, -0.25, -0.21, -0.37/
C *** eta cross-section pT(D*) > 4GeV
	real eta4cr(6), eta4sta(6), eta4sysa(6), eta4sysb(6)
	data eta4cr   /  1.021,  1.641,  1.877,  1.662,  1.090,  1.186 /
        data eta4sta  /  0.096,  0.137,  0.143,  0.137,  0.117,  0.142 /
        data eta4sysa /  0.079,  0.074,  0.117,  0.077,  0.138,  0.132 /
	data eta4sysb / -0.095, -0.118, -0.112, -0.067, -0.079, -0.137 /
C *** eta cross-section pT(D*) > 6GeV
	real eta6cr(6), eta6sta(6), eta6sysa(6), eta6sysb(6)
	data eta6cr   /  0.052,  0.331,  0.460,  0.398,  0.374,  0.242 /
        data eta6sta  /  0.052,  0.047,  0.061,  0.056,  0.045,  0.062 /
        data eta6sysa /  0.048,  0.054,  0.031,  0.032,  0.029,  0.030 /
	data eta6sysb / -0.037, -0.054, -0.039, -0.028, -0.036, -0.043 /

C *** Data for ET>7,6 GeV
	REAL cr(8),sta(8),sysa(8),sysb(8),banda(8),bandb(8)
C *** Actual data
	DATA cr    / 0.32, 1.06, 1.20, 0.98, 1.24, 1.80, 3.70, 2.87 /
	DATA sta   / 0.19, 0.30, 0.28, 0.31, 0.27, 0.36, 0.53, 0.37 /
	DATA sysa  / 0.14, 0.17, 0.17, 0.26, 0.33, 0.48, 0.61, 0.36 /
        DATA sysb  / -.26, -.22, -.36, -.26, -.24, -.20, -.65, -.33 /
	DATA banda / 0.00, 0.10, 0.17, 0.15, 0.18, 0.24, 0.54, 0.23 /
        DATA bandb / -.06, -.13, -.14, -.11, -.12, -.19, -.37, -.18 / 

*       Determine the process type.
	iproc=(mod(iflag,10000)/1000)
	if ((iproc.lt.0).or.(iproc.gt.2)) then
	   write(HZUNIT,*) 'HZ',xxxx,': illegal IPROC:',iproc
	   return
	endif
	
*       --- choose jet finder
	IF ((MOD(iflag,100)/10.gt.0).and.MOD(iflag,10).eq.1) THEN
	   chjet=INT(MOD(iflag,100)/10)
	ENDIF
	
	IF (MOD(iflag,10).eq.1) THEN
C *** Initialisation run ***
C *** Create directories ***
	  IF (iproc.le.1) THEN
	    CALL HCDIR('//HISTO',' ')
	    CALL HMDIR(xxxx,'S')
	    CALL HCDIR('//PAWC',' ')
	    CALL HMDIR(xxxx,'S')
C --- create proper cross-section data
	    call hbookb(-20, 'Data pt(D*) cross-section (nb)'
     ,                     , 6, ptdsbins, 0.)
	    call hbook1(-30, 
     &           'Data eta(D*) cross-section, pT(D*)>2 GeV (nb)'
     ,                     , 6, -1.5, 1.5, 0.)
	    call hbook1(-40, 
     &           'Data eta(D*) cross-section, pT(D*)>3 GeV (nb)'
     ,                     , 6, -1.5, 1.5, 0.)
	    call hbook1(-50, 
     &           'Data eta(D*) cross-section, pT(D*)>4 GeV (nb)'
     ,                     , 6, -1.5, 1.5, 0.)
	    call hbook1(-60, 
     &           'Data eta(D*) cross-section, pT(D*)>6 GeV (nb)'
     ,                     , 6, -1.5, 1.5, 0.)
	    CALL HBOOK1(-10,
     &           'Data xgamma cross-section (nb)'
     ,	                   , 8, 0., 1., 0.)
 
	    call hpak(-20, ptcr)
	    call hpak(-30, eta2cr)
	    call hpak(-40, eta3cr)
	    call hpak(-50, eta4cr)
	    call hpak(-60, eta6cr)
	    CALL HPAK(-10, cr)

	    call hpake(-20, ptsta)
	    call hpake(-30, eta2sta)
	    call hpake(-40, eta3sta)
	    call hpake(-50, eta4sta)
	    call hpake(-60, eta6sta)
	    CALL HPAKE(-10, sta)

	  ENDIF

	  CALL HCDIR('//HISTO/'//xxxx,' ')
	  CALL HCDIR('//PAWC/'//xxxx,' ')

	  CALL HBOOK1(iproc*100+10
     ,         , 'Monte Carlo xgamma cross-section (nb)'
     ,         , 8, 0., 1., 0.)

	  call hbookb(iproc*100+20
     ,         , 'Monte Carlo pt(D*) cross-section (nb)'
     ,         , 6, ptdsbins, 0.)
	  call hbook1(iproc*100+30
     ,         , 'Monte Carlo eta(D*) cross-section, pT(D*)>2 GeV (nb)'
     ,         , 6, -1.5, 1.5, 0.)
	  call hbook1(iproc*100+40
     ,         , 'Monte Carlo eta(D*) cross-section, pT(D*)>3 GeV (nb)'
     ,         , 6, -1.5, 1.5, 0.)
	  call hbook1(iproc*100+50
     ,         , 'Monte Carlo eta(D*) cross-section, pT(D*)>4 GeV (nb)'
     ,         , 6, -1.5, 1.5, 0.)
	  call hbook1(iproc*100+60
     ,         , 'Monte Carlo eta(D*) cross-section, pT(D*)>6 GeV (nb)'
     ,         , 6, -1.5, 1.5, 0.)
	  
	  CALL HIDOPT(0,'stat')
	  CALL HBARX(0)

C --- standard cone radius is 1.0
	  CALL HZJETRAD(2,CONER)
	  IF (CONER.lt.0.0) THEN
	    CONER=1.0
	  ENDIF
          CALL HZJTNAME(chjet,jetf)
 
	  WRITE(6,*)'**********************************************'
	  WRITE(6,*)'* HZ98085 called, histograms will be output. *'
	  WRITE(6,*)'*     Jet finder selected : ',jetf,'           *'
	  WRITE(6,1001)'*        with cone radius : ',CONER,'   *'
	  WRITE(6,*)'**********************************************'
 
 1001	  FORMAT(A29,F5.2,A13)
 
	ELSE IF (MOD(iflag,10).eq.2) THEN

C *** Filling runs ***
	
C *** Change directory to our graphs ***
	  CALL HCDIR('//PAWC/'//xxxx,' ')
 
	  q2=hzphokin(1)
	  IF (q2.eq.-1) THEN
	    PRINT*,'HZ98085: HZPHOKIN returns error for Q2'
	  ELSEIF (q2.lt.0) THEN
	    q2=abs(q2)
	  ENDIF
	  ybj=hzphokin(3)
	  IF (ybj.eq.-1) THEN
	    PRINT*,'HZ98085: HZPHOKIN returns error for ybj'
	  ENDIF
 
C *** Fill our histogram ***
 
	  IF ((ybj.ge.ybjcut(1)).and.(ybj.le.ybjcut(2))
     &		.and.(q2.lt.q2cut(2))) Then
 
	  hzflag=HZIBEAM(myip,myil)
	  IF (hzflag.ge.0) Then
	     elecE=abs(PHEP(4,myil))
	  ELSE
	     PRINT*,'HZ98085: Electron not found. E=0.001'
	     PRINT*,'         event will be rejected'
	     elecE=0.001
	  ENDIF
	
	  DO i=1,NHEP
	    IF (abs(IDHEP(i)).eq.413) THEN !D*+-
	      px(1)=PHEP(1,i)
	      py(1)=PHEP(2,i)
	      pz(1)=PHEP(3,i)
	      ptsq(1)=px(1)**2+py(1)**2
	      ppsq(1)=(sqrt(ptsq(1)+pz(1)**2)+ABS(pz(1)))**2
	      IF (ptsq(1).le.4.25E-18*ppsq(1)) THEN
		partrap=20
	      ELSE
		partrap=0.5*LOG(ppsq(1)/ptsq(1))
	      ENDIF
	      partrap=SIGN(partrap,pz(1))
	      IF (abs(partrap).lt.1.5
     +       .and.ptsq(1).gt.4.0.and.ptsq(1).lt.144.0) THEN

		call hfill(iproc*100+20, real(sqrt(ptsq(1))),0.0, wtx)
		call hfill(iproc*100+30, real(partrap),0.0, wtx)

		if (ptsq(1).gt.9.0) then
		  call hfill(iproc*100+40, real(partrap), 0.0, wtx)
		  CALL hzjtfind(chjet,CONER,NJET,Jets)
		  IF (NJET.ge.2) THEN
c       Find the two highest ET jets in the allowed rapidity range.
		     j=0
		     DO loop=1,njet
c       Search jets in the rapidity interval.
			IF ( ABS(jets(loop,1)).lt.rapcut ) THEN
			   j=j+1
			   IF (j.LE.2) THEN
			      Et(j)=Jets(loop,3)
			      rap(j)=Jets(loop,1)
			   ENDIF
			ENDIF
		     ENDDO

		    IF (Et(1).gt.Etcut(1).and.Et(2).gt.Etcut(2)) THEN
		      xgam=(Et(1)*exp(-1*rap(1))+Et(2)*exp(-1*rap(2)))
     +		          /(2*ybj*elecE)
		      CALL HFILL(iproc*100+10,REAL(xgam),0.0,wtx)
		    end if
		  end if
	        end if
		if (ptsq(1).gt.16.0) then
		  call hfill(iproc*100+50, real(partrap),0.0,wtx)
	        end if
		if (ptsq(1).gt.36.0) then
		  call hfill(iproc*100+60, real(partrap), 0.0,wtx)
	        end if
	      ENDIF		! momentum and eta cuts
	    ENDIF		! Dstar
	  ENDDO			! loop over HEPEVTP
 
	  ENDIF		! Q2 & y cuts
 
	ELSE IF (MOD(iflag,10).eq.3) THEN

	   if (iproc.eq.0.or.iproc.eq.2) then
	      write(hzunit,*) 'HZ98085: finishing off.'
	   endif

C *** Termination run ***
C *** Change directory ***
	  CALL HCDIR('//PAWC/'//xxxx,' ')
C *** Finish off histograms ***
 
	  IF (Xsec.eq.0) THEN
	    PRINT*,'HZ98085: termination called with zero cross section'
	    PRINT*,'          cross section graph meaningless'
	    PRINT*,'	      in process :',iproc*1000
	    Xsec=1
	  ENDIF
	  IF (Ntot.eq.0) THEN
	    PRINT*,'HZ98085: termination called with no total events'
	    PRINT*,'          cross section graph meaningless'
	    PRINT*,'	      in process :',iproc*1000
	    Ntot=1
	  ENDIF
 
C --- store Xsec and Ntot for both direct and resolved events.
	  lumnb = REAL(ntot/xsec)
	  DO i=10,60,10
	     CALL HZHINRM(100*iproc+i,0,lumnb,1)
	  ENDDO

	  IF (iproc.eq.2) THEN
C       --- add the cross sections together.
	     
	     DO i=10,60,10
		CALL HOPERA(100+i, '+e', 200+i, i,1,1)
	     ENDDO
	  ENDIF
 
	ELSE
C *** End ***
	  PRINT*,'HZ98085: Please run routine with iflag set to 1,2 or 3'
	  PRINT*,'         with +1000 or +2000 for DIR or RES events.'
	ENDIF
 
	RETURN
	END
