*CMZU:  1.02/04 07/12/98  15.41.41  by  Douglas M. Jansen
*-- Author :    Douglas M. Jansen   10/11/98
      subroutine HZ98169(IFLAG)
****************************************************************************
* Purpose: produced histograms for the distributions...
*          Event selection:
*          Reference:
* Arguments: iflag=1 initialisation
*            iflag=1 filling
*            iflag=3 termination
*
* written by: Douglas M. Jansen on 10/11/98
*
* This routine plots the semi-inclusive structure functions for
* leading proton and neutron production with pt <= 200 MeV/c.
* The semi-inclusive cross section e p -> e N X, where N is either
* a final state proton or neutron, is parameteried by F2LB(x,Q2,z)
* similar to the definition of F2D(3).
*
* Beams: 27.5 GeV positrons, 820 GeV protons   [1995/96 HERA running]
*
* Event selection:
* 2 GeV2 < Q2 < 50 GeV2
* 6E-5 < x < 6E-3
* Baryon pt, defined with respect to the incident proton
* beam, is less than or equal to 200 MeV
*
* Histograms:
* 101 - 112: The proton data points for F2LB(3).
* 201 - 212: The Monte Carlo predictions for protons.
* 301 - 312: The neutron data points for F2LB(3).
* 401 - 412: The Monte Carlo predictions for neutrons.
*
* The kumac k_98169 makes two nice plots.
*
* Reference: DESY: 98-169
*
****************************************************************************
      IMPLICIT NONE
*
#include "hzfunc.inc"
#include "hzhbook.inc"
#include "hepevtp.inc"
#include "heracmn.inc"
*
      Integer ihep,iflag
*     DESY Preprint number 98-169
      Character*5 xxxx
      Data xxxx/'98169'/

      real q2,x,pt,enel,enep,rsec
      real xlow, xhigh
      integer nentry,ierr,ibeam,ilep

      real pbin1(4),pbin2(4),pbin3(4),pbin4(4)
      real pbin5(4),pbin6(4),pbin7(4),pbin8(4)
      real pbin9(4),pbin10(4),pbin11(4),pbin12(4)
      real pebin1(4),pebin2(4),pebin3(4),pebin4(4)
      real pebin5(4),pebin6(4),pebin7(4),pebin8(4)
      real pebin9(4),pebin10(4),pebin11(4),pebin12(4)
      real nbin1(4),nbin2(4),nbin3(4),nbin4(4)
      real nbin5(4),nbin6(4),nbin7(4),nbin8(4)
      real nbin9(4),nbin10(4),nbin11(4),nbin12(4)
      real nebin1(4),nebin2(4),nebin3(4),nebin4(4)
      real nebin5(4),nebin6(4),nebin7(4),nebin8(4)
      real nebin9(4),nebin10(4),nebin11(4),nebin12(4)

      real scale1,scale2,scale3,scale4,scale5,scale6
      real scale7,scale8,scale9,scale10,scale11,scale12

      data pbin1 / 0.0514, 0.0590, 0.0580, 0.0499/
      data pbin2 / 0.0392, 0.0550, 0.0595, 0.0528/
      data pbin3 / 0.0357, 0.0354, 0.0403, 0.0461/
      data pbin4 / 0.0684, 0.0807, 0.1030, 0.0887/
      data pbin5 / 0.0631, 0.0619, 0.0626, 0.0572/
      data pbin6 / 0.0770, 0.0652, 0.0873, 0.1080/
      data pbin7 / 0.0411, 0.0859, 0.0736, 0.0631/
      data pbin8 / 0.0419, 0.1126, 0.0699, 0.0832/
      data pbin9 / 0.1024, 0.1168, 0.1253, 0.1114/
      data pbin10 / 0.0839, 0.0715, 0.0759, 0.0500/
      data pbin11 / 0.0737, 0.1030, 0.0398, 0.0840/
      data pbin12 / 0.1179, 0.1047, 0.0734, 0.0784/

      data pebin1 / 0.01223, 0.01069, 0.01018, 0.01354/
      data pebin2 / 0.00827, 0.00807, 0.00828, 0.01275/
      data pebin3 / 0.00956, 0.00742, 0.00800, 0.01272/
      data pebin4 / 0.01573, 0.01296, 0.01488, 0.02205/
      data pebin5 / 0.01615, 0.01254, 0.01205, 0.01638/
      data pebin6 / 0.02244, 0.01528, 0.01789, 0.03059/
      data pebin7 / 0.01404, 0.01745, 0.01530, 0.01945/
      data pebin8 / 0.01650, 0.02614, 0.01911, 0.02802/
      data pebin9 / 0.02746, 0.02278, 0.02322, 0.03218/
      data pebin10 / 0.02538, 0.01796, 0.01890, 0.01896/
      data pebin11 / 0.03003, 0.02822, 0.01676, 0.03201/
      data pebin12 / 0.03514, 0.02442, 0.01958, 0.02730/

      data nbin1 / 0.0829, 0.0470, 0.0396, 0.0125/
      data nbin2 / 0.0673, 0.0378, 0.0315, 0.0066/
      data nbin3 / 0.0582, 0.0296, 0.0257, 0.0059/
      data nbin4 / 0.1150, 0.0664, 0.0526, 0.0091/
      data nbin5 / 0.0952, 0.0450, 0.0414, 0.0101/
      data nbin6 / 0.1228, 0.0752, 0.0442, 0.0220/
      data nbin7 / 0.0984, 0.0527, 0.0488, 0.0080/
      data nbin8 / 0.0812, 0.0458, 0.0412, 0.0033/
      data nbin9 / 0.1295, 0.0749, 0.0533, 0.0202/
      data nbin10 / 0.0918, 0.0512, 0.0457, 0.0141/
      data nbin11 / 0.1650, 0.0952, 0.0509, 0.0433/
      data nbin12 / 0.1018, 0.0702, 0.0499, 0.0148/

      data nebin1 / 0.0225, 0.0168, 0.0074, 0.0076/
      data nebin2 / 0.0179, 0.0133, 0.0055, 0.0040/
      data nebin3 / 0.0155, 0.0105, 0.0047, 0.0037/
      data nebin4 / 0.0310, 0.0236, 0.0097, 0.0059/
      data nebin5 / 0.0257, 0.0161, 0.0076, 0.0062/
      data nebin6 / 0.0347, 0.0275, 0.0100, 0.0138/
      data nebin7 / 0.0272, 0.0192, 0.0095, 0.0055/
      data nebin8 / 0.0227, 0.0168, 0.0082, 0.0034/
      data nebin9 / 0.0362, 0.0274, 0.0116, 0.0133/
      data nebin10 / 0.0255, 0.0187, 0.0091, 0.0090/
      data nebin11 / 0.0497, 0.0365, 0.0165, 0.0283/
      data nebin12 / 0.0297, 0.0260, 0.0114, 0.0112/

      save nentry

*
*
* Data points from plots
*
*
      IF (iflag.eq.1) then
*
*      Initialisation: The following MUST always be done
*      (i) make subdirectory in PAWC
*       - use the name as the xxxxxx in HZxxxxxx subroutine
*      (i) make subdirectory in o/p file
*
       nentry=0
       Call hcdir('//PAWC',' ')
       call hmdir(xxxx,'S')
       Call hcdir('//HISTO',' ')
       call hmdir(xxxx,'S')
*
*      book your histograms
*

* *** For the proton data points ***
      xlow =580./820.
      xhigh=740./820.

      CALL HBOOK1(101,'proton x?L!',4,xlow,xhigh,0.)
      CALL HBOOK1(102,'proton x?L!',4,xlow,xhigh,0.)
      CALL HBOOK1(103,'proton x?L!',4,xlow,xhigh,0.)
      CALL HBOOK1(104,'proton x?L!',4,xlow,xhigh,0.)
      CALL HBOOK1(105,'proton x?L!',4,xlow,xhigh,0.)
      CALL HBOOK1(106,'proton x?L!',4,xlow,xhigh,0.)
      CALL HBOOK1(107,'proton x?L!',4,xlow,xhigh,0.)
      CALL HBOOK1(108,'proton x?L!',4,xlow,xhigh,0.)
      CALL HBOOK1(109,'proton x?L!',4,xlow,xhigh,0.)
      CALL HBOOK1(110,'proton x?L!',4,xlow,xhigh,0.)
      CALL HBOOK1(111,'proton x?L!',4,xlow,xhigh,0.)
      CALL HBOOK1(112,'proton x?L!',4,xlow,xhigh,0.)

* *** For the proton Monte Carlo prediction ***
      CALL HBOOK1(201,'proton x?L!',50,0.,1.,0.)
      CALL HBOOK1(202,'proton x?L!',50,0.,1.,0.)
      CALL HBOOK1(203,'proton x?L!',50,0.,1.,0.)
      CALL HBOOK1(204,'proton x?L!',50,0.,1.,0.)
      CALL HBOOK1(205,'proton x?L!',50,0.,1.,0.)
      CALL HBOOK1(206,'proton x?L!',50,0.,1.,0.)
      CALL HBOOK1(207,'proton x?L!',50,0.,1.,0.)
      CALL HBOOK1(208,'proton x?L!',50,0.,1.,0.)
      CALL HBOOK1(209,'proton x?L!',50,0.,1.,0.)
      CALL HBOOK1(210,'proton x?L!',50,0.,1.,0.)
      CALL HBOOK1(211,'proton x?L!',50,0.,1.,0.)
      CALL HBOOK1(212,'proton x?L!',50,0.,1.,0.)

* *** For the neutron data points ***
      CALL HBOOK1(301,'neutron x?L!',4,0.2,1.,0.)
      CALL HBOOK1(302,'neutron x?L!',4,0.2,1.,0.)
      CALL HBOOK1(303,'neutron x?L!',4,0.2,1.,0.)
      CALL HBOOK1(304,'neutron x?L!',4,0.2,1.,0.)
      CALL HBOOK1(305,'neutron x?L!',4,0.2,1.,0.)
      CALL HBOOK1(306,'neutron x?L!',4,0.2,1.,0.)
      CALL HBOOK1(307,'neutron x?L!',4,0.2,1.,0.)
      CALL HBOOK1(308,'neutron x?L!',4,0.2,1.,0.)
      CALL HBOOK1(309,'neutron x?L!',4,0.2,1.,0.)
      CALL HBOOK1(310,'neutron x?L!',4,0.2,1.,0.)
      CALL HBOOK1(311,'neutron x?L!',4,0.2,1.,0.)
      CALL HBOOK1(312,'neutron x?L!',4,0.2,1.,0.)

* *** For the neutron Monte Carlo prediction ***

      CALL HBOOK1(401,'neutron x?L!',50,0.,1.,0.)
      CALL HBOOK1(402,'neutron x?L!',50,0.,1.,0.)
      CALL HBOOK1(403,'neutron x?L!',50,0.,1.,0.)
      CALL HBOOK1(404,'neutron x?L!',50,0.,1.,0.)
      CALL HBOOK1(405,'neutron x?L!',50,0.,1.,0.)
      CALL HBOOK1(406,'neutron x?L!',50,0.,1.,0.)
      CALL HBOOK1(407,'neutron x?L!',50,0.,1.,0.)
      CALL HBOOK1(408,'neutron x?L!',50,0.,1.,0.)
      CALL HBOOK1(409,'neutron x?L!',50,0.,1.,0.)
      CALL HBOOK1(410,'neutron x?L!',50,0.,1.,0.)
      CALL HBOOK1(411,'neutron x?L!',50,0.,1.,0.)
      CALL HBOOK1(412,'neutron x?L!',50,0.,1.,0.)

* *** FILL THE HISTOGRAMS WITH THE DATA POINTS ***
      CALL HPAK(101,PBIN1)
      CALL HPAK(102,PBIN2)
      CALL HPAK(103,PBIN3)
      CALL HPAK(104,PBIN4)
      CALL HPAK(105,PBIN5)
      CALL HPAK(106,PBIN6)
      CALL HPAK(107,PBIN7)
      CALL HPAK(108,PBIN8)
      CALL HPAK(109,PBIN9)
      CALL HPAK(110,PBIN10)
      CALL HPAK(111,PBIN11)
      CALL HPAK(112,PBIN12)

      CALL HPAKE(101,PEBIN1)
      CALL HPAKE(102,PEBIN2)
      CALL HPAKE(103,PEBIN3)
      CALL HPAKE(104,PEBIN4)
      CALL HPAKE(105,PEBIN5)
      CALL HPAKE(106,PEBIN6)
      CALL HPAKE(107,PEBIN7)
      CALL HPAKE(108,PEBIN8)
      CALL HPAKE(109,PEBIN9)
      CALL HPAKE(110,PEBIN10)
      CALL HPAKE(111,PEBIN11)
      CALL HPAKE(112,PEBIN12)

      CALL HPAK(301,NBIN1)
      CALL HPAK(302,NBIN2)
      CALL HPAK(303,NBIN3)
      CALL HPAK(304,NBIN4)
      CALL HPAK(305,NBIN5)
      CALL HPAK(306,NBIN6)
      CALL HPAK(307,NBIN7)
      CALL HPAK(308,NBIN8)
      CALL HPAK(309,NBIN9)
      CALL HPAK(310,NBIN10)
      CALL HPAK(311,NBIN11)
      CALL HPAK(312,NBIN12)

      CALL HPAKE(301,NEBIN1)
      CALL HPAKE(302,NEBIN2)
      CALL HPAKE(303,NEBIN3)
      CALL HPAKE(304,NEBIN4)
      CALL HPAKE(305,NEBIN5)
      CALL HPAKE(306,NEBIN6)
      CALL HPAKE(307,NEBIN7)
      CALL HPAKE(308,NEBIN8)
      CALL HPAKE(309,NEBIN9)
      CALL HPAKE(310,NEBIN10)
      CALL HPAKE(311,NEBIN11)
      CALL HPAKE(312,NEBIN12)

      Else if(iflag.eq.2) then
*      Filling: The following MUST always be done
*      (i) move to the correct sub-directory in PAWC
*
       call hcdir('//PAWC/'//xxxx,' ')

       q2 = real(hzdiskin(1))
       x  = real(hzdiskin(2))

       enel=0.
       enep=0.
       ierr=HZIBEAM(ibeam,ilep)
       if(ierr.ne.1) then
          write(6,*) 'HZ'//xxxx,' beams not found ! '
          return
        else
          enel=real(PHEP(4,ilep))
          enep=real(PHEP(4,ibeam))
        endif

        if(abs(enel-27.5).ge.0.2.or.abs(enep-820.0).ge.0.2) then
           nentry=nentry+1
           if(nentry.lt.5) then
             write(6,*)'HZ'//xxxx,' Beam Energies not correct!!',
     &       enel,enep
             return
           elseif(nentry.eq.5) then
             write(6,*)'HZ'//xxxx,
     &       ' Beam Energies not correct - Last Message!!'
             return
           else
             return
           endif
        endif

*
*       Fill the histograms
*
        do 10 ihep = 1, nhep
          if (isthep(ihep).ne.1.or.ihep.eq.ibeam) goto 10

          pt=real(dsqrt(PHEP(1,IHEP)**2+PHEP(2,IHEP)**2))
          if(pt.gt.0.2) goto 10

* *** FILL PROTON HISTOGRAMS ***
          if (idhep(ihep).eq.2212) then

              IF(Q2.GE.2.AND.Q2.LT.3.5.AND.X.GE.0.00006.AND.
     *           X.LT.0.00018) THEN
                 CALL HFILL(201,real(PHEP(4,IHEP)/PHEP(4,IBEAM)),0.,1.)
              ENDIF

              IF(Q2.GE.2.AND.Q2.LT.3.5.AND.X.GE.0.00018.AND.
     *           X.LT.0.00060) THEN
                 CALL HFILL(202,real(PHEP(4,IHEP)/PHEP(4,IBEAM)),0.,1.)
              ENDIF

              IF(Q2.GE.2.AND.Q2.LT.3.5.AND.X.GE.0.00060.AND.
     *           X.LT.0.0018) THEN
                 CALL HFILL(203,real(PHEP(4,IHEP)/PHEP(4,IBEAM)),0.,1.)
              ENDIF

              IF(Q2.GE.3.5.AND.Q2.LT.6.0.AND.X.GE.0.00018.AND.
     *           X.LT.0.00060) THEN
                 CALL HFILL(204,real(PHEP(4,IHEP)/PHEP(4,IBEAM)),0.,1.)
              ENDIF

              IF(Q2.GE.3.5.AND.Q2.LT.6.0.AND.X.GE.0.00060.AND.
     *           X.LT.0.0018) THEN
                 CALL HFILL(205,real(PHEP(4,IHEP)/PHEP(4,IBEAM)),0.,1.)
              ENDIF

              IF(Q2.GE.6.0.AND.Q2.LT.10.0.AND.X.GE.0.00018.AND.
     *           X.LT.0.00060) THEN
                 CALL HFILL(206,real(PHEP(4,IHEP)/PHEP(4,IBEAM)),0.,1.)
              ENDIF

              IF(Q2.GE.6.0.AND.Q2.LT.10.0.AND.X.GE.0.00060.AND.
     *           X.LT.0.0018) THEN
                 CALL HFILL(207,real(PHEP(4,IHEP)/PHEP(4,IBEAM)),0.,1.)
              ENDIF

              IF(Q2.GE.6.0.AND.Q2.LT.10.0.AND.X.GE.0.0018.AND.
     *           X.LT.0.0060) THEN
                 CALL HFILL(208,real(PHEP(4,IHEP)/PHEP(4,IBEAM)),0.,1.)
              ENDIF

              IF(Q2.GE.10.0.AND.Q2.LT.20.0.AND.X.GE.0.00060.AND.
     *           X.LT.0.0018) THEN
                 CALL HFILL(209,real(PHEP(4,IHEP)/PHEP(4,IBEAM)),0.,1.)
              ENDIF

              IF(Q2.GE.10.0.AND.Q2.LT.20.0.AND.X.GE.0.0018.AND.
     *           X.LT.0.0060) THEN
                 CALL HFILL(210,real(PHEP(4,IHEP)/PHEP(4,IBEAM)),0.,1.)
              ENDIF

              IF(Q2.GE.20.0.AND.Q2.LT.50.0.AND.X.GE.0.00060.AND.
     *           X.LT.0.0018) THEN
                 CALL HFILL(211,real(PHEP(4,IHEP)/PHEP(4,IBEAM)),0.,1.)
              ENDIF

              IF(Q2.GE.20.0.AND.Q2.LT.50.0.AND.X.GE.0.0018.AND.
     *           X.LT.0.0060) THEN
                 CALL HFILL(212,real(PHEP(4,IHEP)/PHEP(4,IBEAM)),0.,1.)
              ENDIF

          endif

* *** FILL NEUTRON HISTOGRAMS ***
          if (idhep(ihep).eq.2112) then

              IF(Q2.GE.2.AND.Q2.LT.3.5.AND.X.GE.0.00006.AND.
     *           X.LT.0.00018) THEN
                 CALL HFILL(401,real(PHEP(4,IHEP)/PHEP(4,IBEAM)),0.,1.)
              ENDIF

              IF(Q2.GE.2.AND.Q2.LT.3.5.AND.X.GE.0.00018.AND.
     *           X.LT.0.00060) THEN
                 CALL HFILL(402,real(PHEP(4,IHEP)/PHEP(4,IBEAM)),0.,1.)
              ENDIF

              IF(Q2.GE.2.AND.Q2.LT.3.5.AND.X.GE.0.00060.AND.
     *           X.LT.0.0018) THEN
                 CALL HFILL(403,real(PHEP(4,IHEP)/PHEP(4,IBEAM)),0.,1.)
              ENDIF

              IF(Q2.GE.3.5.AND.Q2.LT.6.0.AND.X.GE.0.00018.AND.
     *           X.LT.0.00060) THEN
                 CALL HFILL(404,real(PHEP(4,IHEP)/PHEP(4,IBEAM)),0.,1.)
              ENDIF

              IF(Q2.GE.3.5.AND.Q2.LT.6.0.AND.X.GE.0.00060.AND.
     *           X.LT.0.0018) THEN
                 CALL HFILL(405,real(PHEP(4,IHEP)/PHEP(4,IBEAM)),0.,1.)
              ENDIF

              IF(Q2.GE.6.0.AND.Q2.LT.10.0.AND.X.GE.0.00018.AND.
     *           X.LT.0.00060) THEN
                 CALL HFILL(406,real(PHEP(4,IHEP)/PHEP(4,IBEAM)),0.,1.)
              ENDIF

              IF(Q2.GE.6.0.AND.Q2.LT.10.0.AND.X.GE.0.00060.AND.
     *           X.LT.0.0018) THEN
                 CALL HFILL(407,real(PHEP(4,IHEP)/PHEP(4,IBEAM)),0.,1.)
              ENDIF

              IF(Q2.GE.6.0.AND.Q2.LT.10.0.AND.X.GE.0.0018.AND.
     *           X.LT.0.0060) THEN
                 CALL HFILL(408,real(PHEP(4,IHEP)/PHEP(4,IBEAM)),0.,1.)
              ENDIF

              IF(Q2.GE.10.0.AND.Q2.LT.20.0.AND.X.GE.0.00060.AND.
     *           X.LT.0.0018) THEN
                 CALL HFILL(409,real(PHEP(4,IHEP)/PHEP(4,IBEAM)),0.,1.)
              ENDIF

              IF(Q2.GE.10.0.AND.Q2.LT.20.0.AND.X.GE.0.0018.AND.
     *           X.LT.0.0060) THEN
                 CALL HFILL(410,real(PHEP(4,IHEP)/PHEP(4,IBEAM)),0.,1.)
              ENDIF

              IF(Q2.GE.20.0.AND.Q2.LT.50.0.AND.X.GE.0.00060.AND.
     *           X.LT.0.0018) THEN
                 CALL HFILL(411,real(PHEP(4,IHEP)/PHEP(4,IBEAM)),0.,1.)
              ENDIF

              IF(Q2.GE.20.0.AND.Q2.LT.50.0.AND.X.GE.0.0018.AND.
     *           X.LT.0.0060) THEN
                 CALL HFILL(412,real(PHEP(4,IHEP)/PHEP(4,IBEAM)),0.,1.)
              ENDIF

          endif

 10     continue
*
      Else if(iflag.eq.3) then
*      Termination: The following MUST always be done
*      (i) Move to the correct PAW subdirectory
*
       call hcdir('//PAWC/'//xxxx,' ')
*
*      Histo manipulation
*
      rsec=REAL(XSEC)/1000.0

      SCALE1 = RSEC/NTOT/0.02/46.34171
      SCALE2 = RSEC/NTOT/0.02/61.27304
      SCALE3 = RSEC/NTOT/0.02/59.57490
      SCALE4 = RSEC/NTOT/0.02/31.85264
      SCALE5 = RSEC/NTOT/0.02/32.39826
      SCALE6 = RSEC/NTOT/0.02/16.07404
      SCALE7 = RSEC/NTOT/0.02/17.52318
      SCALE8 = RSEC/NTOT/0.02/20.34958
      SCALE9 = RSEC/NTOT/0.02/12.28553
      SCALE10= RSEC/NTOT/0.02/14.92930
      SCALE11= RSEC/NTOT/0.02/6.23943
      SCALE12= RSEC/NTOT/0.02/8.44929

      call hopera(201,'+',201,201,0.,SCALE1)
      call hopera(202,'+',202,202,0.,SCALE2)
      call hopera(203,'+',203,203,0.,SCALE3)
      call hopera(204,'+',204,204,0.,SCALE4)
      call hopera(205,'+',205,205,0.,SCALE5)
      call hopera(206,'+',206,206,0.,SCALE6)
      call hopera(207,'+',207,207,0.,SCALE7)
      call hopera(208,'+',208,208,0.,SCALE8)
      call hopera(209,'+',209,209,0.,SCALE9)
      call hopera(210,'+',210,210,0.,SCALE10)
      call hopera(211,'+',211,211,0.,SCALE11)
      call hopera(212,'+',212,212,0.,SCALE12)

      call hopera(401,'+',401,401,0.,SCALE1)
      call hopera(402,'+',402,402,0.,SCALE2)
      call hopera(403,'+',403,403,0.,SCALE3)
      call hopera(404,'+',404,404,0.,SCALE4)
      call hopera(405,'+',405,405,0.,SCALE5)
      call hopera(406,'+',406,406,0.,SCALE6)
      call hopera(407,'+',407,407,0.,SCALE7)
      call hopera(408,'+',408,408,0.,SCALE8)
      call hopera(409,'+',409,409,0.,SCALE9)
      call hopera(410,'+',410,410,0.,SCALE10)
      call hopera(411,'+',411,411,0.,SCALE11)
      call hopera(412,'+',412,412,0.,SCALE12)

*      Deal with data plots
*
      Endif
*
      RETURN
*
      END
