*CMZ :  2.00/06 02/05/2002  15.02.41  by  Tancredi Carli
*CMZ :  2.00/05 01/08/2000  10.32.33  by  Tancredi Carli
*CMZ :  2.00/00 13/08/99  10.06.19  by  Hannes Jung
*-- Author :    Thorsten Wengler   14/04/98
      subroutine HZ99094(IFLAG)
**********************************************************************
* Purpose: histograms for the fwd pi0 x-sections 1996
*
* to normalize histgrams. This routine is included at the end of this
* file.
*
* The Routine is intended to run with the HzTool Package
* Theta is measured wrt the incoming proton direction (=z direction)
*
* DIS Event selection: 2.0<Q2<70.0 GeV2
*                      0.1<y<0.6
*
* pi0 selection: 5 < Theta < 25 degrees (in lab)
*                Pt > 2.5 GeV (in hcms)
*                x_pi=E_pi/E_proton> 0.01 (in lab)
*
**********************************************************************
* Arguments: iflag=1 initialisation
*            iflag=1 filling
*            iflag=3 termination
*
* Output Histograms:
*
*--------------------------------------------------------------------
* Histogram   Description
*--------------------------------------------------------------------
*                        Data Plots
*--------------------------------------------------------------------
*     -114    ds/dq2  [pb/GeV2]
*     -115    ds/dq2  [pb/GeV2] for Pt > 3.5 GeV (in hcms)
*     -116    ds/dx  [pb/GeV2] for Pt > 3.5 GeV (in hcms)
*     -101    ds/dx [nb] 2.0 <q2< 4.5  GeV2
*     -102    ds/dx [nb] 4.5 <q2< 15.0 GeV2
*     -103    ds/dx [nb] 15.0<q2< 70.0 GeV2
*     -131    ds/deta [pb] 2.0 <q2< 4.5  GeV2
*     -132    ds/deta [pb] 4.5 <q2< 15.0 GeV2
*     -133    ds/deta [pb] 15.0<q2< 70.0 GeV2
*     -171    ds/dpt [pb/GeV] 2.0 <q2< 4.5  GeV2
*     -172    ds/dpt [pb/GeV] 4.5 <q2< 15.0 GeV2
*     -173    ds/dpt [pb/GeV] 15.0<q2< 70.0 GeV2
*--------------------------------------------------------------------
*                                              Monte Carlo Histograms
*--------------------------------------------------------------------
*     114    ds/dq2  [pb/GeV2]
*     115    ds/dq2  [pb/GeV2] for Pt > 3.5 GeV (in hcms)
*     116    ds/dx  [pb/GeV2] for Pt > 3.5 GeV (in hcms)
*     101    ds/dx [nb] 2.0 <q2< 4.5  GeV2
*     102    ds/dx [nb] 4.5 <q2< 15.0 GeV2
*     103    ds/dx [nb] 15.0<q2< 70.0 GeV2
*     131    ds/deta [pb] 2.0 <q2< 4.5  GeV2
*     132    ds/deta [pb] 4.5 <q2< 15.0 GeV2
*     133    ds/deta [pb] 15.0<q2< 70.0 GeV2
*     171    ds/dpt [pb/GeV] 2.0 <q2< 4.5  GeV2
*     172    ds/dpt [pb/GeV] 4.5 <q2< 15.0 GeV2
*     173    ds/dpt [pb/GeV] 15.0<q2< 70.0 GeV2
*
* rate histos:  divided by DIS histo
*     -100101    drate/dx [nb] 2.0 <q2< 4.5  GeV2
*     -100102    drate/dx [nb] 4.5 <q2< 15.0 GeV2
*     -100103    drate/dx [nb] 15.0<q2< 70.0 GeV2
*--------------------------------------------------------------------
*
* written by: T.Wengler 01. Sptember 1998
*********************************************************************
      IMPLICIT NONE

#include "hzfunc.inc"
#include "hzhbook.inc"
#include "hepevtp.inc"
#include "heracmn.inc"
      Logical Debug
      Parameter(Debug=.false.)

*Lepto common
      integer LST
      real  CUT,PARL,X,Y,W2,Q2,U
      COMMON /LEPTOU/ CUT(14),LST(40),PARL(30),X,Y,W2,Q2,U

*---PYTHIA common
      real parp
      integer mstp
      COMMON /PYPARS/ MSTP(200),PARP(200)
*
*
*
      Double precision phad,pgam(4),pcm(4),ph(5),plab(5,NMXHEP)
      Double precision plep,eout(4),p(4,1),q(4,1),myxsecdp,myxsdisdp
      Double precision newxs
      real ethcms,myxsec,myxsdis,wtxdis
      integer retc
*
      Integer i,iflag,ihep
      Character*5 xxxx
      Integer nentry
      Data xxxx/'99094'/,NENTRY/0/
      save xxxx
*
      real a(3),b(3),thgg,thmin,enpi
      Real pi,rd,eb
      Integer iel,idum,ipbeam,iebeam
      Parameter (pi=3.1415927,rd=180./pi)
      Real enel,thel,efwd,th,lx,lxj
      real xwght,q2wght,ywght,pdfset,rewghttw
      Integer ibin,nj,ij,naj,naj2,np,np4,np5,j,nsel
*
      Integer ierr
      Integer maxhi,modjet,maxjet
      Parameter (maxhi=100,modjet=2,maxjet=30)
      Real  selj(maxjet,4),help,empz,empzh,xg,empzl,empzhl,xgl
      Integer ipjet(maxjet),l,k
      Real nev(maxhi),lum
      Double precision rcone
      Parameter (rcone=1.)
***********************************
*
* Define bins for histograms
*
* Q2
      INTEGER piq2anb
      PARAMETER (piq2anb=10)
      REAL piq2ab(piq2anb+1)
      DATA piq2ab/2.0, 2.4, 3.0, 3.8, 4.9, 6.5, 9.1,
     +     13.0, 20.0, 32.0, 70.0/
*
* Q2 for higher pt cut of 3.5 GeV
      INTEGER piq3anb
      PARAMETER (piq3anb=5)
      REAL piq3ab(piq3anb+1)
      DATA piq3ab/2.0, 3.0, 4.9, 9.1, 20.0, 70.0/
*
* 3x x_bj
      INTEGER pixbj1nb
      PARAMETER (pixbj1nb=5)
      REAL pixbj1b(pixbj1nb+1)
      DATA pixbj1b/4.2E-5,7.9E-5,1.1E-4,1.7E-4,2.5E-4,4.2E-4/
*
      INTEGER pixbj2nb
      PARAMETER (pixbj2nb=5)
      REAL pixbj2b(pixbj2nb+1)
      DATA pixbj2b/1.1E-4,2.0E-4,2.9E-4,3.9E-4,5.5E-4,1.1E-3/
*
      INTEGER pixbj3nb
      PARAMETER (pixbj3nb=4)
      REAL pixbj3b(pixbj3nb+1)
      DATA pixbj3b/3.9E-4,7.9E-4,1.3E-3,1.9E-3,6.3E-3/
*
* x for higher pt cut of 3.5 GeV
      INTEGER pixbj4nb
      PARAMETER (pixbj4nb=5)
      REAL pixbj4b(pixbj4nb+1)
      DATA pixbj4b/1.0E-4, 1.9E-4, 3.0E-4, 5.0E-4,
     +     1.1E-3, 6.3E-3/
*
*3x eta
      INTEGER pieta1nb
      PARAMETER (pieta1nb=5)
      REAL pieta1b(pieta1nb+1)
      DATA pieta1b/1.5,1.93,2.23,2.56,2.85,3.13/
*
      INTEGER pieta2nb
      PARAMETER (pieta2nb=5)
      REAL pieta2b(pieta2nb+1)
      DATA pieta2b/1.5,1.91,2.22,2.5,2.82,3.13/
*
      INTEGER pieta3nb
      PARAMETER (pieta3nb=4)
      REAL pieta3b(pieta3nb+1)
      DATA pieta3b/1.5,1.93,2.26,2.69,3.13/
*
*3x pt
      INTEGER piet1nb
      PARAMETER (piet1nb=5)
      REAL piet1b(piet1nb+1)
      DATA piet1b/2.5,2.8,3.3,4.0,5.2,8.0/
*
      INTEGER piet2nb
      PARAMETER (piet2nb=5)
      REAL piet2b(piet2nb+1)
      DATA piet2b/2.5,2.8,3.4,4.1,5.2,8.0/
*
      INTEGER piet3nb
      PARAMETER (piet3nb=4)
      REAL piet3b(piet3nb+1)
      DATA piet3b/2.5,2.9,3.5,4.7,8.0/
**********************************************************************
*
* Define Variables
*
* Nevt -> number of events
      real Nevt,Nevtw,Npass
      real npdis
      save Nevt
      save Nevtw
      save Npass
      save npdis
* Ee -> electron energy
* The -> electron theta
* Phe -> electron phi
* xpi ->  pi E/820.
* Thpi -> pi theta
* Phpi -> pi phi
* Ptpi -> pi pt
      real Ee, The, Phe, xpi, Thpi, Phpi, Ptpi, etapi
*
* --- define 3 Q2 regions in which to plot x-Bj dependencies
*
      REAL q2re1lo,q2re2lo,q2re3lo,q2re1hi,q2re2hi,q2re3hi
      PARAMETER (q2re1lo=2.0,q2re1hi=4.5)
      PARAMETER (q2re2lo=4.5,q2re2hi=15.0)
      PARAMETER (q2re3lo=15.0,q2re3hi=70.0)

* define cut parameters (prelim. secenario)
*
      real ylow,yhigh,eelow,thelow,thehigh
      real xpilow,xpihigh,ptpilow,ptpihigh,etapilow,etapihigh
      parameter(ylow=0.1)
      parameter(yhigh=0.6)
      parameter(xpilow=0.01)
      parameter(xpihigh=99.0)
      parameter(ptpilow=2.5)
      parameter(ptpihigh=99.0)
      parameter(etapilow=1.5)
      parameter(etapihigh=3.13)
* --- define higher pt-cut for additional plots
      real addpt
      parameter(addpt=3.5)
*
* define H1 pi0 x-sections
*
* for pt > 2.5 GeV
      real pq2(10),pq2se(10),pq2te(10)
      real px1(5),px1se(5),px1te(5)
      real px2(5),px2se(5),px2te(5)
      real px3(4),px3se(4),px3te(4)
      real peta1(5),peta1se(5),peta1te(5)
      real peta2(5),peta2se(5),peta2te(5)
      real peta3(4),peta3se(4),peta3te(4)
      real ppt1(5),ppt1se(5),ppt1te(5)
      real ppt2(5),ppt2se(5),ppt2te(5)
      real ppt3(4),ppt3se(4),ppt3te(4)
* for pt > 3.5 GeV
      real pq22(5),pq22se(5),pq22te(5)
      real px4(5),px4se(5),px4te(5)

* rel. err. for rates
      real pq2rsy(10),pq3rsy(5)
      real px1rsy(5),px2rsy(5),px3rsy(4),px4rsy(5)

* for pt > 2.5 GeV
      data pq2/139.35,106.168,78.1601,54.4862,35.5119,18.961,
     +     11.5121,7.11915,3.6436,1.08484/
      data pq2se/16.179,10.8842,7.34308,4.75706,3.12018,1.81492,
     +     1.14794,0.681489,0.360846,0.101381/
      data pq2te/29.5678,23.5225,13.3422,10.4635,5.27709,3.61389,
     +     1.81803,1.23043,0.627047,0.219166/
      data px1/1189.4,1260.71,970.313,609.441,159.994/
      data px1se/133.668,142.596,96.4167,75.2521,26.8072/
      data px1te/188.539,253.011,201.361,120.972,52.5223/
      data px2/397.611,366.411,336.702,192.65,63.5073/
      data px2se/42.8972,41.0226,38.9405,21.4877,6.84674/
      data px2te/61.3354,59.0537,64.2526,33.8743,13.7813/
      data px3/72.4478,55.811,43.0783,6.44535/
      data px3se/8.74659,6.25791,5.49562,0.742444/
      data px3te/16.778,12.864,9.80771,1.23017/
      data peta1/141.393,164.624,129.465,107.787,111.024/
      data peta1se/15.7144,20.3144,14.5706,13.0785,15.1506/
      data peta1te/27.9372,31.3005,28.5719,26.0788,26.348/
      data peta2/103.374,144.833,130.951,114.562,89.029/
      data peta2se/11.1443,14.8855,14.305,12.3191,10.2301/
      data peta2te/16.1466,29.1717,21.4435,20.7014,22.1032/
      data peta3/66.1792,93.4857,73.2534,43.6127/
      data peta3se/7.93505,11.2445,8.11797,5.38275/
      data peta3te/14.8967,17.5536,13.527,10.6018/
      data ppt1/186.682,131.718,65.3347,25.1165,6.47844/
      data ppt1se/20.4757,12.7326,7.52587,3.36217,1.23232/
      data ppt1te/91.939,22.0191,17.5243,4.2398,1.85798/
      data ppt2/156.887,93.484,56.8597,16.5697,4.4512/
      data ppt2se/15.3586,7.77076,6.35731,2.39455,0.751803/
      data ppt2te/74.0591,21.7324,9.50821,3.77732,1.18351/
      data ppt3/89.0546,54.0469,19.9416,4.71727/
      data ppt3se/9.53043,5.88907,2.46298,0.717909/
      data ppt3te/34.6819,8.40795,3.91176,0.939199/

* for pt > 3.5 GeV
      data pq22/35.0229,22.543,10.0461,3.199,0.645272/
      data pq22se/4.38384,2.3367,1.02995,0.355149,0.0689242/
      data pq22te/7.65896,3.8773,1.54139,0.554968,0.112668/
      data px4/374.066,249.41,165.41,60.2614,5.89598/
      data px4se/41.5554,31.8851,20.6637,6.48355,0.686599/
      data px4te/69.5626,45.3052,29.9305,10.511,1.50017/

* relative errors +10 % for rates
      data pq2rsy/0.203819,0.220405,0.174108,0.198132,0.156084,
     +     0.192782,0.158102,0.175237,0.172652,0.205137/
      data px1rsy/0.149992,0.193605,0.207826,0.184809,0.299487/
      data px2rsy/0.14885,0.153103,0.18177,0.168751,0.213231/
      data px3rsy/0.221488,0.224843,0.213447,0.182097/
      data pq3rsy/0.205317,0.169818,0.151758,0.166645,0.170522/
      data px4rsy/0.179558,0.163257,0.16473,0.169847,0.247343/

*********************************************************************
*
*                      Initialization
*
*********************************************************************
      NENTRY=NENTRY+1
*
      IF (iflag.eq.1) then
*
         Nevt=0.
         Nevtw=0.
         Npass=0.
         npdis=0.
*
*
* Initialisation: The following MUST always be done
* (i) make subdirectory in PAWC
* - use the name as the xxxxxx in HZxxxxxx subroutine
* (i) make subdirectory in o/p file
*
         Call hcdir('//PAWC',' ')
         call hmdir(xxxx,'S')
         Call hcdir('//HISTO',' ')
         call hmdir(xxxx,'S')
*
* book histograms -------------------
*
* pi0 x-sections
         CALL HBOOKB(101,'[p]^o![:] x?Bj,1!',pixbj1nb,pixbj1b,0.)
         CALL HBOOKB(102,'[p]^o![:] x?Bj,2!',pixbj2nb,pixbj2b,0.)
         CALL HBOOKB(103,'[p]^o![:] x?Bj,3!',pixbj3nb,pixbj3b,0.)
         CALL HBOOKB(131,'[p]^o![:] [c]?1!',pieta1nb,pieta1b,0.)
         CALL HBOOKB(132,'[p]^o![:] [c]?2!',pieta2nb,pieta2b,0.)
         CALL HBOOKB(133,'[p]^o![:] [c]?3!',pieta3nb,pieta3b,0.)
         CALL HBOOKB(171,'[p]^o![:] E?T,1!(hcms)',piet1nb,piet1b,0.)
         CALL HBOOKB(172,'[p]^o![:] E?T,2!(hcms)',piet2nb,piet2b,0.)
         CALL HBOOKB(173,'[p]^o![:] E?T,3!(hcms)',piet3nb,piet3b,0.)
         CALL HBOOKB(114,'[p]^o![:] Q^2!',piq2anb,piq2ab,0.)
         CALL HBOOKB(115,'[p]^o![:] Q^2!',piq3anb,piq3ab,0.)
         CALL HBOOKB(116,'[p]^o![:] Q^2!',pixbj4nb,pixbj4b,0.)
*
* DIS histos for calc. of Rate
         CALL HCOPY(101,110101,' ')
         CALL HCOPY(102,110102,' ')
         CALL HCOPY(103,110103,' ')
*
* DIS histos for calc. of Rate in data
         CALL HCOPY(101,-110101,' ')
         CALL HCOPY(102,-110102,' ')
         CALL HCOPY(103,-110103,' ')
*
* histograms for data (with stat. error, 10000+hid has tot. error)
         CALL HCOPY(114,-114,' ')
         CALL HCOPY(114,-10114,' ')
         CALL HCOPY(115,-115,' ')
         CALL HCOPY(115,-10115,' ')
         CALL HCOPY(116,-116,' ')
         CALL HCOPY(116,-10116,' ')
         CALL HCOPY(101,-101,' ')
         CALL HCOPY(101,-10101,' ')
         CALL HCOPY(102,-102,' ')
         CALL HCOPY(102,-10102,' ')
         CALL HCOPY(103,-103,' ')
         CALL HCOPY(103,-10103,' ')
         CALL HCOPY(131,-131,' ')
         CALL HCOPY(131,-10131,' ')
         CALL HCOPY(132,-132,' ')
         CALL HCOPY(132,-10132,' ')
         CALL HCOPY(133,-133,' ')
         CALL HCOPY(133,-10133,' ')
         CALL HCOPY(171,-171,' ')
         CALL HCOPY(171,-10171,' ')
         CALL HCOPY(172,-172,' ')
         CALL HCOPY(172,-10172,' ')
         CALL HCOPY(173,-173,' ')
         CALL HCOPY(173,-10173,' ')
* reserve space to store errors
         CALL HRESET(0,' ')
         call hbarx(0)

* - fill the data histograms

         call hpak(-114,pq2)
         call hpake(-114,pq2se)
         call hpak(-10114,pq2)
         call hpake(-10114,pq2te)

         call hpak(-101,px1)
         call hpake(-101,px1se)
         call hpak(-10101,px1)
         call hpake(-10101,px1te)
         call hpak(-102,px2)
         call hpake(-102,px2se)
         call hpak(-10102,px2)
         call hpake(-10102,px2te)
         call hpak(-103,px3)
         call hpake(-103,px3se)
         call hpak(-10103,px3)
         call hpake(-10103,px3te)

         call hpak(-131,peta1)
         call hpake(-131,peta1se)
         call hpak(-10131,peta1)
         call hpake(-10131,peta1te)
         call hpak(-132,peta2)
         call hpake(-132,peta2se)
         call hpak(-10132,peta2)
         call hpake(-10132,peta2te)
         call hpak(-133,peta3)
         call hpake(-133,peta3se)
         call hpak(-10133,peta3)
         call hpake(-10133,peta3te)

         call hpak(-171,ppt1)
         call hpake(-171,ppt1se)
         call hpak(-10171,ppt1)
         call hpake(-10171,ppt1te)
         call hpak(-172,ppt2)
         call hpake(-172,ppt2se)
         call hpak(-10172,ppt2)
         call hpake(-10172,ppt2te)
         call hpak(-173,ppt3)
         call hpake(-173,ppt3se)
         call hpak(-10173,ppt3)
         call hpake(-10173,ppt3te)

* for higher pt cut of 3.5 GeV
         call hpak(-115,pq22)
         call hpake(-115,pq22se)
         call hpak(-10115,pq22)
         call hpake(-10115,pq22te)

         call hpak(-116,px4)
         call hpake(-116,px4se)
         call hpak(-10116,px4)
         call hpake(-10116,px4te)
* book histo for stat. of event types
         call HBOOK1(299,'event type',8,0.,7.,0.)
         call hbook1(297,'gg-op-ang',30,0.,30.,0.)
         call hbook1(296,'gg-op-ang',20,0.,20.,0.)
         call hbprof(298,'gg-op-ang',30,0.,30.,0.,25.0,' ')
         CALL HRESET(298,' ')
         call hbar2(298)
*
*********************************************************************
*
*                      Event Processing
*
*********************************************************************
      Else if (iflag.eq.2) then
*
* Filling: The following MUST always be done
* (i) move to the correct sub-directory in PAWC
*
         call hcdir('//PAWC/'//xxxx,' ')

* weight is not used - set to 1.0
         wtx=1.0

* count events
         Nevt = Nevt + wtx
* find beam particles
         ierr=HZIBEAM(ipbeam,iebeam)
         if (.not.(ierr.eq.1)) then
            write(6,*) 'HZ'//xxxx,' beams not found ! '
            return
         else
            phad=DBLE(PHEP(3,IPBEAM))
            plep=DBLE(PHEP(3,IEBEAM))
         endif

* get kinematics
         q2=real(HZDISKIN(1))
         x =real(HZDISKIN(2))
         y =real(HZDISKIN(3))
         xwght=x
         ywght=y
         q2wght=q2
channes         pdfset=MSTP(51)
channes       if(pdfset.gt.9999) pdfset=pdfset-1000000
channes       wtxdis = wtx * REWGHTTW(Xwght,Q2wght,Ywght,pdfset)
         wtxdis = wtx
         Nevtw = Nevtw + wtxdis
*       print *,q2,x,y,wtx,wtxdis

* get electron
         iel=HZIDELEC(idum)
         if (iel.eq.-1) then
            write(*,*) 'Hz'//xxxx,' electron not found '
            return
         endif
*       print *,'e->',PHEP(1,iel),PHEP(2,iel),PHEP(3,iel),
*     +      PHEP(4,iel)
         eout(1)=(PHEP(1,iel))
         eout(2)=(PHEP(2,iel))
         eout(3)=(PHEP(3,iel))
         eout(4)=(PHEP(4,iel))

* select DIS events
         if (y.lt.ylow) return
         if (y.gt.yhigh) return
         if (q2.lt.q2re1lo) return
         if (q2.gt.q2re3hi) return

* count number of events passing the DIS cuts
         Npass=Npass+WTX
         Npdis=Npdis+wtxdis
channes         if (q2.lt.q2re3hi.and.q2.gt.q2re1lo) then
channes            call hf1(110114,q2,wtx)
channes            call hf1(-110114,q2,wtxdis)
channes            call hf1(110115,q2,wtx)
channes            call hf1(-110115,q2,wtxdis)
channes            call hf1(110116,x,wtx)
channes            call hf1(-110116,x,wtxdis)
channes         endif
         if (q2.lt.q2re1hi.and.q2.gt.q2re1lo) then
            call hf1(110101,x,wtx)
            call hf1(-110101,x,wtxdis)
         endif
         if (q2.lt.q2re2hi.and.q2.gt.q2re2lo) then
            call hf1(110102,x,wtx)
            call hf1(-110102,x,wtxdis)
         endif
         if (q2.lt.q2re3hi.and.q2.gt.q2re3lo) then
            call hf1(110103,x,wtx)
            call hf1(-110103,x,wtxdis)
         endif


* find pi0s, boost pt, apply selection, fill histos
         do i=1,nhep
            if (iabs(idhep(i)).eq.111) then
*             print *,PHEP(1,i),PHEP(2,i),PHEP(3,i),PHEP(4,i)
               p(1,1)=DBLE(PHEP(1,i))
               p(2,1)=DBLE(PHEP(2,i))
               p(3,1)=DBLE(PHEP(3,i))
               p(4,1)=DBLE(PHEP(4,i))
               thpi=real(HZPHMANG(PHEP(3,i), sqrt(PHEP(1,i)**2+PHEP(2,
     +         i)**2)))
               etapi=-LOG(TAN(thpi/2))
               xpi=real(PHEP(4,i))/820.0
               CALL KTHADR(0,plep,phad,eout,1,P,Q,*999)
               ethcms=sngl(sqrt(q(1,1)**2+q(2,1)**2))

* calculate opening of two gammas from pi0
               a(1)=sngl(PHEP(1,jdahep(1,i)))
               a(2)=sngl(PHEP(2,jdahep(1,i)))
               a(3)=sngl(PHEP(3,jdahep(1,i)))
               b(1)=sngl(PHEP(1,jdahep(2,i)))
               b(2)=sngl(PHEP(2,jdahep(2,i)))
               b(3)=sngl(PHEP(3,jdahep(2,i)))
               thgg=57.2958*acos( (a(1)*b(1)+a(2)*b(2)+a(3)*b(3))/
     +         (sqrt(a(1)**2+a(2)**2+a(3)**2)* sqrt(b(1)**2+b(2)**2+
     +         b(3)**2)) )
               thmin=57.2958*2*atan(0.13498/PHEP(4,i))
               enpi=sngl(PHEP(4,i))

               call hfill(298,enpi,thgg,wtx)
               call hf1(297,enpi,wtx)
               call hf1(296,thgg,wtx)

*             print *,thpi*57.29578,etapi,xpi,
*     +            sqrt(PHEP(1,i)**2+PHEP(2,i)**2)

               if ((ethcms.gt.ptpilow.and.ethcms.lt.ptpihigh).and.
     +         (etapi.gt.etapilow.and.etapi.lt.etapihigh).and. (xpi.gt.
     +         xpilow.and.xpi.lt.xpihigh) )then
c                print *,'hz99094 ',PHEP(4,i),thgg,thmin,thgg-thmin
                  if (q2.lt.q2re3hi.and.q2.gt.q2re1lo) then
                     call hf1(299,float(lst(24)),wtx)
*                   print *,'hi',lst(24)
                     call hf1(114,q2,wtx)
                     if(ethcms.gt.addpt) then
                        call hf1(115,q2,wtx)
                        call hf1(116,x,wtx)
                     endif
                  endif

                  if (q2.lt.q2re1hi.and.q2.gt.q2re1lo) then
                     call hf1(101,x,wtx)
                     call hf1(131,etapi,wtx)
                     call hf1(171,ethcms,wtx)
                  endif
                  if (q2.lt.q2re2hi.and.q2.gt.q2re2lo) then
                     call hf1(102,x,wtx)
                     call hf1(132,etapi,wtx)
                     call hf1(172,ethcms,wtx)
                  endif
                  if (q2.lt.q2re3hi.and.q2.gt.q2re3lo) then
                     call hf1(103,x,wtx)
                     call hf1(133,etapi,wtx)
                     call hf1(173,ethcms,wtx)
                  endif
               endif
            endif
         enddo
*********************************************************************
*
*                      Termination
*
*********************************************************************

      Else if(iflag.eq.3) then

* Termination: The following MUST always be done
* (i) Move to the correct PAW subdirectory
*
         call hcdir('//PAWC/'//xxxx,' ')
*
c         call pdfsta
         lum=999999999.
         if (xsec.ne.0.) then
            lum=Nevt/real(xsec)
            print *,' HZ99094 '
            print *,'hz',xxxx,' pdfset:',pdfset
            print *,'hz',xxxx,' Number of events procesed:',Nevt
            print *,'hz',xxxx,' Number of wght events procesed:',Nevtw
            print *,'hz',xxxx,' Number of events passing DIS cuts ',
     +      Npass
            print *,'hz',xxxx,' Number of wght events passing DIS cuts '
     +      ,Npdis
            print *,'hz',xxxx,' Total cross section:',xsec
            newxs=xsec*(dble(nevtw)/dble(nevt))
            print *,'hz',xxxx,' New Total cross section:',newxs
            print *,'hz',xxxx,' Equivalent luminosity:',lum
            myxsecdp=dble(npass)/dble(nevt)*(xsec)
            myxsec=real(myxsecdp)
            myxsdisdp=dble(npdis)/dble(nevt)*(xsec)
            myxsdis=real(myxsdisdp)
            write (*,40) 'hz',xxxx,' Total x-sect in my PS:',myxsec
            write (*,40) 'hz',xxxx,' Total x-sect in my PS dis:',
     +      myxsdis
         else
            write(6,*) 'hz',xxxx,' xsec=0 ! '
         endif
   40    FORMAT(A,F15.4)
*
* calc. rate for x and Q2 distr.
channes         call hopera(114,'/E',110114,100114,1.0,1.0)
channes         call hopera(115,'/E',110115,100115,1.0,1.0)
channes         call hopera(116,'/E',110116,100116,1.0,1.0)
         call hopera(101,'/E',110101,100101,1.0,1.0)
         call hopera(102,'/E',110102,100102,1.0,1.0)
         call hopera(103,'/E',110103,100103,1.0,1.0)


*      normalize the histos (also div. xbj plots by 1000. to get nb)
*
         call hzhinrm(114,0,lum,1)
         call hzhinrm(115,0,lum,1)
         call hzhinrm(116,0,lum*1000,1)
         call hzhinrm(101,0,lum*1000.0,1)
         call hzhinrm(102,0,lum*1000.0,1)
         call hzhinrm(103,0,lum*1000.0,1)
         call hzhinrm(131,0,lum,1)
         call hzhinrm(132,0,lum,1)
         call hzhinrm(133,0,lum,1)
         call hzhinrm(171,0,lum,1)
         call hzhinrm(172,0,lum,1)
         call hzhinrm(173,0,lum,1)


* calc. rate for x and Q2 distr. for data with wght distr.
channes         call normal(110114,0,lum,1,1)
channes         call normal(110115,0,lum,1,1)
channes         call normal(110116,0,lum*1000,1,1)
         call hzhinrm(-110101,0,lum*1000.0,1)
         call hzhinrm(-110102,0,lum*1000.0,1)
         call hzhinrm(-110103,0,lum*1000.0,1)
channes         call hopera(-114,'/E',-110114,-100114,1.0,1.0)
channes         call hopera(-115,'/E',-110115,-100115,1.0,1.0)
channes         call hopera(-116,'/E',-110116,-100116,1.0,1.0)
         call hopera(-101,'/E',-110101,-100101,1.0,1.0)
         call hopera(-102,'/E',-110102,-100102,1.0,1.0)
         call hopera(-103,'/E',-110103,-100103,1.0,1.0)

* get histos with tot. error for rates

channes         call seterror(-100114,-111114,pq2rsy)
channes         call seterror(-100115,-111115,pq3rsy)
channes         call seterror(-100116,-111116,px4rsy)
         call seterror(-100101,-111101,px1rsy)
         call seterror(-100102,-111102,px2rsy)
         call seterror(-100103,-111103,px3rsy)

* ds/deta and ds/dpt divided by total x-section in analysis Phase Sp.
* for data with wght total xsection
         call hzhinrm(131,100131,myxsdis,1)
         call hzhinrm(132,100132,myxsdis,1)
         call hzhinrm(133,100133,myxsdis,1)
         call hzhinrm(171,100171,myxsdis,1)
         call hzhinrm(172,100172,myxsdis,1)
         call hzhinrm(173,100173,myxsdis,1)
*
      endif
*
      RETURN
 999  continue
      write(6,*) 'hz',xxxx,' error in KTHADR '
      END
