      SUBROUTINE HZH0404033(iflag)
************************************************************************
*     This routine reproduces the ZEUS plots on dijets for the
*     virtual photon, reference hep-ex/0404033.
*
*     Initial state: 27 GeV positrons vs 820 GeV protons.
*
*     Reference: [reference]
*     Arguments: iflag=10000*[A]+1000*[B]+10*[D]+1 initialisation
*                iflag=10000*[A]+1000*[B]+10*[D]+2 filling
*                iflag=10000*[A]+1000*[B]+10*[D]+3 termination
*
*     [A]= IPS   >0 for partons shower running, 0 otherwise
*     [B]= IPROC decides which process is being run
*     [D]= JETF  decides jet finder type. Ignored if IRUN=1.
*     See manual section "The analysis routines" for more details.
*
*     It is assumed you run either (a) with iproc = 0
*                               or (b) with iproc = 1, 2. in order.
*
*     Histograms: -N data for Fig.Xa
*                  N simulation for Fig.Xa
*                  ... etc.
*
*     Routine by Matt Lightwood and Jon Butterworth
*
****************************************************************************

      IMPLICIT NONE

#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
#include "hzjetcmn.inc"
c     Jetfinder stuff

      CHARACTER*6 jetf
      
      INTEGER NJET
      DOUBLE PRECISION et(10),rap(10),phi(10)
      DOUBLE PRECISION Jets_lab(MAXHZJETS,8),et_lab(10),rap_lab(10)
      DOUBLE PRECISION CONER
      SAVE CONER

      DOUBLE PRECISION elecE
      INTEGER myil,myip

c     Iterators. i_jet not an iterator.

      INTEGER i_bw,i_plot,i_jet,j_jet,i_loop,j_loop,i

c     Physics stuff

      DOUBLE PRECISION q2,ybj,xgamma,E_e,E_p,x

      PARAMETER (E_e = 27.5, E_p = 820.0)
c stuff for boost msl
      DOUBLE PRECISION pgam(4),pgam4(4),pbeam(4),phcm(4)
      DOUBLE PRECISION phc(4),jets_hcm(50,4),plab(4)
      DOUBLE PRECISION ppp(4,512)
      INTEGER lepid,IErr,pbid,ebid,J,p_count
      REAL p,pt,e_t(100),th,rap_hcm,eta(100)

c stuff for ktclus call
      integer ktmode,npjets,npart, parttojet(512), nsjets
      integer mjet(4,512),nppp
      REAL ecut, yy(512), ktpjet(4,512), drad
      REAL ktireco, scale, p_t
      double precision px,py,pz,ptsq,ppsq, eta_hcm,et_hcm

c     Cuts

      DOUBLE PRECISION ybjcut(2)
      DATA ybjcut / 0.2, 0.55 /      
      REAL xgcuts(3)
      DATA xgcuts / 0., 0.75, 1.001 /
      REAL etacut(2)
      DATA etacut / -2.4, 2.4 /
      REAL etcut(2)
      DATA etcut / 6.5, 7.5 /

c     declare the shift in eta and mean_et variable
      REAL eta_shift, mean_et, mean_et2, eta_f

      REAL lumnb

c     q2 bin information

      REAL q2_bins_php(2)
      REAL q2_bins_dis(7)
      REAL q2_bins_dis0(2)
      REAL q2_bins_dis1(2)
      REAL q2_bins_dis2(2)
      REAL q2_bins_dis3(2)
      REAL q2_bins_dis4(2)
      DATA q2_bins_php / 0., 1.0/
      DATA q2_bins_dis / 0.1, 0.55, 1.5, 4.5, 10.5, 49.0, 5000.0 /
      DATA q2_bins_dis0 / 0.1, 0.55 /
      DATA q2_bins_dis1 / 1.5, 4.5 /
      DATA q2_bins_dis2 / 4.5, 10.5 /
      DATA q2_bins_dis3 / 10.5, 49. /
      DATA q2_bins_dis4 / 49., 5000. /
c      
c     eta bin information

      REAL eta_bins(5)
      DATA eta_bins / -3., -1.8, 0., 0.8, 2.4 /

c     dsigma/det bin information

      REAL dsigma(7)
      DATA dsigma / 7.5, 9., 12.5, 15. , 22., 30. , 40. /
    
c     xgamma bin information

      REAL xg_bins(3)
      DATA xg_bins / 0., 0.75, 1.001 /

c     mean_et2 bin information
      REAL meanet2_bins(4)
      DATA meanet2_bins /49.,85.,150.,700./

c     Some useful declarations
      INTEGER chjet, iproc, iflag
      SAVE chjet

c     Choose KT clus. Frixiones code uses ES Jet. Doesn't matter.

      DATA chjet/3/
c
      INTEGER hzflag
c
c     Don't know if all these are neccessary. Check.
      
      LOGICAL parton_showers, lp 
      DATA lp /.false./      

      Character*8 xxxx
      DATA xxxx /'H0404033'/

c     The data ds/dQ2 plots.

c     All et ----------------------------------
      REAL q2_xsec_dis(6), q2_xsec_php
      REAL q2_xsechi_dis(6), q2_xsechi_php
      REAL q2_xseclo_dis(6), q2_xseclo_php
      REAL q2_rat_dis(6), q2_rat_php
c     total errors
      REAL q2_err_dis(6), q2_err_php
      REAL q2_errhi_dis(6), q2_errhi_php
      REAL q2_errlo_dis(6), q2_errlo_php
      REAL q2_erat_dis(6), q2_erat_php
c     statistical errors
      REAL q2_err1_dis(6), q2_err1_php
      REAL q2_errhi1_dis(6), q2_errhi1_php
      REAL q2_errlo1_dis(6), q2_errlo1_php
      REAL q2_erat1_dis(6), q2_erat1_php
c     systematic errors
      REAL q2_err2_dis(6), q2_err2_php
      REAL q2_errhi2_dis(6), q2_errhi2_php
      REAL q2_errlo2_dis(6), q2_errlo2_php
      REAL q2_erat2_dis(6), q2_erat2_php
c     energy scale errors
      REAL q2_err3_dis(6), q2_err3_php
      REAL q2_errhi3_dis(6), q2_errhi3_php
      REAL q2_errlo3_dis(6), q2_errlo3_php
      REAL q2_erat3_dis(6), q2_erat3_php


c     ET2 49-85
      REAL q2_xsec_dis1(6), q2_xsec_php1
      REAL q2_xsechi_dis1(6), q2_xsechi_php1
      REAL q2_xseclo_dis1(6), q2_xseclo_php1
      REAL q2_rat_dis1(6), q2_rat_php1
c     total errors
      REAL q2_err_dis1(6), q2_err_php1
      REAL q2_errhi_dis1(6), q2_errhi_php1
      REAL q2_errlo_dis1(6), q2_errlo_php1
      REAL q2_erat_dis1(6), q2_erat_php1
c     statistical errors
      REAL q2_err1_dis1(6), q2_err1_php1
      REAL q2_errhi1_dis1(6), q2_errhi1_php1
      REAL q2_errlo1_dis1(6), q2_errlo1_php1
      REAL q2_erat1_dis1(6), q2_erat1_php1
c     systematic errors
      REAL q2_err2_dis1(6), q2_err2_php1
      REAL q2_errhi2_dis1(6), q2_errhi2_php1
      REAL q2_errlo2_dis1(6), q2_errlo2_php1
      REAL q2_erat2_dis1(6), q2_erat2_php1
c     energy scale errors
      REAL q2_err3_dis1(6), q2_err3_php1
      REAL q2_errhi3_dis1(6), q2_errhi3_php1
      REAL q2_errlo3_dis1(6), q2_errlo3_php1
      REAL q2_erat3_dis1(6), q2_erat3_php1


c     ET 85-150
      REAL q2_xsec_dis2(6), q2_xsec_php2
      REAL q2_xsechi_dis2(6), q2_xsechi_php2
      REAL q2_xseclo_dis2(6), q2_xseclo_php2
      REAL q2_rat_dis2(6), q2_rat_php2
c     total errors
      REAL q2_err_dis2(6), q2_err_php2
      REAL q2_errhi_dis2(6), q2_errhi_php2
      REAL q2_errlo_dis2(6), q2_errlo_php2
      REAL q2_erat_dis2(6), q2_erat_php2
c     statistical errors
      REAL q2_err1_dis2(6), q2_err1_php2
      REAL q2_errhi1_dis2(6), q2_errhi1_php2
      REAL q2_errlo1_dis2(6), q2_errlo1_php2
      REAL q2_erat1_dis2(6), q2_erat1_php2
c     systematic errors
      REAL q2_err2_dis2(6), q2_err2_php2
      REAL q2_errhi2_dis2(6), q2_errhi2_php2
      REAL q2_errlo2_dis2(6), q2_errlo2_php2
      REAL q2_erat2_dis2(6), q2_erat2_php2
c     energy scale errors
      REAL q2_err3_dis2(6), q2_err3_php2
      REAL q2_errhi3_dis2(6), q2_errhi3_php2
      REAL q2_errlo3_dis2(6), q2_errlo3_php2
      REAL q2_erat3_dis2(6), q2_erat3_php2

c     et2 150-700
      REAL q2_xsec_dis3(6), q2_xsec_php3
      REAL q2_xsechi_dis3(6), q2_xsechi_php3
      REAL q2_xseclo_dis3(6), q2_xseclo_php3
      REAL q2_rat_dis3(6), q2_rat_php3
c     total errors
      REAL q2_err_dis3(6), q2_err_php3
      REAL q2_errhi_dis3(6), q2_errhi_php3
      REAL q2_errlo_dis3(6), q2_errlo_php3
      REAL q2_erat_dis3(6), q2_erat_php3
c     statistical errors
      REAL q2_err1_dis3(6), q2_err1_php3
      REAL q2_errhi1_dis3(6), q2_errhi1_php3
      REAL q2_errlo1_dis3(6), q2_errlo1_php3
      REAL q2_erat1_dis3(6), q2_erat1_php3
c     systematic errors
      REAL q2_err2_dis3(6), q2_err2_php3
      REAL q2_errhi2_dis3(6), q2_errhi2_php3
      REAL q2_errlo2_dis3(6), q2_errlo2_php3
      REAL q2_erat2_dis3(6), q2_erat2_php3
c     energy scale errors
      REAL q2_err3_dis3(6), q2_err3_php3
      REAL q2_errhi3_dis3(6), q2_errhi3_php3
      REAL q2_errlo3_dis3(6), q2_errlo3_php3
      REAL q2_erat3_dis3(6), q2_erat3_php3

      DATA q2_xsec_php, q2_xsec_dis / 9280., 2250., 167., 54.5, 11.9, 2
     $     .27, 0.095 /
      DATA q2_xsechi_php, q2_xsechi_dis / 3620., 980., 79.5, 30., 7.74,
     $     1.63, 0.077 /
      DATA q2_xseclo_php, q2_xseclo_dis / 5710., 1270., 87.8, 24.5, 4.21
     $     , 0.72, 0.022 /
      DATA q2_rat_php, q2_rat_dis / 2.12, 1.57, 1.42, 0.92, 0.66, 0.35,
     $     0.44/

      parton_showers=.FALSE.
      IF (iflag/10000.gt.0) THEN
         parton_showers=.TRUE.           
      ENDIF
      

*     Determine the process type.
      iproc=(mod(iflag,10000)/1000)
      if ((iproc.lt.0).or.(iproc.gt.2)) then
         write(HZUNIT,*) 'HZ',xxxx,': illegal IPROC:',iproc
         return
      endif

*     --- choose jet finder
      IF ((MOD(iflag,100)/10.gt.0).and.MOD(iflag,10).eq.1) THEN
         chjet=INT(MOD(iflag,100)/10)
      ENDIF

      IF (parton_showers) THEN
         chjet=-1*chjet
      ENDIF

      IF (MOD(iflag,10).eq.1) then
*     Initialisation: The following MUST always be done
*     (i) make subdirectory in PAWC
*     - use the name as the xxxx in HZxxxx subroutine
*     (i) make subdirectory in o/p file

*     
*     book your MC histograms
*
         if (iproc.le.1) then
            
            IF (parton_showers) THEN
               Call hcdir('//HISTO',' ')
               CALL HMDIR('PS'//xxxx,'S')
               Call hcdir('//PAWC',' ')
               CALL HMDIR('PS'//xxxx,'S')
            ELSE
               Call hcdir('//HISTO',' ')
               CALL HMDIR(xxxx,'S')
               Call hcdir('//PAWC',' ')
               CALL HMDIR(xxxx,'S')
            ENDIF
            
c     book the data histograms

c     eta plot
            CALL HBOOKB(-13,'dsig/deta_f php',4,eta_bins,0.)
            CALL HBOOKB(-25,'dsig/deta_f bpc',4,eta_bins,0.)
            CALL HBOOKB(-26,'dsig/deta_f dis1',4,eta_bins,0.)
            CALL HBOOKB(-27,'dsig/deta_f dis2',4,eta_bins,0.)
            CALL HBOOKB(-28,'dsig/deta_f dis3',4,eta_bins,0.)
            CALL HBOOKB(-29,'dsig/deta_f dis4',4,eta_bins,0.)
         
c     dsigma/detj1 plot
c     
            CALL HBOOKB(-14,'dsigma/detj1 php',6,dsigma,0.)
            CALL HBOOKB(-31,'dsigma/detj1 bpc',6,dsigma,0.)
            CALL HBOOKB(-32,'dsigma/detj1 dis1',6,dsigma,0.)
            CALL HBOOKB(-33,'dsigma/detj1 dis2',6,dsigma,0.)
            CALL HBOOKB(-34,'dsigma/detj1 dis3',6,dsigma,0.)
            CALL HBOOKB(-35,'dsigma/detj1 dis4',6,dsigma,0.) 
c     
c     dsig/dQ^2 in bins of xgamma and etj1^2 php
c     
            CALL HBOOKB(-40,'dsigma/dq2 php tot',1,q2_bins_php ,0.)
            CALL HBOOKB(-41,'dsigma/dq2 php low',1,q2_bins_php ,0.)
            CALL HBOOKB(-42,'dsigma/dq2 php hi',1,q2_bins_php,0 .)
            CALL HBOOKB(-43,'ratio, php',1,q2_bins_php,0 .)

            CALL HBOOKB(-44,'dsigma/dq2 php1 tot',1,q2_bins_php ,0.)
            CALL HBOOKB(-45,'dsigma/dq2 php1 low',1,q2_bins_php ,0.)
            CALL HBOOKB(-46,'dsigma/dq2 php1 hi',1,q2_bins_php,0 .)
            CALL HBOOKB(-47,'ratio, php1',1,q2_bins_php,0 .)

            CALL HBOOKB(-48,'dsigma/dq2 php2 tot',1,q2_bins_php ,0.)
            CALL HBOOKB(-49,'dsigma/dq2 php2 low',1,q2_bins_php ,0.)
            CALL HBOOKB(-50,'dsigma/dq2 php2 hi',1,q2_bins_php,0 .)
            CALL HBOOKB(-51,'ratio, php2',1,q2_bins_php,0 .)

            CALL HBOOKB(-52,'dsigma/dq2 php3 tot',1,q2_bins_php ,0.)
            CALL HBOOKB(-53,'dsigma/dq2 php3 low',1,q2_bins_php ,0.)
            CALL HBOOKB(-54,'dsigma/dq2 php3 hi',1,q2_bins_php,0 .)
            CALL HBOOKB(-55,'ratio, php3',1,q2_bins_php,0 .)

c      dis
c     
            CALL HBOOKB(-60,'dsigma/dq2 dis tot',6,q2_bins_dis ,0.)
            CALL HBOOKB(-61,'dsigma/dq2 dis low',6,q2_bins_dis ,0.)
            CALL HBOOKB(-62,'dsigma/dq2 dis hi',6,q2_bins_dis ,0.)
            CALL HBOOKB(-63,'ratio, dis',6,q2_bins_dis ,0.)

            CALL HBOOKB(-64,'dsigma/dq2 dis1 tot',6,q2_bins_dis ,0.)
            CALL HBOOKB(-65,'dsigma/dq2 dis1 low',6,q2_bins_dis ,0.)
            CALL HBOOKB(-66,'dsigma/dq2 dis1 hi',6,q2_bins_dis ,0.)
            CALL HBOOKB(-67,'ratio, dis1',6,q2_bins_dis ,0.)

            CALL HBOOKB(-68,'dsigma/dq2 dis2 tot',6,q2_bins_dis ,0.)
            CALL HBOOKB(-69,'dsigma/dq2 dis2 low',6,q2_bins_dis ,0.)
            CALL HBOOKB(-70,'dsigma/dq2 dis2 hi',6,q2_bins_dis ,0.)
            CALL HBOOKB(-71,'ratio, dis2',6,q2_bins_dis ,0.)

            CALL HBOOKB(-72,'dsigma/dq2 dis3 tot',6,q2_bins_dis ,0.)
            CALL HBOOKB(-73,'dsigma/dq2 dis3 low',6,q2_bins_dis ,0.)
            CALL HBOOKB(-74,'dsigma/dq2 dis3 hi',6,q2_bins_dis ,0.)
            CALL HBOOKB(-75,'ratio, dis3',6,q2_bins_dis ,0.)

c     calculate the full errors
            q2_err_php = sqrt(q2_err1_php**2+q2_err2_php**2+q2_err3_php
     $           **2)
            q2_errhi_php = sqrt(q2_errhi1_php**2+q2_errlo2_php**2
     $           +q2_errlo3_php**2)
            q2_errlo_php = sqrt(q2_errlo1_php**2+q2_errlo2_php**2
     $           +q2_errlo3_php**2)
            q2_erat_php = sqrt(q2_erat1_php**2+q2_erat2_php**2
     $           +q2_erat3_php**2)

            q2_err_php1 = sqrt(q2_err1_php1**2+q2_err2_php1**2
     $           +q2_err3_php1**2)
            q2_errhi_php1 = sqrt(q2_errhi1_php1**2+q2_errlo2_php1**2
     $           +q2_errlo3_php1**2)
            q2_errlo_php1 = sqrt(q2_errlo1_php1**2+q2_errlo2_php1**2
     $           +q2_errlo3_php1**2)
            q2_erat_php1 = sqrt(q2_erat1_php1**2+q2_erat2_php1**2
     $           +q2_erat3_php1**2)

            q2_err_php2 = sqrt(q2_err1_php2**2+q2_err2_php2**2
     $           +q2_err3_php2**2)
            q2_errhi_php2 = sqrt(q2_errhi1_php2**2+q2_errlo2_php2**2
     $           +q2_errlo3_php2**2)
            q2_errlo_php2 = sqrt(q2_errlo1_php2**2+q2_errlo2_php2**2
     $           +q2_errlo3_php2**2)
            q2_erat_php2 = sqrt(q2_erat1_php2**2+q2_erat2_php2**2
     $           +q2_erat3_php2**2)

            q2_err_php3 = sqrt(q2_err1_php3**2+q2_err2_php3**2
     $           +q2_err3_php3**2)
            q2_errhi_php3 = sqrt(q2_errhi1_php3**2+q2_errlo2_php3**2
     $           +q2_errlo3_php3**2)
            q2_errlo_php3 = sqrt(q2_errlo1_php3**2+q2_errlo2_php3**2
     $           +q2_errlo3_php3**2)
            q2_erat_php3 = sqrt(q2_erat1_php3**2+q2_erat2_php3**2
     $           +q2_erat3_php3**2)

            do i=1,6
               q2_err_dis(i)=sqrt(q2_err1_dis(i)**2+q2_err2_dis(i)**2
     $              +q2_err3_dis(3)**2)
               q2_errhi_dis(i)=sqrt(q2_errhi1_dis(i)**2+q2_errhi2_dis(i)
     $              **2+q2_errhi3_dis(3)**2)
               q2_errlo_dis(i)=sqrt(q2_errlo1_dis(i)**2+q2_errlo2_dis(i)
     $              **2+q2_errlo3_dis(3)**2)
               q2_erat_dis(i)=sqrt(q2_erat1_dis(i)**2+q2_erat2_dis(i)**2
     $              +q2_erat3_dis(3)**2)

               q2_err_dis1(i)=sqrt(q2_err1_dis1(i)**2+q2_err2_dis1(i)**2
     $              +q2_err3_dis1(3)**2)
               q2_errhi_dis1(i)=sqrt(q2_errhi1_dis1(i)**2
     $              +q2_errhi2_dis1(i)**2+q2_errhi3_dis1(3)**2)
               q2_errlo_dis1(i)=sqrt(q2_errlo1_dis1(i)**2
     $              +q2_errlo2_dis1(i)**2+q2_errlo3_dis1(3)**2)
               q2_erat_dis1(i)=sqrt(q2_erat1_dis1(i)**2+q2_erat2_dis1(i)
     $              **2+q2_erat3_dis1(3)**2)

               q2_err_dis2(i)=sqrt(q2_err1_dis2(i)**2+q2_err2_dis2(i)**2
     $              +q2_err3_dis2(3)**2)
               q2_errhi_dis2(i)=sqrt(q2_errhi1_dis2(i)**2
     $              +q2_errhi2_dis2(i)**2+q2_errhi3_dis2(3)**2)
               q2_errlo_dis2(i)=sqrt(q2_errlo1_dis2(i)**2
     $              +q2_errlo2_dis2(i)**2+q2_errlo3_dis2(3)**2)
               q2_erat_dis2(i)=sqrt(q2_erat1_dis2(i)**2+q2_erat2_dis2(i)
     $              **2+q2_erat3_dis2(3)**2)

               q2_err_dis3(i)=sqrt(q2_err1_dis3(i)**2+q2_err2_dis3(i)**2
     $              +q2_err3_dis3(3)**2)
               q2_errhi_dis3(i)=sqrt(q2_errhi1_dis3(i)**2
     $              +q2_errhi2_dis3(i)**2+q2_errhi3_dis3(3)**2)
               q2_errlo_dis3(i)=sqrt(q2_errlo1_dis3(i)**2
     $              +q2_errlo2_dis3(i)**2+q2_errlo3_dis3(3)**2)
               q2_erat_dis3(i)=sqrt(q2_erat1_dis3(i)**2+q2_erat2_dis3(i)
     $              **2+q2_erat3_dis3(3)**2)

            enddo

c     now fill the data plots.
            CALL HPAK(-40,q2_xsec_php)
            CALL HPAKE(-40,q2_err_php)
            CALL HPAK(-41,q2_xseclo_php)
            CALL HPAKE(-41,q2_errlo_php)
            CALL HPAK(-42,q2_xsechi_php)
            CALL HPAKE(-42,q2_errhi_php)
            CALL HPAK(-43,q2_rat_php)
            CALL HPAKE(-43,q2_erat_php)

            CALL HPAK(-44,q2_xsec_php1)
            CALL HPAKE(-44,q2_err_php1)
            CALL HPAK(-45,q2_xseclo_php1)
            CALL HPAKE(-45,q2_errlo_php1)
            CALL HPAK(-46,q2_xsechi_php1)
            CALL HPAKE(-46,q2_errhi_php1)
            CALL HPAK(-47,q2_rat_php1)
            CALL HPAKE(-47,q2_erat_php1)

            CALL HPAK(-48,q2_xsec_php2)
            CALL HPAKE(-48,q2_err_php2)
            CALL HPAK(-49,q2_xseclo_php2)
            CALL HPAKE(-49,q2_errlo_php2)
            CALL HPAK(-50,q2_xsechi_php2)
            CALL HPAKE(-50,q2_errhi_php2)
            CALL HPAK(-51,q2_rat_php2)
            CALL HPAKE(-51,q2_erat_php2)

            CALL HPAK(-52,q2_xsec_php3)
            CALL HPAKE(-52,q2_err_php3)
            CALL HPAK(-53,q2_xseclo_php3)
            CALL HPAKE(-53,q2_errlo_php3)
            CALL HPAK(-54,q2_xsechi_php3)
            CALL HPAKE(-54,q2_errhi_php3)
            CALL HPAK(-55,q2_rat_php3)
            CALL HPAKE(-55,q2_erat_php3)

            CALL HPAK(-60,q2_xsec_dis)
            CALL HPAKE(-60,q2_err_dis)
            CALL HPAK(-61,q2_xseclo_dis)
            CALL HPAKE(-61,q2_errlo_dis)
            CALL HPAK(-62,q2_xsechi_dis)
            CALL HPAKE(-62,q2_errhi_dis)
            CALL HPAK(-63,q2_rat_dis)
            CALL HPAKE(-63,q2_erat_dis)

            CALL HPAK(-64,q2_xsec_dis1)
            CALL HPAKE(-64,q2_err_dis1)
            CALL HPAK(-65,q2_xseclo_dis1)
            CALL HPAKE(-65,q2_errlo_dis1)
            CALL HPAK(-66,q2_xsechi_dis1)
            CALL HPAKE(-66,q2_errhi_dis1)
            CALL HPAK(-67,q2_rat_dis1)
            CALL HPAKE(-67,q2_erat_dis1)

            CALL HPAK(-68,q2_xsec_dis2)
            CALL HPAKE(-68,q2_err_dis2)
            CALL HPAK(-69,q2_xseclo_dis2)
            CALL HPAKE(-69,q2_errlo_dis2)
            CALL HPAK(-70,q2_xsechi_dis2)
            CALL HPAKE(-70,q2_errhi_dis2)
            CALL HPAK(-71,q2_rat_dis2)
            CALL HPAKE(-71,q2_erat_dis2)

            CALL HPAK(-72,q2_xsec_dis3)
            CALL HPAKE(-72,q2_err_dis3)
            CALL HPAK(-73,q2_xseclo_dis3)
            CALL HPAKE(-73,q2_errlo_dis3)
            CALL HPAK(-74,q2_xsechi_dis3)
            CALL HPAKE(-74,q2_errhi_dis3)
            CALL HPAK(-75,q2_rat_dis3)
            CALL HPAKE(-75,q2_erat_dis3)
            
         ENDIF
         
c     Change in to the correct directories.

*******************************************************
*     event processing
*******************************************************
         
         IF (parton_showers) THEN
            CALL HCDIR('//HISTO/PS'//XXXX,' ')
            CALL HCDIR('//PAWC/PS'//XXXX,' ')
         ELSE
            CALL HCDIR('//HISTO/'//XXXX,' ')
            CALL HCDIR('//PAWC/'//XXXX,' ')
         ENDIF     

c     eta plot
         CALL HBOOKB(100*iproc+13,'dsig/deta_f php',4,eta_bins,0.)
         CALL HBOOKB(100*iproc+25,'dsig/deta_f bpc',4,eta_bins,0.)
         CALL HBOOKB(100*iproc+26,'dsig/deta_f dis1',4,eta_bins,0.)
         CALL HBOOKB(100*iproc+27,'dsig/deta_f dis2',4,eta_bins,0.)
         CALL HBOOKB(100*iproc+28,'dsig/deta_f dis3',4,eta_bins,0.)
         CALL HBOOKB(100*iproc+29,'dsig/deta_f dis4',4,eta_bins,0.)
         
c     dsigma/detj1 plot
c     
         CALL HBOOKB(100*iproc+14,'dsigma/detj1 php',6,dsigma,0.)
         CALL HBOOKB(100*iproc+31,'dsigma/detj1 bpc',6,dsigma,0.)
         CALL HBOOKB(100*iproc+32,'dsigma/detj1 dis1',6,dsigma,0.)
         CALL HBOOKB(100*iproc+33,'dsigma/detj1 dis2',6,dsigma,0.)
         CALL HBOOKB(100*iproc+34,'dsigma/detj1 dis3',6,dsigma,0.)
         CALL HBOOKB(100*iproc+35,'dsigma/detj1 dis4',6,dsigma,0.) 
c     
c     dsig/dQ^2 in bins of xgamma and etj1^2 php
         CALL HBOOKB(100*iproc+40,'dsigma/dq2 php tot',1,q2_bins_php ,0.
     $        )
         CALL HBOOKB(100*iproc+41,'dsigma/dq2 php low',1,q2_bins_php ,0.
     $        )
         CALL HBOOKB(100*iproc+42,'dsigma/dq2 php hi',1,q2_bins_php,0 .)
         CALL HBOOKB(100*iproc+43,'ratio, php',1,q2_bins_php,0 .)
         
         CALL HBOOKB(100*iproc+44,'dsigma/dq2 php1 tot',1,q2_bins_php ,0
     $        .)
         CALL HBOOKB(100*iproc+45,'dsigma/dq2 php1 low',1,q2_bins_php ,0
     $        .)
         CALL HBOOKB(100*iproc+46,'dsigma/dq2 php1 hi',1,q2_bins_php,0 .
     $        )
         CALL HBOOKB(100*iproc+47,'ratio, php1',1,q2_bins_php,0 .)
         
         CALL HBOOKB(100*iproc+48,'dsigma/dq2 php2 tot',1,q2_bins_php ,0
     $        .)
         CALL HBOOKB(100*iproc+49,'dsigma/dq2 php2 low',1,q2_bins_php ,0
     $        .)
         CALL HBOOKB(100*iproc+50,'dsigma/dq2 php2 hi',1,q2_bins_php,0 .
     $        )
         CALL HBOOKB(100*iproc+51,'ratio, php2',1,q2_bins_php,0 .)
         
         CALL HBOOKB(100*iproc+52,'dsigma/dq2 php3 tot',1,q2_bins_php ,0
     $        .)
         CALL HBOOKB(100*iproc+53,'dsigma/dq2 php3 low',1,q2_bins_php ,0
     $        .)
         CALL HBOOKB(100*iproc+54,'dsigma/dq2 php3 hi',1,q2_bins_php,0 .
     $        )
         CALL HBOOKB(100*iproc+55,'ratio, php3',1,q2_bins_php,0 .)
         
c      dis
c     
         CALL HBOOKB(100*iproc+60,'dsigma/dq2 dis tot',6,q2_bins_dis ,0.
     $        )
         CALL HBOOKB(100*iproc+61,'dsigma/dq2 dis low',6,q2_bins_dis ,0.
     $        )
         CALL HBOOKB(100*iproc+62,'dsigma/dq2 dis hi',6,q2_bins_dis ,0.)
         CALL HBOOKB(100*iproc+63,'ratio, dis',6,q2_bins_dis ,0.)
         
         CALL HBOOKB(100*iproc+64,'dsigma/dq2 dis1 tot',6,q2_bins_dis ,0
     $        .)
         CALL HBOOKB(100*iproc+65,'dsigma/dq2 dis1 low',6,q2_bins_dis ,0
     $        .)
         CALL HBOOKB(100*iproc+66,'dsigma/dq2 dis1 hi',6,q2_bins_dis ,0.
     $        )
         CALL HBOOKB(100*iproc+67,'ratio, dis1',6,q2_bins_dis ,0.)
         
         CALL HBOOKB(100*iproc+68,'dsigma/dq2 dis2 tot',6,q2_bins_dis ,0
     $        .)
         CALL HBOOKB(100*iproc+69,'dsigma/dq2 dis2 low',6,q2_bins_dis ,0
     $        .)
         CALL HBOOKB(100*iproc+70,'dsigma/dq2 dis2 hi',6,q2_bins_dis ,0.
     $        )
         CALL HBOOKB(100*iproc+71,'ratio, dis2',6,q2_bins_dis ,0.)
         
         CALL HBOOKB(100*iproc+72,'dsigma/dq2 dis3 tot',6,q2_bins_dis ,0
     $        .)
         CALL HBOOKB(100*iproc+73,'dsigma/dq2 dis3 low',6,q2_bins_dis ,0
     $        .)
         CALL HBOOKB(100*iproc+74,'dsigma/dq2 dis3 hi',6,q2_bins_dis ,0.
     $        )
         CALL HBOOKB(100*iproc+75,'ratio, dis3',6,q2_bins_dis ,0.)

        
c     HIDOPT sets the mean value and RMS to be computed at filling time
c     (double precision).  See HBOOK manual
        
        CALL HIDOPT(0,'stat')

c     Store the errors for 1D histograms in memory and superimpose them
c     on the plot during output. This routine must be called after
c     booking but before filling.

        CALL HBARX(0)

c     Standard cone radius is 1.0

        CALL HZJETRAD(2,CONER)
        
        IF (CONER.lt.0.0) THEN
           CONER=1.0
        ENDIF
        
        CALL HZJTNAME(chjet,jetf)
        
        WRITE(HZUNIT,*)'**********************************************'
        WRITE(HZUNIT,*
     $       )'* HZ'//xxxx//'  called, histograms will be output. *'
        WRITE(HZUNIT,*)'*     Jet finder selected : ',jetf
     $       ,'           *'
        WRITE(HZUNIT,1001)'*        with cone radius : ',CONER,'   *'
        WRITE(HZUNIT,*)'**********************************************'
        
 1001   FORMAT(A29,F5.2,A13)         
        
      ELSE IF (MOD(iflag,10).eq.2) THEN
         
c     Filling runs 
         
c     Change directory to our graphs
         
         IF (parton_showers) THEN              
            CALL HCDIR('//PAWC/PS'//XXXX,' ')
         ELSE
            CALL HCDIR('//PAWC/'//XXXX,' ')
         ENDIF
         

c- get useful variables
         q2=real(HZDISKIN(1))
         if (lp) write(hzunit,*) 'hz'//xxxx//': q2=',q2
	 ybj=HZDISKIN(3)
	 x = real(hzdiskin(2))
c
c
c- some checks on the variables
         IF (q2.EQ.-1.) THEN
            WRITE(HZUNIT,*) 'HZ'//xxxx/
     $           /': HZPHOKIN returns error for Q2'
         ELSEIF (q2.LT.0) THEN
            q2=ABS(q2)
         ENDIF
         IF (ybj.EQ.-1) THEN
            WRITE(HZUNIT,*) 'HZH'//xxxx/
     $           /': HZPHOKIN returns error for y'
         ENDIF
c     check yjb cut and the Q2 cut.
 
         IF ((ybj.GE.ybjcut(1)).and.(ybj.LE.ybjcut(2)) .AND.q2.GT.0.
     $        .AND.(q2.LT.q2_bins_dis(7))) THEN
c
c if we get this far initialize boost
c some checks of the photon and electron properties                        
c
           
c go get the beam properties
c
            hzflag=HZIBEAM(pbid,ebid)

            if (hzflag.ne.1) then
               write(hzunit,*) 'ERROR: Problem finding beams'
            endif

c
c     Get the initial state electron energy.(incomin e+) lepton ID -
            lepid=hzidelec(0)
            if (lepid.lt.0) then
               write (6,*) 'Id of electron not found'
               return
            endif
            
            IF (hzflag.GE.0) THEN
               elecE=ABS(PHEP(4,ebid))
            ELSE
               WRITE(HZUNIT,*) 'HZ'//xxxx/
     $              /': Electron not found. E=0.001'
               WRITE(HZUNIT,*) '         event will be rejected'
               RETURN
            ENDIF

c
c     Find jets
            DO i_loop=1,50
               DO j_loop=1,8
                  Jets(i_loop,j_loop)=0.
                  Jets_lab(i_loop,j_loop)=0.			       
               ENDDO
            ENDDO
c     
            NJET=0
            
            CALL HZJTFIND(3,CONER,NJET,Jets)
            
            IF (lp) THEN
               WRITE(*,*) 'Njet=',NJET
            ENDIF
            
c     calculate the shift in eta if we are in photoproduction
             eta_shift=0.
c            eta_shift=0.5*LOG((E_p)/(E_e*ybj)) 
            i_jet=0
            DO j_jet = 1,NJET

c     Find the two highest ET jets in the rapidity interval.
c     Assumes jets are ordered in ET.

               IF (((Jets(j_jet,1)).GT.etacut(1))
     &              .AND.((Jets(j_jet,1)).LT.etacut(2))
     &              .AND.(Jets(j_jet,3).GT.etcut(1))) THEN
                  i_jet=i_jet+1
                  IF (i_jet.LE.2) THEN
                     et(i_jet)=Jets(j_jet,3)
                     rap(i_jet)=Jets(j_jet,1)
                  ENDIF
               ENDIF
            ENDDO
c     
c     If there are not at least two jets satisfying the cuts exit
c                                  
            IF (i_jet.LT.2) GOTO 99
            
            
c     Remember, I am assuming the jets are ordered in ET. 
              
            IF ((et(1).GT.etcut(2))) THEN
               
c     - calculate xgamma using jets selected in the CMS frame
c     - but boosted back to the lab frame
c                  
               xgamma=((et(1)*EXP(-1*rap(1)))+(et(2)*
     &              EXP(-1*rap(2))))/(2*elecE*ybj)          
               
               mean_et = (et(1)+et(2))/2.
               mean_et2 = mean_et**2

               eta_f = -9999
               DO i=1,i_jet
                  IF((rap(i)).GT.eta_f) THEN
                     eta_f = rap(i)
                  ENDIF                  
               ENDDO
               
c     fill the eta_f distributions
	       IF(q2.GT.q2_bins_php(1).AND.q2.LT.q2_bins_php(2)) THEN
                  CALL HFILL((100*iproc)+13,
     &                 REAL(eta_f),0.,wtx)
	       ENDIF
	       IF(q2.GT.q2_bins_dis(1).AND.q2.LT.q2_bins_dis0(2)) THEN
                  CALL HFILL((100*iproc)+25,
     &                 REAL(eta_f),0.,wtx)
	       ENDIF
	       IF(q2.GT.q2_bins_dis1(1).AND.q2.LT.q2_bins_dis1(2)) THEN
                  CALL HFILL((100*iproc)+26,
     &                 REAL(eta_f),0.,wtx)
	       ENDIF
	       IF(q2.GT.q2_bins_dis2(1).AND.q2.LT.q2_bins_dis2(2)) THEN
                  CALL HFILL((100*iproc)+27,
     &                 REAL(eta_f),0.,wtx)
	       ENDIF
	       IF(q2.GT.q2_bins_dis3(1).AND.q2.LT.q2_bins_dis3(2)) THEN
                  CALL HFILL((100*iproc)+28,
     &                 REAL(eta_f),0.,wtx)
	       ENDIF
	       IF(q2.GT.q2_bins_dis4(1).AND.q2.LT.q2_bins_dis4(2)) THEN
                  CALL HFILL((100*iproc)+29,
     &                 REAL(eta_f),0.,wtx)
	       ENDIF
c
c     Q2 cross sections

c     all xgamma
	       IF ((xgamma.GT.xgcuts(1)).AND.
     &              (xgamma.LT.xgcuts(3))) THEN
                  CALL HFILL(100*iproc+40, REAL(q2),0.,wtx)
                  CALL HFILL(100*iproc+60, REAL(q2),0.,wtx)
                  do i=1,3
                     if (mean_et2.gt.meanet2_bins(i).and.mean_et2.lt
     $                    .meanet2_bins(i+1)) then
                        CALL HFILL((100*iproc)+40+4*i, REAL(q2),0.
     $                       ,wtx)
                        CALL HFILL((100*iproc)+60+4*i, REAL(q2),0.
     $                       ,wtx)
                     endif
                  enddo
	       ENDIF
c     
               IF ((xgamma.GT.xgcuts(1)).AND. (xgamma.LT.xgcuts(2)))
     $              THEN
                  CALL HFILL(100*iproc+41, REAL(q2),0.,wtx)
                  CALL HFILL(100*iproc+61, REAL(q2),0.,wtx)
                  do i=1,3
                     if (mean_et2.gt.meanet2_bins(i).and.mean_et2.lt
     $                    .meanet2_bins(i+1)) then
                        CALL HFILL((100*iproc)+41+4*i, REAL(q2),0.
     $                       ,wtx)
                        CALL HFILL((100*iproc)+61+4*i, REAL(q2),0.
     $                       ,wtx)
                     endif
                  enddo
	       ENDIF
c     
               IF ((xgamma.GT.xgcuts(2)).AND.
     &              (xgamma.LT.xgcuts(3))) THEN
                  CALL HFILL(100*iproc+42, REAL(q2),0.,wtx)
                  CALL HFILL(100*iproc+62, REAL(q2),0.,wtx)
                  do i=1,3
                     if (mean_et2.gt.meanet2_bins(i).and.mean_et2.lt
     $                    .meanet2_bins(i+1)) then
                        CALL HFILL((100*iproc)+42+4*i, REAL(q2),0.
     $                       ,wtx)
                        CALL HFILL((100*iproc)+62+4*i, REAL(q2),0.
     $                       ,wtx)
                     endif
                  enddo
	       ENDIF
               

c     et cross sections
               IF (et(1).LT.dsigma(7)) THEN
                  IF(q2.GT.q2_bins_php(1).AND.q2.LT.q2_bins_php(2)) THEN
                     CALL HFILL((100*iproc)+14,REAL(et(1)), 0.,wtx)
                  ENDIF
                  IF(q2.GT.q2_bins_dis0(1).AND.q2.LT.q2_bins_dis0(2))
     $                 THEN
                     CALL HFILL((100*iproc)+31,REAL(et(1)), 0.,wtx)
                  ENDIF               
                  IF(q2.GT.q2_bins_dis1(1).AND.q2.LT.q2_bins_dis1(2))
     $                 THEN
                     CALL HFILL((100*iproc)+32,REAL(et(1)), 0.,wtx)
                  ENDIF               
                  IF(q2.GT.q2_bins_dis2(1).AND.q2.LT.q2_bins_dis2(2))
     $                 THEN
                     CALL HFILL((100*iproc)+33,REAL(et(1)), 0.,wtx)
     $                    
                  ENDIF
                  IF(q2.GT.q2_bins_dis3(1).AND.q2.LT.q2_bins_dis3(2))
     $                 THEN
                     CALL HFILL((100*iproc)+34,REAL(et(1)),
     &                    0.,wtx)               
                  ENDIF
                  IF(q2.GT.q2_bins_dis4(1).AND.q2.LT.q2_bins_dis4(2))
     $                 THEN
                     CALL HFILL((100*iproc)+35,REAL(et(1)), 0.,wtx) 
                  ENDIF
                  
               ENDIF  
               
c     end et cut
               
            ENDIF
            
c     For q2 and ybj cut
         ENDIF
         
      ELSE IF (MOD(iflag,10).eq.3) THEN

c     Termination run 
         if (iproc.eq.0.or.iproc.eq.2) then
            write(hzunit,*) 'HZ'//xxxx//': finishing off'
         endif
         
c     Change directory
         
         IF (parton_showers) THEN
            CALL HCDIR('//PAWC/PS'//XXXX,' ')
         ELSE
            CALL HCDIR('//PAWC/'//XXXX,' ')
         ENDIF
         
c     Finish off histograms
         
         IF (Xsec.eq.0) THEN
            WRITE(HZUNIT,*) 
     & 'HZ'//xxxx//': termination called with zero cross section'
            WRITE(HZUNIT,*) 
     &   '       cross section graph meaningless'
            WRITE(HZUNIT,*) '        in process :',iproc*1000
            Xsec=1
         ENDIF
         IF (Ntot.eq.0) THEN
            WRITE(HZUNIT,*) 'HZ'//xxxx/
     $           /': termination called with no total events'
            WRITE(HZUNIT,*) '      cross section graph meaningless'
            WRITE(HZUNIT,*) '         in process :',iproc*1000
            Ntot=1
         ENDIF


c - now weight the histograms 

         lumnb = ntot/xsec
         
c     - eta plots
         CALL HZHINRM(100*iproc+13,0,lumnb,1)
         DO i=25,29
            CALL HZHINRM(100*iproc+i,0,lumnb,1)
         ENDDO
c     - et plots
         CALL HZHINRM(100*iproc+14,0,lumnb,1)
         DO i=31,35
            CALL HZHINRM(100*iproc+i,0,lumnb,1)
         ENDDO
         
c     - q2 plots
         DO i=40,55
            CALL HZHINRM(100*iproc+i,0,lumnb,1)
            CALL HZHINRM(100*iproc+20+i,0,lumnb,1)
         ENDDO

         
         if (iproc.eq.2) then
            CALL HOPERA(113,'+e',213,13,1.,1.)
            CALL HOPERA(114,'+e',214,14,1.,1.)
            DO i=25,29
               CALL HOPERA(100+i,'+e',200+i,i,1.,1.)
            ENDDO
            DO i=31,35
               CALL HOPERA(100+i,'+e',200+i,i,1.,1.)
            ENDDO
            DO i=40,55
               CALL HOPERA(100+i,'+e',200+i,i,1.,1.)
               CALL HOPERA(120+i,'+e',220+i,20+i,1.,1.)
            ENDDO
         endif
         
         if (iproc.eq.0.or.iproc.eq.2) then
c     produce the ratios
            do i=41,53,4
               CALL HOPERA(i,'/e',i+1,i+2,1.,1.)
               CALL HOPERA(20+i,'/e',20+i+1,20+i+2,1.,1.)
            enddo
         endif
         
c     dsig/det^2 - need to still devide by q2 binning

      ELSE

c     End 

         WRITE(HZUNIT,*) 'HZH'//xxxx/
     $        /':Please run routine with iflag set to 1,2 or 3'
         WRITE(HZUNIT,*) 
     &    '     with +1000 or +2000 for DIR or RES events.'
      
      ENDIF
      
99    CONTINUE

      RETURN

      END
