      SUBROUTINE HZh0407018(IFLAG)
**********************************************************************
* Running: 	Ee=27.5, Ep=920
*
* Purpose: 	Produce histograms for:
*
*          	Measurement of Prompt Photon Cross Section in Photoproduction a HERA
* 	  	Eur.Phys.J.C38:437-445,2005
*	   	DESY 04-118
*          	hep-ex/0407018
*
*
* Selection:	       Q^2 < 1 GeV^2
*		0.2 <  y   < 0.7
*               
*		     E_T^gamma > 5 GeV
*		-1 < eta^gamma < 0.9
*            
*          	Jets are found in Lab frame using the k_{T} algorithm with d=1.
*		     E_T^jet > 4.5 GeV
*		-1 < eta^jet < 2.3
*
* Produced histos (Figure numbers refer to hep-ex/0407018):
*
*		ID = 31,32	Fig. 3 a,b and Fig. 4 a,b
*		ID = 41,42	Fig. 4 c,d
*		ID = 51-54	Fig. 5 a-d
*		ID = 61,62	Fig. 6 a-d
*		ID = 63,64	Fig. 6 a-d but normalized to lumi 
*				(e.g. for adding direct and resolved contributions)
*
* H1 data histograms have corresponding negative numbers:
*
*               ID =    XX MC
*		ID =   -XX Data with statistical errors
*		ID =  -1XX Data with systematic errors
*		ID =  -2XX Data with total errors (stat and sys added in quadrature)
*
* Arguments: 	IFLAG = 1: Initialization
*            	IFLAG = 2: Filling
*            	IFLAG = 3: Termination
*
*
* Called subroutines and functions:
*            
*		From HBOOK lib:  HCDIR,HMDIR,HBOOK1,HFILL,HPAK,HPAKE,HBARX                            	 
*            	From HzTool lib: HZIBEAM, HZDISKIN, HZET, HZETA, HZHINRM
*
*
*
* Adapted by: Magnus Hansson
*
*
**********************************************************************

      IMPLICIT NONE

**********************************************************************
*
*                      Declarations
*
**********************************************************************

#include "hzfunc.inc"
#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzjetcmn.inc"      
*---Constants:

      REAL PI,RD
      PARAMETER (PI = 3.1415927, RD = 180./PI)

*---General variables:
 
      INTEGER NENTRY, IFLAG, I
      CHARACTER*8 XXXX
      DATA XXXX/'h0407018'/,NENTRY/0/
      INTEGER HISTID(12), IPHO
      DATA HISTID/31,32,41,42,51,52,53,54,61,62,63,64/      
      DOUBLE PRECISION EVENTS, XSECTMP

*--- Beams
      
      INTEGER IEBEAM,IPBEAM
      REAL EE, EP
      PARAMETER (EE=27.6, EP=920)
      DOUBLE PRECISION PBEAM(4), EBEAM(4)
      
*---Kinematic variables:

      REAL Q2,X,Y

*--- Jet stuff
 
      INTEGER IERR
      INTEGER MODJET, KTMODE
      DOUBLE PRECISION RCONE
      PARAMETER (KTMODE=3212)
      PARAMETER (MODJET=3)
      PARAMETER (RCONE=1.)
      
      INTEGER NUMJET, NJ, IJET
      PARAMETER (NUMJET=50)
      DOUBLE PRECISION PJ(MAXHZJETS,8), PTJETMIN
      
      DOUBLE PRECISION PTPHOMIN, PHOET, PHOETA
      DOUBLE PRECISION XGAM, XP, DPHI, PT  
      DOUBLE PRECISION LUMI, NTOT61, NTOT62
      INTEGER NPHOT,NPHOTJET, PHOIST
            
*--- Define bins (binsXX where XX is histogram ID)
      
      REAL bins31(7)
      DATA bins31/5.,5.8,6.7,7.5,8.3,9.2,10./
      
      REAL bins32(7)
      DATA bins32/-1.,-0.7,-0.4,0.,0.3,0.6,0.9/
      
      REAL bins41(7)
      DATA bins41/5.,5.8,6.7,7.5,8.3,9.2,10./
      
      REAL bins42(7)
      DATA bins42/-1.,-0.7,-0.4,0.,0.3,0.6,0.9/
      
      REAL bins51(4)
      DATA bins51/4.5,6.7,8.8,11./
      
      REAL bins52(6)
      DATA bins52/-1.,-0.3,0.3,1.,1.6,2.3/
      
      REAL bins53(5)
      DATA bins53/0.1,0.3,0.6,0.9,1.1/
      
      REAL bins54(6)
      DATA bins54/0.0018,0.0034,0.0063,0.012,0.022,0.04/
      
      REAL bins61(5)
      DATA bins61/0.,2.,4.,6.,8./
      
      REAL bins62(4)
      DATA bins62/0.,2.,4.,6/


*--- Data
      
      REAL DA31(6), ST31(6), SY31(6), TO31(6)       
      DATA DA31/17.,11.6,8.3,6.67,5.46,2.42/
      DATA ST31/1.8,1.2,0.9,0.69,0.55,0.48/
      DATA SY31/2.9,1.9,1.3,0.85,0.93,0.57/
      
      REAL DA32(6), ST32(6), SY32(6), TO32(6)       
      DATA DA32/26.8,29.1,26.9,24.7,21.2,11./
      DATA ST32/2.6,3.4,3.0,2.7,1.9,1.6/
      DATA SY32/4.2,5.2,4.3,3.4,3.5,2.2/
      
      REAL DA41(6), ST41(6), SY41(6), TO41(6)       
      DATA DA41/8.9,6.24,6.1,5.28,3.98,2.3/
      DATA ST41/1.2,0.9,0.77,0.64,0.51,0.48/
      DATA SY41/2.4,1.02,0.97,0.71,0.71,0.52/
      
      REAL DA42(6), ST42(6), SY42(6), TO42(6)       
      DATA DA42/16.2,18.2,18.2,13.7,14.5,6.8/
      DATA ST42/2.1,2.5,2.4,2.0,1.5,1.3/
      DATA SY42/2.8,3.2,2.9,2.2,2.6,1.6/
            
      REAL DA51(3), ST51(3), SY51(3), TO51(3)       
      DATA DA51/5.97,3.92,2.07/
      DATA ST51/0.54,0.42,0.26/
      DATA SY51/0.91,0.72,0.44/
      
      REAL DA52(5), ST52(5), SY52(5), TO52(5)       
      DATA DA52/7.5,12.4,12.3,6.58,2.81/
      DATA ST52/1.2,1.2,1.3,0.87,1.01/
      DATA SY52/1.4,1.6,2.0,1.10,0.94/

      REAL DA53(4), ST53(4), SY53(4), TO53(4)       
      DATA DA53/7.8,16.4,39.9,49.3/
      DATA ST53/2.2,2.8,4.1,4.1/
      DATA SY53/2.5,2.6,7.4,7.4/
      
      REAL DA54(5), ST54(5), SY54(5), TO54(5)       
      DATA DA54/247.,2350.,1900.,640.,167./
      DATA ST54/149.,300.,180.,70.,31./
      DATA SY54/51.,350.,290.,113.,39./
      
      REAL DA61(4), ST61(4), SY61(4), TO61(4)       
      DATA DA61/0.216,0.117,0.124,0.0225/
      DATA ST61/0.030,0.022,0.019,0.0081/
      DATA SY61/0.015,0.011,0.011,0.0077/
      
      REAL DA62(3), ST62(3), SY62(3), TO62(3)       
      DATA DA62/0.420,0.061,0.0054/
      DATA ST62/0.033,0.017,0.0078/
      DATA SY62/0.024,0.014,0.0026/


      save NTOT61, NTOT62, NENTRY, EVENTS, NPHOT, NPHOTJET


*********************************************************************
*
*                      Initialization
*
*********************************************************************
      
      IF (IFLAG.EQ.1) THEN

*---Initialization: The following MUST always be done
*      (i) make subdirectory in PAWC
*       - use the name as the xxxxxx in HZxxxxxx subroutine
*      (ii) make subdirectory in o/p file

         CALL HCDIR('//PAWC',' ')
         CALL HMDIR(XXXX,'S')
         CALL HCDIR('//HISTO',' ')
         CALL HMDIR(XXXX,'S')

	 WRITE(6,*)' '
	 WRITE(6,*)'***************************************************'
	 WRITE(6,*)'               HZ'//xxxx//' called           '
         WRITE(6,*)'***************************************************'
         WRITE(6,*)' '

*---Book histograms, fill data histograms
*---Reminder: HistoID's:  ID > 0  : MC, stat. uncertainty
*                         ID < 0  : H1 data, stat. uncertainty
*                         ID <-100: H1 data, syst. uncertainty
*                         ID <-200: H1 data, tot. uncertainty
*
	 CALL HBOOKB(31,'dSig/dE_{T}^{gam} Incl.'          ,6,bins31,0.)
	 CALL HBOOKB(32,'dSig/dEta^{gam} Incl.'            ,6,bins32,0.)
         CALL HBOOKB(41,'dSig/dE_{T}^{gam}'                ,6,bins41,0.)
	 CALL HBOOKB(42,'dSig/dEta^{gam} '                 ,6,bins42,0.)
         CALL HBOOKB(51,'dSig/dE_{T}^{jet}'                ,3,bins51,0.)
	 CALL HBOOKB(52,'dSig/dEta^{jet}'                  ,5,bins52,0.)
         CALL HBOOKB(53,'dSig/dx_{gam}'                    ,4,bins53,0.)
	 CALL HBOOKB(54,'dSig/dx_{p} '                     ,5,bins54,0.)
	 CALL HBOOKB(61,'1/ Sig dSig/dp_{T} x_{gam}<0.85'  ,4,bins61,0.)
	 CALL HBOOKB(62,'1/ Sig dSig/dp_{T} x_{gam}>0.85'  ,3,bins62,0.)
	 CALL HBOOKB(63,'dSig/dp_{T} x_{gam}<0.85'         ,4,bins61,0.)
	 CALL HBOOKB(64,'dSig/dp_{T} x_{gam}>0.85'         ,3,bins62,0.)

	 CALL HBOOKB(-31,'dSig/dE_{T}^{gam} Incl.'         ,6,bins31,0.)
	 CALL HBOOKB(-32,'dSig/dEta^{gam} Incl.'           ,6,bins32,0.)
         CALL HBOOKB(-41,'dSig/dE_{T}^{gam}'               ,6,bins41,0.)
	 CALL HBOOKB(-42,'dSig/dEta^{gam} '                ,6,bins42,0.)
         CALL HBOOKB(-51,'dSig/dE_{T}^{jet}'               ,3,bins51,0.)
	 CALL HBOOKB(-52,'dSig/dEta^{jet}'                 ,5,bins52,0.)
         CALL HBOOKB(-53,'dSig/dx_{gam}'                   ,4,bins53,0.)
	 CALL HBOOKB(-54,'dSig/dx_{p} '                    ,5,bins54,0.)
	 CALL HBOOKB(-61,'1/ Sig dSig/dp_{T} x_{gam}<0.85' ,4,bins61,0.)
	 CALL HBOOKB(-62,'1/ Sig dSig/dp_{T} x_{gam}>0.85' ,3,bins62,0.)

	 CALL HBOOKB(-131,'dSig/dE_{T}^{gam} Incl.'        ,6,bins31,0.)
	 CALL HBOOKB(-132,'dSig/dEta^{gam} Incl.'          ,6,bins32,0.)
         CALL HBOOKB(-141,'dSig/dE_{T}^{gam}'              ,6,bins41,0.)
	 CALL HBOOKB(-142,'dSig/dEta^{gam} '               ,6,bins42,0.)
         CALL HBOOKB(-151,'dSig/dE_{T}^{jet}'              ,3,bins51,0.)
	 CALL HBOOKB(-152,'dSig/dEta^{jet}'                ,5,bins52,0.)
         CALL HBOOKB(-153,'dSig/dx_{gam}'                  ,4,bins53,0.)
	 CALL HBOOKB(-154,'dSig/dx_{p} '                   ,5,bins54,0.)
	 CALL HBOOKB(-161,'1/ Sig dSig/dp_{T} x_{gam}<0.85',4,bins61,0.)
	 CALL HBOOKB(-162,'1/ Sig dSig/dp_{T} x_{gam}>0.85',3,bins62,0.)
	 
	 CALL HBOOKB(-231,'dSig/dE_{T}^{gam} Incl.'        ,6,bins31,0.)
	 CALL HBOOKB(-232,'dSig/dEta^{gam} Incl.'          ,6,bins32,0.)
         CALL HBOOKB(-241,'dSig/dE_{T}^{gam}'              ,6,bins41,0.)
	 CALL HBOOKB(-242,'dSig/dEta^{gam} '               ,6,bins42,0.)
         CALL HBOOKB(-251,'dSig/dE_{T}^{jet}'              ,3,bins51,0.)
	 CALL HBOOKB(-252,'dSig/dEta^{jet}'                ,5,bins52,0.)
         CALL HBOOKB(-253,'dSig/dx_{gam}'                  ,4,bins53,0.)
	 CALL HBOOKB(-254,'dSig/dx_{p} '                   ,5,bins54,0.)
	 CALL HBOOKB(-261,'1/ Sig dSig/dp_{T} x_{gam}<0.85',4,bins61,0.)
	 CALL HBOOKB(-262,'1/ Sig dSig/dp_{T} x_{gam}>0.85',3,bins62,0.)

	 DO i=1,12
	   CALL HBARX(HISTID(i))
	   IF (i.le.10) THEN
	     CALL HBARX(-HISTID(i))
	     CALL HBARX(-100-HISTID(i))
	     CALL HBARX(-200-HISTID(i))
	   ENDIF
	 ENDDO	 
	 
*--- Calculate total error

         Do i=1,6
	   TO31(i) = sqrt(ST31(i)**2 + SY31(i)**2)
	   TO32(i) = sqrt(ST32(i)**2 + SY32(i)**2)
	   TO41(i) = sqrt(ST41(i)**2 + SY41(i)**2)
	   TO42(i) = sqrt(ST42(i)**2 + SY42(i)**2)
	 Enddo
         
	 Do i=1,5
	   TO52(i) = sqrt(ST52(i)**2 + SY52(i)**2)
	   TO54(i) = sqrt(ST54(i)**2 + SY54(i)**2)
	 Enddo
	 
         Do i=1,4
	   TO53(i) = sqrt(ST53(i)**2 + SY53(i)**2)
	   TO61(i) = sqrt(ST61(i)**2 + SY61(i)**2)
	 Enddo
	 
	 Do i=1,3
	   TO51(i) = sqrt(ST51(i)**2 + SY51(i)**2)
	   TO62(i) = sqrt(ST62(i)**2 + SY62(i)**2)
	 Enddo

*--- Fill data histos
	 
	 DO i=0,2
	   CALL HPAK (-31-(100*i), DA31)
	   CALL HPAK (-32-(100*i), DA32)
	   CALL HPAK (-41-(100*i), DA41)
	   CALL HPAK (-42-(100*i), DA42)
	   CALL HPAK (-51-(100*i), DA51)
	   CALL HPAK (-52-(100*i), DA52)
	   CALL HPAK (-53-(100*i), DA53)
	   CALL HPAK (-54-(100*i), DA54)
	   CALL HPAK (-61-(100*i), DA61)
	   CALL HPAK (-62-(100*i), DA62)
	 ENDDO
	 
	 CALL HPAKE (-31, ST31)
	 CALL HPAKE (-32, ST32)
	 CALL HPAKE (-41, ST41)
	 CALL HPAKE (-42, ST42)
	 CALL HPAKE (-51, ST51)
	 CALL HPAKE (-52, ST52)
	 CALL HPAKE (-53, ST53)
	 CALL HPAKE (-54, ST54)
	 CALL HPAKE (-61, ST61)
	 CALL HPAKE (-62, ST62)
	 
	 CALL HPAKE (-131, SY31)
	 CALL HPAKE (-132, SY32)
	 CALL HPAKE (-141, SY41)
	 CALL HPAKE (-142, SY42)
	 CALL HPAKE (-151, SY51)
	 CALL HPAKE (-152, SY52)
	 CALL HPAKE (-153, SY53)
	 CALL HPAKE (-154, SY54)
	 CALL HPAKE (-161, SY61)
	 CALL HPAKE (-162, SY62)
	 
	 CALL HPAKE (-231, TO31)
	 CALL HPAKE (-232, TO32)
	 CALL HPAKE (-241, TO41)
	 CALL HPAKE (-242, TO42)
	 CALL HPAKE (-251, TO51)
	 CALL HPAKE (-252, TO52)
	 CALL HPAKE (-253, TO53)
	 CALL HPAKE (-254, TO54)
	 CALL HPAKE (-261, TO61)
	 CALL HPAKE (-262, TO62)

	 NTOT61 = 0.
	 NTOT62 = 0.
	 NENTRY = 0
	 EVENTS = 0.
	 NPHOT    = 0
	 NPHOTJET = 0
	 
         IF (GEN.EQ.'HERWIG') THEN
	   XSECTMP = XSEC*1000.
	 ELSE
	   XSECTMP = XSEC
	 ENDIF
	 
*********************************************************************
*
*                      Event Processing
*
*********************************************************************

      ELSEIF (IFLAG.EQ.2) THEN

	NENTRY = NENTRY + 1
        EVENTS = EVENTS + WTX
	PHOIST = 0

*--- Flip sign of pz for HERWIG
        
	IF (GEN.EQ.'HERWIG') THEN 
	  DO i=1,NHEP
            PHEP(3,i)=-PHEP(3,i)
          ENDDO
	ENDIF
	
*---Filling: The following MUST always be done
*   (i) move to the correct sub-directory in PAWC
*
         CALL HCDIR('//PAWC/'//XXXX,' ')

*--- Find position of beams (ipbeam->proton, iebeam->electron)
*--- Four vectors saved in PBEAM and EBEAM         
        
	IERR=HZIBEAM(IPBEAM,IEBEAM)
        IF (.NOT.(IERR.EQ.1)) THEN
          WRITE(6,*) 'HZ'//xxxx,' beams not found ! '
          RETURN
        ELSE
          DO I=1,4
            PBEAM(i)=PHEP(i,IPBEAM)
            EBEAM(i)=PHEP(i,IEBEAM)
          ENDDO
        ENDIF
	
*--- Check that electron beam energy = 27.5     

        IF (ABS(ABS(EBEAM(4))-EE).GT.0.2) THEN
          IF (NENTRY.lt.10) THEN
            WRITE(6,*) 'HZ'//xxxx,' Electron not at ',EE,' ! ',EBEAM(4)
          ELSEIF (NENTRY.eq.10) then
            WRITE(6,*) 'HZ'//xxxx,' Electron not at ',EE,
     &           ' ...last message ! '
          ENDIF
          RETURN
        ENDIF

*--- Check that proton beam energy = 920     

        IF (ABS(ABS(PBEAM(4))-EP).GT.1.) THEN
          IF (NENTRY.lt.10) THEN
            WRITE(6,*) 'HZ'//xxxx,' Proton not at ',EP,' ! ',PBEAM(4)
          ELSEIF (NENTRY.eq.10) then
            WRITE(6,*) 'HZ'//xxxx,' Proton not at ',EP,
     &           ' ...last message ! '
          ENDIF
          RETURN
        ENDIF

*---Get Q2 and Y and cut

         Q2 = REAL(HZDISKIN(1))
         Y  = REAL(HZDISKIN(3))
*	 Q2 = REAL(HZPHOKIN(1))
*        Y  = REAL(HZPHOKIN(3))	 

         IF (Q2.GE.1) RETURN
         IF (Y.GE.0.7) RETURN
         IF (Y.LE.0.2) RETURN

*--- Find Prompt Photon
*--- Loop over all particles, find photon which came from a quark
*--- If more than one photon, take the hardest one
         
         IF (GEN.EQ.'HERWIG') THEN
*--- special tratment for HERWIG
	   PTPHOMIN = 5.
	   IPHO = -1
	   DO i=1,NHEP
	     IF ((IDHEP(i).EQ.22).AND.(ISTHEP(i).eq.1)) THEN
*--- Mother or mothers mother is a photon with ID 94
               IF ((IDHEP(JMOHEP(1,i)).EQ.94).OR.
     &	           (IDHEP(JMOHEP(1,JMOHEP(1,i))).EQ.94)) THEN
	         IF (HZET(i).GT.PTPHOMIN) THEN
	           IF ((-1.LT.HZETA(i)).AND.(HZETA(i).LT.0.9)) THEN
	             IPHO = i
	  	     PTPHOMIN = HZET(i)
	           ENDIF
	         ENDIF
	       ENDIF
	     ENDIF
	   ENDDO
 98	   FORMAT(A,I3,I5,I5,I5,I5,I5)	 
	 ELSE
*--- Assume that all other MC behave like PYTHIA...	 
	   PTPHOMIN = 5.
	   IPHO = -1
	   DO i=1,NHEP
	     IF ((IDHEP(i).EQ.22).AND.(ISTHEP(i).eq.1)) THEN
               IF (((IDHEP(JMOHEP(1,i)).NE.0).AND.
     &	           (ABS(IDHEP(JMOHEP(1,i))).LE.6)).OR.
*--- ... or mother is a photon marked for documentation, which has a quark mother
*--- (this is at least needed for Pythia)
     &            ((IDHEP(JMOHEP(1,i)).EQ.22).AND.
     &            (ISTHEP(JMOHEP(1,i)).NE. 1).AND.
     &            (IDHEP(JMOHEP(1,JMOHEP(1,i))).NE.0).AND.
     &	       (ABS(IDHEP(JMOHEP(1,JMOHEP(1,i)))).LE.6))) THEN
	         IF (HZET(i).GT.PTPHOMIN) THEN
	           IF ((-1.LT.HZETA(i)).AND.(HZETA(i).LT.0.9)) THEN
	             IPHO = i
		     PTPHOMIN = HZET(i)
	           ENDIF
	         ENDIF
	       ENDIF
	     ENDIF
	   ENDDO	 
	 
	 ENDIF
	 
	 IF (IPHO.GT.0) THEN
	   PHOET  = PTPHOMIN
	   PHOETA = HZETA(IPHO)
	   NPHOT = NPHOT+1
*-.-- Exclude prompt photon from jet finder
	   PHOIST = ISTHEP(IPHO)
	   ISTHEP(IPHO)=999
	 ELSE
	   RETURN
	 ENDIF
	 
*--- Find jets in lab frame
       
	 CALL HZJTFIND(MODJET,RCONE,NJ,PJ)	 

         IF (IPHO.GT.0) THEN
	   ISTHEP(IPHO)=PHOIST	 
	 ENDIF
	 
*--- Do jet selection
 
	 IJET = -1
	 PTJETMIN = 4.5
	 DO i=1,NJ
	   IF (PJ(i,3).GT.PTJETMIN) THEN
	     IF ((-1.LT.PJ(i,1)).AND.(PJ(i,1).LT.2.3)) THEN
	       IJET = i
	       PTJETMIN = PJ(i,3)
	     ENDIF
	   ENDIF
	 ENDDO 
 	 
*--- If jet and photon is found, do some calculations
 	 
	 IF (IJET.GT.0) THEN
	   NPHOTJET = NPHOTJET+1
	   XGAM = (PHOET*(EXP(-PJ(IJET,1)) + EXP(-PHOETA)))
     &	   	   /(2.*Y*EBEAM(4))
	   XP   = (PHOET*(EXP(PJ(IJET,1)) + EXP(PHOETA)))
     &	   	   /(2.*PBEAM(4))
           DPHI = HZPHI(IPHO)-PJ(IJET,2)
	   IF (DPHI < 0 ) DPHI = DPHI + 2*Pi;
           IF (DPHI > Pi) DPHI = 2*Pi - DPHI;
	   PT   =  PHOET*sin(DPHI)
	 ENDIF	 
	 	 
*--- Fill Histograms

	 CALL HFILL(31,REAL(PHOET       ) ,0.,wtx)
	 CALL HFILL(32,REAL(PHOETA      ) ,0.,wtx)
	 
	 IF (IJET.GT.0) THEN
	   CALL HFILL(41,REAL(PHOET     ) ,0.,wtx)
	   CALL HFILL(42,REAL(PHOETA    ) ,0.,wtx)
	   CALL HFILL(51,REAL(PJ(IJET,3)) ,0.,wtx)
	   CALL HFILL(52,REAL(PJ(IJET,1)) ,0.,wtx)
	   CALL HFILL(53,REAL(XGAM      ) ,0.,wtx)
	   CALL HFILL(54,REAL(XP        ) ,0.,wtx)
	   IF (XGAM.LT.0.85) THEN
	     CALL HFILL(61,REAL(PT      ) ,0.,wtx)
	     CALL HFILL(63,REAL(PT      ) ,0.,wtx)
	     IF (PT.LT.8) NTOT61 = NTOT61 + wtx
	   ELSE
	     CALL HFILL(62,REAL(PT      ) ,0.,wtx)
	     CALL HFILL(64,REAL(PT      ) ,0.,wtx)
	     IF (PT.LT.6) NTOT62 = NTOT62 + wtx
	   ENDIF
	 ENDIF
	 
	 
*********************************************************************
*
*                      Termination
*
*********************************************************************

      ELSEIF (IFLAG.EQ.3) THEN

*---Termination: The following MUST always be done
*   (i) Move to the correct PAW subdirectory
*
        CALL HCDIR('//PAWC/'//XXXX,' ')

        lumi=999999999.
        if (XSECTMP.NE.0.) then
          lumi=REAL(events)/REAL(XSECTMP)
        else
          write(6,*) 'hz',xxxx,' xsec=0 ! '
        endif
        
	DO i=1,8
	  CALL HZHINRM(HISTID(i), 0, REAL(LUMI), 1)
	ENDDO
	
	DO i=11,12
	  CALL HZHINRM(HISTID(i), 0, REAL(LUMI), 1)
	ENDDO
	
	IF (NTOT61.GT.0.) THEN
	  CALL HZHINRM(61, 0, REAL(NTOT61) , 1)
	ELSE
	  write(6,*)'NTOT61 = ',NTOT61,' Hist 61 not normalised!'
	ENDIF
	
	IF (NTOT62.GT.0.) THEN
	  CALL HZHINRM(62, 0, REAL(NTOT62) , 1)
	ELSE
	  write(6,*)'NTOT62 = ',NTOT62,' Hist 62 not normalised!'
	ENDIF
	
	write(6,*)' '
	write(6,*)'*********************************************'
	write(6,*)'*                                           *'
	write(6,*)'*            Summary of hz',xxxx,'            *'
	write(6,*)'*                                           *'
	write(6,*)'*********************************************'
	write(6,*)' '
	write(6,*)' Events ',events
	write(6,*)' x-sec  ',XSECTMP
	write(6,*)' Lumi   ',Lumi
	write(6,*)' Found  ',NPHOT,' events containing prompt photon'
	write(6,*)' Found  ',NPHOTJET,' events containing prompt photon
     & and a jet'
        write(6,*)' NTOT61 ',NTOT61
	write(6,*)' NTOT62 ',NTOT62
        write(6,*)' '
	write(6,*)'*********************************************'
	write(6,*)' '
      
      ENDIF
      RETURN
      END
