      subroutine hzh0408149(IFLAG)
************************************************************************
* Purpose: Inclusive Production of D+,D0,D*+,D+s in DIS at HERA
*
* Cuts :    2  <=  Q^2  <= 100 GeV^2
*         0.05 <  y_bj  < 0.7
*          2.5 < p_t(D) < 10 GeV
*
* H1 publication: DESY-04-156
* 
* Histograms:
*   ID = Monte Carlo prediction
*  -ID = Data distribution 
*  (ID -x0x -> data + stat. error; ID -x1x -> data + stat. + sys. error
*   ID 55x,44x,33x b-quark times 4.3 as written in the publication)
*
* Author: Andreas Werner Jung  (andreas.werner.jung@desy.de)  31.03.2005
************************************************************************
      IMPLICIT NONE
*
#include "hzfunc.inc"
#include "hzhbook.inc"
#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzjetcmn.inc"
*
**********************************************
* Variables from AROMA for cross checks
c      INTEGER LSTHFL
c      REAL*8 PARHFL,CUTHFL
c      COMMON /HFDATA/ PARHFL(10),CUTHFL(8),LSTHFL(10)
c      INTEGER LST
*
***********************************************
* user variables - the parameter ee is the desired value
* for the electron beam. there is a cut on the spread of this beamenergy 
* app. line 460
*
      real enel,thel
      Real rd,ee,eb,dphi
      Logical evcut,lp,lbtag,lctag,lstag
      data lp/.false./
*
      Character string*20
      Character strng*9999
*
      Integer nev,nentry,nd0,ndplus,ndminus
      Real qqradp,xntot,q2,x,y,Wyp2
      Real ptl,pth,wl,wh,etal,etah
      Integer nbpt,nbw,nbeta
      Data nev/0/
      Parameter (ee=27.6)
*
      Integer ierr,ibeam,igam,iel,idum,idhe,idmo1,idmo2
      Double precision pbeam(4),pgam(4),pcm(4),ph(5),phc(4)
*
      integer err_flag
      real xnd0,sigma_tot,x_d,p,pt,et,th,phdum,rap
      real lum,lumnb,lumpb,binw
      data xnd0/0./
*
***************************************************
*     Data points from plots
***************************************************
*     101 dsigma_vis/dpt D^+ data, stat.err., stat.err. + sys. err. 
      real e101(6),xse101(6),xye101(6),xLA101(6)
      data e101 / 1.95 , 1.05 , 0.56 , 0.353 , 0.063 , 0.025 /
      data xse101 / 0.32 , 0.18 , 0.128 , 0.054 , 0.023 , 0.005 /
       data xye101 / 0.41 , 0.22 , 0.121 , 0.104 , 0.072 , 0.005 /

*     102 dsigma_vis/dpt D^0 data, stat.err.,  stat.err. + sys. err. 
      real e102(6),xse102(6),xye102(6),xLA102(6)
      data e102 / 5.29 , 3.04 , 2.29 , 1.13 , 0.516 , 0.056 /
      data xse102 / 0.72 , 0.41 , 0.34 , 0.15 , 0.120 , 0.018 /
       data xye102 / 1.11 , 0.78 , 0.81 , 0.21 , 0.129 , 0.021 /

*     103 dsigma_vis/dpt D^+_s data, stat.err., sys. err. + sys. err. 
      real e103(3),xse103(3),xye103(3),xLA103(3)
      data e103 / 1.09 , 0.44 , 0.071 /
      data xse103 / 0.33 , 0.22 , 0.021 /
      data xye103 / 0.27 , 0.16 , 0.021 /

*     104 dsigma_vis/dpt D^*+ data, stat.err., sys. err. + sys. err. 
      real e104(6),xse104(6),xye104(6),xLA104(6)
      data e104 / 2.31 , 1.15 , 1.03 , 0.344 , 0.224 , 0.027 /
      data xse104 / 0.28 , 0.16 , 0.13 , 0.055 , 0.041 , 0.007 /
       data xye104 / 0.50 , 0.23 , 0.21 , 0.085 , 0.048 , 0.015 /

*     201 dsigma_vis/deta D^+  data, stat. err.,sys. err. + sys. err. 
      real e201(6),xse201(6),xye201(6),xLA201(6)
      data e201 / 0.84 , 0.97 , 0.66 , 0.83 , 0.76 , 0.28 /
      data xse201 / 0.16 , 0.15 , 0.12 , 0.13 , 0.15 , 0.13 /
      data xye201 / 0.25 , 0.2 , 0.14 , 0.18 , 0.36 , 0.15 /

*     202 dsigma_vis/deta D^0 data, stat. err.,sys. err. + sys. err. 
      real e202(6),xse202(6),xye202(6),xLA202(6)
      data e202 / 1.85 , 1.74 , 2.43 , 2.88 , 2.67 , 1.91 /
      data xse202 / 0.38 , 0.3 , 0.34 , 0.39 , 0.37 , 0.41 /
      data xye202 / 0.46 , 0.28 , 0.42 , 0.87 , 0.87 , 0.77 /

*     203 dsigma_vis/deta D^+_s data, stat. err.,sys. err. + sys. err. 
      real e203(3),xse203(3),xye203(3),xLA203(3)
      data e203 / 0.59 , 0.50 , 0.81 /
      data xse203 / 0.19 , 0.17 , 0.25 /
      data xye203 / 0.12 , 0.12 , 0.43 /

*     204 dsigma_vis/deta D^*+ data, stat. err.,sys. err. + sys. err. 
      real e204(6),xse204(6),xye204(6),xLA204(6)
      data e204 / 0.62 , 1.10 , 1.2 , 1.15 , 0.99 , 0.65 /
      data xse204 / 0.13 , 0.13 , 0.14 , 0.16 , 0.15 , 0.16 /
      data xye204 / 0.14 , 0.23 , 0.24 , 0.22 , 0.31 , 0.32 /

*     301 dsigma_vis/dq2 D^+ data, stat. err.,sys. err. + sys. err. 
      real e301(6),xse301(6),xye301(6),xLA301(6)
      data e301 / 0.2 , 0.155 , 0.095 , 0.0373 , 0.0153, 0.0042 /
      data xse301 / 0.038 , 0.025 , 0.015 , 0.0072 , 0.0036, 0.0009 /
      data xye301 / 0.047 , 0.034 , 0.021 , 0.0105 , 0.0033, 0.0009 /

*     302 dsigma_vis/dq2 D^0 data, stat. err.,sys. err. + sys. err. 
      real e302(6),xse302(6),xye302(6),xLA302(6)
      data e302 / 0.662 , 0.386 , 0.267 , 0.102 , 0.0474, 0.0186 /
      data xse302 / 0.099 , 0.058 , 0.043 , 0.018 , 0.0113, 0.00227 /
      data xye302 / 0.161 , 0.088 , 0.066 , 0.023 , 0.0107, 0.0038 /

*     303 dsigma_vis/dq2 D^+_s data, stat. err.,sys. err. + sys. err. 
      real e303(3),xse303(3),xye303(3),xLA303(3)
      data e303 / 0.172 , 0.053 , 0.0065 /
      data xse303 / 0.044 , 0.017 , 0.0028 /
      data xye303 / 0.048 , 0.012 , 0.0014 /

*     304 dsigma_vis/dq2 D^*+ data, stat. err.,sys. err. + sys. err. 
      real e304(6),xse304(6),xye304(6),xLA304(6)
      data e304 / 0.359 , 0.193 , 0.107 , 0.0486 , 0.0180, 0.0064 /
      data xse304 / 0.043 , 0.027 , 0.014 , 0.0068 , 0.0038, 0.0010 /
      data xye304 / 0.079 , 0.045 , 0.021 , 0.0109 , 0.0036, 0.0013 /

*************************************************************************
*     ndstare - number D^{*+}
*     ndpluse - number D^{+}
*     ndsquark - number D_{s}^{+}
*     ndnull - number D^{0}
*
      Integer iflag,ihep,i,j,k,l,idhisto,ndstare,ndpluse,ndsquark,ndnull
      Integer nnotc,nb,nc,ncb,ndplee,ndstee,ndspee,ndoee
      Real xw,xnorm
*     DESY Preprint number
      Character*8 xxxx
      Data xxxx/'h0408149'/
      Integer nbs

* p_t stuff
      Data nbs/6/
      Real ptbs(7)
      Data ptbs/2.5,3.,3.5,4.,5.,6.,10./
      Integer nbptd
      Data nbptd/3/
      Real ptbd(4)
      Data ptbd/2.5,3.5,4.,10./

* eta stuff
      Real etabs(7)
      Data etabs/-1.5,-1.,-0.5,0.,0.5,1.,1.5 /
      Integer nbetad
      Data nbetad/3/
      real etabd(4)
      Data etabd/-1.5,-0.5,0.75,1.5 /

*q2 stuff
      real q2bs(7)
      Data q2bs/2.,4.,7.,12.,22.,35.,100./
      Integer q2bsm
      Data q2bsm/3/
      Real q2bsmb(4)
      Data q2bsmb/2.,6.,16.,100./

      save nc, ncb, nb

*
*********************************************************************
*
*                      Initialization
*
*********************************************************************
      NENTRY=NENTRY+1
*
      IF (iflag.eq.1) then
*
         nd0=0
         ndplus=0
         write(hzunit,*) 'HZ'//xxxx,' Inclusive Production of D+,D0,D*+,D+s 
     +   in DIS at HERA' 
*
*      Initialisation: The following MUST always be done
*      (i) make subdirectory in PAWC
*       - use the name as the xxxxxx in HZxxxxxx subroutine
*      (i) make subdirectory in o/p file
*
         Call hcdir('//PAWC',' ')
         call hmdir(xxxx,'S')
         Call hcdir('//HISTO',' ')
         call hmdir(xxxx,'S')
*
         call hbookb(-101,'dsigma_vis/dpt D^+ data and Aroma '
     &        ,nbs,ptbs,0.0)
         call hbookb(-111,'dsigma_vis/dpt D^+ data and Aroma '
     &        ,nbs,ptbs,0.0)
         call hbookb(666,' D+ pt-MC'
     &        ,nbs,ptbs,0.0)
         call hbookb(556,' D+ from b; pt-MC'
     &        ,nbs,ptbs,0.0)
         call hbookb(667,' D*+ pt-MC'
     &        ,nbs,ptbs,0.0)
         call hbookb(557,' D*+ from b; pt-MC'
     &        ,nbs,ptbs,0.0)
         call hbookb(668,' Ds+ pt-MC'
     &        ,nbptd,ptbd,0.0)
         call hbookb(558,' Ds+ from b; pt-MC'
     &        ,nbptd,ptbd,0.0)
         call hbookb(669,' D0 pt-MC'
     &        ,nbs,ptbs,0.0)
         call hbookb(559,' D0 from b; pt-MC'
     &        ,nbs,ptbs,0.0)
         call hbookb(-102,' dsigma_vis/dpt D^0 data and Aroma'
     &        ,nbs,ptbs,0.0)
         call hbookb(-112,' dsigma_vis/dpt D^0 data and Aroma'
     &        ,nbs,ptbs,0.0)
         call hbookb(-103,' dsigma_vis/dpt D^+_s data and Aroma'
     &        ,nbptd,ptbd,0.0)
         call hbookb(-113,' dsigma_vis/dpt D^+_s data and Aroma'
     &        ,nbptd,ptbd,0.0)
         call hbookb(-104,' dsigma_vis/dpt D^*+ data and Aroma'
     &        ,nbs,ptbs,0.0)
         call hbookb(-114,' dsigma_vis/dpt D^*+ data and Aroma'
     &        ,nbs,ptbs,0.0)
         call hbookb(-201,' dsigma_vis/deta D^+ data and Aroma'
     &        ,nbs,etabs,0.0)
         call hbookb(-211,' dsigma_vis/deta D^+ data and Aroma'
     &        ,nbs,etabs,0.0)
         call hbookb(776,' D+ eta-MC'
     &        ,nbs,etabs,0.0)
         call hbookb(446,' D+ from b; eta-MC'
     &        ,nbs,etabs,0.0)
         call hbookb(777,' D*+ eta-MC'
     &        ,nbs,etabs,0.0)
         call hbookb(447,' D+ from b; eta-MC'
     &        ,nbs,etabs,0.0)
         call hbookb(778,' Ds+ eta-MC'
     &        ,nbetad,etabd,0.0)
         call hbookb(448,' Ds+ from b; eta-MC'
     &        ,nbetad,etabd,0.0)
         call hbookb(779,' D0 eta-MC'
     &        ,nbs,etabs,0.0)
         call hbookb(449,' D0 from b; eta-MC'
     &        ,nbs,etabs,0.0)
         call hbookb(-202,' dsigma_vis/deta D^0 data and Aroma'
     &        ,nbs,etabs,0.0)
         call hbookb(-212,' dsigma_vis/deta D^0 data and Aroma'
     &        ,nbs,etabs,0.0)
         call hbookb(-203,' dsigma_vis/deta D^+_s data and Aroma'
     &        ,nbetad,etabd,0.0)
         call hbookb(-213,' dsigma_vis/deta D^+_s data and Aroma'
     &        ,nbetad,etabd,0.0)
         call hbookb(-204,' dsigma_vis/deta D^+* data and Aroma'
     &        ,nbs,etabs,0.0)
         call hbookb(-214,' dsigma_vis/deta D^+* data and Aroma'
     &        ,nbs,etabs,0.0)
         call hbookb(886,' D+ q2-MC'
     &        ,nbs,q2bs,0.0)
         call hbookb(336,' D+ from b; q2-MC'
     &        ,nbs,q2bs,0.0)
         call hbookb(887,' D*+ q2-MC'
     &        ,nbs,q2bs,0.0)
         call hbookb(337,' D*+ from b; q2-MC'
     &        ,nbs,q2bs,0.0)
         call hbookb(888,' Ds+ q2-MC'
     &        ,q2bsm,q2bsmb,0.0)
         call hbookb(338,' Ds+ from b; q2-MC'
     &        ,q2bsm,q2bsmb,0.0)
         call hbookb(889,' D0 q2-MC'
     &        ,nbs,q2bs,0.0)
         call hbookb(339,' D0 from b; q2-MC'
     &        ,nbs,q2bs,0.0)
         call hbookb(-301,' dsigma_vis/dq2 D^+ data and Aroma'
     &        ,nbs,q2bs,0.0)
         call hbookb(-311,' dsigma_vis/dq2 D^+ data and Aroma'
     &        ,nbs,q2bs,0.0)
         call hbookb(-302,' dsigma_vis/dq2 D^0 data and Aroma'
     &        ,nbs,q2bs,0.0)
         call hbookb(-312,' dsigma_vis/dq2 D^0 data and Aroma'
     &        ,nbs,q2bs,0.0)
         call hbookb(-303,' dsigma_vis/dq2 D^+_s data and Aroma'
     &        ,q2bsm,q2bsmb,0.0)
         call hbookb(-313,' dsigma_vis/dq2 D^+_s data and Aroma'
     &        ,q2bsm,q2bsmb,0.0)
         call hbookb(-304,' dsigma_vis/dq2 D^+* data and Aroma'
     &        ,nbs,q2bs,0.0)
         call hbookb(-314,' dsigma_vis/dq2 D^+* data and Aroma'
     &        ,nbs,q2bs,0.0)
*
         call hbookb(1,' dsigma_vis/dpt/f(c->D) '
     &        ,nbs,ptbs,0.0)
         call hbookb(2,' dsigma_vis/deta/f(c->D) '
     &        ,nbs,etabs,0.0)
         call hbookb(3,' dsigma_vis/dq2/f(c->D) '
     &        ,nbs,q2bs,0.0)
*
*     Pack H1 data into histos
*     
         call hpak(-101,e101)
         call hpake(-101,xse101)
         call hpak(-111,e101)
         call hpake(-111,xye101)
*         call hpak(1,e101)
*         call hpake(1,xye101)
         call hpak(-102,e102)
         call hpake(-102,xse102)
         call hpak(-112,e102)
         call hpake(-112,xye102)
         call hpak(-103,e103)
         call hpake(-103,xse103)
         call hpak(-113,e103)
         call hpake(-113,xye103)
         call hpak(-104,e104)
         call hpake(-104,xse104)
         call hpak(-114,e104)
         call hpake(-114,xye104)
         call hpak(-201,e201)
         call hpake(-201,xse201)
         call hpak(-211,e201)
         call hpake(-211,xye201)
         call hpak(-202,e202)
         call hpake(-202,xse202)
         call hpak(-212,e202)
         call hpake(-212,xye202)
         call hpak(-203,e203)
         call hpake(-203,xse203)
         call hpak(-213,e203)
         call hpake(-213,xye203)
         call hpak(-204,e204)
         call hpake(-204,xse204)
         call hpak(-214,e204)
         call hpake(-214,xye204)
         call hpak(-301,e301)
         call hpake(-301,xse301)
         call hpak(-311,e301)
         call hpake(-311,xye301)
         call hpak(-302,e302)
         call hpake(-302,xse302)
         call hpak(-312,e302)
         call hpake(-312,xye302)
         call hpak(-303,e303)
         call hpake(-303,xse303)
         call hpak(-313,e303)
         call hpake(-313,xye303)
         call hpak(-304,e304)
         call hpake(-304,xse304)
         call hpak(-314,e304)
         call hpake(-314,xye304)
*
* counter for c- and b-quarks 
         nc = 0
         ncb = 0
         nb = 0
*
*********************************************************************
*                      Event Processing
*
*********************************************************************
*
      Else if(iflag.eq.2) then
*
         call hcdir('//PAWC/'//xxxx,' ')
*
* total number of generated events
         Ntot=Ntot+1
         xw = wtx
*
         q2=real(HZDISKIN(1))
       IF (q2.eq.-1) THEN
            PRINT*,'HZn653: HZDISKIN returns error for Q2'
         ELSEIF (q2.lt.0) THEN
            q2=abs(q2)
         ENDIF

         x =real(HZDISKIN(2))
         y =real(HZDISKIN(3))
         IF (y.eq.-1) THEN
            PRINT*,'HZn653: HZDISKIN returns error for y'
         ENDIF
         Wyp2 =real(HZDISKIN(4))
         ierr=HZIBEAM(ibeam,idum)
         if (.not.(ierr.eq.1)) then
            write(hzunit,*) 'HZ'//xxxx,' beams not found ! '
            goto 20
         endif

c define the cuts here
         if(y.lt.0.05.or.y.gt.0.7) then
            goto 20
         endif
*
************************************
* check if data is ok
*
         iel=HZIDELEC(idum)
         if (iel.eq.-1) then
            write(6,*) 'Hz'//xxxx,' electron not found '
            goto 20
         endif
         enel=real(PHEP(4,iel))
         thel=real(HZPHMANG(PHEP(3,iel), sqrt(PHEP(1,iel)**2+PHEP(2,
     +   iel)**2)))*rd
         if (lp) write(6,*) 'Hz'//xxxx, ' enel= ',enel,' thel= ',thel
*
         ierr=HZIBEAM(ibeam,idum)
         if (.not.(ierr.eq.1)) then
            write(6,*) 'HZ'//xxxx,' beams not found ! '
            goto 20
         else
            Do i=1,4
               pbeam(i)=PHEP(i,IBEAM)
            enddo
         endif
*
*     comparison between desired value and the given value of the 
*     electron beam
*
         eb=real(PHEP(4,idum))
         if (abs(abs(eb)-ee).gt.0.2) then
            if (nentry.lt.10) then
               write(6,*) 'HZ'//xxxx,' Electron beam not at ',ee,'  ! '
            elseif (nentry.eq.10) then
               write(6,*) 'HZ'//xxxx,' Electron beam not at ',ee,
     +         ' ...last message ! '
            endif
            goto 20
         endif
*
         ierr=HZIPGAM(ph)
         if (ierr.eq.-1) then
            write(6,*) 'HZ'//xxxx,' boson vector not found ! '
            call VZERO(ph,5)
         else
            Do i=1,4
               pgam(i)=ph(i)
            enddo
         endif
*
         call HZHCMINI(pbeam,pgam,ierr)
         if (ierr.eq.1) then
            write(6,*) 'HZ'//xxxx,' problem with boost to cms ! '
            goto 20
         endif
*
 9999    continue
         lbtag=.false.
         lctag=.false.
         xnd0=0
         nev=nev+1
c         ndstare = 0
c         ndpluse = 0
c         ndsquark = 0
c         ndnull = 0
*********************************************************************
*     Search for D-Mesons starts !
*
         Do 10 ihep=1,nhep
            idhe=idhep(ihep)

*****************************************************************
* isthep(ihep)==2 means fragmented or decayed state and therefor
*     not in the final state
*
            if ((abs(idhe).eq.4).and.(isthep(ihep).eq.2)) lctag=.true.
            if ((abs(idhe).eq.5).and.(isthep(ihep).eq.2)) lbtag=.true.
*
            if(idhe.eq. 4.and.isthep(ihep).eq.2) nc  = nc  + 1
            if(.not.(idhe.eq.4.and.isthep(ihep).eq.2)) nnotc = nnotc + 1
            if(idhe.eq. 5.and.isthep(ihep).eq.2) nb  = nb  + 1
            if(idhe.eq.-4.and.isthep(ihep).eq.2) ncb = ncb + 1
c            write(6,*) Ntot,nc,nb,ihep,idhe
            if (idhe .eq. 411 ) ndplee = ndplee + 1
            if (idhe .eq. 413 ) ndstee = ndstee + 1
            if (idhe .eq. 431 ) ndspee = ndspee + 1
            if (idhe .eq. 421 ) ndoee  = ndoee  + 1

**********************************************************************
*     look for D+ Mesons
**********************************************************************
            ndpluse=0
            if ( abs(idhe) .eq. 411 ) then
               ndpluse = ndpluse + 1
*
               Do i=1,4
                  Phc(i)=PHEP(i,ihep)
               enddo
*
*     pcm in had cms
               call HZHCM(phc,pcm,ierr)
               if (ierr.eq.1) then
c                 write(6,*) 'HZ'//xxxx,' problem with boost to cms ! '
                  goto 20
               endif
*     get particle momenta
*
               idmo1=jmohep(1,ihep)
               idmo2=idhep(idmo1)
c              write(6,*)' D+ found ',Ntot,lctag,ndpluse
*
*     LAB variables
               call hzgetvar(phc,p,pt,et,th,phdum,rap,err_flag)
               if ( err_flag.ne.0 ) goto 10
*     CMS variables
c               call hzgetvar(pcm,p,pt,et,th,phdum,rap,err_flag)
c               if ( err_flag.ne.0 ) goto 20
*
               rap = -0.5*log((phc(4)-phc(3))/(phc(4)+phc(3)))
*
*     Cuts as mentioned in the analyses
               if(ndpluse.eq.1) then
       if((rap.ge.-1.5.and.rap.le.1.5).and.(pt.ge.2.5.and.pt.le.10.0)
     +  .and.(q2.ge.2.and.q2.le.100.)) then
c                 write(6,*) 'FILLED'
                  if(lbtag) then
                     call hfill(556,pt,0.,4.3)
                     call hfill(446,rap,0.,4.3)
                     call hfill(336,q2,0.,4.3)
                  endif
                  call hfill(666,pt,0.,xw)
                  call hfill(776,rap,0.,xw)
                  call hfill(886,q2,0.,xw)
               endif
            endif
*
         endif
*
**********************************************************************
*     look for D*+ Mesons
**********************************************************************
*
         ndstare=0
            if ( abs(idhe) .eq. 413 ) then
               ndstare = ndstare + 1

               Do i=1,4
                  Phc(i)=PHEP(i,ihep)
               enddo
               call HZHCM(phc,pcm,ierr)
               if (ierr.eq.1) then
                  write(6,*) 'HZ'//xxxx,' problem with boost to cms ! '
                  goto 20
               endif
*
*     get particle momenta
*
               idmo1=jmohep(1,ihep)
               idmo2=idhep(idmo1)
*
c              write(6,*)' D*+ found ',Ntot,lctag,ndstare
*
*     LAB variables
               call hzgetvar(phc,p,pt,et,th,phdum,rap,err_flag)
               if ( err_flag.ne.0 ) goto 10
*     CMS variables
c               call hzgetvar(pcm,p,pt,et,th,phdum,rap,err_flag)
c               if ( err_flag.ne.0 ) goto 20
*
               rap = -0.5*log((phc(4)-phc(3))/(phc(4)+phc(3)))
*
*     Cuts as mentioned in the analyses
               if(ndstare.eq.1) then
       if((rap.ge.-1.5.and.rap.le.1.5).and.(pt.ge.2.5.and.pt.le.10.0)
     +  .and.(q2.ge.2.and.q2.le.100.)) then
c                 write(6,*) 'FILLED'
                  if(lbtag) then
                     call hfill(557,pt,0.,4.3)
                     call hfill(447,rap,0.,4.3)
                     call hfill(337,q2,0.,4.3)
                  endif
                  call hfill(667,pt,0.,xw)
                  call hfill(777,rap,0.,xw)
                  call hfill(887,q2,0.,xw)
               endif
           endif
*
         endif
*
**********************************************************************
*     look for Ds+ Mesons
**********************************************************************
*
         ndsquark=0
         if ( abs(idhe) .eq. 431 ) then
               ndsquark = ndsquark + 1
*
               Do i=1,4
                  Phc(i)=PHEP(i,ihep)
               enddo
               call HZHCM(phc,pcm,ierr)
               if (ierr.eq.1) then
                  write(6,*) 'HZ'//xxxx,' problem with boost to cms ! '
                  goto 20
               endif
*
*     get particle momenta
*
               idmo1=jmohep(1,ihep)
               idmo2=idhep(idmo1)
*
c              write(6,*)' Ds+ found ',Ntot,lctag,ndsquark
*
*     LAB variables
               call hzgetvar(phc,p,pt,et,th,phdum,rap,err_flag)
               if ( err_flag.ne.0 ) goto 10
*     CMS variables
c               call hzgetvar(pcm,p,pt,et,th,phdum,rap,err_flag)
c               if ( err_flag.ne.0 ) goto 20
*
               rap = -0.5*log((phc(4)-phc(3))/(phc(4)+phc(3)))
*
*     Cuts as mentioned in the analyses
               if(ndsquark.eq.1) then
        if((rap.ge.-1.5.and.rap.le.1.5).and.(pt.ge.2.5.and.pt.le.10.0)
     +  .and. (q2.ge.2.and.q2.le.100.)) then
c                 write(6,*) 'FILLED'
                  if(lbtag) then
                     call hfill(558,pt,0.,4.3)
                     call hfill(448,rap,0.,4.3)
                     call hfill(338,q2,0.,4.3)
                  endif
                  call hfill(668,pt,0.,xw)
                  call hfill(778,rap,0.,xw)
                  call hfill(888,q2,0.,xw)
               endif
            endif
*
         endif
*
**********************************************************************
*     look for D0 Mesons
**********************************************************************
*
         ndnull=0
            if ( abs(idhe) .eq. 421 ) then
               ndnull = ndnull + 1

               Do i=1,4
                  Phc(i)=PHEP(i,ihep)
               enddo
               call HZHCM(phc,pcm,ierr)
               if (ierr.eq.1) then
                  write(6,*) 'HZ'//xxxx,' problem with boost to cms ! '
                  goto 20
               endif
*
*     get particle momenta
*
               idmo1=jmohep(1,ihep)
               idmo2=idhep(idmo1)
*
c              write(6,*)' D0 found ',Ntot,lctag,ndnull
*
*     LAB variables
               call hzgetvar(phc,p,pt,et,th,phdum,rap,err_flag)
               if ( err_flag.ne.0 ) goto 10
*     CMS variables
c               call hzgetvar(pcm,p,pt,et,th,phdum,rap,err_flag)
c               if ( err_flag.ne.0 ) goto 20
*
              rap = -0.5*log((phc(4)-phc(3))/(phc(4)+phc(3)))
*
*     Cuts as mentioned in the analyses
               if(ndnull.eq.1) then
      if((rap.ge.-1.5.and.rap.le.1.5).and.(pt.ge.2.5.and.pt.le.10.0).
     + and.(q2.ge.2.and.q2.le.100.)) then
c                 write(6,*) 'FILLED'
                  if(lbtag) then
                     call hfill(559,pt,0.,4.3)
                     call hfill(449,rap,0.,4.3)
                     call hfill(339,q2,0.,4.3)
                  endif
                   call hfill(669,pt,0.,xw)
                   call hfill(779,rap,0.,xw)
                   call hfill(889,q2,0.,xw)
               endif
            endif
*
         endif
*
c      endif
*
 10   continue
*
c      if(.not.lctag) then
c        call H1PRNT('GTR ',0)
c        print*,'lcflag false'
c      endif
*
 20   continue
*
*********************************************************************
*
*                      Termination
*
*********************************************************************
*
      Else if(iflag.eq.3) then
*
*      Termination: The following MUST always be done
*      (i) Move to the correct PAW subdirectory
*
         call hcdir('//PAWC/'//xxxx,' ')
*
         lum=999999999.
c        Xsec=PARHFL(6)
*
* x section in nb
         sigma_tot=Xsec
         if (sigma_tot.ne.0.) then
            lumnb=real((Ntot))/real(sigma_tot)
c            lumpb=real((Ntot))/real(Xsec)
         else
            write(6,*) 'hz',xxxx,' sigma_tot=0 ! '
         endif
         call hzhinrm(336, 0, lumnb, 1)
         call hzhinrm(446, 0, lumnb, 1)
         call hzhinrm(556, 0, lumnb, 1)
         call hzhinrm(666, 0, lumnb, 1)
         call hzhinrm(667, 0, lumnb, 1)
         call hzhinrm(337, 0, lumnb, 1)
         call hzhinrm(447, 0, lumnb, 1)
         call hzhinrm(557, 0, lumnb, 1)
         call hzhinrm(668, 0, lumnb, 1)
         call hzhinrm(338, 0, lumnb, 1)
         call hzhinrm(448, 0, lumnb, 1)
         call hzhinrm(558, 0, lumnb, 1)
         call hzhinrm(669, 0, lumnb, 1)
         call hzhinrm(339, 0, lumnb, 1)
         call hzhinrm(449, 0, lumnb, 1)
         call hzhinrm(559, 0, lumnb, 1)
         call hzhinrm(776, 0, lumnb, 1)
         call hzhinrm(777, 0, lumnb, 1)
         call hzhinrm(778, 0, lumnb, 1)
         call hzhinrm(779, 0, lumnb, 1)
         call hzhinrm(886, 0, lumnb, 1)
         call hzhinrm(887, 0, lumnb, 1)
         call hzhinrm(888, 0, lumnb, 1)
         call hzhinrm(889, 0, lumnb, 1)
*
*     merge c- and 4.3*b-quark fraction
         call hopera(666,'+',556,666,1,1)
         call hopera(776,'+',446,776,1,1)
         call hopera(886,'+',336,886,1,1)
*
         call hopera(667,'+',557,667,1,1)
         call hopera(777,'+',447,777,1,1)
         call hopera(887,'+',337,887,1,1)
*
         call hopera(668,'+',558,668,1,1)
         call hopera(778,'+',448,778,1,1)
         call hopera(888,'+',338,888,1,1)
*
         call hopera(669,'+',559,669,1,1)
         call hopera(779,'+',449,779,1,1)
         call hopera(889,'+',339,889,1,1)
*
         write(6,*) ' lumnb = ',lumnb
         write(6,*) ' sigma_tot = ',sigma_tot
         write(6,*) ' Xsec = ',Xsec
*
* Obtain total cross-section

c         WRITE(6,*) ' Heavy flavour xsec delivered from Aroma',PARHFL(6)
*
         write(6,*) ' Nr notc = ',nnotc,' Nr Ntot = ',Ntot
	 write(6,*) ' Nr totc = ',nc+nb
         write(6,*) ' Nr charm = ',nc,' Nr beauty = ',nb
         write(6,*) ' Nr D+ = ',ndplee,' D0 = ',ndoee
         write(6,*) ' Nr D*+ = ',ndstee,' Ds+ = ',ndspee
         write(6,*) ' c -> D*+  BR = ',real(ndstee)/max(real(nc),1.)
         write(6,*) ' c -> D+  BR = ',real(ndplee)/max(real(nc),1.)
         write(6,*) ' c -> D0  BR = ',real(ndoee)/max(real(nc),1.)
         write(6,*) ' c -> Ds+  BR = ',real(ndspee)/max(real(nc),1.)
      endif
*
      RETURN
      END
*
