      SUBROUTINE HZH0507089(IFLAG)
*
* corrections by T. Toll(DESY)      
* cut was wrongly done on IPROC instead of xgamma
* 
****************************************************************************
*
*     Purpose: This routine reproduces the data from Fig.3 to 12
*     from hep-ex/0507089 (DESY-05-132), and produces histograms of 
*     simulated data which can be compared to them.
*
*     ZEUS, Inclusive jet cross sections and dijet correlations in
*     D*+-  photoproduction at HERA
*
*     Initial state: 920GeV protons vs 27.5GeV electrons
*
*     Reference: hep-ex/0507089

*     Arguments: iflag=10000*[A]+1000*[B]+100*[C]+10*[D]+1 initialisation
*                iflag=10000*[A]+1000*[B]+100*[C]+10*[D]+2 filling
*                iflag=10000*[A]+1000*[B]+100*[C]+10*[D]+3 termination
*
*     [A]= IPS   =0 to run on final state particles. 
*     [B]= IPROC =0 for direct and resolved processes together
*                =1 for direct component
*                =2 for resolved component
*                The routine should be run twice, once with iproc = 1 
*                and once with iproc = 2, to produce all histograms
*     [C]= IRUN  =0 rerun jetfinder
*     [D]= JETF  =jetfinder number but the default jetfinder is KTCLUS
*
*     Histograms:
*
*     Inclusive cross sections:
*     ID:
*        -11, -12, -13, -14, -15          data for Fig.3
*        -16, -17                         data for Fig.4
*        -21, -22, -23                    data for Fig.5
*        -24, -25, -26, -27, -28, -29     data for Fig.6
*        -18, -19, -20, -30, -31, -32     data for Fig.7
*        -33                              data for Fig.8
*
*     Dijet Cross sections:
*     ID:       
*        -34, -35, -38, -41               data for Fig.9
*        -42, -43                         data for Fig.10
*        -36, -37                         data for Fig.11
*        -39, -40                         data for Fig.12
*
*     Note: the systematic and statistical errors are added in quadrature 
*           using the average of the positive and negative systematic errors
*
*
*     Positive histogram IDs are the monte carlo simulations of 
*     the above histograms
*
*     written by: Sarah Boutle on 16/02/06
****************************************************************************
      IMPLICIT NONE

#include "hzfunc.inc"
#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzjetcmn.inc"
#include "hzsubjet.inc"


****** declare variables
      INTEGER ihep,iflag,chjet,chtjet,iproc,njet,iloop,i,j,jloop
      INTEGER ip,il,flag,xjet,iloopx,loop
      REAL rap,q2,q2cut,pt,pp,et,rapd,ybj,elecE,mjj,phijj,rapx,etx
      REAL jEt(2), jRap(2), xgamma,xdjet,pjj2,jPtx(2),jPty(2),jPhi(2)
      REAL deltaRx,phid,phix,theta
      DOUBLE PRECISION myjets(maxhzjets,8)
      DOUBLE PRECISION tjets(maxhzjets,8)
      DOUBLE PRECISION coner
      SAVE coner
      CHARACTER*8 xxxx

      SAVE chjet
      DATA chjet/3/
      SAVE chtjet
      DATA chtjet/14/
      DATA q2cut/1./
      REAL ybjcut(2)
      DATA ybjcut / 0.19, 0.87 /
      REAL etjetcut(2)
      DATA etjetcut /6.0,7.0/

      REAL pival

****** Binning for Et(jet) distribution
      REAL etdbins(5)
      DATA etdbins  /  6, 9, 13, 18, 25 /
      REAL etdbins2(4)
      DATA etdbins2  /  6, 9, 13, 18 /
****** Binning for eta(jet) distribution
      REAL rapbins(9)
      DATA rapbins / -1.5, -1.0, -0.5, 0.0, 0.5, 1.0, 1.5, 2.0, 2.4 / 
****** Binning for xgamma distribution
      REAL xbins(7)
      DATA xbins   / 0.000, 0.250, 0.375, 0.500, 0.625, 0.750, 1.000 /
****** Binning for delta(Phi) distribution
      REAL phibins(7)
      DATA phibins / 0.00, 1.57, 2.09, 2.36, 2.62, 2.88, 3.14 /
****** Binning for pt squared distribution
      REAL ptbins(6)
      DATA ptbins  / 0, 9, 36, 81, 144, 324 /
****** Binning for M distribution
      REAL mbins(7)
      DATA mbins   / 15, 20, 25, 30, 35, 40, 50 /

****** Pt(D*) binning
      REAL ptcut(4)
      DATA ptcut  / 3, 5, 8, 20 /
****** Rapidity cuts
      REAL rapcut(5)
      DATA rapcut / -1.5, -0.5, 0.5, 1.5, 2.4 /

****** Initialize data points and errors from plots/tables

c     Et cross section -1.5 < etajet < 2.4
      REAL e1cr(4), e1st(4), e1sysa(4), e1sysb(4)
      REAL e1escala(4), e1escalb(4),e1er(4)
      DATA e1cr     /  1.583,  0.367,  0.0775,  0.0269 /
      DATA e1st     /  0.039,  0.018,  0.0092,  0.0046 /
      DATA e1sysa   /  0.145,  0.035,  0.0149,  0.0027 /
      DATA e1sysb   / -0.063, -0.016, -0.0077, -0.0048 /
      DATA e1escala /  0.032,  0.013,  0.0088,  0.0020 /
      DATA e1escalb / -0.029, -0.014, -0.0061, -0.0024 /

c     Et cross section -1.5 < etajet < -0.5
      REAL e2cr(4), e2st(4), e2sysa(4), e2sysb(4)
      REAL e2escala(4), e2escalb(4),e2er(4)
      DATA e2cr     /  0.321,  0.0408, 0.0088,  0.00   /
      DATA e2st     /  0.017,  0.0062, 0.0033,  0.00   /
      DATA e2sysa   /  0.032,  0.0054, 0.0013,  0.00   /
      DATA e2sysb   / -0.010, -0.0034,-0.0056,  0.00   /
      DATA e2escala /  0.011,  0.0023, 0.0031,  0.00   /
      DATA e2escalb / -0.009, -0.0022,-0.0014,  0.00   /

c     Et cross section -0.5 < etajet < 0.5
      REAL e3cr(4), e3st(4), e3sysa(4), e3sysb(4)
      REAL e3escala(4), e3escalb(4),e3er(4)
      DATA e3cr     /  0.669,  0.159,  0.0315,  0.0089 /
      DATA e3st     /  0.023,  0.011,  0.0050,  0.0025 /
      DATA e3sysa   /  0.062,  0.015,  0.0059,  0.0029 /
      DATA e3sysb   / -0.025, -0.008, -0.0041, -0.0018 /
      DATA e3escala /  0.010,  0.005,  0.0034,  0.0007 /
      DATA e3escalb / -0.008, -0.006, -0.0031, -0.0009 /

c     Et cross section 0.5 < etajet < 1.5
      REAL e4cr(4), e4st(4), e4sysa(4), e4sysb(4)
      REAL e4escala(4), e4escalb(4),e4er(4)
      DATA e4cr     /  0.446,  0.122,  0.0306,  0.0126 /
      DATA e4st     /  0.022,  0.011,  0.0063,  0.0033 /
      DATA e4sysa   /  0.049,  0.015,  0.0120,  0.0007 /
      DATA e4sysb   / -0.017, -0.010, -0.0032, -0.0038 /
      DATA e4escala /  0.007,  0.004,  0.0028,  0.0008 /
      DATA e4escalb / -0.007, -0.003, -0.0014, -0.0014 /

c     Et cross section 1.5 < etajet < 2.4
      REAL e5cr(4), e5st(4), e5sysa(4), e5sysb(4)
      REAL e5escala(4), e5escalb(4),e5er(4)
      DATA e5cr     /  0.151,  0.0462, 0.0095,  0.0053 /
      DATA e5st     /  0.017,  0.0072, 0.0041,  0.0019 /
      DATA e5sysa   /  0.015,  0.0049, 0.0026,  0.0014 /
      DATA e5sysb   / -0.025, -0.0058,-0.0045, -0.0010 /
      DATA e5escala /  0.005,  0.0032, 0.0019,  0.0005 /
      DATA e5escalb / -0.007, -0.0037,-0.0011, -0.0000 /

c     Et cross section -1.5 < etajet < 2.4 (D* tagged jets)
      REAL e6cr(4), e6st(4), e6sysa(4), e6sysb(4)
      REAL e6escala(4), e6escalb(4),e6er(4)
      DATA e6cr     /  0.889,  0.208,  0.0374,  0.0145 /
      DATA e6st     /  0.028,  0.012,  0.0059,  0.0034 /
      DATA e6sysa   /  0.135,  0.032,  0.0066,  0.0016 /
      DATA e6sysb   / -0.032, -0.012, -0.0060, -0.0046 /
      DATA e6escala /  0.010,  0.006,  0.0036,  0.0013 /
      DATA e6escalb / -0.006, -0.005, -0.0020, -0.0021 /

c     Et cross section -1.5 < etajet < 2.4 (untagged jets)
      REAL e7cr(4), e7st(4), e7sysa(4), e7sysb(4)
      REAL e7escala(4), e7escalb(4),e7er(4)
      DATA e7cr     /  0.704,  0.161,  0.0423,  0.0124 /
      DATA e7st     /  0.028,  0.014,  0.0076,  0.0030 /
      DATA e7sysa   /  0.029,  0.004,  0.0109,  0.0039 /
      DATA e7sysb   / -0.037, -0.006, -0.0043, -0.0021 /
      DATA e7escala /  0.027,  0.009,  0.0062,  0.0007 /
      DATA e7escalb / -0.027, -0.010, -0.0050, -0.0002 /

c     Et cross section 3 < pt < 5 GeV
      REAL e8cr(4), e8st(4), e8sysa(4), e8sysb(4)
      REAL e8escala(4), e8escalb(4),e8er(4)
      DATA e8cr     /  1.072,  0.169,  0.030,   0.0112 /
      DATA e8st     /  0.038,  0.018,  0.011,   0.0056 /
      DATA e8sysa   /  0.159,  0.019,  0.012,   0.0089 /
      DATA e8sysb   / -0.060, -0.018, -0.010,  -0.0051 /
      DATA e8escala /  0.044,  0.014,  0.004,   0.0000 /
      DATA e8escalb / -0.039, -0.012, -0.005,  -0.0024 /

c     Et cross section 5 < pt < 8 GeV
      REAL e9cr(4), e9st(4), e9sysa(4), e9sysb(4)
      REAL e9escala(4), e9escalb(4),e9er(4)
      DATA e9cr     /  0.510,  0.1432, 0.0200,  0.0093 /
      DATA e9st     /  0.016,  0.0091, 0.0040,  0.0034 /
      DATA e9sysa   /  0.041,  0.0115, 0.0055,  0.0007 /
      DATA e9sysb   / -0.036, -0.0056,-0.0021, -0.0036 /
      DATA e9escala /  0.001,  0.0074, 0.0026,  0.0022 /
      DATA e9escalb / -0.000, -0.0060,-0.0016, -0.0024 /

c     Et cross section 8 < pt < 20 GeV
      REAL e10cr(4), e10st(4), e10sysa(4), e10sysb(4)
      REAL e10escala(4), e10escalb(4),e10er(4)
      DATA e10cr    /  0.0377, 0.0731, 0.0338,  0.0092 /
      DATA e10st    /  0.0036, 0.0050, 0.0036,  0.0019 /
      DATA e10sysa  /  0.0020, 0.0015, 0.0013,  0.0012 /
      DATA e10sysb  / -0.0069,-0.0068,-0.0023, -0.0017 /
      DATA e10escala/  0.0009, 0.0002, 0.0033,  0.0010 /
      DATA e10escalb/ -0.0005,-0.0017,-0.0022, -0.0004 /

c     eta cross section Etjet < 6 GeV
      REAL eta1cr(8), eta1st(8), eta1sysa(8), eta1sysb(8)
      REAL eta1escala(8), eta1escalb(8), eta1er(8)
      DATA eta1cr     /  0.548,  1.745,  2.79,   2.90, 
     +                   2.30,   1.85,   0.871,  0.74  /
      DATA eta1st     /  0.069,  0.094,  0.12,   0.12,
     +                   0.12,   0.13,   0.093,  0.11  /
      DATA eta1sysa   /  0.088,  0.210,  0.25,   0.29,
     +                   0.29,   0.16,   0.087,  0.06  /
      DATA eta1sysb   / -0.154, -0.063, -0.16,  -0.12,
     +                  -0.08,  -0.11,  -0.086, -0.15  /
      DATA eta1escala /  0.040,  0.054,  0.07,   0.06,
     +                   0.05,   0.06,   0.045,  0.05  /
      DATA eta1escalb / -0.032, -0.046, -0.07,  -0.06,
     +                  -0.05,  -0.05,  -0.051, -0.04  /

c     eta cross section 6 < Etjet < 9 GeV
      REAL eta2cr(8), eta2st(8), eta2sysa(8), eta2sysb(8)
      REAL eta2escala(8), eta2escalb(8), eta2er(8)
      DATA eta2cr     /  0.546,  1.374,  2.067,  1.944, 
     +                   1.429,  1.30,   0.542,  0.463 /
      DATA eta2st     /  0.066,  0.081,  0.097,  0.096,
     +                   0.093,  0.10,   0.075,  0.089 /
      DATA eta2sysa   /  0.069,  0.166,  0.196,  0.211,
     +                   0.186,  0.12,   0.071,  0.049 /
      DATA eta2sysb   / -0.110, -0.053, -0.136, -0.083, 
     +                  -0.053, -0.09,  -0.070, -0.122 /
      DATA eta2escala /  0.038,  0.034,  0.043,  0.021,
     +                   0.019,  0.03,   0.013,  0.032 /
      DATA eta2escalb / -0.027, -0.030, -0.033, -0.018,
     +                  -0.018, -0.03,  -0.030, -0.008 /

c     eta cross section Etjet > 9 GeV
      REAL eta3cr(8), eta3st(8), eta3sysa(8), eta3sysb(8)
      REAL eta3escala(8), eta3escalb(8), eta3er(8)
      DATA eta3cr     /  0.000,  0.377,  0.727,  0.950, 
     +                   0.872,  0.559,  0.326,  0.276 /
      DATA eta3st     /  0.000,  0.051,  0.070,  0.074,
     +                   0.080,  0.079,  0.056,  0.063 /
      DATA eta3sysa   /  0.000,  0.043,  0.073,  0.116,
     +                   0.132,  0.080,  0.046,  0.036 /
      DATA eta3sysb   /  0.000, -0.050, -0.057, -0.042,
     +                  -0.058, -0.071, -0.043, -0.066 /
      DATA eta3escala /  0.000,  0.024,  0.034,  0.041,
     +                   0.037,  0.028,  0.034,  0.020 /
      DATA eta3escalb /  0.000, -0.018, -0.034, -0.047,
     +                  -0.030, -0.019, -0.021, -0.030 /

c     eta cross section Etjet < 6 GeV (D* tagged jets)
      REAL eta4cr(8), eta4st(8), eta4sysa(8), eta4sysb(8)
      REAL eta4escala(8), eta4escalb(8), eta4er(8)
      DATA eta4cr     /  0.412,  1.170,  1.831,  1.78, 
     +                   1.18,   0.99,   0.19,   0.00   /
      DATA eta4st     /  0.055,  0.075,  0.097,  0.11,
     +                   0.11,   0.13,   0.17,   0.00   /
      DATA eta4sysa   /  0.061,  0.236,  0.323,  0.34,
     +                   0.30,   0.24,   0.27,   0.00   /
      DATA eta4sysb   / -0.042, -0.070, -0.120, -0.09,
     +                  -0.07,  -0.09,  -0.13,   0.00   /
      DATA eta4escala /  0.023,  0.019,  0.027,  0.01,
     +                   0.01,   0.01,   0.00,   0.00   /
      DATA eta4escalb / -0.012, -0.013, -0.013, -0.01,
     +                  -0.01,  -0.00,  -0.01,   0.00   /

c     eta cross section 6 < Etjet < 9 GeV (D* tagged jets)
      REAL eta5cr(8), eta5st(8), eta5sysa(8), eta5sysb(8)
      REAL eta5escala(8), eta5escalb(8), eta5er(8)
      DATA eta5cr     /  0.386,  0.935,  1.367,  1.224, 
     +                   0.771,  0.651,  0.113,  0.000  /
      DATA eta5st     /  0.052,  0.060,  0.073,  0.074,
     +                   0.069,  0.086,  0.100,  0.000  /
      DATA eta5sysa   /  0.052,  0.157,  0.185,  0.180,
     +                   0.162,  0.132,  0.190,  0.000  /
      DATA eta5sysb   / -0.040, -0.044, -0.056, -0.054, 
     +                  -0.049, -0.062, -0.081,  0.000  /
      DATA eta5escala /  0.022,  0.015,  0.020,  0.007,
     +                   0.004,  0.005,  0.000,  0.000  /
      DATA eta5escalb / -0.011, -0.010, -0.009, -0.005,
     +                  -0.006, -0.001, -0.005,  0.000  /

c     eta cross section Etjet > 9 GeV (D* tagged jets)
      REAL eta6cr(8), eta6st(8), eta6sysa(8), eta6sysb(8)
      REAL eta6escala(8), eta6escalb(8), eta6er(8)
      DATA eta6cr     /  0.000,  0.239,  0.514,  0.620, 
     +                   0.520,  0.231,  0.140,  0.000  /
      DATA eta6st     /  0.000,  0.036,  0.050,  0.055,
     +                   0.056,  0.057,  0.080,  0.000  /
      DATA eta6sysa   /  0.000,  0.044,  0.063,  0.093,
     +                   0.100,  0.043,  0.047,  0.000  /
      DATA eta6sysb   /  0.000, -0.023, -0.054, -0.044,
     +                  -0.032, -0.042, -0.076,  0.000  /
      DATA eta6escala /  0.000,  0.012,  0.019,  0.023,
     +                   0.019,  0.010,  0.022,  0.000  /
      DATA eta6escalb /  0.000, -0.009, -0.018, -0.025,
     +                  -0.010, -0.006, -0.005,  0.000  /

c     eta cross section Etjet < 6 GeV (untagged jets)
      REAL eta7cr(8), eta7st(8), eta7sysa(8), eta7sysb(8)
      REAL eta7escala(8), eta7escalb(8), eta7er(8)
      DATA eta7cr     /  0.173,  0.560,  1.010,  1.113, 
     +                   1.08,   1.014,  0.75,   0.70  /
      DATA eta7st     /  0.043,  0.075,  0.094,  0.095,
     +                   0.10,   0.098,  0.10,   0.13  /
      DATA eta7sysa   /  0.055,  0.034,  0.070,  0.094,
     +                   0.10,   0.089,  0.10,   0.07  /
      DATA eta7sysb   / -0.097, -0.030, -0.200, -0.064,
     +                  -0.07,  -0.113, -0.10,  -0.18  /
      DATA eta7escala /  0.017,  0.030,  0.038,  0.024,
     +                   0.03,   0.043,  0.02,   0.05  /
      DATA eta7escalb / -0.014, -0.028, -0.040, -0.023,
     +                  -0.02,  -0.052, -0.04,  -0.01  /

c     eta cross section 6 < Etjet < 9 GeV (untagged jets)
      REAL eta8cr(8), eta8st(8), eta8sysa(8), eta8sysb(8)
      REAL eta8escala(8), eta8escalb(8), eta8er(8)
      DATA eta8cr     /  0.160,  0.438,  0.714,  0.725, 
     +                   0.665,  0.623,  0.473,  0.466 /
      DATA eta8st     /  0.040,  0.058,  0.067,  0.062,
     +                   0.063,  0.061,  0.066,  0.089 /
      DATA eta8sysa   /  0.052,  0.035,  0.048,  0.061,
     +                   0.060,  0.053,  0.064,  0.047 /
      DATA eta8sysb   / -0.089, -0.023, -0.132, -0.042, 
     +                  -0.039, -0.070, -0.070, -0.122 /
      DATA eta8escala /  0.016,  0.023,  0.027,  0.016,
     +                   0.021,  0.026,  0.012,  0.032 /
      DATA eta8escalb / -0.013, -0.022, -0.028, -0.015,
     +                  -0.015, -0.032, -0.026, -0.008 /

c     eta cross section Etjet > 9 GeV (untagged jets)
      REAL eta9cr(8), eta9st(8), eta9sysa(8), eta9sysb(8)
      REAL eta9escala(8), eta9escalb(8), eta9er(8)
      DATA eta9cr     /  0.000,  0.148,  0.194,  0.328, 
     +                   0.358,  0.335,  0.263,  0.275 /
      DATA eta9st     /  0.000,  0.043,  0.055,  0.051,
     +                   0.063,  0.055,  0.048,  0.062 /
      DATA eta9sysa   /  0.000,  0.020,  0.053,  0.039,
     +                   0.039,  0.065,  0.043,  0.036 /
      DATA eta9sysb   /  0.000, -0.085, -0.015, -0.016,
     +                  -0.052, -0.039, -0.033, -0.065 /
      DATA eta9escala /  0.000,  0.019,  0.014,  0.019,
     +                   0.020,  0.020,  0.027,  0.020 /
      DATA eta9escalb /  0.000, -0.011, -0.014, -0.023,
     +                  -0.025, -0.015, -0.017, -0.030 /

c     eta cross section 3 < pt < 5 GeV
      REAL eta10cr(8), eta10st(8), eta10sysa(8), eta10sysb(8)
      REAL eta10escala(8), eta10escalb(8), eta10er(8)
      DATA eta10cr    /  0.350,  1.087,  1.64,   1.65, 
     +                   1.32,   1.20,   0.553,  0.43  /
      DATA eta10st    /  0.065,  0.088,  0.11,   0.11,
     +                   0.12,   0.13,   0.093,  0.11  /
      DATA eta10sysa  /  0.088,  0.294,  0.19,   0.28,
     +                   0.26,   0.20,   0.079,  0.06  /
      DATA eta10sysb  / -0.136, -0.117, -0.13,  -0.15,
     +                  -0.09,  -0.22,  -0.099, -0.17  /
      DATA eta10escala/  0.036,  0.061,  0.07,   0.05,
     +                   0.06,   0.05,   0.036,  0.03  /
      DATA eta10escalb/ -0.029, -0.051, -0.06,  -0.06,
     +                  -0.05,  -0.05,  -0.040, -0.03  /

c     eta cross section 5 < pt < 8 GeV
      REAL eta11cr(8), eta11st(8), eta11sysa(8), eta11sysb(8)
      REAL eta11escala(8), eta11escalb(8), eta11er(8)
      DATA eta11cr    /  0.211,  0.602,  0.981,  1.015, 
     +                   0.745,  0.534,  0.231,  0.240 /
      DATA eta11st    /  0.033,  0.046,  0.056,  0.057,
     +                   0.051,  0.053,  0.033,  0.045 /
      DATA eta11sysa  /  0.059,  0.041,  0.065,  0.102,
     +                   0.101,  0.046,  0.029,  0.027 /
      DATA eta11sysb  / -0.037, -0.030, -0.048, -0.069,
     +                  -0.035, -0.050, -0.015, -0.045 /
      DATA eta11escala/  0.011,  0.007,  0.015,  0.012,
     +                   0.007,  0.015,  0.013,  0.019 /
      DATA eta11escalb/ -0.004, -0.006, -0.013, -0.013,
     +                  -0.010, -0.011, -0.013, -0.010 /

c     eta cross section 8 < pt < 20 GeV
      REAL eta12cr(8), eta12st(8), eta12sysa(8), eta12sysb(8)
      REAL eta12escala(8), eta12escalb(8), eta12er(8)
      DATA eta12cr    /  0.000,  0.101,  0.217,  0.274, 
     +                   0.296,  0.178,  0.108,  0.092 /
      DATA eta12st    /  0.000,  0.020,  0.024,  0.027,
     +                   0.028,  0.024,  0.021,  0.020 /
      DATA eta12sysa  /  0.000,  0.009,  0.010,  0.014,
     +                   0.015,  0.007,  0.011,  0.014 /
      DATA eta12sysb  /  0.000, -0.021, -0.016, -0.010,
     +                  -0.021, -0.017, -0.020, -0.017 /
      DATA eta12escala/  0.000,  0.001,  0.002,  0.001,
     +                   0.003,  0.003,  0.001,  0.005 /
      DATA eta12escalb/  0.000, -0.002, -0.001, -0.000,
     +                  -0.001, -0.003, -0.004, -0.004 /

c     x(D*, jet) cross section 
      REAL x1cr(6), x1st(6), x1sysa(6), x1sysb(6)
      REAL x1escala(6), x1escalb(6), x1er(6)
      DATA x1cr     /  1.23,  2.15,  2.85,  3.82,  5.71,  2.82 /
      DATA x1st     /  0.24,  0.28,  0.31,  0.30,  0.32,  0.15 /
      DATA x1sysa   /  0.13,  0.38,  0.30,  0.46,  0.22,  0.19 /
      DATA x1sysb   / -0.33, -0.25, -0.22, -0.36, -0.33, -0.23 /
      DATA x1escala /  0.07,  0.10,  0.07,  0.05,  0.00,  0.29 /
      DATA x1escalb / -0.08, -0.11, -0.10, -0.07, -0.06, -0.25 /

c     Dijet x(D*, jj) cross section 
      REAL x2cr(6), x2st(6), x2sysa(6), x2sysb(6)
      REAL x2escala(6), x2escalb(6), x2er(6)
      DATA x2cr     /  0.19,  0.78,  0.99,  1.52,  2.03,  4.35 /
      DATA x2st     /  0.15,  0.22,  0.19,  0.22,  0.24,  0.21 /
      DATA x2sysa   /  0.33,  0.26,  0.20,  0.16,  0.19,  0.37 /
      DATA x2sysb   / -0.06, -0.22, -0.22, -0.29, -0.26, -0.11 /
      DATA x2escala /  0.02,  0.06,  0.08,  0.09,  0.12,  0.16 /
      DATA x2escalb / -0.01, -0.06, -0.07, -0.10, -0.12, -0.16 /

c     Delta Phi(jj) cross section 0 < x < 1 
      REAL ph1cr(6), ph1st(6), ph1sysa(6), ph1sysb(6)
      REAL ph1escala(6), ph1escalb(6), ph1er(6)
      DATA ph1cr     /  0.020,  0.149,  0.318,  0.67,   1.50,   4.06   /
      DATA ph1st     /  0.011,  0.035,  0.074,  0.10,   0.14,   0.22   /
      DATA ph1sysa   /  0.012,  0.046,  0.064,  0.15,   0.17,   0.31   /
      DATA ph1sysb   / -0.013, -0.046, -0.089, -0.007, -0.11,  -0.29   /
      DATA ph1escala /  0.002,  0.017,  0.015,  0.05,   0.07,   0.16   /
      DATA ph1escalb / -0.002, -0.013, -0.022, -0.03,  -0.08,  -0.17   /

c     Delta Phi(jj) cross section x > 0.75
      REAL ph2cr(6), ph2st(6), ph2sysa(6), ph2sysb(6)
      REAL ph2escala(6), ph2escalb(6), ph2er(6)
      DATA ph2cr     /  0.0158,  0.036,  0.126,  0.389,  0.829,  2.68  /
      DATA ph2st     /  0.0068,  0.014,  0.038,  0.055,  0.091,  0.16  /
      DATA ph2sysa   /  0.0058,  0.004,  0.033,  0.076,  0.099,  0.21  /
      DATA ph2sysb   / -0.0103, -0.026, -0.027, -0.016, -0.059, -0.11  /
      DATA ph2escala /  0.0008,  0.005,  0.007,  0.020,  0.031,  0.08  /
      DATA ph2escalb / -0.0009, -0.002, -0.007, -0.017, -0.033, -0.09  /

c     Delta Phi(jj) cross section x < 0.75
      REAL ph3cr(6), ph3st(6), ph3sysa(6), ph3sysb(6)
      REAL ph3escala(6), ph3escalb(6), ph3er(6)
      DATA ph3cr     /  0.0066,  0.115,  0.190,  0.280,  0.67,   1.37  /
      DATA ph3st     /  0.0093,  0.034,  0.062,  0.083,  0.11,   0.15  /
      DATA ph3sysa   /  0.0124,  0.047,  0.044,  0.121,  0.11,   0.12  /
      DATA ph3sysb   / -0.0083, -0.035, -0.075, -0.080, -0.10,  -0.25  /
      DATA ph3escala /  0.0006,  0.011,  0.008,  0.025,  0.05,   0.09  /
      DATA ph3escalb / -0.0008, -0.011, -0.016, -0.011, -0.05,  -0.08  /

c     Pt(jj) squared cross section 0 < x < 1 
      REAL pt1cr(5), pt1st(5), pt1sysa(5), pt1sysb(5) 
      REAL pt1escala(5), pt1escalb(5), pt1er(5)
      DATA pt1cr     /  0.0941,  0.0209,  0.00508,  0.00167,  0.00036  /
      DATA pt1st     /  0.0059,  0.0016,  0.00063,  0.00038,  0.00012  /
      DATA pt1sysa   /  0.0080,  0.0021,  0.00075,  0.00028,  0.00013  /
      DATA pt1sysb   / -0.0054, -0.0017, -0.00041, -0.00059, -0.00012  /
      DATA pt1escala /  0.0026,  0.0011,  0.00029,  0.00027,  0.00003  /
      DATA pt1escalb / -0.0032, -0.0008, -0.00044, -0.00013, -0.00003  /

c     Pt(jj) squared cross section x > 0.75
      REAL pt2cr(5), pt2st(5), pt2sysa(5), pt2sysb(5)
      REAL pt2escala(5), pt2escalb(5), pt2er(5)
      DATA pt2cr     /  0.0677,  0.0119,  0.00200,  0.00067,  0.000220 /
      DATA pt2st     /  0.0044,  0.0010,  0.00032,  0.00021,  0.000062 /
      DATA pt2sysa   /  0.0062,  0.0013,  0.00037,  0.00012,  0.000026 /
      DATA pt2sysb   / -0.0047, -0.0010, -0.00033, -0.00027, -0.000123 /
      DATA pt2escala /  0.0008,  0.0005,  0.00012,  0.00012,  0.000020 /
      DATA pt2escalb / -0.0020, -0.0003, -0.00015, -0.00005, -0.000009 /

c     Pt(jj) squared cross section x < 0.75
      REAL pt3cr(5), pt3st(5), pt3sysa(5), pt3sysb(5)
      REAL pt3escala(5), pt3escalb(5), pt3er(5)
      DATA pt3cr     /  0.0262,  0.0091,  0.00320,  0.00098,  0.00015  /
      DATA pt3st     /  0.0038,  0.0012,  0.00058,  0.00030,  0.00010  /
      DATA pt3sysa   /  0.0029,  0.0010,  0.00066,  0.00024,  0.00016  /
      DATA pt3sysb   / -0.0047, -0.0013, -0.00048, -0.00042, -0.00006  /
      DATA pt3escala /  0.0017,  0.0006,  0.00017,  0.00015,  0.00001  /
      DATA pt3escalb / -0.0011, -0.0005, -0.00032, -0.00008, -0.00002  /

c     M(jj) cross section 0 < x < 1 
      REAL m1cr(6), m1st(6), m1sysa(6), m1sysb(6)
      REAL m1escala(6), m1escalb(6), m1er(6)
      DATA m1cr     /  0.1299, 0.0823, 0.0484, 0.0328, 0.0223, 0.0081 /
      DATA m1st     /  0.0088, 0.0076, 0.0062, 0.0056, 0.0048, 0.0022 /
      DATA m1sysa   /  0.0095, 0.0085, 0.0087, 0.0047, 0.0018, 0.0013 /
      DATA m1sysb   / -0.0120,-0.0076,-0.0056,-0.0074,-0.0047,-0.0018 /
      DATA m1escala /  0.0052, 0.0043, 0.0036, 0.0021, 0.0023, 0.0006 /
      DATA m1escalb / -0.0046,-0.0050,-0.0017,-0.0037, 0.0000,-0.0008 /

c     M(jj) squared cross section x > 0.75
      REAL m2cr(6), m2st(6), m2sysa(6), m2sysb(6)
      REAL m2escala(6), m2escalb(6), m2er(6)
      DATA m2cr     /  0.0788, 0.0470, 0.0319, 0.0221, 0.0136, 0.0040 /
      DATA m2st     /  0.0056, 0.0049, 0.0045, 0.0045, 0.0035, 0.0016 /
      DATA m2sysa   /  0.0058, 0.0058, 0.0036, 0.0041, 0.0016, 0.0013 /
      DATA m2sysb   / -0.0063,-0.0024,-0.0029,-0.0036,-0.0042,-0.0010 /
      DATA m2escala /  0.0012, 0.0029, 0.0018, 0.0012, 0.0011, 0.0003 /
      DATA m2escalb / -0.0015,-0.0022,-0.0007,-0.0032, 0.0000,-0.0004 /

c     M(jj) squared cross section x < 0.75
      REAL m3cr(6), m3st(6), m3sysa(6), m3sysb(6)
      REAL m3escala(6), m3escalb(6), m3er(6)
      DATA m3cr     /  0.0512, 0.0357, 0.0166, 0.0109, 0.0086, 0.0038 /
      DATA m3st     /  0.0069, 0.0060, 0.0041, 0.0033, 0.0033, 0.0015 /
      DATA m3sysa   /  0.0054, 0.0038, 0.0064, 0.0023, 0.0021, 0.0009 /
      DATA m3sysb   / -0.0077,-0.0082,-0.0036,-0.0043,-0.0015,-0.0015 /
      DATA m3escala /  0.0046, 0.0012, 0.0017, 0.0009, 0.0012, 0.0003 /
      DATA m3escalb / -0.0033,-0.0031,-0.0008,-0.0006, 0.0000,-0.0005 /

****** Set xxxx equal to the arXiv:hep-ex number.
      DATA xxxx /'0507089'/

****** Determine the process type (single/double resolved etc)
****** (only necessary if > 1 process types!)

      iproc=(mod(iflag,10000)/1000)
      IF ((iproc.LT.0).OR.(iproc.GT.2)) THEN
         WRITE(HZUNIT,*) xxxx,': illegal IPROC:',iproc
         RETURN
      ENDIF

****** choose jet finder
      IF ((MOD(iflag,100)/10.GT.0).AND.MOD(iflag,10).eq.1) THEN
         chjet=INT(MOD(iflag,100)/10)
      ENDIF

****** Decide if we are running on parton showers or final state particles.
      IF (MOD(iflag,100000)/10000.GT.0) THEN

****** use a different paw directory for parton showers.
         xxxx = 'PS'//xxxx
****** tell jet finder that we running on parton showers 
         chjet=-1*ABS(chjet)
      ENDIF

******************************************************************************
********************************  Initialise ********************************* 
      IF (MOD(iflag,10).EQ.1) THEN
****** Initialisation: The following MUST always be done
******     (i) make subdirectory in PAWC
******     - use the name as the xxxxxx in HZHxxxxxx subroutine
******     (i) make subdirectory in o/p file

         IF (iproc.LE.1) THEN

****** Combine the stat.and sys. errors in quadrature

            DO loop = 1, 4
               e1er(loop) = sqrt((e1st(loop)**2)+
     +              (((e1sysa(loop)-e1sysb(loop))/2)**2))
               e2er(loop) = sqrt((e2st(loop)**2)+
     +              (((e2sysa(loop)-e2sysb(loop))/2)**2))
               e3er(loop) = sqrt((e3st(loop)**2)+
     +              (((e3sysa(loop)-e3sysb(loop))/2)**2))
               e4er(loop) = sqrt((e4st(loop)**2)+
     +              (((e4sysa(loop)-e4sysb(loop))/2)**2))
               e5er(loop) = sqrt((e5st(loop)**2)+
     +              (((e5sysa(loop)-e5sysb(loop))/2)**2))
               e6er(loop) = sqrt((e6st(loop)**2)+
     +              (((e6sysa(loop)-e6sysb(loop))/2)**2))
               e7er(loop) = sqrt((e7st(loop)**2)+
     +              (((e7sysa(loop)-e7sysb(loop))/2)**2))
               e8er(loop) = sqrt((e8st(loop)**2)+
     +              (((e8sysa(loop)-e8sysb(loop))/2)**2))
               e9er(loop) = sqrt((e9st(loop)**2)+
     +              (((e9sysa(loop)-e9sysb(loop))/2)**2))
               e10er(loop) = sqrt((e10st(loop)**2)+
     +              (((e10sysa(loop)-e10sysb(loop))/2)**2))
            ENDDO

            DO loop = 1, 8
               eta1er(loop) = sqrt((eta1st(loop)**2)+
     +              (((eta1sysa(loop)-eta1sysb(loop))/2)**2))
               eta2er(loop) = sqrt((eta2st(loop)**2)+
     +              (((eta2sysa(loop)-eta2sysb(loop))/2)**2))
               eta3er(loop) = sqrt((eta3st(loop)**2)+
     +              (((eta3sysa(loop)-eta3sysb(loop))/2)**2))
               eta4er(loop) = sqrt((eta4st(loop)**2)+
     +              (((eta4sysa(loop)-eta4sysb(loop))/2)**2))
               eta5er(loop) = sqrt((eta5st(loop)**2)+
     +              (((eta5sysa(loop)-eta5sysb(loop))/2)**2))
               eta6er(loop) = sqrt((eta6st(loop)**2)+
     +              (((eta6sysa(loop)-eta6sysb(loop))/2)**2))
               eta7er(loop) = sqrt((eta7st(loop)**2)+
     +              (((eta7sysa(loop)-eta7sysb(loop))/2)**2))
               eta8er(loop) = sqrt((eta8st(loop)**2)+
     +              (((eta8sysa(loop)-eta8sysb(loop))/2)**2))
               eta9er(loop) = sqrt((eta9st(loop)**2)+
     +              (((eta9sysa(loop)-eta9sysb(loop))/2)**2))
               eta10er(loop) = sqrt((eta10st(loop)**2)+
     +              (((eta10sysa(loop)-eta10sysb(loop))/2)**2))
               eta11er(loop) = sqrt((eta11st(loop)**2)+
     +              (((eta11sysa(loop)-eta11sysb(loop))/2)**2))
               eta12er(loop) = sqrt((eta12st(loop)**2)+
     +              (((eta12sysa(loop)-eta12sysb(loop))/2)**2))
            ENDDO

            DO loop = 1, 6
               x1er(loop) = sqrt((x1st(loop)**2)+
     +              (((x1sysa(loop)-x1sysb(loop))/2)**2))
               x2er(loop) = sqrt((x2st(loop)**2)+
     +              (((x2sysa(loop)-x2sysb(loop))/2)**2))
               ph1er(loop) = sqrt((ph1st(loop)**2)+
     +              (((ph1sysa(loop)-ph1sysb(loop))/2)**2))
               ph2er(loop) = sqrt((ph2st(loop)**2)+
     +              (((ph2sysa(loop)-ph2sysb(loop))/2)**2))
               ph3er(loop) = sqrt((ph3st(loop)**2)+
     +              (((ph3sysa(loop)-ph3sysb(loop))/2)**2))
               m1er(loop) = sqrt((m1st(loop)**2)+
     +              (((m1sysa(loop)-m1sysb(loop))/2)**2))
               m2er(loop) = sqrt((m2st(loop)**2)+
     +              (((m2sysa(loop)-m2sysb(loop))/2)**2))
               m3er(loop) = sqrt((m3st(loop)**2)+
     +              (((m3sysa(loop)-m3sysb(loop))/2)**2))
            ENDDO

            DO loop = 1, 5
               pt1er(loop) = sqrt((pt1st(loop)**2)+
     +              (((pt1sysa(loop)-pt1sysb(loop))/2)**2))
               pt2er(loop) = sqrt((pt2st(loop)**2)+
     +              (((pt2sysa(loop)-pt2sysb(loop))/2)**2))
               pt3er(loop) = sqrt((pt3st(loop)**2)+
     +              (((pt3sysa(loop)-pt3sysb(loop))/2)**2))               
            ENDDO

            CALL hcdir('//PAWC',' ')
            IF (iproc.LE.1) THEN
               CALL hmdir(xxxx,'S')
            ELSE 
               CALL hcdir(xxxx,' ')
            ENDIF
            
            CALL hcdir('//HISTO',' ')
            IF (iproc.LE.1) THEN
               CALL hmdir(xxxx,'S')
            ELSE 
               CALL hcdir(xxxx,' ')
            ENDIF
            
****** Book histograms -11 to -43

            CALL hbookb(-11,'Data Et(jet) cross sec,-1.5<eta(jet)<2.4'
     +                 ,4,etdbins,0.)
            CALL hbookb(-12,'Data Et(jet) cross sec,-1.5<eta(jet)<-0.5'
     +                 ,4,etdbins,0.)
            CALL hbookb(-13,'Data Et(jet) cross sec, -0.5<eta(jet)<0.5'
     +                 ,4,etdbins,0.)
            CALL hbookb(-14,'Data Et(jet) cross sec, 0.5<eta(jet)<1.5'
     +                 ,4,etdbins,0.)
            CALL hbookb(-15,'Data Et(jet) cross sec, 1.5<eta(jet)<2.4'
     +                 ,4,etdbins,0.)

            CALL hbookb(-16,'Data Et(jet) cross section, D* Tagged Jets'
     +           ,4,etdbins,0.)
            CALL hbookb(-17,'Data Et(jet) cross section, Untagged Jets'
     +           ,4,etdbins,0.)
            
            CALL hbookb(-18,'Data Et(jet) cross section, 3 < pt < 5GeV'
     +           ,4,etdbins,0.)
            CALL hbookb(-19,'Data Et(jet) cross section, 5 < pt < 8GeV'
     +           ,4,etdbins,0.)
            CALL hbookb(-20,'Data Et(jet) cross section, 8 < pt < 20GeV'
     +           ,4,etdbins,0.)
            
            CALL hbookb(-21,'Data Eta(jet) cross section, Et(jet)>6GeV'
     +           ,8,rapbins,0.)
            CALL hbookb(-22,'Data Eta(jet) cross section,
     +            6<Et(jet)<9GeV',8,rapbins,0.)
            CALL hbookb(-23,'Data Eta(jet) cross section, Et(jet)>9GeV'
     +           ,8,rapbins,0.)

            CALL hbookb(-24,'Data Eta(jet) cross section, Et(jet)>6GeV
     +           D* tagged jets' ,8,rapbins,0.)
            CALL hbookb(-25,'Data Eta(jet) cross section, 6<Et(jet)<9GeV
     +           D* tagged jets' ,8,rapbins,0.)
            CALL hbookb(-26,'Data Eta(jet) cross section, Et(jet)>9GeV
     +           D* tagged jets' ,8,rapbins,0.)
            CALL hbookb(-27,'Data Eta(jet) cross section, Et(jet)>6GeV
     +           Untagged jets' ,8,rapbins,0.)
            CALL hbookb(-28,'Data Eta(jet) cross section, 6<Et(jet)<9GeV
     +           Untagged jets' ,8,rapbins,0.)
            CALL hbookb(-29,'Data Eta(jet) cross section, Et(jet)>9GeV
     +           Untagged jets' ,8,rapbins,0.)
            
            CALL hbookb(-30,'Data Eta(jet) cross section, 3 < pt < 5GeV'
     +           ,8,rapbins,0.)
            CALL hbookb(-31,'Data Eta(jet) cross section, 5 < pt < 8GeV'
     +           ,8,rapbins,0.)
            CALL hbookb(-32,'Data Eta(jet) cross section,
     +            8 < pt < 20GeV',8,rapbins,0.)

            CALL hbookb(-33,'Data x(gamma)(D*,jet) cross section',
     +            6,xbins,0.)

            CALL hbookb(-34,'Data dijet x(gamma) cross section',
     +            6,xbins,0.)

            CALL hbookb(-35,'Data dijet deltaphi 0<x<1 cross section',
     +            6,phibins,0.)
            CALL hbookb(-36,'Data dijet deltaphi x>0.75 cross section',
     +            6,phibins,0.)
            CALL hbookb(-37,'Data dijet deltaphi x<0.75 cross section',
     +            6,phibins,0.)

            CALL hbookb(-38,'Data dijet pt^2 0<x<1 cross section',
     +            5,ptbins,0.)
            CALL hbookb(-39,'Data dijet pt^2 x>0.75 cross section',
     +            5,ptbins,0.)
            CALL hbookb(-40,'Data dijet pt^2 x<0.75 cross section',
     +            5,ptbins,0.)

            CALL hbookb(-41,'Data dijet Mjj 0<x<1 cross section',
     +            6,mbins,0.)
            CALL hbookb(-42,'Data dijet Mjj x>0.75 cross section',
     +            6,mbins,0.)
            CALL hbookb(-43,'Data dijet Mjj x<0.75 cross section',
     +            6,mbins,0.)
            
****** Fill histograms -11 to -43 with data and errors

            CALL hpak(-11, e1cr)
            CALL hpak(-12, e2cr)
            CALL hpak(-13, e3cr)
            CALL hpak(-14, e4cr)
            CALL hpak(-15, e5cr)
            CALL hpak(-16, e6cr)
            CALL hpak(-17, e7cr)
            CALL hpak(-18, e8cr)
            CALL hpak(-19, e9cr)
            CALL hpak(-20, e10cr)
        
            CALL hpak(-21, eta1cr)
            CALL hpak(-22, eta2cr)
            CALL hpak(-23, eta3cr)
            CALL hpak(-24, eta4cr)
            CALL hpak(-25, eta5cr)
            CALL hpak(-26, eta6cr)
            CALL hpak(-27, eta7cr)
            CALL hpak(-28, eta8cr)
            CALL hpak(-29, eta9cr)
            CALL hpak(-30, eta10cr)
            CALL hpak(-31, eta11cr)
            CALL hpak(-32, eta12cr)

            CALL hpak(-33, x1cr)
            CALL hpak(-34, x2cr)
            
            CALL hpak(-35, ph1cr)
            CALL hpak(-36, ph2cr)
            CALL hpak(-37, ph3cr)

            CALL hpak(-38, pt1cr)
            CALL hpak(-39, pt2cr)
            CALL hpak(-40, pt3cr)

            CALL hpak(-41, m1cr)
            CALL hpak(-42, m2cr)
            CALL hpak(-43, m3cr)

            CALL hpake(-11, e1er)
            CALL hpake(-12, e2er)
            CALL hpake(-13, e3er)
            CALL hpake(-14, e4er)
            CALL hpake(-15, e5er)
            CALL hpake(-16, e6er)
            CALL hpake(-17, e7er)
            CALL hpake(-18, e8er)
            CALL hpake(-19, e9er)
            CALL hpake(-20, e10er)
            
            CALL hpake(-21, eta1er)
            CALL hpake(-22, eta2er)
            CALL hpake(-23, eta3er)
            CALL hpake(-24, eta4er)
            CALL hpake(-25, eta5er)
            CALL hpake(-26, eta6er)
            CALL hpake(-27, eta7er)
            CALL hpake(-28, eta8er)
            CALL hpake(-29, eta9er)
            CALL hpake(-30, eta10er)
            CALL hpake(-31, eta11er)
            CALL hpake(-32, eta12er)

            CALL hpake(-33, x1er)
            CALL hpake(-34, x2er)

            CALL hpake(-35, ph1er)
            CALL hpake(-36, ph2er)
            CALL hpake(-37, ph3er)

            CALL hpake(-38, pt1er)
            CALL hpake(-39, pt2er)
            CALL hpake(-40, pt3er)

            CALL hpake(-41, m1er)
            CALL hpake(-42, m2er)
            CALL hpake(-43, m3er)

         ENDIF
******  Book histograms 1011 to 1043 or 2011 to 2043
         CALL hbookb(1000*iproc+11,'Et cross sec(MC),-1.5<eta(jet)<2.4'
     +        ,4,etdbins,0.)
         CALL hbookb(1000*iproc+12,'Et cross sec(MC),-1.5<eta(jet)<-0.5'
     +        ,3,etdbins2,0.)
         CALL hbookb(1000*iproc+13,'Et cross sec(MC),-0.5<eta(jet)<0.5'
     +        ,4,etdbins,0.)
         CALL hbookb(1000*iproc+14,'Et cross sec(MC),0.5<eta(jet)<1.5'
     +        ,4,etdbins,0.)
         CALL hbookb(1000*iproc+15,'Et cross sec(MC),1.5<eta(jet)<2.4'
     +        ,4,etdbins,0.)

         CALL hbookb(1000*iproc+16,'Et cross sec(MC),-1.5<eta(jet)<2.4
     +        ,D* Taggged jets',4,etdbins,0.)
         CALL hbookb(1000*iproc+17,'Et cross sec(MC),-1.5<eta(jet)<2.4
     +        ,D* UnTaggged jets',4,etdbins,0.)


         CALL hbookb(1000*iproc+18,'Et cross sec(MC), 3 < pt < 5GeV'
     +           ,4,etdbins,0.)
         CALL hbookb(1000*iproc+19,'Et cross sec(MC), 5 < pt < 8GeV'
     +           ,4,etdbins,0.)
         CALL hbookb(1000*iproc+20,'Et cross sec(MC), 8 < pt < 20GeV'
     +           ,4,etdbins,0.)

         CALL hbookb(1000*iproc+21,'Eta cross sec(MC), Et(jet)>6GeV'
     +           ,8,rapbins,0.)
         CALL hbookb(1000*iproc+22,'Eta cross sec(MC), 6<Et(jet)<9GeV'
     +           ,8,rapbins,0.)
         CALL hbookb(1000*iproc+23,'Eta cross sec(MC), Et(jet)>9GeV'
     +           ,8,rapbins,0.)

         CALL hbookb(1000*iproc+24,'Eta cross sec(MC), Et(jet)>6GeV,
     +           D* Tagged jets',8,rapbins,0.)
         CALL hbookb(1000*iproc+25,'Eta cross sec(MC), 6<Et(jet)<9GeV,
     +           D* Tagged jets',8,rapbins,0.)
         CALL hbookb(1000*iproc+26,'Eta cross sec(MC), Et(jet)>9GeV,
     +           D* Tagged jets',8,rapbins,0.)

         CALL hbookb(1000*iproc+27,'Eta cross sec(MC), Et(jet)>6GeV,
     +           D* UnTagged jets',8,rapbins,0.)
         CALL hbookb(1000*iproc+28,'Eta cross sec(MC), 6<Et(jet)<9GeV,
     +           D* UnTagged jets',8,rapbins,0.)
         CALL hbookb(1000*iproc+29,'Eta cross sec(MC), Et(jet)>9GeV,
     +           D* UnTagged jets',8,rapbins,0.)

         CALL hbookb(1000*iproc+30,'Eta cross sec(MC), 3 < pt < 5GeV'
     +           ,8,rapbins,0.)
         CALL hbookb(1000*iproc+31,'Eta cross sec(MC), 5 < pt < 8GeV'
     +           ,8,rapbins,0.)
         CALL hbookb(1000*iproc+32,'Eta cross sec(MC), 8 < pt < 20GeV'
     +           ,8,rapbins,0.)

         CALL hbookb(1000*iproc+33,'x(gamma)(D*,jet) cross sec(MC)'
     +           ,6,xbins,0.)
         CALL hbookb(1000*iproc+34,'Dijet x(gamma) cross sec(MC)'
     +           ,6,xbins,0.)

         CALL hbookb(1000*iproc+35,
     +            'dijet delta phi 0<x<1 cross sec(MC)',6,phibins,0.)
         CALL hbookb(1000*iproc+36,
     +            'dijet delta phi x>0.75 cross sec(MC)',6,phibins,0.)
         CALL hbookb(1000*iproc+37,
     +            'dijet delta phi x<0.75 cross sec(MC)',6,phibins,0.)

         CALL hbookb(1000*iproc+38,'dijet pt^2 0<x<1 cross sec(MC)',
     +            5,ptbins,0.)
         CALL hbookb(1000*iproc+39,'dijet pt^2 x>0.75 cross sec(MC)',
     +            5,ptbins,0.)
         CALL hbookb(1000*iproc+40,'dijet pt^2 x<0.75 cross sec(MC)',
     +            5,ptbins,0.)

         CALL hbookb(1000*iproc+41,'dijet Mjj cross sec(MC) 0<x<1 ',
     +            6,mbins,0.)
         CALL hbookb(1000*iproc+42,'dijet Mjj cross sec(MC) x>0.75',
     +            6,mbins,0.)
         CALL hbookb(1000*iproc+43,'dijet Mjj cross sec(MC) x<0.75',
     +            6,mbins,0.)


******************************************************************************
**********************  Loop over events (fill) ******************************
      ELSE IF(MOD(iflag,10).EQ.2) THEN
******  Filling: The following MUST always be done
******    (i) move to the correct sub-directory in PAWC
         
         CALL hcdir('//PAWC/'//xxxx,' ')

         q2  = hzphokin(1)
         ybj = hzphokin(3)
         pival = 4*atan(1.)

         IF(abs(q2).LT.q2cut .AND. ybj.GE.ybjcut(1) .AND. 
     +      ybj.LE.ybjcut(2)) THEN

            flag = hzibeam(ip, il)
            IF(flag.GE.0) THEN
               elecE = abs(phep(4,il))
            ELSE 
               Print*,'HZH0507089: No Electron Found'
               Print*,'         E=0.001, event will be rejected'
               elecE = 0.001
            ENDIF
******  Loop through all particles in the events     
            DO ihep = 1, nhep
               IF (abs(idhep(ihep)).EQ.413) THEN

******  Get D* kinematics       
                  pt = sqrt(((phep(1,ihep))**2)+((phep(2,ihep))**2))
                  pp = sqrt((pt**2)+((phep(3,ihep))**2))
     +                 +abs(phep(3,ihep))
                  rapd = 0.5*log((pp**2)/(pt**2))
                  rapd = SIGN(rapd,sngl(phep(3,ihep)))

******  Take cuts on D* Kinematics
                  IF (abs(rapd).LE.1.5 .AND. pt.GE.3) THEN
                     CALL hzjtfind(chjet,coner,njet,myjets)
                     
******  Loop through the jets
                     DO iloop = 1, njet
                        rap=myjets(iloop,1)
                        et=myjets(iloop,3)
                        IF (rap.GE.rapcut(1).AND.rap.LE.rapcut(5)
     +                       .AND.et.GE.etjetcut(1)) THEN
                           CALL hfill(1000*iproc+11,et,0.,wtx)
                           IF(rap.GE.rapcut(1).AND.rap.LE.rapcut(2))THEN
                              CALL hfill(1000*iproc+12,et,0.,wtx)
                           ENDIF
                           IF(rap.GE.rapcut(2).AND.rap.LE.rapcut(3))THEN
                              CALL hfill(1000*iproc+13,et,0.,wtx)
                           ENDIF
                           IF(rap.GE.rapcut(3).AND.rap.LE.rapcut(4))THEN
                              CALL hfill(1000*iproc+14,et,0.,wtx)
                           ENDIF
                           IF(rap.GE.rapcut(4).AND.rap.LE.rapcut(5))THEN
                              CALL hfill(1000*iproc+15,et,0.,wtx)
                           ENDIF
                           IF(pt.GE.ptcut(1).AND.pt.LE.ptcut(2))THEN
                              CALL hfill(1000*iproc+18,et,0.,wtx)
                              CALL hfill(1000*iproc+30,rap,0.,wtx)
                           ENDIF
                           IF(pt.GE.ptcut(2).AND.pt.LE.ptcut(3))THEN
                              CALL hfill(1000*iproc+19,et,0.,wtx)
                              CALL hfill(1000*iproc+31,rap,0.,wtx)
                           ENDIF
                           IF(pt.GE.ptcut(3).AND.pt.LE.ptcut(4))THEN
                              CALL hfill(1000*iproc+20,et,0.,wtx)
                              CALL hfill(1000*iproc+32,rap,0.,wtx)
                           ENDIF
                           IF(et.GE.6)THEN
                              CALL hfill(1000*iproc+21,rap,0.,wtx)
                              IF(et.LE.9)THEN
                                 CALL hfill(1000*iproc+22,rap,0.,wtx)
                              ENDIF
                              IF(et.GE.9)THEN
                                 CALL hfill(1000*iproc+23,rap,0.,wtx)
                              ENDIF
                           ENDIF
                        ENDIF
                     ENDDO

****** Dijet Cross sections
                     IF(njet.GE.2) THEN
                        j = 0
                        DO jloop = 1, njet
                           rap = myjets(jloop,1)
                           IF(rap.GE.rapcut(1).AND.rap.LE.rapcut(5))THEN
                              j = j + 1
                              IF(j.LE.2) THEN
                                 jRap(j) = myjets(jloop,1)
                                 jEt(j) = myjets(jloop,3)
                                 jPtx(j) = myjets(jloop,5)
                                 jPty(j) = myjets(jloop,6)
                                 jPhi(j) = myjets(jloop,2)
                              ENDIF
                           ENDIF
                        ENDDO
                        IF(jEt(1).GE.etjetcut(2).AND.
     +                       jEt(2).GE.etjetcut(1)) THEN
                           xgamma = ((jEt(1)*exp(-1*jRap(1)))+
     +                          (jEt(2)*exp(-1*jRap(2))))/(2*ybj*elecE)
                           CALL hfill(1000*iproc+34,xgamma,0.,wtx)
                           pjj2 = ((jPtx(1)+jPtx(2))*(jPtx(1)+jPtx(2)))+
     +                          ((jPty(1)+jPty(2))*(jPty(1)+jPty(2)))
                           CALL hfill(1000*iproc+38,pjj2,0.,wtx)
                           phijj = abs(jPhi(1)-jPhi(2))
                           IF(phijj.GE.pival)THEN
                              phijj = (2*pival)- phijj
                           ENDIF
                           CALL hfill(1000*iproc+35,phijj,0.,wtx)
                           mjj = sqrt(2*jEt(1)*jEt(2)*
     +                          (cosh(jRap(1)-jRap(2))
     +                          -cos(jPhi(1)-jPhi(2))))
                           CALL hfill(1000*iproc+41,mjj,0.,wtx)
                           
******Fill histograms binned in xgamma
                           if(xgamma.gt.0.75) then 
                              CALL hfill(1000*iproc+39,pjj2,0.,wtx)
                              CALL hfill(1000*iproc+36,phijj,0.,wtx)
                              CALL hfill(1000*iproc+42,mjj,0.,wtx)
                           ELSEIF(xgamma.lt.0.75)THEN 
                              CALL hfill(1000*iproc+40,pjj2,0.,wtx)
                              CALL hfill(1000*iproc+37,phijj,0.,wtx)
                              CALL hfill(1000*iproc+43,mjj,0.,wtx)
                           ENDIF
                        ENDIF
                     ENDIF

******  Find the jets containing final state particles + D* - D* decay products

                     CALL hzjtfind(chtjet,coner,njet,tjets)
                     xjet = 0

******  Loop through the jets
                     DO iloop = 1, njet
                        rap=tjets(iloop,1)
                        et=tjets(iloop,3)
                        IF (rap.GE.rapcut(1).AND.rap.LE.rapcut(5)
     +                       .AND.et.GE.etjetcut(1)) THEN
 
******  Find Tagged Jets
                           IF (iloop.EQ.iJetNo(ihep))THEN
                              CALL hfill(1000*iproc+16,et,0.,wtx)
                              CALL hfill(1000*iproc+24,rap,0.,wtx)
                              IF(et.LE.9)THEN
                                 CALL hfill(1000*iproc+25,rap,0.,wtx)
                              ENDIF
                              IF(et.GE.9)THEN
                                 CALL hfill(1000*iproc+26,rap,0.,wtx)
                              ENDIF
                           ELSE
******  Untagged Jets     
                              xjet = xjet + 1
                              CALL hfill(1000*iproc+17,et,0.,wtx)
                              CALL hfill(1000*iproc+27,rap,0.,wtx)
                              IF(et.LE.9)THEN
                                 CALL hfill(1000*iproc+28,rap,0.,wtx)
                              ENDIF
                              IF(et.GE.9)THEN
                                 CALL hfill(1000*iproc+29,rap,0.,wtx)
                              ENDIF
                              IF(xjet.EQ.1)THEN
                                 xdjet = ((pt*exp(-1*rapd))
     +                                +(et*exp(-1*rap)))
     +                                /(2*ybj*elecE)
                                 CALL hfill(1000*iproc+33,
     +                                xdjet,0.,wtx)
                              ENDIF
                           ENDIF    
                        ENDIF
                     ENDDO                     
                  ENDIF                           
               ENDIF
            ENDDO
         ENDIF

         
*******************************************************************************
*******************************  Terminate ************************************
         
      ELSE IF(MOD(iflag,10).EQ.3) THEN

******  Termination: The following MUST always be done
******     (i) Move to the correct PAW subdirectory     
         CALL hcdir('//PAWC/'//xxxx,' ')

******  write the number of events and xsec for the process just finished.
         WRITE(6,*) xxxx,' X-section is',xsec, ' and number of events ' 
     +    ,ntot, ' for process ',iproc

******  normalization
         DO i=11, 43
            CALL hzhinrm(1000*iproc+i,0,real(ntot/xsec),0)
         ENDDO

******  Add together the resolved and direct histograms
         IF (iproc.EQ.2) THEN
            DO i=11, 43
               CALL hopera(1000+i,'+',2000+i,i,1.,1.)
            ENDDO
         ENDIF
      ENDIF

      RETURN

      END

