      SUBROUTINE hzh09012226(IFLAG)
****************************************************************************
*     
* produces histograms for the ZEUS paper hep-ex/09012226
* Measurement of beauty photoproduction using decays into muons in dijet events at HERA.
* By ZEUS Collaboration (S. Chekanov et al.). DESY-08-210, Dec 2008. 19pp.
* Published in JHEP 0904:133,2009.
* e-Print: arXiv:0901.2226
* Based on the routine hzh0312057
*
* Author: Tobias Toll
*
*****************************************************************************
      IMPLICIT NONE
     
#include "hzhbook.inc"
#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
#include "hzjetcmn.inc"
     
      Character xxxx*8
      Data xxxx/'HZH09012226'/
     
C     constants
      DOUBLE PRECISION pi
c      double precision hzp
      PARAMETER(pi=3.141592653589793238)
     
C     variables
      INTEGER IFLAG,IPROC,ID,I,J,K
      LOGICAL first
      Data first/.true./
      Save first

*     bins, data and errors for cross section B. x_gamma
      INTEGER NBBIN
      PARAMETER (NBBIN=5)
      REAL BBIN(NBBIN+1)
      REAL BDATA(NBBIN),BERROR(NBBIN)
      
      DATA BBIN /0.,0.25,0.375,0.5,0.75,1./
      SAVE BBIN
      DATA BDATA /11.85, 17.17, 14.81, 22.19, 106.63/
      SAVE BDATA
      DATA BERROR /5.51, 9.07, 8.25, 6.36, 17.30/
      SAVE BERROR
      
*     bins, larger errors for cross section B. x_gamma
      INTEGER NBEBIN
      PARAMETER (NBEBIN=5)
      REAL BEBIN(NBEBIN+1)
      REAL BEDATA(NBEBIN),BEERROR(NBEBIN)
      
      DATA BEBIN /0.,0.25,0.375,0.5,0.75,1./
      SAVE BEBIN
      DATA BEDATA /0., 0., 0., 0., 0./
      SAVE BEDATA
      DATA BEERROR /5.97, 11.02, 8.58, 8.71, 17.94/
      SAVE BEERROR
      
*     bins, data and errors for cross section C. eta(mu)
      INTEGER NCBIN
      PARAMETER (NCBIN=3)
      REAL CBIN(NCBIN+1)
      REAL CDATA(NCBIN),CERROR(NCBIN)
      
      DATA CBIN /-1.6,-0.75,0.25,1.3/
      SAVE CBIN
      DATA CDATA /3.86, 16.81, 19.7/
      SAVE CDATA
      DATA CERROR /1.65, 3.15, 3.44/
      SAVE CERROR
      
*     bins and larger errors for cross section C. eta(mu)
      INTEGER NCEBIN
      PARAMETER (NCEBIN=3)
      REAL CEBIN(NCEBIN+1)
      REAL CEDATA(NCEBIN),CEERROR(NCEBIN)
      
      DATA CEBIN /-1.6,-0.75,0.25,1.3/
      SAVE CEBIN
      DATA CEDATA /0., 0., 0./
      SAVE CEDATA
      DATA CEERROR /1.96, 3.28, 3.93/
      SAVE CEERROR
      
*     bins, data and errors for cross section D. p_t(mu)
      INTEGER NDBIN
      PARAMETER (NDBIN=4)
      REAL DBIN(NDBIN+1)
      REAL DDATA(NDBIN),DERROR(NDBIN)
      
      DATA DBIN /1.5,2.5,4.,6.,10./
      SAVE DBIN
      DATA DDATA /41.05, 15.78, 4.87, 0.84/
      SAVE DDATA
      DATA DERROR /11.32, 2.79, 1.23, 0.29/
      SAVE DERROR
      
*     bins and larger errors for cross section D. p_t(mu)
      INTEGER NDEBIN
      PARAMETER (NDEBIN=4)
      REAL DEBIN(NDEBIN+1)
      REAL DEDATA(NDEBIN),DEERROR(NDEBIN)
      
      DATA DEBIN /1.5,2.5,4.,6.,10./
      SAVE DEBIN
      DATA DEDATA /0., 0., 0., 0./
      SAVE DEDATA
      DATA DEERROR /11.5, 2.82, 1.24, 0.29/
      SAVE DEERROR
      
*     bins, data and errors for cross section E. eta(mu-jet)
      INTEGER NEBIN
      PARAMETER (NEBIN=3)
      REAL EBIN(NEBIN+1)
      REAL EDATA(NEBIN),EERROR(NEBIN)
      
      DATA EBIN /-1.6,-0.6,0.4,1.4/
      SAVE EBIN
      DATA EDATA /6.13,13.89,16.42/
      SAVE EDATA
      DATA EERROR /1.63, 3.03, 2.85/
      SAVE EERROR
      
*     bins and larger errors for cross section E. eta(mu-jet)
      INTEGER NEEBIN
      PARAMETER (NEEBIN=3)
      REAL EEBIN(NEEBIN+1)
      REAL EEDATA(NEEBIN),EEERROR(NEEBIN)
      
      DATA EEBIN /-1.6,-0.6,0.4,1.4/
      SAVE EEBIN
      DATA EEDATA /0., 0., 0./
      SAVE EEDATA
      DATA EEERROR /2.06, 3.12, 3.24/
      SAVE EEERROR
      
*     bins, data and errors for cross section F. p_t(mu-jet)
      INTEGER NFBIN
      PARAMETER (NFBIN=3)
      REAL FBIN(NFBIN+1)
      REAL FDATA(NFBIN),FERROR(NFBIN)
      
      DATA FBIN /6.,11.,16.,30./
      SAVE FBIN
      DATA FDATA /4.74, 1.78, 0.33/
      SAVE FDATA
      DATA FERROR /0.82, 0.39, 0.11/
      SAVE FERROR
      
*     bins and larger errors for cross section F. p_t(mu-jet)
      INTEGER NFEBIN
      PARAMETER (NFEBIN=3)
      REAL FEBIN(NFEBIN+1)
      REAL FEDATA(NFEBIN),FEERROR(NFEBIN)
      
      DATA FEBIN /6.,11.,16.,30./
      SAVE FEBIN
      DATA FEDATA /0., 0., 0./
      SAVE FEDATA
      DATA FEERROR /0.83, 0.40, 0.11/
      SAVE FEERROR
      
*     bins, data and errors for cross section dphi
      INTEGER NdphiBIN
      PARAMETER (NdphiBIN=4)
      REAL dphiBIN(NdphiBIN+1)
      REAL dphiDATA(NdphiBIN),dphiERROR(NdphiBIN)
      
      DATA dphiBIN /0., 2.094, 2.618, 2.880, 3.142/
      SAVE dphiBIN
      DATA dphiDATA /2.26, 7.35, 24.7, 92.91/
      SAVE dphiDATA
      DATA dphiERROR /1.73, 2.52, 7.63, 15.25/
      SAVE dphiERROR

*     bins and larger errors for cross section dphi
      INTEGER NdphiEBIN
      PARAMETER (NdphiEBIN=4)
      REAL dphiEBIN(NdphiEBIN+1)
      REAL dphiEDATA(NdphiEBIN),dphiEERROR(NdphiEBIN)
      
      DATA dphiEBIN /0., 2.094, 2.618, 2.880, 3.142/
      SAVE dphiEBIN
      DATA dphiEDATA /0., 0., 0., 0./
      SAVE dphiEDATA
      DATA dphiEERROR /1.97, 2.53, 7.92, 16.96/
      SAVE dphiEERROR

*     bins, data and errors for cross section dphi-low xgamma
      INTEGER NdphilBIN
      PARAMETER (NdphilBIN=4)
      REAL dphilBIN(NdphiBIN+1)
      REAL dphilDATA(NdphiBIN),dphilERROR(NdphiBIN)
      
      DATA dphilBIN /0., 2.094, 2.618, 2.880, 3.142/
      SAVE dphilBIN
      DATA dphilDATA /1.38, 3.36, 7.75, 18.84/
      SAVE dphilDATA
      DATA dphilERROR /0.97, 1.82, 3.74, 4.91/
      SAVE dphilERROR

*     bins and larger errors for cross section dphi-low xgamma
      INTEGER NdphilEBIN
      PARAMETER (NdphilEBIN=4)
      REAL dphilEBIN(NdphilEBIN+1)
      REAL dphilEDATA(NdphilEBIN),dphilEERROR(NdphilEBIN)
      
      DATA dphilEBIN /0., 2.094, 2.618, 2.880, 3.142/
      SAVE dphilEBIN
      DATA dphilEDATA /0., 0., 0., 0./
      SAVE dphilEDATA
      DATA dphilEERROR /1.03, 1.87, 4.98, 5.58/
      SAVE dphilEERROR

*     bins, data and errors for cross section dphi-high xgamma
      INTEGER NdphihBIN
      PARAMETER (NdphihBIN=3)
      REAL dphihBIN(NdphihBIN+1)
      REAL dphihDATA(NdphihBIN),dphihERROR(NdphihBIN)
      
      DATA dphihBIN /0., 2.618, 2.880, 3.142/
      SAVE dphihBIN
      DATA dphihDATA /1.62, 15.27, 65.69/
      SAVE dphihDATA
      DATA dphihERROR /0.78, 5.24, 13.41/
      SAVE dphihERROR

*     bins and larger errors for cross section dphi-high xgamma
      INTEGER NdphihEBIN
      PARAMETER (NdphihEBIN=3)
      REAL dphihEBIN(NdphihEBIN+1)
      REAL dphihEDATA(NdphihEBIN),dphihEERROR(NdphihEBIN)
      
      DATA dphihEBIN /0., 2.618, 2.880, 3.142/
      SAVE dphihEBIN
      DATA dphihEDATA /0., 0., 0./
      SAVE dphihEDATA
      DATA dphihEERROR /1.3, 5.37, 14.07/
      SAVE dphihEERROR

      INTEGER NTEMPBIN
      PARAMETER (NTEMPBIN=5)
      REAL TEMPBIN(NTEMPBIN+1)
      DATA TEMPBIN/8.,9.,13.1,14.1,21.75,22.75/
      REAL temparr(NTEMPBIN+2),temperr(NTEMPBIN+2)

      REAL Q2,Y,LUMPB
      INTEGER BEAMFLAG,PROTID,LEPID
      REAL EE
     
C     final state muon list including IHEP id (1), 
C     id of originating b-quark from BBBAR list (2),
C     id of originating B-hadron from BHAD list (3),
C     jet id in which muon is contained (4)
      INTEGER NMAXMU,NMUIND,MUMOID
      PARAMETER (NMAXMU=6,NMUIND=4)
      INTEGER NMU,MU(NMAXMU,NMUIND)
      REAL MUPT,MUETA

*     b quark variables
      INTEGER NB,NMAXB
      INTEGER NBBAR,NMAXBBAR
      INTEGER NBBBAR,NMAXBBBAR
      PARAMETER (NMAXB=3,NMAXBBAR=3,NMAXBBBAR=6)
      INTEGER B(NMAXB),BBAR(NMAXBBAR),BBBAR(NMAXBBBAR)
      LOGICAL replace
     
C     list of B hadrons
      INTEGER NBHADMAX,NBHADIND
      PARAMETER (NBHADMAX=10,NBHADIND=2)
      INTEGER NBHAD,BHAD(NBHADMAX,NBHADIND)
     
      INTEGER nbrange
      PARAMETER (nbrange = 84)
      INTEGER brange(nbrange)
      data brange / 521,-521,511,-511,553,551,20553,555,100553,10551,
     +     120553,100555,200553,300553,400553,9010553,5122,-5122,-531,
     +     531,-541,541,-523,523,-513,513,-533,533,-543,543,5222,-5222,
     +     5212,-5212,5112,-5112,5322,-5322,5232,-5232,5312,-5312,5132,
     +     -5132,5332,-5332,5224,-5224,5214,-5214,5114,-5114,5334,-5334,
     +     10511,-10511,10513,-10513,10521,-10521,10523,-10523,10531,
     +     -10531,10533,-10533,10541,-10541,10543,-10543,515,-515,525,
     +     -525,535,-535,20513,-20513,20523,-20523,20533,-20533,20543,
     +     -20543 /
      LOGICAL inlist

C     jet finder
C     array has 10 digits, first id from jetfinder, following 8 correspond to
C     jetfinder output array, 10 is pt
      INTEGER NJETS,TEMPNJETS
      DOUBLE PRECISION JET(MAXHZJETS,10),TEMPJET(MAXHZJETS,8)
      LOGICAL twojetcut
     
C     div. variables
      REAL XGAMMA
      INTEGER exchange,loop_1,loop_2,loop_3
      LOGICAL filled,matched
      INTEGER MAXARRAY
      PARAMETER(MAXARRAY=1024)
      INTEGER STEPINDEX,TOTINDEX,ARRAY(MAXARRAY)

c     deltaphi
      real dphi, nevt
*     Determine the process type.
      iproc=(mod(iflag,10000)/1000)
      if ((iproc.lt.0).or.(iproc.gt.2)) then
         write(HZUNIT,*) 'HZ',xxxx,': illegal IPROC:',iproc
         return
      endif


      IF (MOD(iflag,10).eq.1) then
C     *** Initialisation run ***
    
         IF (first) THEN
            CALL HCDIR('//PAWC',' ')
            CALL HMDIR(xxxx,'S')
            CALL HCDIR('//HISTO',' ')
            CALL HMDIR(xxxx,'S')
         ELSE
            CALL HCDIR('//PAWC/'//xxxx,' ')
            CALL HCDIR('//HISTO/'//xxxx,' ')
         ENDIF

         write(hzunit,*) 'HZ'//xxxx//': Booking histograms'
     
C     book mc histos
         ID=2+100*iproc
         CALL HBOOKB(ID,' d[s]/dx?[g](ep->e bbbar X->e jj [m] X) ',
     +        NBBIN,BBIN,0.)
         CALL HBARX(ID)

         ID=3+100*iproc
         CALL HBOOKB(ID,' d[s]/d[e]^[m](ep->e bbbar X->e jj [m] X) ',
     +        NCBIN,CBIN,0.)
         CALL HBARX(ID)

         ID=4+100*iproc
         CALL HBOOKB(ID,' d[s]/dp?T^[m](ep->e bbbar X->e jj [m] X) ',
     +        NDBIN,DBIN,0.)
         CALL HBARX(ID)

         ID=5+100*iproc
         CALL HBOOKB(ID,' d[s]/d[e]^{[m]-jet}(ep->e bbbar X->
     +        e jj [m] X) ',NEBIN,EBIN,0.)
         CALL HBARX(ID)

         ID=6+100*iproc
         CALL HBOOKB(ID,' d[s]/dp?T^{[m]-jet}(ep->e bbbar X->
     +        e jj [m] X) ',NFBIN,FBIN,0.)
         CALL HBARX(ID)

         id=10+100*iproc
         CALL HBOOKB(ID,' d[s]/ddpi^{jj}(ep->e bbbar X->e jj X) ',
     +        NdphiBIN, dphiBIN,0.)
         CALL HBARX(ID)

         id=11+100*iproc
         CALL HBOOKB(ID,' d[s]/ddpi^{jj}(ep->e bbbar X->e jj X) ',
     +        NdphilBIN, dphilBIN,0.)
         CALL HBARX(ID)

         id=12+100*iproc
         CALL HBOOKB(ID,' d[s]/ddpi^{jj}(ep->e bbbar X->e jj X) ',
     +        NdphihBIN, dphihBIN,0.)
         CALL HBARX(ID)

         IF (first) THEN
c     Book the data.
            ID=2
            CALL HBOOKB(-ID,' d[s]/dx?[g](ep->e bbbar X->e jj [m] X) ',
     +           NBBIN,BBIN,0.)
            CALL HPAK(-ID,BDATA)
            CALL HPAKE(-ID,BERROR)
         
            ID=22
            CALL HBOOKB(-ID,' d[s]/dx?[g](ep->e bbbar X->e jj [m] X) ',
     +           NBEBIN,BEBIN,0.)
            CALL HPAK(-ID,BEDATA)
            CALL HPAKE(-ID,BEERROR)
         
            ID=3
            CALL HBOOKB(-ID,' d[s]/d[e]^[m](ep->e bbbar X->
     +           e jj [m] X) ',NCBIN,CBIN,0.)
            CALL HPAK(-ID,CDATA)
            CALL HPAKE(-ID,CERROR)
            
            ID=23
            CALL HBOOKB(-ID,' d[s]/d[e]^[m](ep->e bbbar X->
     +           e jj [m] X) ',NCEBIN,CEBIN,0.)
            CALL HPAK(-ID,CEDATA)
            CALL HPAKE(-ID,CEERROR)

            ID=4
            CALL HBOOKB(-ID,' d[s]/dp?T^[m](ep->e bbbar X->
     +           e jj [m] X) ',NDBIN,DBIN,0.)
            CALL HPAK(-ID,DDATA)
            CALL HPAKE(-ID,DERROR)
            
            ID=24
            CALL HBOOKB(-ID,' d[s]/dp?T^[m](ep->e bbbar X->
     +           e jj [m] X) ',NDEBIN,DEBIN,0.)
            CALL HPAK(-ID,DEDATA)
            CALL HPAKE(-ID,DEERROR)
            
            ID=5
            CALL HBOOKB(-ID,' d[s]/d[e]^{j [m]-jet}
     +           (ep->e bbbar X->e jj [m] X) ',
     +           NEBIN,EBIN,0.)
            CALL HPAK(-ID,EDATA)
            CALL HPAKE(-ID,EERROR)
            
            ID=25
            CALL HBOOKB(-ID,' d[s]/d[e]^{j [m]-jet}
     +           (ep->e bbbar X->e jj [m] X) ',
     +           NEEBIN,EEBIN,0.)
            CALL HPAK(-ID,EEDATA)
            CALL HPAKE(-ID,EEERROR)

            ID=6
            CALL HBOOKB(-ID,' d[s]/dp?T^{j [m]-jet}'//
     +           '(ep->e bbbar X->e jj [m] X) ',
     +           NFBIN,FBIN,0.)
            CALL HPAK(-ID,FDATA)
            CALL HPAKE(-ID,FERROR)
            
            ID=26
            CALL HBOOKB(-ID,' d[s]/dp?T^{j [m]-jet}'//
     +           '(ep->e bbbar X->e jj [m] X) ',
     +           NFEBIN,FEBIN,0.)
            CALL HPAK(-ID,FEDATA)
            CALL HPAKE(-ID,FEERROR)

            ID=10
            CALL HBOOKB(-ID,' d[s]/ddphi^{jj}(ep->e bbbar X->e jj X) ',
     +           NdphiBIN,dphiBIN,0.)
            CALL HPAK(-ID,dphiDATA)
            CALL HPAKE(-ID,dphiERROR)

            ID=30
            CALL HBOOKB(-ID,' d[s]/ddphi^{jj}(ep->e bbbar X->e jj X) ',
     +           NdphiEBIN,dphiEBIN,0.)
            CALL HPAK(-ID,dphiEDATA)
            CALL HPAKE(-ID,dphiEERROR)

            ID=11
            CALL HBOOKB(-ID,' d[s]/ddphi^{jj}(ep->e bbbar X->e jj X) ',
     +           NdphilBIN,dphilBIN,0.)
            CALL HPAK(-ID,dphilDATA)
            CALL HPAKE(-ID,dphilERROR)

            ID=31
            CALL HBOOKB(-ID,' d[s]/ddphi^{jj}(ep->e bbbar X->e jj X) ',
     +           NdphilEBIN,dphilEBIN,0.)
            CALL HPAK(-ID,dphilEDATA)
            CALL HPAKE(-ID,dphilEERROR)

            ID=12
            CALL HBOOKB(-ID,' d[s]/ddphi^{jj}(ep->e bbbar X->e jj X) ',
     +           NdphihBIN,dphihBIN,0.)
            CALL HPAK(-ID,dphihDATA)
            CALL HPAKE(-ID,dphihERROR)

            ID=32
            CALL HBOOKB(-ID,' d[s]/ddphi^{jj}(ep->e bbbar X->e jj X) ',
     +           NdphihEBIN,dphihEBIN,0.)
            CALL HPAK(-ID,dphihEDATA)
            CALL HPAKE(-ID,dphihEERROR)

            first=.FALSE.
*     
         ENDIF
*     
C==================================================================
*     
      ELSE IF (MOD(iflag,10).EQ.2) THEN
C     ***  Event loop, fill MC histograms ***

C     initialise variables
         nevt=nevt+1.
         Q2=0
         Y=0
         EE=0
         BEAMFLAG=99
         PROTID=0
         LEPID=0
         NB=0
         DO i=1,NMAXB
            B(i)=0
         ENDDO
         NBBAR=0
         DO i=1,NMAXBBAR
            BBAR(i)=0
         ENDDO
         NBBBAR=0
         DO i=1,NMAXBBBAR
            BBBAR(i)=0
         ENDDO
         NMU=0
         DO I=1,NMAXMU
            DO J=1,NMUIND
               MU(i,j)=0
            ENDDO
         ENDDO
         MUMOID=0
         NBHAD=0
         DO I=1,NBHADMAX
            DO J=1,NBHADIND
               BHAD(I,J)=0
            ENDDO
         ENDDO
         TEMPNJETS=0
         NJETS=0
         DO I=1,MAXHZJETS
            DO J=1,8
               TEMPJET(i,j)=0
               JET(i,j)=0
            ENDDO
            JET(i,9)=0
            JET(i,10)=0
         ENDDO
         XGAMMA=0.0
         inlist=.false.
         twojetcut=.false.
         replace=.false.
         stepindex=0
         totindex=0
         DO I=1,MAXARRAY
            ARRAY(I)=0
         ENDDO
         loop_1=0
         loop_2=0
         loop_3=0
         filled=.false.
         matched=.false.         

         CALL HCDIR('//PAWC/'//xxxx,' ')
         CALL HCDIR('//HISTO/'//xxxx,' ')

C     get beam particle and lepton energy
         BEAMFLAG=HZIBEAM(PROTID,LEPID)
         IF ( BEAMFLAG.eq.0 ) THEN
            WRITE(HZUNIT,*) 'Beam: Only Lepton was found, Skip Event'
            GOTO 65
         ELSE IF ( BEAMFLAG.eq.-1 ) THEN
            WRITE(HZUNIT,*) 'Beam: Only Proton was found, Skip Event'
            GOTO 65
         ELSE IF ( BEAMFLAG.gt.1.OR.BEAMFLAG.lt.-1 ) THEN
            WRITE(HZUNIT,*) 'Beam: Something went wrong, Skip Event'
            GOTO 65
         ENDIF
         EE=ABS(PHEP(4,LEPID))
     
C     test kinematic range
         Q2=REAL(HZPHOKIN(1))
         Y=REAL(HZPHOKIN(3))     
         IF (Q2.GT.1.) GOTO 65
         IF (Y.LE.0.2.OR.Y.GE.0.8) GOTO 65
         
         DO I=1,NHEP

C     search for mu+- in final state, make mu list
            IF ( ISTHEP(i).eq.1 ) THEN
               IF ( IDHEP(i).eq.13.OR.IDHEP(i).eq.-13 ) THEN
C     particle id of mother of muon
                  MUMOID=IDHEP(JMOHEP(1,i))
                  IF ( MUMOID.eq.0) THEN
                     WRITE(HZUNIT,*) 'Muon Mother = 0, reject event'
                     GOTO 65
                  ENDIF
C     check if muon mother is not a pion or kaon
C     ID's: 211,-211,311,321,-321
                  IF ( (.NOT.(MUMOID.eq.211)).AND.
     +                 (.NOT.(MUMOID.eq.-211)).AND.
     +                 (.NOT.(MUMOID.eq.311)).AND.
     +                 (.NOT.(MUMOID.eq.321)).AND.
     +                 (.NOT.(MUMOID.eq.-321))) THEN
                     NMU=NMU+1
                     IF ( NMU.gt.NMAXMU ) THEN
                        WRITE(HZUNIT,*) 'Number of muons ',
     +                       NMU,
     1                       ' larger than array with maximal ',
     2                       NMAXMU,' entries, skip event'
                        GOTO 65
                     ENDIF
                     MU(NMU,1)=i
                  ENDIF
               ENDIF
            ENDIF
               
*     search for b, pythia style, ISTHEP<21
            IF ( GEN.eq.'PYT' ) THEN
*     first found b quark is kept, chain b quarks are discarded
               IF ( IDHEP(i).eq.5.AND.ISTHEP(i).ge.10 ) THEN
                  IF (NB.ge.NMAXB) THEN
                     write(hzunit,*) xxxx,' ERROR: More than ',NMAXB
     $                    ,' b quarks'
                     RETURN
                  ELSE
*     check 3 consecutive mother if they are already in list 
                     inlist=.false.
                     DO j=1,NB
                        IF ( B(j).eq.JMOHEP(1,i) ) THEN
                           inlist=.true.
                        ELSE IF ( JMOHEP(1,JMOHEP(1,i)).gt.0.AND.
     $                          JMOHEP(1,JMOHEP(1,i)).eq.B(j)) THEN
                           inlist=.true.
                        ELSE IF ( JMOHEP(1,JMOHEP(1,JMOHEP(1,i))). gt.0
     $                          .AND. JMOHEP(1,JMOHEP(1,JMOHEP(1,i))).eq
     $                          . B(j)) THEN
                           inlist=.true.
                        ENDIF
                     ENDDO
                     IF ( .NOT.inlist ) THEN
                        NB=NB+1
                        B(NB)=i
                        NBBBAR=NBBBAR+1
                        BBBAR(NBBBAR)=i
                     ENDIF
                  ENDIF
               ELSE IF ( IDHEP(i).eq.-5.AND.ISTHEP(i).ge.10 ) 
     +                 THEN
                  IF (NBBAR.ge.NMAXBBAR) THEN
                     write(hzunit,*) xxxx,' ERROR: More than ',NMAXB,
     $                    ' anti b quarks'
                     RETURN
                  ELSE
*     check 3 consecutive mother if they are already in list 
                     inlist=.false.
                     DO j=1,NBBAR
                        IF ( BBAR(j).eq.JMOHEP(1,i) ) THEN
                           inlist=.true.
                        ELSE IF ( JMOHEP(1,JMOHEP(1,i)).gt.0.AND.
     +                          JMOHEP(1,JMOHEP(1,i)).eq.
     +                          BBAR(j) ) THEN
                           inlist=.true.
                        ELSE IF ( JMOHEP(1,JMOHEP(1,JMOHEP(1,i))).
     +                          gt.0.AND.
     +                          JMOHEP(1,JMOHEP(1,JMOHEP(1,i))).eq.
     +                          BBAR(j)
     +                          ) THEN
                           inlist=.true.
                        ENDIF
                     ENDDO
                     IF ( .NOT.inlist ) THEN
                        NBBAR=NBBAR+1
                        BBAR(NBBAR)=i
                        NBBBAR=NBBBAR+1
                        BBBAR(NBBBAR)=i
                     ENDIF
                  ENDIF
               ENDIF
            ELSE IF ( GEN.eq.'HRW' ) THEN
*     take all b quarks which have ISTHEP=2 (parton before hadronisation
*     quarks have as second mother color information, so only consider
*     first mother
               IF ( IDHEP(i).eq.5.AND.ISTHEP(i).eq.2 ) THEN
                  IF (NB.ge.NMAXB) THEN
                     write(hzunit,*) xxxx,' ERROR: More than ',NMAXB
     $                    ,' b quarks'
                     RETURN
                  ELSE
*     check 3 consecutive mother if they are already in list 
                     inlist=.false.
                     DO j=1,NB
                        IF ( B(j).eq.JMOHEP(1,i) ) THEN
                           inlist=.true.
                        ELSE IF ( JMOHEP(1,JMOHEP(1,i)).gt.0.AND.
     +                          JMOHEP(1,JMOHEP(1,i)).eq.B(j)
     +                          ) THEN
                           inlist=.true.
                        ELSE IF ( JMOHEP(1,JMOHEP(1,JMOHEP(1,i))).
     +                          gt.0.AND.
     +                          JMOHEP(1,JMOHEP(1,JMOHEP(1,i))).eq.
     +                          B(j)
     +                          ) THEN
                           inlist=.true.
                        ENDIF
                     ENDDO
                     IF ( .NOT.inlist ) THEN
                        NB=NB+1
                        B(NB)=i
                        NBBBAR=NBBBAR+1
                        BBBAR(NBBBAR)=i
                     ENDIF
                  ENDIF
               ELSE IF ( IDHEP(i).eq.-5.AND.ISTHEP(i).eq.2 ) 
     +                 THEN
                  IF (NBBAR.ge.NMAXBBAR) THEN
                     write(hzunit,*) xxxx,' ERROR: More than ',NMAXB,
     $                    ' anti b quarks'
                     RETURN
                  ELSE
*     check 3 consecutive mother if they are already in list 
                     inlist=.false.
                     DO j=1,NBBAR
                        IF ( BBAR(j).eq.JMOHEP(1,i) ) THEN
                           inlist=.true.
                        ELSE IF ( JMOHEP(1,JMOHEP(1,i)).gt.0.AND.
     +                          JMOHEP(1,JMOHEP(1,i)).eq.
     +                          BBAR(j) ) THEN
                           inlist=.true.
                        ELSE IF ( JMOHEP(1,JMOHEP(1,JMOHEP(1,i))).
     +                          gt.0.AND.
     +                          JMOHEP(1,JMOHEP(1,JMOHEP(1,i))).eq.
     +                          BBAR(j)
     +                          ) THEN
                           inlist=.true.
                        ENDIF
                     ENDDO
                     IF ( .NOT.inlist ) THEN
                        NBBAR=NBBAR+1
                        BBAR(NBBAR)=i
                        NBBBAR=NBBBAR+1
                        BBBAR(NBBBAR)=i
                     ENDIF
                  ENDIF
               ENDIF
            ELSE
               write(hzunit,*) xxxx
     $              ,' generator not implemented for this routine'
               RETURN
            ENDIF
            
*     look for B hadrons, select first in decay chain
            DO j=1,nbrange
               IF ( IDHEP(i).eq.brange(j) ) THEN
*     check if 3 consecutive mothers of i are included in BHAD
                  inlist=.false.
                  DO k=1,NBHAD
                     IF ( BHAD(k,1).eq.JMOHEP(1,i) ) THEN
                        inlist=.true.
                     ELSE IF ( JMOHEP(1,JMOHEP(1,i)).gt.0.AND.
     +                       JMOHEP(1,JMOHEP(1,i)).eq.BHAD(k,1)
     +                       ) THEN
                        inlist=.true.
                     ELSE IF ( JMOHEP(1,JMOHEP(1,JMOHEP(1,i))).gt.
     +                       0.AND.
     +                       JMOHEP(1,JMOHEP(1,JMOHEP(1,i))).eq.
     +                       BHAD(k,1)
     +                       ) THEN
                        inlist=.true.
                     ENDIF
                  ENDDO
                  IF ( .NOT.inlist ) THEN   
                     IF ( NBHAD.ge.NBHADMAX ) THEN
                        write(hzunit,*) xxxx,' ERROR: More than ',
     +                       NBHADMAX,' B hadrons'
                        RETURN
                     ELSE
                        NBHAD=NBHAD+1
                        BHAD(NBHAD,1)=i
                     ENDIF
                  ENDIF
               ENDIF
            ENDDO
            
         ENDDO
     
C     check if b or bbar exist
         IF ( NB.eq.0.AND.NBBAR.eq.0 ) GOTO 65
       
C     sort in pt 
         DO i=1,NB
            DO j=(i+1),NB
               IF ( HZPT(B(j)).gt.HZPT(B(i)) ) THEN
                  exchange = B(j)
                  B(j) = B(i)
                  B(i) = exchange
               ENDIF
            ENDDO
         ENDDO
         DO i=1,NBBAR
            DO j=(i+1),NBBAR
               IF ( HZPT(BBAR(j)).gt.HZPT(BBAR(i)) ) THEN
                  exchange = BBAR(j)
                  BBAR(j) = BBAR(i)
                  BBAR(i) = exchange
               ENDIF
            ENDDO
         ENDDO
         DO i=1,NBBBAR
            DO j=(i+1),NBBBAR
               IF ( HZPT(BBBAR(j)).gt.HZPT(BBBAR(i)) ) THEN
                  exchange = BBBAR(j)
                  BBBAR(j) = BBBAR(i)
                  BBBAR(i) = exchange
               ENDIF
            ENDDO
         ENDDO
         DO i=1,NBHAD
            DO j=(i+1),NBHAD
               IF ( HZPT(BHAD(j,1)).gt.HZPT(BHAD(i,1)) ) THEN
                  exchange = BHAD(j,1)
                  BHAD(j,1) = BHAD(i,1)
                  BHAD(i,1) = exchange
               ENDIF
            ENDDO
         ENDDO
         DO i=1,NMU
            DO j=(i+1),NMU
               IF ( HZPT(MU(j,1)).gt.HZPT(MU(i,1)) ) THEN
                  exchange = MU(j,1)
                  MU(j,1) = MU(i,1)
                  MU(i,1) = exchange
               ENDIF
            ENDDO
         ENDDO




C     call jet finder in massive kt mode, returns pt ordered jets
         CALL HZJTFIND(10,1.0D0,TEMPNJETS,TEMPJET)

C     check for jet eta cut
C     fill in new list, first array digit is old jetindex
         DO I=1,TEMPNJETS
            IF ( ABS(REAL(TEMPJET(I,1))).le.2.5 ) THEN
               NJETS=NJETS+1
               JET(NJETS,1)=I
               DO J=2,9
                  JET(NJETS,J)=TEMPJET(I,J-1)
               ENDDO
               JET(NJETS,10)=SQRT(JET(NJETS,6)**2+JET(NJETS,7)**2)
            ENDIF
         ENDDO

*     check for jet cuts
         IF ( NJETS.ge.2 ) THEN
            IF ( REAL(JET(1,10)).gt.7.0.AND.REAL(JET(2,10)).gt.6.0 )
     $           THEN
               twojetcut=.true.
            ENDIF
         ENDIF

*     check in which jet the b hadron with highest pt is
         IF ( twojetcut ) THEN
            DO I=1,NBHAD
               DO J=1,NJETS
                  IF ( REAL(JET(J,10)).gt.6.0 ) THEN
                     IF ( IJETNO(BHAD(I,1)).eq.JET(J,1) ) THEN
                        BHAD(I,2)=J
                     ENDIF
                  ENDIF
               ENDDO
            ENDDO
         ENDIF

*     if one or more muons, check for muons from which b-quark in list bbbar they come from
         matched=.false.
         stepindex=0
         totindex=0
         IF ( NMU.ge.1 ) THEN
            DO loop_1=1,NMU
               IF ( NBBBAR.ge.1 ) THEN
                  DO loop_2=1,NBBBAR
                     matched=.false.
                     stepindex=0
                     totindex=0
                     IF ( JDAHEP(1,BBBAR(loop_2)).gt.0 ) THEN
                        IF ( JDAHEP(1,BBBAR(loop_2)).le.
     +                       JDAHEP(2,BBBAR(loop_2)) ) THEN
                           DO loop_3=JDAHEP(1,BBBAR(loop_2)),
     +                          JDAHEP(2,BBBAR(loop_2))
                              IF ( loop_3.eq.MU(loop_1,1) ) THEN
                                 MU(loop_1,2)=BBBAR(loop_2)
                                 matched=.true.
                                 stepindex=totindex
                              ELSE
                                 IF ( totindex.ge.maxarray ) THEN
                                    write(hzunit,*) xxxx
     $                               ,' ERROR: More than ',MAXARRAY,'
     $                                   entries in array'
                                    RETURN
                                 ELSE
                                    totindex=totindex+1
                                    array(totindex)=loop_3
                                 ENDIF
                              ENDIF
                           ENDDO
                        ELSE
                           IF ( JDAHEP(1,BBBAR(loop_2)).eq.MU(loop_1,1)
     +                          ) THEN
                              MU(loop_1,2)=BBBAR(loop_2)
                              matched=.true.
                              stepindex=totindex
                           ELSE
                              IF ( totindex.ge.maxarray ) THEN
                                 write(hzunit,*) xxxx
     $                                ,' ERROR: More than ',MAXARRAY
     $                                ,' entries in array'
                                 RETURN
                              ELSE
                                 totindex=totindex+1
                                 array(totindex)=JDAHEP(1,BBBAR(loop_2))
                              ENDIF
                           ENDIF
                        ENDIF
                     ENDIF
                     IF ( .NOT.matched ) THEN
                        DO WHILE ( stepindex.lt.totindex )
                           IF ( .NOT.matched ) THEN
                              stepindex=stepindex+1
                              IF ( JDAHEP(1,array(stepindex)).gt.0) THEN
                                 IF ( JDAHEP(1,array(stepindex)).le.
     +                                JDAHEP(2,array(stepindex)) ) THEN
                                    DO loop_3=JDAHEP(1,
     +                                   array(stepindex)),
     +                                JDAHEP(2,array(stepindex))
                                       IF ( .NOT.Matched ) THEN
                                          IF ( loop_3.eq.MU(loop_1,1) 
     +                                         ) THEN
                                             MU(loop_1,2)=
     +                                            BBBAR(loop_2)
                                             matched=.true.
                                             stepindex=totindex
                                          ELSE
                                             IF ( totindex.ge.
     +                                            maxarray ) 
     +                                            THEN
                                                write(hzunit,*) xxxx,
     $                                               ' ERROR:',
     $                                               ' More than ',
     $                                               MAXARRAY,
     $                                               ' entries in ',
     $                                               'array'
                                                RETURN
                                             ELSE
                                                totindex=totindex+1
                                                array(totindex)=
     +                                               loop_3
                                             ENDIF
                                          ENDIF
                                       ENDIF
                                    ENDDO
                                 ELSE
                                    IF ( JDAHEP(1,array(stepindex))
     +                                   .eq.MU(loop_1,1) ) THEN
                                       MU(loop_1,2)=BBBAR(loop_2)
                                       matched=.true.
                                       stepindex=totindex
                                    ELSE
                                       IF ( totindex.ge.maxarray ) 
     +                                      THEN
                                          write(hzunit,*) xxxx,
     $                                         ' ERROR: More than ',
     $                                         MAXARRAY,
     $                                         ' entries in array'
                                          RETURN
                                       ELSE
                                          totindex=totindex+1
                                          array(totindex)=
     +                                         JDAHEP(1,
     +                                         array(stepindex))
                                       ENDIF
                                    ENDIF
                                 ENDIF
                              ENDIF
                           ENDIF
                        ENDDO
                     ENDIF
                  ENDDO
               ENDIF
            ENDDO
         ENDIF
         
*     if one or more muons, check for muons from which B-hadron in list
*     bhad they come from
         matched=.false.
         stepindex=0
         totindex=0
         IF ( NMU.ge.1 ) THEN
            DO loop_1=1,NMU
               IF ( NBHAD.ge.1 ) THEN
                  DO loop_2=1,NBHAD
                     matched=.false.
                     stepindex=0
                     totindex=0
                     IF ( JDAHEP(1,BHAD(loop_2,1)).gt.0 ) THEN
                        IF ( JDAHEP(1,BHAD(loop_2,1)).le.
     +                       JDAHEP(2,BHAD(loop_2,1)) ) THEN
                           DO loop_3=JDAHEP(1,BHAD(loop_2,1)),
     +                          JDAHEP(2,BHAD(loop_2,1))
                              IF ( loop_3.eq.MU(loop_1,1) ) THEN
                                 MU(loop_1,3)=BHAD(loop_2,1)
                                 matched=.true.
                                 stepindex=totindex
                              ELSE
                                 IF ( totindex.ge.maxarray ) THEN
                                    write(hzunit,*) xxxx
     $                                 ,' ERROR: More than ',MAXARRAY,'
     $                                   entries in array'
                                    RETURN
                                 ELSE
                                    totindex=totindex+1
                                    array(totindex)=loop_3
                                 ENDIF
                              ENDIF
                           ENDDO
                        ELSE
                           IF ( JDAHEP(1,BHAD(loop_2,1)).eq.MU(loop_1,1)
     +                          ) THEN
                              MU(loop_1,3)=BHAD(loop_2,1)
                              matched=.true.
                              stepindex=totindex
                           ELSE
                              IF ( totindex.ge.maxarray ) THEN
                                 write(hzunit,*) xxxx
     $                                ,' ERROR: More than ',MAXARRAY
     $                                ,' entries in array'
                                 RETURN
                              ELSE
                                 totindex=totindex+1
                                 array(totindex)=
     +                                JDAHEP(1,BHAD(loop_2,1))
                              ENDIF
                           ENDIF
                        ENDIF
                     ENDIF
                     IF ( .NOT.matched ) THEN
                        DO WHILE ( stepindex.lt.totindex )
                           IF ( .NOT.matched ) THEN
                              stepindex=stepindex+1
                              IF ( JDAHEP(1,array(stepindex)).gt.0) THEN
                                 IF ( JDAHEP(1,array(stepindex)).le.
     +                                JDAHEP(2,array(stepindex)) ) THEN
                                    DO loop_3=JDAHEP(1,
     +                                   array(stepindex)),
     +                                JDAHEP(2,array(stepindex))
                                       IF ( .NOT.Matched ) THEN
                                          IF ( loop_3.eq.MU(loop_1,1) 
     +                                         ) THEN
                                             MU(loop_1,3)=
     +                                            BHAD(loop_2,1)
                                             matched=.true.
                                             stepindex=totindex
                                          ELSE
                                             IF ( totindex.ge.
     +                                            maxarray ) 
     +                                            THEN
                                                write(hzunit,*) xxxx,
     $                                               ' ERROR:',
     $                                               ' More than ',
     $                                               MAXARRAY,
     $                                               ' entries in ',
     $                                               'array'
                                                RETURN
                                             ELSE
                                                totindex=totindex+1
                                                array(totindex)=
     +                                               loop_3
                                             ENDIF
                                          ENDIF
                                       ENDIF
                                    ENDDO
                                 ELSE
                                    IF ( JDAHEP(1,array(stepindex))
     +                                   .eq.MU(loop_1,1) ) THEN
                                       MU(loop_1,3)=BHAD(loop_2,1)
                                       matched=.true.
                                       stepindex=totindex
                                    ELSE
                                       IF ( totindex.ge.maxarray ) 
     +                                      THEN
                                          write(hzunit,*) xxxx,
     $                                         ' ERROR: More than ',
     $                                         MAXARRAY,
     $                                         ' entries in array'
                                          RETURN
                                       ELSE
                                          totindex=totindex+1
                                          array(totindex)=
     +                                         JDAHEP(1,
     +                                         array(stepindex))
                                       ENDIF
                                    ENDIF
                                 ENDIF
                              ENDIF
                           ENDIF
                        ENDDO
                     ENDIF
                  ENDDO
               ENDIF
            ENDDO
         ENDIF

*     get jet id's for muons by connection via the B-hadron the muon
*     comes from
         DO I=1,NMU
            IF ( MU(I,3).gt.0 ) THEN
               DO J=1,NBHAD
                  IF ( BHAD(J,1).eq.MU(I,3) ) THEN
                     MU(I,4)=BHAD(J,2)
                  ENDIF
               ENDDO
            ENDIF
         ENDDO
         
C         IF ( nmu.gt.0 ) THEN
C            write(hzunit,*) xxxx,' twojetcut ',twojetcut
C            write(hzunit,*) xxxx,' bbbar information'
C            DO I=1,NBBBAR
C               write(hzunit,*) 'bbbar ',I,' id ',BBBAR(I)
C            ENDDO
C            write(hzunit,*) xxxx,' bhad information'
C            DO I=1,NBHAD
C               write(hzunit,*) 'bhad ',I,' id ',BHAD(I,1),' jet ',BHAD(I,2)
C            ENDDO
C            write(hzunit,*) xxxx,' Muon information'
C            DO I=1,NMU
C               write(hzunit,*) 'muon ',I,' id ',MU(I,1),' quark ',MU(I,2),
C     +              ' bhad ',MU(I,3),' jet ',MU(I,4)
C            ENDDO
C         ENDIF


*     fill histo B and the deltaphi histos
         filled=.false.
         IF ( twojetcut.AND.NMU.gt.0 ) THEN
            XGAMMA=((JET(1,5)-JET(1,8))+
     +           (JET(2,5)-JET(2,8)))/(2*Y*EE)
            DO I=1,NMU
               IF ( .NOT.filled ) THEN
                  IF ( REAL(HZPT(MU(I,1))).gt.2.5.AND.
     +                 REAL(HZETA(MU(I,1))).gt.-1.6.AND.
     +                 REAL(HZETA(MU(I,1))).LT.1.3 ) THEN
*     fill if muon comes from b decay
                     IF ( MU(I,2).gt.0 ) THEN
			CALL HFILL(2+100*IPROC,XGAMMA,0.,wtx)
                        dphi=abs(jet(1,3)-jet(2,3))
                        if(dphi.gt.pi) dphi=2.*pi-dphi
                        call hfill(10+100*iproc,dphi,0.,wtx)
                        if(xgamma.lt.0.75) then
                           call hfill(11+100*iproc,dphi,0.,wtx)
                        elseif(xgamma.gt.0.75) then
                           call hfill(12+100*iproc,dphi,0.,wtx)
                        endif
*                        filled=.true.
                     ENDIF
                  ENDIF
               ENDIF
            ENDDO
         ENDIF
         
*     fill histo C
         filled=.false.
         IF ( twojetcut.AND.NMU.gt.0 ) THEN
            DO I=1,NMU
               IF ( .NOT.filled ) THEN
*     muon cuts
                  IF ( REAL(HZPT(MU(I,1))).gt.2.5 ) THEN
*     fill if muon comes from b decay
                     IF ( MU(I,2).gt.0 ) THEN
                        CALL HFILL(3+100*IPROC,REAL(HZETA(MU(I,1))),
     +                       0.,wtx)
*                        filled=.true.
                     ENDIF
                  ENDIF
               ENDIF
            ENDDO
         ENDIF

*     fill histo D
         filled=.false.
         IF ( twojetcut.AND.NMU.gt.0 ) THEN
            DO I=1,NMU
               IF ( .NOT.filled ) THEN
*     muon cuts
                  IF ( REAL(HZPT(MU(I,1))).gt.1.5 ) THEN
                     IF ( REAL(HZETA(MU(I,1))).gt.-1.6.AND.
     +                    REAL(HZETA(MU(I,1))).lt.1.3 ) THEN
*     fill if muon comes from b decay
                        IF ( MU(I,2).gt.0 ) THEN
                           CALL HFILL(4+100*IPROC,REAL(HZPT(MU(I,1))),
     +                          0.,wtx)
*                           filled=.true.
                        ENDIF
                     ENDIF
                  ENDIF
               ENDIF
            ENDDO
         ENDIF

*     fill histo E
         filled=.false.
         IF ( twojetcut.AND.NMU.gt.0 ) THEN
            DO I=1,NMU
               IF ( .NOT.filled ) THEN
*     fill if muon comes from b decay
                  IF ( MU(I,2).gt.0 ) THEN
*     get jet which contains muon and fill in muon regions
                     IF ( MU(I,4).gt.0) THEN 
                        IF ( REAL(HZPT(MU(I,1))).gt.2.5.AND.
     +                       REAL(HZETA(MU(I,1))).gt.-1.6.AND.
     +                       REAL(HZETA(MU(I,1))).LT.1.3 ) THEN
                           CALL HFILL(5+100*IPROC,REAL(JET(MU(I,4),2)),
     +                          0.,wtx)
*                           filled=.true.
                        ENDIF
                     ENDIF
                  ENDIF
               ENDIF
            ENDDO
         ENDIF

*     fill histo F
         filled=.false.
         IF ( twojetcut.AND.NMU.gt.0 ) THEN
            DO I=1,NMU
               IF ( .NOT.filled ) THEN
*     fill if muon comes from b decay
                  IF ( MU(I,2).gt.0 ) THEN
*     get jet which contains muon and fill in muon regions
                     IF ( MU(I,4).gt.0) THEN 
                        IF ( REAL(HZPT(MU(I,1))).gt.2.5.AND.
     +                       REAL(HZETA(MU(I,1))).gt.-1.6.AND.
     +                       REAL(HZETA(MU(I,1))).LT.1.3 ) THEN
                           CALL HFILL(6+100*IPROC,REAL(JET(MU(I,4),10)),
     +                          0.,wtx)
*                           filled=.true.
                        ENDIF
                     ENDIF
                  ENDIF
               ENDIF
            ENDDO
         ENDIF

 65      continue

C==================================================================

      ELSE IF (MOD(iflag,10).eq.3) THEN
C     *** Termination, fix normalisation of MC histograms ***
c         ntot=nevt

         IF (IPROC.EQ.0.OR.IPROC.EQ.2) THEN
            WRITE(HZUNIT,*) 'HZ'//xxxx//': finishing off'
         ENDIF

         CALL HCDIR('//PAWC/'//xxxx,' ')
         CALL HCDIR('//HISTO/'//xxxx,' ')

         IF (Xsec.eq.0) THEN
            WRITE(HZUNIT,*) xxxx
     $           ,': termination called with zero cross section'
            WRITE(HZUNIT,*) '          cross section graph meaningless'
            Xsec=1
         ENDIF
         IF (Ntot.eq.0) THEN
            WRITE(HZUNIT,*) xxxx,' no total events'
            WRITE(HZUNIT,*) '          cross section graph meaningless'
            Ntot=1
         ENDIF
*     norm factor for pb
         lumpb=REAL(Ntot/(xsec))
         write(hzunit,*) '#events=', ntot, 'xsec=', xsec, 'lumpb=',lumpb

c         write(hzunit,*) ' iproc= ',iproc
c         write(hzunit,*) 'HZ',xxxx,': ntot= ', ntot,' xsec (nb)= ',xsec
c         write(hzunit,*) 'norm(',iproc,'): ',norm(iproc)

*     normalise histograms
         CALL HOPERA(100*iproc+1,'+e',100*iproc+1,100*iproc+1,
     +        1/lumpb,0.)
         DO I=2,6
            ID=100*iproc+i
            CALL HZHINRM(ID,0,lumpb,1)
         ENDDO
         id=10+100*iproc
         CALL HZHINRM(ID,0,lumpb,1)
         id=11+100*iproc
         CALL HZHINRM(ID,0,lumpb,1)
         id=12+100*iproc
         CALL HZHINRM(ID,0,lumpb,1)
         
C     adding up direct and resolved
         IF (iproc.eq.2) THEN
            CALL HOPERA(102,'+e',202,2,1.,1.)
            CALL HOPERA(103,'+e',203,3,1.,1.)
            CALL HOPERA(104,'+e',204,4,1.,1.)
            CALL HOPERA(105,'+e',205,5,1.,1.)
            CALL HOPERA(106,'+e',206,6,1.,1.)
            CALL HOPERA(110,'+e',210,10,1.,1.)
            CALL HOPERA(111,'+e',211,11,1.,1.)
            CALL HOPERA(112,'+e',212,12,1.,1.)
         ENDIF
*     
      ELSE
         WRITE(HZUNIT,*) 'HZ',xxxx,' Unknown flag',iflag
      ENDIF
      RETURN
      END
                          
