        Subroutine hzh9505001(iflag)
*****************************************************************************
*
*    Study of the photon remnant in resolved photoproduction at HERA
*    Phys.Lett.B354:163-177,1995
*    hep-ex/9505001
*
*    The histograms are 1,2,3 (-1,-2,-3 for data) corresponding
*    to Figure 3a,b,c of the paper. 
*
*    These histograms are area normalised to one. The corresponding
*    simulated cross sections (i.e. normalised to luminosity) 
*    are in 10,11,12.
*    NB The normalisation in the paper is not clear (it is not to unity, but
*    also not the cross section. Since all conclusions are based on the shapes,
*    we have normalised the data and MC to one - the data therefore has the same
*    shape, but a different normalisation, to that in the paper.
*
* Arguments: iflag=1000*[IPROC]+1 initialisation
*            iflag=1000*[IPROC]+2 filling
*            iflag=1000*[IPROC]+3 termination
*
* IPROC = 1 direct photoproduction
* IPROC = 2 resolved photoproduction
* IPROC = 0 both together
*
*    Beta version: still to be validated.
*    Ben Waugh, Jon Butterworth, UCL
*    Contact: hztool@hep.ucl.ac.uk
*****************************************************************************
*
*     HERA Tuning defined commons
      
      IMPLICIT NONE
      
#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
#include "hzjetcmn.inc"

      REAL HSUM

      integer iflag,iproc
c      DOUBLE PRECISION JETS(50,8)
      DOUBLE PRECISION CONER
      DOUBLE PRECISION px,py,pt,rap,e,pt1,pt2,e3
      
c     cuts arrays
      DOUBLE PRECISION ybJcut(2),q2cut(2)
      DOUBLE PRECISION Rapcut,ptcut,rapcut3,ecut3
      DOUBLE PRECISION rapjet(3), ptjet(3), ejet(3)
      integer njet,chjet,i,j,jetord(3)
      DATA chjet/13/
      
      SAVE ybjcut,q2cut,ptcut,rapcut,rapcut3,ecut3
      
c     Actual data for x-gamma 

      REAL bin(9),band(8),toter(8)

      DATA bin/-4.0,-3.25,-2.45,-1.80,-0.90,-0.25,0.50,1.25,2.00/
      Data band/0.042,0.126,0.237,0.250,0.197,0.147,0.163,0.168/
      data toter/0.01,0.021,0.029,0.021,0.016,0.016,0.016,0.013/

      REAL bin2(8),band2(7),toter2(7)

      DATA bin2/0.0,1.0,2.0,3.0,4.0,5.0,6.0,7.0/
      DATA band2/0.20,0.37,0.23,0.095,0.08,0.028,0.008/
      DATA toter2/0.036,0.05,0.024,0.016,0.02,0.012,0.004/

      REAL bin3(7),band3(6),toter3(6)

      Data bin3/2.0,5.1,8.8,11.5,14.7,17.9,21.0/
      Data band3/0.05,0.091,0.083,0.05,0.032,0.004/
      Data toter3/0.01,0.015,0.01,0.01,0.007,0.003/

      Character*8 xxxx
      Data xxxx /'H9505001'/
      save xxxx


*     Determine the process type.
      iproc=iflag/1000
      if ((iproc.lt.0).or.(iproc.gt.2)) then
         write(hzunit,*) 'HZ',xxxx,': illegal IPROC:',iproc
         return
      endif

      IF (MOD(iflag,10).eq.1) THEN
C     *** Initialisation run ***
C     *** Create directories ***
                  
         write (hzunit,*) 'HZ',xxxx,': booking histograms'

*     Change to  //PAWC (' ' means CHANGE, 'R' means READ)
         call hcdir('//PAWC',' ')
*     Subdirectory xxxx is created. 'S' changes it to the current directory
*     (xxxx is a character variable, which was set to xxxx = '98148')
         if (iproc.le.1) then
            call hmdir(xxxx,'S')
         else
            call hcdir(xxxx,' ')  
         endif
            
         call hcdir('//HISTO',' ')
         if (iproc.le.1) then 
            call hmdir(xxxx,'S')
         else
            call hcdir(xxxx,' ')  
         endif
         
         if (iproc.le.1) then
            CALL HBOOK1(-1,
     &           'Fig. 3a data'
     &           ,8 ,-4.0,2.0,0.)
            CALL HBOOK1(-2,
     &           'Fig. 3b data'
     &           ,7 ,0.0,7.0 ,0.)
            CALL HBOOK1(-3,
     &           'Fig. 3c data'
     &           ,6,2.0,21.0,0.)
            
            
            CALL HPAK(-1,band)
            CALL HPAKE(-1,toter)
            CALL HPAK(-2,band2)
            CALL HPAKE(-2,toter2)
            CALL HPAK(-3,band3)
            CALL HPAKE(-3,toter3)

         END IF
         
         CALL HBOOK1(iproc*100+10,
     &        'Fig. 3a MC component'
     &        ,8 ,-4.0,2.0,0.)
         CALL HBOOK1(iproc*100+11,
     &        'Fig. 3b MC component'
     &        ,7 ,0.0,7.0 ,0.)
         CALL HBOOK1(iproc*100+12,
     &        'Fig. 3c MC component'
     &        ,6,2.0,21.0,0.)
         
         CALL HIDOPT(0,'stat')
         CALL HBARX(0)

c     photoproduction cuts

         ybjcut(1)=0.2
         ybjcut(2)=0.85
         q2cut(1)=0
         q2cut(2)=4
         
c     hzh9505001 cuts for rapidity plots
c     cuts on first two jets     
         ptcut=6.0
         rapcut=1.6
c     cuts on third jet
         rapcut3=-1.0
         ecut3=2.0
         

      ELSE IF (MOD(iflag,10).eq.2) THEN

C     *** Filling runs ***

c         write (*,*) '*** HZH9505001: filling histograms'
         
         CALL HCDIR('//PAWC/'//xxxx,' ')
         CALL HCDIR('//HISTO/'//xxxx,' ')

         
C     *** Fill our histogram ***
         CONER=0
         NJET=3
         
         CALL hzjtfind(chjet,CONER,NJET,Jets)
         
c     search jets in the rapidity interval
                          
         IF ( jets(1,1).lt.rapcut.and.jets(2,1).lt.rapcut) then
            px=jets(1,5)
            py=jets(1,6)
            pt1=SQRT(px*px + py*py)
            
            px=jets(2,5)
            py=jets(2,6)
            pt2=SQRT(px*px + py*py)
            
            
            IF( pt1.gt.ptcut.and.pt2.gt.ptcut) then
               IF (jets(3,4).gt.Ecut3) then


                  rap=jets(3,1)
                  CALL HFILL(INT(iproc*100+10), REAL(rap) , 0. ,wtx)

                  IF (jets(3,1).lt.rapcut3) THEN
         
                     px=jets(3,5)
                     py=jets(3,6)
                     pt=SQRT(px*px + py*py)

                     CALL HFILL(INT(iproc*100+11), REAL(pt) , 0. ,wtx)
                     
                     e3 = jets(3,4)

                     CALL HFILL(INT(iproc*100+12), REAL(e3) , 0. ,wtx)
                  ENDIF
               END IF
            END IF
            
         END IF
                   
      
      ELSE IF (MOD(iflag,10).eq.3) THEN
C     *** Termination run ***
         
C     *** Change directory ***
         CALL HCDIR('//PAWC/'//xxxx,' ')
         CALL HCDIR('//HISTO/'//xxxx,' ')
                     
C     --- normalise resolved/direct component histograms
         call hzhinrm(100*iproc+10,0,real(ntot/xsec),1)         
         call hzhinrm(100*iproc+11,0,real(ntot/xsec),1)         
         call hzhinrm(100*iproc+12,0,real(ntot/xsec),1)         

         IF (iproc.EQ.2) THEN

c     --  combine direct & resolved
            CALL HOPERA(110,'+e',210,10,1.,1.)
            CALL HOPERA(111,'+e',211,11,1.,1.)
            CALL HOPERA(112,'+e',212,12,1.,1.)
         endif

         if (iproc.EQ.0.or.iproc.EQ.2) then
            write (HZUNIT,*) 'HZ'//xxxx//': finishing off'
C     --- now normalise to one
            CALL hzhnorm(10,1,HSUM(10),1)
            CALL hzhnorm(11,2,HSUM(11),1)
            CALL hzhnorm(12,3,HSUM(12),1)
            CALL hzhnorm(-1,0,HSUM(-1),1)
            CALL hzhnorm(-2,0,HSUM(-2),1)
            CALL hzhnorm(-3,0,HSUM(-3),1)

         endif
            
      ELSE
         
C     *** End ***
         PRINT*,'HZH9505001: Please run with iflag set to 1,2 or 3'
         PRINT*,'        with +1000 or +2000 for DIR or RES events.'
      ENDIF
      
      RETURN
      END
      
