*CMZ :  1.01/01 28/03/96  15.50.59  by  Tancredi Carli
*-- Author :
      subroutine HZTH002(IFLAG)
****************************************************************************
* Purpose: Produce fraction of events with a forward jet using a cone
* algorithm (R=1). The forward jet must have xjet > 0.025, 6 < Thetajet < 20
* degrees, 0.25 < pt2jet/q2 < 4 and ptjet > 5
*
* Event selection:
* Eel>12 GeV, 173.0>Thel>157.0, y>0.1 and cut on forward energy
*
* Running: Ee=26.7, Ep=820, no polarization, HERA running 1993
* Reference: Event selection as in Phys. Lett. B356 (1995) 118, DESY 95-108
*            data from publication corrected very crudely with Ariadne 4.03
* Arguments: iflag=1 initialisation
*            iflag=1 filling
*            iflag=3 termination
*
* written by: Leif Lonnblad <leif@nordita.dk> adapted from the hz95108
*             routine by Michael Kuhlen (kuhlen@desy.de)
****************************************************************************
      IMPLICIT NONE
#include "hzfunc.inc"
#include "hepevtp.inc"
#include "heracmn.inc"
*
      double precision pp(4,2000),pp4(4,2000),pp5(4,2000),pj(5,100)
      integer ia(2000),jm(100)
*
      Integer iflag,ihep
*     Not DESY Preprint number
      Character*5 xxxx
*
      Real pi,rd
      Integer iel,idum
      Parameter (pi=3.1415927,rd=180./pi)
      Integer nentry
      Real x,y,q2,enel,thel,efwd,th,lx,lxj
      Real pt2r,thj,xj
      Integer ibin,nj,ij,naj,naj2,np,np4,np5,j
*
      Integer ierr,nev(3),njet(3)
      Double precision ptj2
*
*
*********************************************************************
*
*                      data points from Ariadne 4.03
*
*********************************************************************
      real fjet(3),efjet(3)
      save xxxx,njet,nev
      Data xxxx/'TH002'/,fjet/0.01203,0.02306,0.003225/,
     $     efjet/0.00077,0.00148,0.000321/,njet/3*0/,nev/3*0/,
     $     NENTRY/0/
*********************************************************************
*
*                      Initialization
*
*********************************************************************
      NENTRY=NENTRY+1
*
      IF (iflag.eq.1) then
*
*
* Initialisation: The following MUST always be done
* (i) make subdirectory in PAWC
* - use the name as the xxxxxx in HZxxxxxx subroutine
* (i) make subdirectory in o/p file
*
        Call hcdir('//PAWC',' ')
        call hmdir(xxxx,'S')
        Call hcdir('//HISTO',' ')
        call hmdir(xxxx,'S')
* book data jetrate histos
        call hbook1(-11,'frac events w. f. jet vs kin. bin',3,0.,3.,0.)
*
*
* pack data into histograms
* pack data et/deta in the 9 bins into histograms
*
        Call hpak (-11,fjet)
        Call hpake(-11,efjet)
*
* book your histograms  (data: ID<0)
*
        call hbook1(11,'fraction events w.f.jet vs kin. bin',3,0.,3.,0.)
        call hbook1(12,'# events vs kin. bin',3,0.,3.,0.)
*
        call hbook2(21,'etamax<5 fraction events w.f.jet vs x and xjet',
     $      4,-4.0,-2.0,4,-2.0,-1.0,0.0)
        call hbook2(22,'# events vs x',4,-4.0,-2.0,4,-2.0,-1.0,0.0)
        call hbook2(31,'etamax<7 fraction events w.f.jet vs x and xjet',
     $      4,-4.0,-2.0,6,-2.0,-0.5,0.0)
        call hbook2(32,'# events vs x',4,-4.0,-2.0,6,-2.0,-0.5,0.0)
        call hbook2(41,'etamax<4 fraction events w.f.jet vs x and xjet',
     $      4,-4.0,-2.0,4,-2.0,-1.0,0.0)

*
*********************************************************************
*
*                      Event Processing
*
*********************************************************************

      Else if(iflag.eq.2) then

* Filling: The following MUST always be done
* (i) move to the correct sub-directory in PAWC
*
       call hcdir('//PAWC/'//xxxx,' ')
*
* Event selection
*
       q2=real(HZDISKIN(1))
       x =real(HZDISKIN(2))
       y =real(HZDISKIN(3))
*
* get electron variables for kinematic selection
*
       if (y.lt.0.1) return
       iel=HZIDELEC(idum)
       if (iel.eq.-1) then
         write(*,*) 'Hz'//xxxx,' electron not found '
         return
       endif
       enel=real(PHEP(4,iel))
       if (enel.lt.12.0) return
       thel=real(HZPHMANG(PHEP(3,iel),
     $      sqrt(PHEP(1,iel)**2+PHEP(2,iel)**2)))*rd
       if (thel.lt.160.0) return
       if (thel.gt.173.0) return
*
* cut on forward energy, copy into new vectors
*
       np=0
       np5=0
       np4=0
       efwd=0.
       Do 1200 ihep=1,nhep
         if (isthep(ihep).ne.1.or.ihep.eq.iel) goto 1200
         th=real(HZPHMANG(PHEP(3,ihep),
     &        sqrt(PHEP(1,ihep)**2+PHEP(2,ihep)**2)))*rd
         if(th.gt.4.4.and.th.lt.15.) then
           efwd=real(PHEP(4,ihep))+efwd
         endif
         if (th.lt.0.105) goto 1200
         np=np+1
         do 2200 j=1,4
           pp(j,np)=phep(j,ihep)
 2200    continue
         if (th.lt.0.772) goto 1200
         np5=np5+1
         do 2300 j=1,4
           pp5(j,np5)=phep(j,ihep)
 2300    continue
         if (th.lt.2.099) goto 1200
         np4=np4+1
         do 2400 j=1,4
           pp4(j,np4)=phep(j,ihep)
 2400    continue
 1200  continue
       if (efwd.lt.0.5) return

*
* Check x-limits
*
       lx=log(x)/log(10.0)
       if (lx.lt.-4.0.or.lx.gt.-2.0) return
*
* Normalization
*
       do 1000 j=1,6
         if (j.le.4) call hf2(22,lx,real(j)*0.25-2.125,1.0)
         call hf2(32,lx,real(j)*0.25-2.125,1.0)
 1000  continue

       ibin = 0
       if(x.gt.0.0002.and.x.le.0.001) ibin=1
       if(x.gt.0.0010.and.x.le.0.002) ibin=2
       if (ibin.eq.0) goto 3000

       call hf1(12,real(ibin)-0.5,1.0)
       call hf1(12,2.5,1.0)
       nev(ibin)=nev(ibin)+1
       nev(3)=nev(3)+1
*
* Current detector reach. Old data
*
       call pxcone(2,np4,4,pp4,0.7d0,5.0d0,0.75d0,100,nj,pj,ia,jm,ierr)
       if (ierr.ne.0) goto 3000

       naj=0
       naj2=0
       do 2000 ij=1,nj
         ptj2=pj(1,ij)**2+pj(2,ij)**2
         if (ptj2.lt.25.0) goto 2000
         pt2r=ptj2/q2
         if (pt2r.gt.4.0.or.pt2r.lt.0.25) goto 2000
         thj=real(HZPHMANG(pj(3,ij),sqrt(ptj2)))*rd
         if (thj.gt.20.0.or.thj.lt.6.0) goto 2000
         xj=(pj(3,ij)+pj(4,ij))/1640.0
         lxj=log(xj)/log(10.0)
         call hf2(41,lx,lxj,1.0)
         if (xj.lt.0.025) goto 2000
         naj=naj+1
         if (xj.lt.0.05) goto 2000
         naj2=naj2+1
 2000  continue

       if (naj.gt.0) then
         call hf1(11,real(ibin)-0.5,1.0)
         njet(ibin)=njet(ibin)+1
       endif

       if (naj2.gt.0) then
         call hf1(11,2.5,1.0)
         njet(3)=njet(3)+1
       endif
*
* Forward plug at ZEUS
*
 3000  call pxcone(2,np5,4,pp5,0.7d0,5.0d0,0.75d0,100,nj,pj,ia,jm,ierr)
       if (ierr.ne.0) goto 4000

       do 3100 ij=1,nj
         ptj2=pj(1,ij)**2+pj(2,ij)**2
         if (ptj2.lt.25.0) goto 3100
         pt2r=ptj2/q2
         if (pt2r.gt.4.0.or.pt2r.lt.0.25) goto 3100
         thj=real(HZPHMANG(pj(3,ij),sqrt(ptj2)))*rd
         if (thj.lt.2.099) goto 3100
         xj=(pj(3,ij)+pj(4,ij))/1640.0
         lxj=log(xj)/log(10.0)
         call hf2(21,lx,lxj,1.0)
 3100  continue
*
* full acceptance detector, sort of...
*
 4000  call pxcone(2,np,4,pp,0.7d0,5.0d0,0.75d0,100,nj,pj,ia,jm,ierr)
       if (ierr.ne.0)  return

       do 4100 ij=1,nj
         ptj2=pj(1,ij)**2+pj(2,ij)**2
         if (ptj2.lt.25.0) goto 4100
         pt2r=ptj2/q2
         if (pt2r.gt.4.0.or.pt2r.lt.0.25) goto 4100
         thj=real(HZPHMANG(pj(3,ij),sqrt(ptj2)))*rd
         if (thj.lt.0.284) goto 4100
         xj=(pj(3,ij)+pj(4,ij))/1640.0
         lxj=log(xj)/log(10.0)
         call hf2(31,lx,lxj,1.0)
 4100  continue


*********************************************************************
*
*                      Termination
*
*********************************************************************

      Else if(iflag.eq.3) then

* Termination: The following MUST always be done
* (i) Move to the correct PAW subdirectory
*
       call hcdir('//PAWC/'//xxxx,' ')
*
       call hopera(11,'/E',12,11,1.0,1.0)
       call hopera(21,'/E',22,21,1.0,1.0)
       call hopera(31,'/E',32,31,1.0,1.0)
       call hopera(41,'/E',22,41,1.0,1.0)

       write(*,610) 'Hz'//xxxx//' fwd jetrates:'
       write(*,600) ' 0.0002 < x < 0.001',
     $       njet(1),nev(1),real(njet(1))/real(nev(1))
       write(*,600) ' 0.001  < x < 0.002',
     $       njet(2),nev(2),real(njet(2))/real(nev(2))
       write(*,600) ' xjet > 0.05       ',
     $       njet(3),nev(3),real(njet(3))/real(nev(3))
 610   format(a)
 600   format(a,i6,' /',i6,' =',f12.5)

      endif
*
      RETURN
      END
