*CMZ :  1.01/07 10/10/97  19.47.29  by  Tancredi Carli
*-- Author :
      subroutine HZPARTON(npart,plist)
****************************************************************************
*
*
* purpose: this routine returns a list of the partons (plist)
*          in the HEP-common and their number (npart).
*
* Lepto uses the string hadronisation model (with soft colour interactions).
* Regarded as Partons are the mothers of strings and clusters and
* stable particles of the proton remnant (including resonances).
*
* If Lepto is run in the ARIADNE-modus it uses colour dipole for parton
* showers.
* A list of possible mothers for mothers of clusters and strings is created.
* The daughters of these particles (the string/cluster mothers!) and
* stable particles of the proton remnant are included in the partonlist.
*
* HERWIG: uses cluster hadronization. A list of partons in the
* parton showering is provided.
* It is instructive to have a look at a typical event bank-printout and
* perhaps the HERWIG-manual!
*
* the routine asks for generator type(logical:lepto,ariadne,herwig).
* If other generators are used and if parameters are changed it has to be
* adjusted to the applied generator and hadronisation sequence
* (strings,cluster) and the steering card!
*
* written by: Reimer Selle (rselle@rec06.desy.de)
*
****************************************************************************
      IMPLICIT NONE
#include "hzfunc.inc"
#include "hzhbook.inc"
#include "hepevtp.inc"
#include "heracmn.inc"
*
      Integer plist(nmxhep),npart,imot,motlist(nmxhep)
      Integer zero,start
      Integer iflag,ihep,i,j,k,l
      Integer iel,idum,ierr,ibeam,ee,eb
      Parameter (ee=27.6)
*     DESY Preprint number
      Character*6 xxxx
      Data xxxx/'parton'/

*
      Real pi,dpi,rd
      Logical lp
      Data lp/.false./
      Parameter (pi=3.1415927,dpi=2.*pi,rd=180./pi)
*

       Logical lepto,ariadne,herwig
       Integer moth1,moth2
       Integer istring,icluster
*
*********************************************************************
*
*
*
*                                      'pre-check' for generated events
       iel=HZIDELEC(idum)
       if (iel.eq.-1) then
        write(6,*) 'Hz'//xxxx,' electron not found '
        goto 9990
       endif


*
       ierr=HZIBEAM(ibeam,idum)
       if (.not.(ierr.eq.1)) then
        write(6,*) 'HZ'//xxxx,' beams not found ! '
        goto 9990
       endif
*
       eb=real(PHEP(4,idum))
       if (abs(abs(eb)-ee).gt.0.2) then
        write(6,*) 'HZ'//xxxx,' Electron beam not at ',ee,'  ! '
        goto 9990
       endif

*                                    all initial values set to 0
       moth1=0
       moth2=0
       icluster=0
       istring=0
       npart=0
C*TC>
C      call vzero(plist,nmxhep)
       do i=1,nmxhep
        plist(i)=0
       enddo
C*TC<
*                                    LEPTO-sequence
      if (GEN(1:3).eq.'LEP'.or.GEN(1:3).eq.'RAP'
     &	.or.GEN(1:3).eq.'CAS') then
       lepto=.true.
      endif

      IF (lepto) then

       Do 2000 ihep=1,nhep
*                               find mothers of strings and clusters

       if ((IDHEP(ihep).eq.91).or.(IDHEP(ihep).eq.92)) then
         moth1= JMOHEP(1,ihep)
         moth2= JMOHEP(2,ihep)

         if (IDHEP(ihep).eq.92) istring=ihep
         if (IDHEP(ihep).eq.91) icluster=ihep

         if (moth2.lt.moth1) then
          write(*,*)'Hzparton: moth2<moth1!'
          goto 2000
         endif
*                                 put partons in plist

          do k=moth1,moth2
*                                 avoid fake-mothers!
*
          if (JDAHEP(1,k).eq.istring.or.JDAHEP(1,k).eq.icluster) then
           npart=npart+1
           plist(npart)= k
           if (lp) write(*,*)'plistnpart',plist(npart)
          endif
          enddo
           if(lp) write(*,*)'npart:',npart
        endif

*                                    put undecayed particles with mother=2
*                                    (proton) in plist

         if (JMOHEP(1,ihep).eq.2.and.ISTHEP(ihep).eq.1) then
          npart=npart+1
          plist(npart)=ihep
         endif

*                                    add resonances
*               --> particles coming from beam proton, decaying (at least
*                   partially)into stable particles
*                   e.g. Delta+ -->p+ + pi0
         if (JMOHEP(1,ihep).eq.2.and.ISTHEP(ihep).eq.2) then
*                                    skip gammas,gluons
          if (IDHEP(ihep).eq.21.or.IDHEP(ihep).eq.22) goto 2000
*                                    skip quarks,diquarks
          j=abs(IDHEP(ihep))
          if (j.le.6.and.j.ne.0) goto 2000
          if (j.eq.2203.or.j.eq.1103.or.j.eq.2101.or.j.eq.3201.or.
     &        j.eq.2103.or.
     &        j.eq.3203.or.j.eq.3103.or.j.eq.3101) goto 2000

           npart=npart+1
           plist(npart)=ihep
         endif

2000     continue
*                                   Terminate lepto-seq.
      ENDIF
*                                    ARIADNE-Sequence
       if (GEN(1:3).eq.'ARI'.or.GEN(1:3).eq.'DJA') then
        ariadne=.true.
       endif

       if (ariadne) then
C*TC>
        imot=0
C        call vzero(motlist,nmxhep)
        do i=1,nmxhep
         motlist(i)=0
        enddo
C*TC<
        do 3000 ihep=1,nhep
*                                    skip documentation,beam particles
        if (ISTHEP(ihep).ne.3.and.ISTHEP(ihep).ne.201)then

*                                    Find particles without mother
*                                    or coming from beam proton and put
*                                    them in motlist

         if (JMOHEP(1,ihep).eq.0.or.JMOHEP(1,ihep).eq.2) then
          imot=imot+1
          motlist(imot)=ihep
         endif

*                                    put undecayed particles with mother=2
*                                    (proton) in plist

         if (JMOHEP(1,ihep).eq.2.and.ISTHEP(ihep).eq.1) then
          npart=npart+1
          plist(npart)=ihep
         endif
        endif
*                                    no partons in hep-common after
*                                    first string

         if (IDHEP(ihep).eq.92) istring=ihep
         if (istring.ne.0) goto 3100
3000     continue
3100     continue

          if (lp) then
           do i=1,imot
            write(*,*) i,'**motlist',motlist(i)
           enddo
          endif
*                                add particles with mother in motlist
        if (imot.ne.0) then
         do ihep=1,nhep
          do i=1,imot
          if (JMOHEP(1,ihep).eq.motlist(i)) then
           npart=npart+1
           plist(npart)=ihep
          endif
          enddo
         enddo
        endif
*                                      Terminate Ariadne-seq.
      ENDIF

*                                      Begin HERWIG-SEQUENCE
      if (GEN(1:3).eq.'HRW') then
       herwig=.true.
      endif

      IF (herwig) then
*                                      set initial values to 0
       npart=0
C*TC>
C      call vzero(plist,nmxhep)
       do i=1, nmxhep
        plist(i)=0
       enddo
C*TC<
       do 4000 ihep=4,nhep
*                                      start at 4 according to actual
*                                      particle listing in herwig.
*                                      (ihep=1-3:initial state)
*                                      for the meanings of the different
*                                      ISTHEPs -->herwig-manual!

*                                      no partons after parton shower
        if (ISTHEP(ihep).gt.144) goto 4100
*                                      get partons!
         if (ISTHEP(ihep).eq.2) then
          npart=npart+1
          plist(npart)=ihep
         endif
 4000    continue
 4100    continue

*                                Terminate HERWIG
       ENDIF


       if(lp) Write(*,*)'npartfin=',npart

*                                 check on correct hadronisation
        if(lp) Write(*,*)'istring=',istring
        if (GEN(1:3).eq.'LEP'.or.GEN(1:3).eq.'RAP'
     &	.or.GEN(1:3).eq.'CAS') then
        if (lp.and.istring.eq.0) write(*,*)'No string found in event'
       endif
9990  continue

      RETURN
      END
