*CMZ :  1.00/01 21/02/96  13.54.17  by  Tancredi Carli
*-- Author :
C ===============================================================
      SUBROUTINE EUCELL( E1,E2,  E3, E4, E5)
C#      SUBROUTINE EUCELL( E1,E2,  E3, E4, E5)
C#      ======================================
C#************************************************************************
C#
C@# EUCELL : adapted version of LUCELL jet finder ( cone algorithme).
C#
C# GENERAL DESCRIPTION  and METHOD :
C# -- This is an adapted version of LUCELL
C#   It is a simple jet finder in an eta-phi coordinate frame.
C#   A cone algorithm is used.   We don't do more energy smearing.
C#
C#   Energy signals (e.g. UCAL cells) are treated as massless particles.
C#   Groups of 9 cells (matched to cone angle)  are used as 'windows'.
C#   If sufficient integrated ET is found in a given window, the
C#   eta-phi value is used as a candidate for a possible jet direction.
C#   ET is the main jet criterion.  The jet is aligned along a
C#   weighted eta, phi direction if this preserves the jet ET.
C#
C#   The main vertex is taken into account  (default = 0., 0., 0.,).
C#
C#-----------------------------------------------------------------------
C#   USAGE:  from an EAZE job two calls are needed.
C#            CALL EUFILL                  (to fill some arrays)
C#            CALL EUCELL(E1,E2,E3,E4,E5)
C#  After this, optionally,
C#            CALL EUPIC
C#  will give line-printer diagrams of the energy flow and found jets.
C#  The calling program must contain  COMMON/EUJETS/     (see below)
C#
C#  INPUT PARAMETERS:      E1,  E2,      E3,     E4,        E5,
C#  (corresponding to    ..PARU(51)..   (52)    (53)       (54)
C#    in LUCELL)        etamax,etamin   ETmin  ETjet0   cone-radius
C#                     rapidity limits   GeV     GeV      radians
C#                      on cells used                    (if fixed)
C#                   (not on jet found)
C#         Defaults :      3.2, -3.0,    1.0,    2.5,      0.7
C#   (E1 =0. AND E2 =0.), E3 =0., E4 =0., or E5 =0. selects default.
C#   ETmin is the minimum ET of a window that we wish to consider
C#   for a jet search.
C#
C#   In addition, threshold E and Etransverse values for individual
C#   EMC, HAC cells must be stored in EEMC0, EHAC0, ETEMC0, ETHAC0.
C#   (The PUCELL defaults are 0.030, 0.080, 0., 0., respectively.)
C#
C#   OUTPUT DATA:  contained in KEEP SEQUENCE EUJETS :
C# +KEEP, EUJETS.
C#       Parameter (ncusmx=500)
C#       COMMON/EUJETS/ NJET,KJ(50,5),PJ(50,12),ETEMC0,ETHAC0,EEMC0,EHAC0
C#      1, NCUSE,KCUSE(ncusmx,5),PCUSE(ncusmx,10)
C#
C#   where :
C#     NJET = number of jets found (up to 50)
C#     PJ(i,j) = parameters of i-th jet.
C#          j  = 1,2     eta, phi  of cone axis.
C#               3,4     eta, phi (ET weighted calculation, may equal
C#                                 1,2)
C#                5      ET
C#               6-10    Momentum 4-vector and effective mass of jet.
C#                11     'Thrust' value of jet
C#                12     Energy of window used.
C#     KJ(i,j) has
C#          j  = 1,2,3   31, 98, number of jet (1,2,3 etc)
C#                4      100*no. of hac cells in jet + no. of emc cells
C#
C#     NCUSE  = no. of cells above threshold (includ. outside eta range)
C#     PCUSE(i,j) contains their parameters.  Useful ones are:
C#          j  = 1,2      eta, phi
C#               4,5      E, ET
C#               6,7      d(eta), d(phi) (+/-) covered by cell.
C#
C#  YOU MAY VARY THE CONE RADIUS with eta by supplying your own sub-
C#  routine to do this.   It must be called  SUBROUTINE PCONRAD(eta,rad)
C#  and output an angle 'rad' in radians.  It must be loaded to
C#  be used in priority over the default version in the EUCELL package.
C#**********************************************************************
C#   MODIFICATION LOG :
C#
C#*  Feb 94 - code tidied up.
C#***********************************************************************
C#   Nov 95 - cuts for quicker termination of the jet search tuned a bit.
C#          - max no. of iterations reduced.
C#          - quicker termination is now OPTIONAL.
C#            TO SWITCH IT ON, make the fourth parameter (min jet ET0) -ve.
C#            i.e. set -5. instead of 5. to get 5 GeV eT jets with quicker
C#            search termination.
C#***********************************************************************
*
      IMPLICIT Real (A-H,O-Z), Integer(I-N)
      Parameter (ncell1=500)
      COMMON/EVOKE2/ nicell,Vmain(3),Uxyze(4,ncell1),ifbreh(2,ncell1)
      COMMON/EUCONE/ifixr
      COMMON/EUPICN/ etamax,netas,nphi,etamin,IE0
#include "eujets.inc"
      DIMENSION ETCELL(32,40), Klist(32,40), etaLIM(41), xyz(3)
     *,E0(2),ET0(2),KCcand(ncusmx)
      Data  par1, par2, par3, par4, par5 /3.2, -3.0, 1.0, 2.5, 0.7/
     1,deta/0.40/, nphi0/16/, pi,twopi/3.140592654,6.283185308/
      Data  indone/0/
      Equivalence (E0(1),Eemc0), (ET0(1),ETemc0)
      Save  dphi,deta,nphi0,pi,twopi,init,NJ

      Logical  termin
*
      termin=.false.
      If(E4.lt.0.) termin=.true.
      ienter=0
      Ifixr=0
      Itmax=12
      If(Indone.eq.0) then
       pi=atan2(0.,-1.)
       if(pi.lt.0.) pi=-pi
       twopi=2.*pi
        Call Uzero(PCUSE,1,10*NCUSMX)
C*TC<
C       Call Uzero(KCUSE,1, 5*NCUSMX)
        do i=1,NCUSMX
         do j=1,5
          KCUSE(i,j)=0
         enddo
        enddo
C*TC>
        Write(6,4)
   4    Format(/' First call to EUCELL, version Nov 95')
        Indone=1
      else
        Call Uzero(PCUSE,1,10*NCUSE)
C*TC<
C        Call Uzero(KCUSE,1, 5*NCUSE)
        do i=1,NCUSE
         do j=1,5
          KCUSE(i,j)=0
         enddo
        enddo
C*TC>
      endif
*
C*TC<
C       Call Uzero(klist,1,1280)
        do i=1,32
         do j=1,40
          klist(i,j)=0
         enddo
        enddo
C*TC>
      Call PCONRAD(0.,rcone0)
*                          -- to find if fixed cone radius is used.
      NJET=0
      If(nicell.eq.0) RETURN
      etamax=E1
      etamin=E2
      If(etamax.le.etamin) then
                    etamax= par1
                    etamin= par2
                      endif
      ETseed=abs(E3)
      If(ETseed.eq.0.) ETseed= par3
      ETjet0=abs(E4)
      If(ETjet0.eq.0.) ETjet0= par4
      cone0 =E5
      If(cone0 .le.0.) cone0 = par5
      If(Ifixr.ne.1) cone0=rcone0
       dphi0=twopi/nphi0
       nphi=nphi0
       If(dphi0.lt..50*cone0) nphi = 3*nphi0/4
       If(dphi0.lt..36*cone0) nphi =   nphi0/2
       If(dphi0.gt..65*cone0) nphi = 3*nphi0/2
       If(dphi0.gt.1.0*cone0) nphi = 2*nphi0
       dphi = twopi/float(nphi)
* --- Try to match cell size to cone size.  Dphi heavily quantized.
      deta=amax1( 0.50*cone0, 0.1)
      neta = max0(3, min0( INT((etamax-etamin)/deta) +1 , 40)  )
      deta1= (etamax-etamin)/neta
      IEhigh = max0(neta-1, 1)
*
*------- load up an array of eta bin boundaries starting from proton end
      eta = etamax
      neta=1
      deta2=deta1
      etalim(1)=etamax
  10  If(ifixr.ne.1) then
        Call PCONRAD(eta,rcone)
        deta2=amax1(0.5*rcone, 0.1)
        endif
      neta=neta+1
      etalim(neta)=etalim(neta-1) - deta2
      If(etalim(neta).gt.etamin.and.neta.lt.41) go to 10
      If(neta.ge.41) Write(6,*) ' EUCELL warning -- cone too small?'
      etafac=1.0001*(etamax-etamin)/(etamax-etalim(neta))
      IE0=0
      Do 15 i=2,neta
      etalim(i) = etamax - (etamax-etalim(i))*etafac
      If(etalim(i).lt.0.and.IE0.eq.0) IE0=i-1
  15  continue
         netas=neta
*
* - First load up the UCAL signals.
      NCUSE = 0
      ETACC=0.
      Do  30 i=1,nicell
      ifbr= ifbreh(1,i)
      ieh = ifbreh(2,i)
      Do 22 j=1,3
  22  xyz(j) = uxyze(j,i) - vmain(j)
      If(vmain(1)**2 + vmain(2)**2.gt.500.) then
          xyz(1) = uxyze(1,i)
          xyz(2) = uxyze(2,i)
          xyz(3) = uxyze(3,i)
               endif
*                         ( bad vertex.)
*
* -- First evaluate the basic quantities.
      x= xyz(1)
      y= xyz(2)
      z= xyz(3)
      r= sqrt(x**2 + y**2)
      rr   = 1./sqrt(r**2 +z**2)
      s = r*rr
      c = z*rr
      ET = uxyze(4,i)*s
*-    ----------- next section won't apply if we're using hadrons. -------
      If(ifbr.ge.1.and.ifbr.le.3.and.ieh.ge.1.and.ieh.le.2) Then
*-                       - In HAC, move longitudinal position in a bit.
        If(ieh.eq.2) then
          If(ifbr.ne.2) z = z - sign( (50. - 40.*abs(c)), z)
          If(ifbr.eq.2) r = r - (45. - 40.*s)
cc        If(ifbr.ne.2) z = z - sign(20.,z)
cc        If(ifbr.eq.2) r = r - 20.
          rr   = 1./sqrt(r**2 +z**2)
          s = r*rr
          c = z*rr
          ET = uxyze(4,i)*s
          endif
* -                                       -- check minimum energy.
        If(uxyze(4,i).lt.E0(ieh)) go to 30
        If(ET.lt.ET0(ieh)) go to 30
        EndIf
*      ---------------------------
      eta = sign(LOG( (1.+ABS(c))/s ),c)
      phi = Atan2(y,x)
      If(ABS(phi).GT.pi) phi = phi-sign(twopi,phi)
      IF(NCUSE.GE.ncell1) NJET=-2
      IF(NCUSE.GE.ncell1) RETURN
      NCUSE=NCUSE+1
      PCUSE(NCUSE,1)=eta
      PCUSE(NCUSE,2)=phi
      PCUSE(NCUSE,4)=uxyze(4,i)
      PCUSE(NCUSE,5)=ET
      PCUSE(NCUSE,6)=0.
      PCUSE(NCUSE,7)=0.
      IE=0
      If(eta.ge.etamin .and. eta.lt.etamax) Then
       ETACC=ETACC+ET
 28    IE=IE+1
       If(IE.gt.41) write(6,*)' SERIOUS warning. EUCELL IE out of range'
       If(IE.gt.41) STOP
         If(eta.lt.etalim(IE+1)) go to 28
       Endif
      KCUSE(NCUSE,1) = IE
      KCUSE(NCUSE,2) = min0(nphi, int((phi+pi)/dphi) + 1 )
      KCUSE(NCUSE,3) = i
      KCUSE(NCUSE,4) = 1 + 99*(ieh-1)
      KCUSE(NCUSE,5) = 0
*                 --- KCUSE(i,5) = or 0 = free, >0 = jet
  30  continue
      If(ETACC.lt.ETjet0) RETURN
      NJ=0
*
C...Put into cells.   Use highest ET as seed for jet attempt.
C                     * * * * Ignore signals already used. * * *
 40   Call Uzero(ETCELL,1,1280)
      IEhigh = max0(neta-1, 1)
      DO 45 I=1,NCUSE
      If(KCUSE(I,1).eq.0.or.KCUSE(I,5).gt.0) go to 45
      IE=KCUSE(I,1)
      IP=KCUSE(I,2)
      ETCELL(IP,IE) = ETCELL(IP,IE) +PCUSE(I,5)
  45  CONTINUE
      ETbest = 0.
      ETAbst = 99.
      PHIbst = 99.
      KLIM=-1
*
C...Look at groups of 9 cells if possible.
  60    EThigh=0.
        IEtop=IEhigh
        IEhigh=0
        DO 90 IP=1,nphi
        ip1=IP+1
        ip2=IP+2
        IF(ip1.gt.nphi) ip1=ip1-nphi
        IF(ip2.gt.nphi) ip2=ip2-nphi
        ip0=IP-1
        ip3=IP+3
        IF(ip0.le.  0 ) ip0=ip0+nphi
        IF(ip3.gt.nphi) ip3=ip3-nphi
        Do 80 IE=1,IEtop
        If(Klist(ip1,IE).lt.KLIM) go to 80
        Klist(ip1,IE) = 0
        E9  =  ETCELL(ip,ie)   + ETCELL(ip1,ie)   + ETCELL(ip2,ie)
     1      +  ETCELL(ip,ie+1) + ETCELL(ip1,ie+1) + ETCELL(ip2,ie+1)
        If(IE.gt.1) E9 =
     2   E9 +  ETCELL(ip,ie-1) + ETCELL(ip1,ie-1) + ETCELL(ip2,ie-1)
        If(E9.lt.ETseed) then
          Klist(ip1,IE)=-2
          Go to 80
          endif
*------ within a broader window see if there could be enough energy.
        IF(termin) THEN
        E25 = E9  + ETCELL(ip0,ie)   +ETCELL(ip3,ie)   +ETCELL(ip0,ie+1)
     1            + ETCELL(ip3,ie+1)
        If(Ie.gt.1)
     1  E25 = E25 + ETCELL(ip0,ie-1) +ETCELL(ip3,ie-1)
        If(Ie.gt.2)
     1  E25 = E25 + ETCELL(ip0,ie-2) +ETCELL(ip1,ie-2)
     2            + ETCELL(ip ,ie-2) +ETCELL(ip2,ie-2) +ETCELL(ip3,ie-2)
        If(Ie.le.neta-2)
     1  E25 = E25 + ETCELL(ip0,ie+2) +ETCELL(ip1,ie+2)
     2            + ETCELL(ip ,ie+2) +ETCELL(ip2,ie+2) +ETCELL(ip3,ie+2)

        If(E25.lt.0.3*ETJET0.or.E25.lt.0.2*ETbest) then
          Klist(ip1,IE)=-3
          Go to 80
          endif
        ENDIF
*
        If(IEhigh.lt.IE) IEhigh=IE
* --               --  aim is to limit maximum of eta scan.
        If(E9.gt.EThigh) then
           IETAS=IE
           IPHIS=ip1
           EThigh=E9
           endif
  80    Continue
  90    Continue
      KLIM=0
      IF(EThigh.lt.ETseed) then
*- No more seeds.  Keep stored best jet if there.  --> Otherwise finish.
         IF(ETbest.lT.ETjet0) go to 210
         NJ=NJ+1
         Klist(IPHISS,IETASS)=-5
         go to 40
*                  ----> look for next jet.
      endif
*
      Klist(IPHIS,IETAS)=-1
      If(IPHIS.gt.nphi) iphis=iphis-nphi
      ETA=0.5*(etalim(IETAS) + etalim(IETAS+1))
      PHI= -pi + (float(IPHIS) -0.5)*dphi
      IF(NJ.GE.500) then
        Write (6,*) 'EUCELL: no more memory left in PJ,KJ '
        NJET=-2
        RETURN
        endif
      Iter =0
      EJmax=0.00
*                         First NT iterations are more tolerant.
      NT=2
* ----------------------- Start of Jet iteration loop.
 150  Iter = Iter +1
      KJ4 =0
      PJ1=ETA
      PJ2=PHI
      PJ3=0.
      PJ4=0.
      PJ5=0.
*
C...Sum up free cells within required distance of initiator.
      If(ifixr.ne.1) call PCONRAD(eta,cone0)
      dcone=amax1(0.1*cone0, 0.1)
      DO 160 IC=1,NCUSE
      KCcand(IC) = 0
      If(KCUSE(IC,1).eq.0) go to 160
      If(KCUSE(IC,5).gt.0.and.KCUSE(IC,5).le.NJ) go to 160
      phic=PCUSE(IC,2)
      IF(abs(phic-phi).GT.pi) phic=phic+SIGN(twopi,phi)
      dazi=phic-phi
      IF( (abs(PCUSE(IC,1)-eta)-PCUSE(IC,6))**2
     1      + (abs(dazi)-PCUSE(IC,7))**2 .GT. cone0**2) go to 160
      KCcand(IC) = 1
      KJ4 = KJ4 +KCUSE(IC,4)
      PJ3 = PJ3 +PCUSE(IC,5)*PCUSE(IC,1)
      PJ4 = PJ4 +PCUSE(IC,5)*PHIC
      PJ5 = PJ5 +PCUSE(IC,5)
  160 CONTINUE
*          --- KJ(n,4) contains 100*no. of hac cells + no. of emc cells
*
      If(PJ5.gt.EJmax) then
       EJmax=PJ5
       EtaJM=eta
       PhiJM=phi
         endif
*
* ------- Stop iterating if it is too bad--------
      IF(termin) THEN
       If((Iter.gt.NT.or.Itmax.le.NT).and.PJ5.LT.0.4*ETjet0) goto 170
       If((Iter.gt.NT.or.Itmax.le.NT).and.EJmax.LT.0.4*ETbest) goto 60
       If((Iter.gt.2*NT.or.Itmax.le.NT).and.PJ5.LT.0.6*ETjet0) goto 170
       If((Iter.gt.2*NT.or.Itmax.le.NT).and.EJmax.LT.0.6*ETbest) goto 60
       ENDIF
      If(PJ5.lt.0.1*ETjet0) go to 170
*                                            (zero division protect)
      PJ3=PJ3/PJ5
      PJ4=PJ4/PJ5
      dazi=abs(PJ4-phi)
      If(ABS(PJ4).GT.pi) PJ4=PJ4-SIGN(twopi,PJ4)
      eta1=PJ3
      If(dazi.gt.pi) dazi=dazi-twopi
      If(Iter.eq.1 .or. (eta1-eta)**2+dazi**2 .gt.dcone**2) then
*--- Another iteration if this is the first, or if jet has changed.
             phi=PJ4
             eta=eta1
             If(Iter.ge.Itmax) go to 170
             Go to 150
           endif
*--- Prepare to accept the jet provisionally.
      If( PJ5.ge.ETjet0.and.PJ5.gt.ETbest) then
           ETAbst=  PJ3
           PHIbst=  PJ4
           ETbest = PJ5
           go to 180
           endif
*--- Otherwise, useless....... look for the next seed.
  170  continue
       If(PJ5.lt.ETjet0) Klist(IPHIS,IETAS)=-4
*                         .... flag cell used as failed seed.
       Go to 60
*----------------------------------------------- end of iteration loop
*
*---------------------- jet has been accepted.  Store its parameters.
*                       (They may be overwritten by a better jet.)
  180 PJ(NJ+1,1) = PJ1
      PJ(NJ+1,2) = PJ2
      PJ(NJ+1,3) = PJ3
      PJ(NJ+1,4) = PJ4
      PJ(NJ+1,5) = PJ5
      DO 190 J=6,9
  190 PJ(NJ+1,J)=0.
      KJ(NJ+1,4)=KJ4
      thr=0.
*-- Collect flagged signals.     KCcand(IC) labels signal in jet.
      DO 200 IC=1,NCUSE
        If(KCUSE(IC,5).gt.NJ) KCUSE(IC,5)=0
        IF(KCcand(IC).eq.0) go to 200
        PJ(NJ+1,6)=PJ(NJ+1,6)+PCUSE(IC,5)*COS(PCUSE(IC,2))
        PJ(NJ+1,7)=PJ(NJ+1,7)+PCUSE(IC,5)*SIN(PCUSE(IC,2))
        PJ(NJ+1,8)=PJ(NJ+1,8)+PCUSE(IC,5)*SINH(PCUSE(IC,1))
        PJ(NJ+1,9)=PJ(NJ+1,9)+PCUSE(IC,5)*COSH(PCUSE(IC,1))
        KCUSE(IC,5)= NJ+1
* calculate a thrust value.
        ce = cos(PCUSE(IC,1)-PJ(NJ+1,1))
        cp = cos(PCUSE(IC,2)-PJ(NJ+1,2))
        thr = thr + PCUSE(IC,5)/sqrt(1./cp**2 +1./ce**2 -1.)
  200   CONTINUE
      PJ(NJ+1,11)=thr/PJ(NJ+1,5)
      PJ(NJ+1,12)=Ethigh
      IETASS=IETAS
      IPHISS=IPHIS
      Go to 60
*------------------------------------------------------------------
*    Finished!   Fill in some more numbers.
 210  NJET=NJ
      DO 250 I=1,NJET
      KJ(I,1)=31
      KJ(I,2)=98
      KJ(I,3)=I
      KJ(I,5)=0
C...Convert to  massive four-vectors.
 250  PJ(I,10)=SQRT(MAX(0.,PJ(I,9)**2-PJ(I,8)**2-PJ(I,7)**2-PJ(I,6)**2))
      RETURN
      END
