*CMZ :  1.01/13 13/02/98  16.11.24  by  Tancredi Carli
*-- Author :    J.Kurzhoefer 19/07/93
*
      SUBROUTINE H1QGCONE(RAPMAX,NRAPB,NAZMB,ETMIN,ETSMIN,RJET,
     +                  NJETS,AJETS)
C---------------------------------------------------------------------
C     Cone algorithm using a pseudorapidity and phi grid. The sequence
C     of jet building is determined by the pt of the jet.
C---------------------------------------------------------------------

      PARAMETER (MAXCE=110,MAXJET=10,MAXJTMP=10000)
      REAL CELL(MAXCE,MAXCE),JETPT(MAXJTMP)
      REAL QQPI,QQ2PI
      PARAMETER (QQPI=3.141593, QQ2PI=QQPI*2.)
      INTEGER LOCK(MAXCE,MAXCE),JETCELL(MAXJTMP,2),INDEX(MAXJTMP)
      REAL AJETS(MAXJET,3)
      PARAMETER (SMALL=1.E-20)

#include "hepevtp.inc"

      NJETS = 0
      JETCAND = 0
C Control some input parameters
      IF(NRAPB.GT.MAXCE) THEN
         PRINT *,'H1QGCONE: TOO MANY ETA CELLS'
         GOTO 99
      ENDIF
      IF(NAZMB.GT.MAXCE) THEN
         PRINT *,'H1QGCONE: TOO MANY PHI CELLS'
         GOTO 99
      ENDIF
C Initialize some arrays
      MAXCEQ = MAXCE*MAXCE

      CALL VZERO(CELL,MAXCEQ)
      CALL VZERO(JETPT,MAXJTMP)
      CALL VZERO(AJETS,3*MAXJET)
      DO 3 I=1,MAXCE
         DO 5 J=1,MAXCE
            LOCK(I,J) = 0
 5       CONTINUE
 3    CONTINUE

*
      BINETA = 2.*RAPMAX/FLOAT(NRAPB)
      BINPHI = 2.*QQPI/FLOAT(NAZMB)
      NBETA = RJET/BINETA + 1
      NBPHI = RJET/BINPHI + 1
*
*     Fill the cell grid here
*==================================================================
*
        COEF=1.
* Is Z axis in the proton direction?
        DO I=1,NHEP
          PZ   = PHEP(3,I)
          ICOD = IDHEP(I)
          IF (ICOD.EQ.2212) then
            IF (PZ.LT.0.) COEF=-1.
          END IF
        ENDDO
*
      DO 10 I=1,NHEP
         X  = PHEP(1,I)
         Y  = PHEP(2,I)
         Z  = PHEP(3,I)*COEF
         E  = PHEP(4,I)
	 ISTA = ISTHEP(I)
	 IF(ISTA.NE.1) GO TO 10
         PT = SQRT(X**2+Y**2)
         IF(PT.LE.SMALL) GOTO 10
         ETA=SIGN(LOG((SQRT(PT**2+Z**2)
     *                +ABS(Z))/PT),Z)
         IF(ABS(ETA).GT.RAPMAX) THEN
            GOTO 10
         ENDIF
         PHI = ATAN2(Y,X)
         IF(PHI.LT.0.) PHI = QQ2PI + PHI
         IETA = MIN(NRAPB,1 + INT((RAPMAX+ETA)/BINETA))
         IPHI = MIN(NAZMB,1 + INT(PHI/BINPHI))
         CELL(IETA,IPHI) = SIGN(1.0,E)*PT + CELL(IETA,IPHI)
 10   CONTINUE
*
****************************************************************
*
C     loop over all cells
*
      ET1 = 0.0
      IET1 = 0
      JET1 = 0
      DO 60 ICELL=1,NRAPB
         DO 70 JCELL=1,NAZMB
            IF(CELL(ICELL,JCELL).LT.ETMIN) GOTO 70
C If the cell can be used as a jet initiator, build the jet
            ET1 = CELL(ICELL,JCELL)
            IET1 = ICELL
            JET1 = JCELL
            ETAET1 = -RAPMAX + BINETA*FLOAT(IET1) - BINETA/2.0
            PHIET1 =           BINPHI*FLOAT(JET1) - BINPHI/2.0

*
*     Collect cells
*
            IMIN = MAX0(1,IET1-NBETA)
            IMAX = MIN0(NRAPB,IET1+NBETA)
*
            ETJET = 0.
            DO 80 II=IMIN,IMAX
               DO 90 JJ=1,NAZMB
                  IF(CELL(II,JJ).EQ.0.) GOTO 90
                  ETA = -RAPMAX + BINETA*FLOAT(II) - BINETA/2.0
                  PHI =           BINPHI*FLOAT(JJ) - BINPHI/2.0
                  DPHI = ABS(PHI-PHIET1)
                  IF(DPHI.GT.QQPI) DPHI = QQ2PI - DPHI
                  DR=SQRT((ETA-ETAET1)**2+DPHI**2)
                  IF(DR.GT.RJET) GOTO 90
                  ETJET = ETJET + CELL(II,JJ)
 90            CONTINUE
 80         CONTINUE
*
C If the PT of the jet is above ETSMIN, store jet parameter
            IF (ETJET.LT.ETSMIN) GOTO 70
            JETCAND=JETCAND+1
            IF (JETCAND.GT.MAXJTMP) THEN
               JETCAND=JETCAND-1
               PRINT *,'H1QGCONE: TOO MANY TEMPORARY JETS'
            ENDIF
            JETPT(JETCAND)=ETJET
            JETCELL(JETCAND,1)=IET1
            JETCELL(JETCAND,2)=JET1
 70      CONTINUE
 60   CONTINUE
*
C  If no jet was found the work is done
      IF (JETCAND.EQ.0) GOTO 99

C Sort the jet candidates using CERNLIB ROUTINE M101
      CALL SORTZV(JETPT,INDEX,JETCAND,1,0,0)

C Build the jet with the highest pt
 150  ETJET = JETPT(INDEX(JETCAND))
      IET1 = JETCELL(INDEX(JETCAND),1)
      JET1 = JETCELL(INDEX(JETCAND),2)

      ETAET1 = -RAPMAX + BINETA*FLOAT(IET1) - BINETA/2.0
      PHIET1 =           BINPHI*FLOAT(JET1) - BINPHI/2.0

      IMIN = MAX0(1,IET1-NBETA)
      IMAX = MIN0(NRAPB,IET1+NBETA)

*        Correct Eta and Phi
*
         ETAJET = 0.
         PHIJET = 0.
         ETJETD = 0.
*
         DO 100 I=IMIN,IMAX
            ETA = -RAPMAX + BINETA*FLOAT(I) - BINETA/2.0
            DO 110 J=1,NAZMB
               IF (CELL(I,J).EQ.0.) GOTO 110
               IF (LOCK(I,J).GT.1)   GOTO 110
               PHI =       BINPHI*FLOAT(J) - BINPHI/2.0
               DPHI = PHI-PHIET1
               IF(ABS(DPHI).GT.QQPI) DPHI = DPHI-SIGN(QQ2PI,DPHI)
               DR=SQRT((ETA-ETAET1)**2+(DPHI)**2)
               IF(DR.GT.RJET) GOTO 110
               LOCK(I,J) = 2
               ETAJET = ETAJET + ETA *MAX(0.,CELL(I,J))
               PHIJET = PHIJET + DPHI*MAX(0.,CELL(I,J))
               ETJETD = ETJETD +      MAX(0.,CELL(I,J))
 110        CONTINUE
 100     CONTINUE

         ETAJET = ETAJET / ETJETD
         PHIJET = PHIJET / ETJETD + PHIET1
         IF (PHIJET.GT.QQ2PI) PHIJET = PHIJET - QQ2PI
         IF (PHIJET.LT.0.) PHIJET = PHIJET + QQ2PI
*
         IF (NJETS.LT.MAXJET) THEN
            NJETS = NJETS + 1
            AJETS(NJETS,1) = ETJET
            AJETS(NJETS,2) = ETAJET
            AJETS(NJETS,3) = PHIJET
         ELSE
            PRINT *,'H1QGCONE: too many jets !'
            GOTO 99
         ENDIF
*
* One jet built means one jet candidates less
         JETCAND = JETCAND - 1
         IF (JETCAND.EQ.0) GOTO 200

C There are still jets left, so recalculate the next jet

 160     IET1 = JETCELL(INDEX(JETCAND),1)
         JET1 = JETCELL(INDEX(JETCAND),2)

         ETAET1 = -RAPMAX + BINETA*FLOAT(IET1) - BINETA/2.0
         PHIET1 =           BINPHI*FLOAT(JET1) - BINPHI/2.0

         IMIN = MAX0(1,IET1-NBETA)
         IMAX = MIN0(NRAPB,IET1+NBETA)

         ETJET = 0.
         DO 130 II=IMIN,IMAX
            DO 140 JJ=1,NAZMB
C Don't use already used cells for new jets
               IF (LOCK(II,JJ).EQ.2) GOTO 140
               IF(CELL(II,JJ).EQ.0.) GOTO 140
               ETA = -RAPMAX + BINETA*FLOAT(II) - BINETA/2.0
               PHI =           BINPHI*FLOAT(JJ) - BINPHI/2.0
               DPHI = ABS(PHI-PHIET1)
               IF(DPHI.GT.QQPI) DPHI = QQ2PI - DPHI
               DR=SQRT((ETA-ETAET1)**2+DPHI**2)
               IF(DR.GT.RJET) GOTO 140
               ETJET = ETJET + CELL(II,JJ)
 140        CONTINUE
 130     CONTINUE
         JETPT(INDEX(JETCAND)) = ETJET

* Now there are several cases
C Only one jet is left and its pt is above ETSMIN => build jet
        IF ((ETJET.GT.ETSMIN).AND.(JETCAND.EQ.1)) GOTO 150
C Only one jet is left and its pt is below ETSMIN => work done
        IF (JETCAND.EQ.1) GOTO 200
C The coreccted jet is below ETSMIN => recalculate the next one
C and forget this one.
        IF (ETJET.LT.ETSMIN) THEN
           JETCAND=JETCAND-1
           GOTO 160
        ENDIF
C The recalculated pt of the jet is still higher than the
C original pt of the next lower pt jet => build jet
        IF (ETJET.GT.JETPT(INDEX(JETCAND-1))) GOTO 150
C Else recalculate also all remaining jets
        NEWJET=0
        DO 170 III=1,JETCAND-1
         IET1 = JETCELL(INDEX(III),1)
         JET1 = JETCELL(INDEX(III),2)

         ETAET1 = -RAPMAX + BINETA*FLOAT(IET1) - BINETA/2.0
         PHIET1 =           BINPHI*FLOAT(JET1) - BINPHI/2.0

         IMIN = MAX0(1,IET1-NBETA)
         IMAX = MIN0(NRAPB,IET1+NBETA)

         ETJET = 0.
         DO 180 II=IMIN,IMAX
            DO 190 JJ=1,NAZMB
               IF(CELL(II,JJ).EQ.0.) GOTO 190
C Don't use already used cells for new jets
               IF (LOCK(II,JJ).EQ.2) GOTO 190
               ETA = -RAPMAX + BINETA*FLOAT(II) - BINETA/2.0
               PHI =           BINPHI*FLOAT(JJ) - BINPHI/2.0
               DPHI = ABS(PHI-PHIET1)
               IF(DPHI.GT.QQPI) DPHI = QQ2PI - DPHI
               DR=SQRT((ETA-ETAET1)**2+DPHI**2)
               IF(DR.GT.RJET) GOTO 190
               ETJET = ETJET + CELL(II,JJ)
 190        CONTINUE
 180     CONTINUE

C If the PT of the jet is above ETSMIN, store jet parameter
         IF (ETJET.GT.ETSMIN) THEN
            NEWJET=NEWJET+1
            JETPT(INDEX(NEWJET))=ETJET
            JETCELL(INDEX(NEWJET),1)=IET1
            JETCELL(INDEX(NEWJET),2)=JET1
         ENDIF
 170  CONTINUE
C     If due to the used cell one jet is now below ETSMIN, the jet is
c     deleted. At the end the last jet that was calculated before has
c     to be appended at the end of the actual jet list.
      IF (NEWJET.NE.JETCAND-1) THEN
         NEWJET = NEWJET+1
         JETPT(INDEX(NEWJET)) = JETPT(INDEX(JETCAND))
         JETCELL(INDEX(NEWJET),1) = JETCELL(INDEX(JETCAND),1)
         JETCELL(INDEX(NEWJET),2) = JETCELL(INDEX(JETCAND),2)
         JETCAND=NEWJET
      ENDIF
C     Sort again and then start building the jet with the highest pt
      CALL SORTZV(JETPT,INDEX,JETCAND,1,0,1)
      GOTO 150

* At this point all jets are built (deleted the rest for HZTOOL version)
 200  CONTINUE
 99   CONTINUE
      RETURN
      END
