      SUBROUTINE hzjtfind(iArgJetf,radius,iArgNJts,ArgJets)
***********************************************************************
*     Purpose: Generic jet finder caller.
*     runs on HEPEVTP table of particles.
*     Associated routine : HZJTNAME returns mnemonic of jet finder
*     
*     INPUT:  iArgJetf = chosen jetfinder, installed so far
*     1=EUCELL, 2=PXCONE, 3=KTCLUS, 4=GPCONE,
*     5=JCLUST(DIS), 6=JCLUST(photoprod.)
*     7=PUCELL 8=KT algorithm optimize for
*     resonance decays to dijets (see M.H.Seymour LUTP93-8)
*     9=ktclus in E recombination scheme (massive mode)
*     10=ktclus in E recombination scheme in meson mode (lists of mesons are
*     considered stable and used for jet-clustering instead of the final
*     state particles originating from them)
*     11=KTCLUS in 'pure' pt-mode, no p=E to achieve Et-mode as in 3!
*     12 = KTCLUS in exclusive mode, angular kt, E scheme, uses "radius" as ycut
*     13 = as 12 but ycut is chosen so as to give number of jets = iArgNJts
*     If ijetf<0 the appropriate jet finder
*     will be run on the "final state" of the
*     parton shower stage of the MC, rather than
*     on the final state particles.
*     14=ktclus in E recombination scheme (including D* mesons in the jet-clustering
*     instead of the final state particles originating from them)
*     
*     radius = radius of jet to find
*     (-1 to get from HZJETRAD, but this slows down the
*     program since HZJETRAD will be called every event)
*     
*     OUTPUT : numjets = Number of jets found (max. 50)
*     Jets(50,x) = information about jets found
*     x= 1,2  rapidity, phi  of jet axis
*     3    Et  of jet
*     4-8  (E,px,py,pz,m) of jet axis
*     note: for PXCONE, KTCLUS: jet momentum components are scaled by E
c     /p
*     
*     iJetNo(NMXHEP) =
*     case >= 1 :  number of jet particle is associated with
*     case <  1 :  particle doesn't belong to any jet
*     (only for follwing jet algorithms: PXCONE, KTCLUS)
*     
*     Author: M. Hayes
*     JMB 25/4/99 Added PS jets.
*     JMB  7/6/99 Set NJETS to zero when PXCONE returns an error.
*     BC   Add mode 7.
*     GF 10/11/04 Add mode 11: KTCLUS in 'plain' pt mode neglecting all masses.
*     SBOUTLE 09/03/06 Add mode 12.
***********************************************************************
      Implicit None
*     
*     
#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
#include "hzjetcmn.inc"
#include "hzsubjet.inc"

C     --- PXCONE stuff ---
      INTEGER  ITKDM,MXTRK
      PARAMETER  (ITKDM=4,MXTRK=500)
      INTEGER  MXJET, MXTRAK, MXPROT
      PARAMETER  (MXJET=50,MXTRAK=500,MXPROT=500)
      INTEGER  IPASS (MXTRAK),IJMUL (MXJET)
      INTEGER  NTRAK,injet,IERR,MODE
      DOUBLE PRECISION  PTRAK (ITKDM,MXTRK),PJET (5,MXJET)
      DOUBLE PRECISION  CONER, EPSLON, OVLIM
*     
C     --- KTCLUS stuff ---
      integer IKTPMX
      parameter (IKTPMX=1512)
      double precision ppp(4,IKTPMX),ktpjet(4,IKTPMX)
      integer          i,j,k,ktmode,ktrecom,npjets
      integer          invmap(IKTPMX),map(IKTPMX)
      double precision etjeti,etjetj,help
      real             yy(IKTPMX),ecut
      integer          parttojet(IKTPMX)
      real             ycut
      integer          nsub,njetreq
*     
* passes yy thruogh hzsubjet.inc commons

      equivalence(yyy(1),yy(1))
c     --- EUCELL stuff ---
#include "eujets.inc"

C     --- GPCONE stuff ---
      REAL JET1(10,3)
      REAL ETALM,ETINI,ETBIN,ETMIN,ETMAX,helpGP
      INTEGER NETA,NPHI
*     
C---- JADE Stuff
      Real EVIS,XJCUT,ee,ep
      Integer JMODE,NPI,NP,ip,il,iel
      REAL PI(4,nmxhep),P(5,nmxhep)
      DOUBLE PRECISION DPI(4,nmxhep),DP(4,nmxhep)

c     --- PUCELL stuff ---
#include "pucell.inc"
      real vtx(3),EnergyCuts(10),Operation(10)

#include "genjet1.inc"

      INTEGER NRclus
*     
*     
c     --- arguments
      INTEGER iArgJetf, iArgNJts, ijetf
      DOUBLE PRECISION radius, ArgJets(MAXHZJETS,8)
*     
c     --- general stuff ---
      character*9 cxxxx
      data cxxxx /'HZJTFIND'/
      
      INTEGER loop,loop1,loop2,count,myil,jetf
      DOUBLE PRECISION phi,px,py,pz,pt,ptsq,ppsq,energy,mass
      DOUBLE PRECISION eta,rap,Et,theta,drad
      INTEGER ref(NMXHEP)
      INTEGER hziflag,hzflag
      integer npart
      
      LOGICAL photoprod,lp,parton_showers
      data lp/.false./
*     
      INTEGER ISTHEP_SEL(2)
*
C     arrays and counters for jetfinder 10
      INTEGER bmdaid,bmda(1024)
      DOUBLE PRECISION enesum
      INTEGER nbrange,ncrange
      PARAMETER (nbrange = 84, ncrange = 68)
      INTEGER brange(nbrange),crange(ncrange)
      data brange / 521,-521,511,-511,553,551,20553,555,100553,10551,
     +     120553,100555,200553,300553,400553,9010553,5122,-5122,-531,
     +     531,-541,541,-523,523,-513,513,-533,533,-543,543,5222,-5222,
     +     5212,-5212,5112,-5112,5322,-5322,5232,-5232,5312,-5312,5132,
     +     -5132,5332,-5332,5224,-5224,5214,-5214,5114,-5114,5334,-5334,
     +     10511,-10511,10513,-10513,10521,-10521,10523,-10523,10531,
     +     -10531,10533,-10533,10541,-10541,10543,-10543,515,-515,525,
     +     -525,535,-535,20513,-20513,20523,-20523,20533,-20533,20543,
     +     -20543 /
      data crange / 411,-411,421,-42,431,-431,433,-433,441,443,10441,
     +     10443,445,100443,200443,9000443,9010443,9020443,413,-413,423,
     +     -423,4122,-4122,4332,-4332,4222,-4222,4212,-4212,4112,-4112,
     +     4232,-4232,4132,-4132,4224,-4224,4214,-4214,4114,-4114,4334,
     +     -4334,20413,-20413,415,-415,20423,-20423,425,-425,20433,
     +     -20433,435,-435,10411,-10411,10413,-10413,10421,-10421,
     +     10423,-10423,10431,-10431,10433,-10433 /
*
C     Are we running on hadrons (good!) or partons (dodgy)?
      IF (iArgJetf.GE.0) THEN
C     Select final state particles.
         parton_showers=.FALSE.
         ISTHEP_SEL(1) = 1
         ISTHEP_SEL(2) = 1
      ELSE
c     Select the results of the parton showering stage.
         parton_showers=.TRUE.
         IF (gen(1:3).eq.'HRW') THEN
            ISTHEP_SEL(1) = 2
            ISTHEP_SEL(2) = 2
         ELSE IF (gen(1:3).eq.'PYT') THEN
            ISTHEP_SEL(1) = 11
            ISTHEP_SEL(2) = 12
         ELSE
            WRITE(6,*)
     &      'Parton shower jets only implemented for HERWIG and PYTHIA'
            RETURN
         ENDIF
      ENDIF
      ijetf = ABS(iArgJetf)
      
**********************************************
*     GENERAL SETTINGS
**********************************************
      
      jetf=ijetf
*     
C     --- This determines if we call JCLUST(6) or not
      IF (jetf.eq.6) THEN
         jetf=5
         photoprod=.TRUE.
      ELSE
         photoprod=.FALSE.
      ENDIF
      
C     --- initializations
      do loop=1,NMXHEP
         IJETNO(loop)=0
      enddo
      do i=1,MAXHZJETS
         do j=1,8
            Jets(i,j)=0.D0
            ArgJets(i,j)=0.D0
         enddo
      enddo
      NUMJETS = 0
*
C     ----standard cone radius is 1.0
      If (radius.lt.0.D0) THEN
         CALL HZJETRAD(2,CONER)
         IF (CONER.lt.0.D0) THEN
            CONER=1.D0
         ENDIF
      Else
         CONER=radius
      Endif
*     
C     --- make sure we disable electron from jet finding
      myil=HZIDELEC(hzflag)
      if (lp) write(6,*) cxxxx,'myil= ',myil
      IF (myil.ge.0) Then
*     status of lepton is changed to 999 and changed back at the end
         ISTHEP(myil)=999
      ENDIF
*     
      
C     --- avoid invalid array-access
      if ((NHEP.lt.1).or.(NHEP.gt.NMXHEP)) then
         write(6,*) cxxxx,'invalid number of particles !',nhep
      endif
      
      
**********************************************
*     CHOOSE JETFINDER
**********************************************
      
***********************************************************************
      IF (jetf.eq.1) THEN
C     ------------------ start EUCELL stuff - find jets
C     ---                EUCELL has implicit cuts for rapidity (5,-5)
C     --- 		     minimum seed jet energy of 2 GeV
C     ---                can not fill the iJetNo array
         
         ETEMC0=0.
         ETHAC0=0.
         EEMC0=0.
         EHAC0=0.
*     
         CALL EUHEPC
         CALL EUCELL(4.0, -4.0, 2.0, 2.0, REAL(CONER))
         IF (NJET.gt.MAXHZJETS) THEN
            NUMJETS=MAXHZJETS
         ELSE
            NUMJETS=NJET
         ENDIF
*     
         DO loop=1,NUMJETS
            Jets(loop,1)=dble(PJ(loop,1)) ! eta
            Jets(loop,2)=dble(PJ(loop,2)) ! phi
            Jets(loop,3)=dble(PJ(loop,5)) ! Et
            Jets(loop,4)=dble(PJ(loop,9)) ! E
            Jets(loop,5)=dble(PJ(loop,6)) ! px
            Jets(loop,6)=dble(PJ(loop,7)) ! py
            Jets(loop,7)=dble(PJ(loop,8)) ! pz
            Jets(loop,8)=dble(PJ(loop,10)) ! m
         ENDDO
C     ---------------- end EUCELL stuff - find jets
***********************************************************************
      ELSEIF (jetf.eq.2) THEN
C     -----------------PXCONE stuff - find jets
C     ---              PXCONE minimum seed jet energy of 2 GeV
         
c     --- prepare pxcone call
         MODE=2
         EPSLON=2.D0
C     EPSLON=5.0
C     OVLIM=0.1
         OVLIM=7.5D-1
*     
         count=0
         DO loop=1,NHEP

*     Skip unstable particles and scattered lepton
            IF (ISTHEP(loop).eq.ISTHEP_SEL(1).OR.ISTHEP(loop).eq
     &           .ISTHEP_SEL(2)) THEN
		 IF (ABS(IDHEP(loop)).eq.11) GOTO 111
c     For running on parton showers: HERWIG generates a second set of
c     parton showers after a b-quark decay. Thus if we select all the
c     particles flagged as the final state of the parton shower stage,
c     we double count beauty quarks. So remove results of beauty quark
c     decays by hand here.
c               write(6,*) ' jetfind ',JMOHEP(1,loop),loop
               if(JMOHEP(1,loop).ne.0) then  ! to prevent out of range crash
               IF (.NOT. (GEN(1:3).EQ.'HRW'.AND.JMOHEP(1,JMOHEP(1,loop))
     &              .GT.25.AND.parton_showers) ) THEN

                  drad = sqrt( PHEP(1,loop)**2+PHEP(2,loop)**2
     &                 +PHEP(3,loop)**2 )
                  if( drad.le.0.D0 ) then
                     if (lp) write(6,*) cxxxx, 'drad = ',drad
                  else
                     IF (count.eq.MXTRK) THEN
                        write(6,*) cxxxx,'Too many particles for PXCONE'
     &                       ,count
                        GOTO 65
                     ELSE
                        count=count+1
*     following 'trick' is used to force PXCONE calculating with Et
*     rather than pt
                        PTRAK(1,count)=PHEP(1,loop)*PHEP(4,loop)/drad
                        PTRAK(2,count)=PHEP(2,loop)*PHEP(4,loop)/drad
                        PTRAK(3,count)=PHEP(3,loop)*PHEP(4,loop)/drad
                        PTRAK(4,count)=PHEP(4,loop)
                        ref(count)=loop

                     ENDIF
                  ENDIF
               ENDIF
               endif
            ENDIF
 111        ENDDO
 65      CONTINUE
         NTRAK=count
*     
         do loop = 1, MXTRAK
            ipass(loop) = 0
         enddo
         
c     --- do pxcone call
         CALL PXCONE (MODE,NTRAK,ITKDM,PTRAK,CONER,EPSLON,OVLIM,
     &        MXJET,injet,PJET,IPASS,IJMUL,IERR)
         IF (IERR.ne.0) THEN
            write(6,*) cxxxx, 'PXCONE returns error'
            NJET=0
         ENDIF
         IF (injet.gt.MXJET) THEN
            write(6,*) cxxxx, 'Too many jets found, increase MXJET'
     &           ,injet
         ENDIF
         
c     --- get output
         
         IF (injet.gt.MAXHZJETS) THEN
            NUMJETS=MAXHZJETS
         ELSE
            NUMJETS=injet
         ENDIF
*     
         DO loop=1,NUMJETS
            px=PJET(1,loop)
            py=PJET(2,loop)
            pz=PJET(3,loop)
            ptsq=px**2+py**2
            ppsq=(SQRT(ptsq+pz**2)+ABS(pz))**2
            IF (ptsq.le.4.25E-18*ppsq) THEN
               eta=20.D0
            ELSE
               eta=0.5*LOG(ppsq/ptsq)
            ENDIF
            eta=SIGN(eta,pz)
            IF (ptsq.EQ.0.D0) THEN
               phi=0.D0
            ELSE
               phi=ATAN2(py,px)
            ENDIF
*     
            Et=PJET(4,loop)*SQRT(ptsq/(ptsq+pz**2))
            Jets(loop,1)=eta    ! eta
            Jets(loop,2)=phi    ! phi
            Jets(loop,3)=Et     ! Et
            Jets(loop,4)=PJET(4,loop) ! E
            Jets(loop,5)=PJET(1,loop) ! px
            Jets(loop,6)=PJET(2,loop) ! py
            Jets(loop,7)=PJET(3,loop) ! pz
            Jets(loop,8)=0      ! m
                        
         Enddo
*     
         DO loop=1,ntrak
            iJetNo(ref(loop))=IPASS(loop)
         ENDDO
         
         
C     ---  end PXCONE stuff - find jets
***********************************************************************
      ELSEIF (jetf.eq.3.or.jetf.eq.11) THEN
C     ---  start KTCLUS stuff - find jets
C     ---  KTCLUS ignores the radius parameter
CGF   ---  why not call ktclur than??         
c     --- prepare KTCLUS call
         ktrecom=2
         ktmode=3212
         ecut   = 1.
         count=0
         DO loop=1,NHEP
*     skip unstable particles and scattered lepton
            IF (isthep(loop).EQ.isthep_sel(1).OR.isthep(loop).EQ
     &           .isthep_sel(2)) THEN

c     For running on parton showers: HERWIG generates a second set of
c     parton showers after a b-quark decay. Thus if we select all the
c     particles flagged as the final state of the parton shower stage,
c     we double count beauty quarks. So remove results of beauty quark
c     decays by hand here.
*               WRITE(*,*) parton_showers
c               WRITE(*,*) ' jtfind ',JMOHEP(1,JMOHEP(1,loop)),JMOHEP(1,loop),loop
               if(JMOHEP(1,loop).ne.0) then  ! to prevent out of range crash
               IF (.NOT. (GEN(1:3).EQ.'HRW'.AND.JMOHEP(1,JMOHEP(1,loop))
     &              .GT.25.AND.parton_showers) ) THEN

                  drad = sqrt( PHEP(1,loop)**2+PHEP(2,loop)**2
     &                 +PHEP(3,loop)**2 )
                  if( drad.le.0.D0 ) then
                     if (lp) write(6,*) cxxxx, 'drad = ',drad
                  else
                     IF (count.eq.IKTPMX) THEN
                        write(6,*) cxxxx,'Too many particles for KTCLUS'
     &                       ,count
                        GOTO 85
                     ELSE
                        count=count+1
*     following 'trick' is used to force KTCLUS calculating with Et
*     rather than pt
                        IF(ijetf.EQ.3) THEN
                           help=PHEP(4,loop)/drad
                        ELSE
                           help=1.0d0
                           IF(ijetf.NE.11) THEN
                              WRITE(6,*) cxxxx,
     &                             'should not reach for ijetf = ',ijetf
                           ENDIF
                        ENDIF
                        ppp(1,count) = PHEP(1,loop)*help
                        ppp(2,count) = PHEP(2,loop)*help
                        ppp(3,count) = PHEP(3,loop)*help
                        ppp(4,count) = PHEP(4,loop)
                        ref(count)=loop
                     ENDIF
                  ENDIF
               ENDIF
               endif
            ENDIF
         ENDDO

 85      CONTINUE
*     
         NPART=count

         if (lp) then
            write(*,*) 'NPART:',npart
            do j=1,npart
               write(*,*) (ppp(i,j),i=1,4)
            enddo
         endif
         
c     --- do KTCLUS call
         call KTCLUS(ktmode,ppp,npart,ecut,yy,*89)
         call KTINCL(ktrecom,ppp,npart,ktpjet,parttojet,NJET,*89)
         
         if (NJET.gt.IKTPMX) goto 89
         
         if (lp) then
            write(*,*) 'KTJETS:',njet
            do j=1,njet
               write(*,*) (ktpjet(i,j),i=1,4)
            enddo
         endif 

c     --- sort jets in Et
         Do loop=1,NJET
            map(loop)=loop
         ENDDO
         
         do i = 1, NJET
            etjeti = sqrt( ktpjet(1,i)**2 + ktpjet(2,i)**2 )
            do j = (i+1), NJET
               etjetj = sqrt( ktpjet(1,j)**2 + ktpjet(2,j)**2 )
               if( etjetj.gt.etjeti )then
                  do k = 1, 4
                     help      = ktpjet(k,i)
                     ktpjet(k,i) = ktpjet(k,j)
                     ktpjet(k,j) = help
                  enddo
                  k=map(i)
                  map(i)=map(j)
                  map(j)=k
                  etjeti = etjetj
               endif
            enddo               ! j
         enddo                  ! i
         DO loop=1,NJET
            invmap(map(loop))=loop
         ENDDO
         
c     --- get output
         IF (NJET.gt.MAXHZJETS) THEN
            NUMJETS=MAXHZJETS
         ELSE
            NUMJETS=NJET
         ENDIF
*     
         DO loop=1,NUMJETS
            px=ktPJET(1,loop)
            py=ktPJET(2,loop)
            pz=ktPJET(3,loop)
            ptsq=px**2+py**2
            ppsq=(SQRT(ptsq+pz**2)+ABS(pz))**2
            IF (ptsq.le.4.25E-18*ppsq) THEN
               eta=20.
            ELSE
               eta=0.5*LOG(ppsq/ptsq)
            ENDIF
            eta=SIGN(eta,pz)
            IF (ptsq.EQ.0) THEN
               phi=0.
            ELSE
               phi=ATAN2(py,px)
            ENDIF
            Et=ktPJET(4,loop)*SQRT(ptsq/(ptsq+pz**2))
            Jets(loop,1)=eta    ! eta
            Jets(loop,2)=phi    ! phi
            Jets(loop,3)=Et     ! Et
            Jets(loop,4)=ktPJET(4,loop) ! E
            Jets(loop,5)=ktPJET(1,loop) ! px
            Jets(loop,6)=ktPJET(2,loop) ! py
            Jets(loop,7)=ktPJET(3,loop) ! pz
            Jets(loop,8)=0      ! m
         Enddo
         
         Do loop=1,npart
            if ((parttojet(loop).ge.1).and.(parttojet(loop).le.NJET))
     &           then
               iJetNo(ref(loop))=invmap(parttojet(loop))
            else
               iJetNo(ref(loop))=0
            endif
         ENDDO
         
         goto 90
 89      CONTINUE
         write(6,*) cxxxx,'Error in KTCLUS !'
         
 90      continue
C     --- end KTCLUS stuff - find jets
***********************************************************************
      ELSEIF (jetf.eq.4) THEN
C     ---    start GPCONE stuff - find jets
C     minimum Et of jet is 2 GeV
C     doesn't return HZJETCMN
         ETALM = 3.0
         NETA = 30
         NPHI = 24
         ETINI = 0.1
         ETBIN = 2.
         ETMIN = 2.0
         ETMAX=49.
         call vzero(jet1,30)
         njet=0
         CALL H1GPCONE(ETALM,NETA,NPHI,ETINI,ETMIN,ETBIN,ETMAX,
     +        REAL(CONER), NJET,JET1)
*     
 	 IF (NJET.gt.1) Then    ! sort the jets in Et
            do i = 1, NJET
               do j = (i+1), NJET
                  if( JET1(j,1).gt.JET1(i,1) )then
                     do k = 1, 3
                        helpGP      = JET1(i,k)
                        JET1(i,k) = JET1(j,k)
                        JET1(j,k) = helpGP
                     enddo
                  endif
               enddo            ! j
            enddo               ! i
         ENDIF                  ! jets sorted.
*     
	 IF (NJET.gt.MAXHZJETS) THEN
            NUMJETS=MAXHZJETS
	 ELSE
            NUMJETS=NJET
	 ENDIF
*     
	 DO loop=1,NUMJETS
            Jets(loop,1)=dble(JET1(loop,2)) ! eta
            Jets(loop,2)=dble(JET1(loop,3)) ! phi
            Jets(loop,3)=dble(JET1(loop,1)) ! Et
            theta=2*datan(exp(-1*Jets(loop,1)))
            Jets(loop,4)= Jets(loop,3)/dcos(theta) ! E
            Jets(loop,5)= Jets(loop,4)*sin(theta)*cos(phi) ! px
            Jets(loop,6)= Jets(loop,4)*sin(theta)*sin(phi) ! py
            Jets(loop,7)= Jets(loop,4)*cos(theta) ! pz
            Jets(loop,8)= 0.d0  ! m
	 ENDDO
C     ---   end GPCONE stuff
      ELSEIF (jetf.eq.5) THEN
C     ---   start JADE stuff
*     Doesn't return HZJETCMN
*     This is both the code for JCLUST(DIS) and JCLUST(photoprod.)
*     Please use with care.
*     
*     evis: visible energy in event
*     xjcut: unscale e fraction x=xjcut/w**2
*     jmode: ?
*     iel: pointer to electron
*     
         evis=0.
*     
         call VZERO(PI,4*nmxhep)
         npi=0
         DO loop=1,NHEP
            IF (ISTHEP(loop).eq.ISTHEP_SEL(1).OR.ISTHEP(loop).eq
     &           .ISTHEP_SEL(2)) THEN

c     For running on parton showers: HERWIG generates a second set of
c     parton showers after a b-quark decay. Thus if we select all the
c     particles flagged as the final state of the parton shower stage,
c     we double count beauty quarks. So remove results of beauty quark
c     decays by hand here.
               if(JMOHEP(1,loop).ne.0) then  ! to prevent out of range crash
               IF (.NOT. (GEN(1:3).EQ.'HRW'.AND.JMOHEP(1,JMOHEP(1,loop))
     &              .GT.25.AND.parton_showers) ) THEN

                  
                  npi=npi+1
                  PI(1,npi)=PHEP(1,loop)
                  PI(2,npi)=PHEP(2,loop)
                  PI(3,npi)=PHEP(3,loop)
                  PI(4,npi)=PHEP(4,loop)
                  evis=evis+PHEP(4,loop)
               ENDIF
               endif
            ENDIF
*     
         ENDDO
*     
         jmode=1
         xjcut=real(radius)
*     
         IF (.NOT.photoprod) THEN
C     IF (myil.ge.0) Then
C     ISTHEP(myil)=1 ! turn on the electron !
C     ENDIF
            evis=real(HZDISKIN(4))
            mass=HZDISKIN(4)
            if (evis.ne.0.) evis=sqrt(evis)
         ENDIF
*     
C     call VZERO(P,5*nmxhep)
         do i=1,nmxhep
            p(1,i)=0.
            p(2,i)=0.
            p(3,i)=0.
            p(4,i)=0.
         enddo
*     
         if (lp) then
            write(6,*) ' calling jade evis= ',evis
            write(6,*) ' xjcut= ',xjcut,' jmode= ',jmode
         endif
*     
         call jclust(evis,xjcut,jmode,npi,Pi,Np,P)
C     call jadejet(dpi,npi,radius,mass,1,1,np,dp)
*     
         if (lp) then
            do i=1,np
               write(6,'(i3,a,4f9.3)') i,' e,px,py,pz= ',
     &              real(p(4,i)),real(p(1,i)),real(p(2,i)),real(p(3,i))
            enddo
         endif
*     
*     sort jets in Et
*     
         do i = 1, np
            do j = (i+1), np
               etjeti = sqrt( p(1,i)**2 + p(2,i)**2 )
               etjetj = sqrt( p(1,j)**2 + p(2,j)**2 )
               if( etjetj.gt.etjeti )then
                  do k = 1, 4
                     help   = p(k,i)
                     p(k,i) = p(k,j)
                     p(k,j) = help
                  enddo
               endif
            enddo               ! j
         enddo                  ! i
*     
         DO loop=1,np
            px=dble(P(1,loop))
            py=dble(P(2,loop))
            pz=dble(P(3,loop))
            ptsq=px**2+py**2
            ppsq=(SQRT(ptsq+pz**2)+ABS(pz))**2
            IF (ptsq.le.4.25E-18*ppsq) THEN
               rap=20.
            ELSE
               rap=0.5*LOG(ppsq/ptsq)
            ENDIF
            rap=SIGN(rap,pz)
            IF (ptsq.EQ.0.) THEN
               phi=0.
            ELSE
               phi=ATAN2(py,px)
            ENDIF
            Et=P(4,loop)*SQRT(ptsq/(ptsq+pz**2))
            Jets(loop,1)=rap
            Jets(loop,2)=phi
            Jets(loop,3)=et
            Jets(loop,4)= dble(P(4,loop))
            Jets(loop,5)= dble(P(1,loop))
            Jets(loop,6)= dble(P(2,loop))
            Jets(loop,7)= dble(P(3,loop))
            Jets(loop,8)= dble(P(5,loop))
C     write(6,*) loop,' Jets= ',Jets(loop,4),Jets(loop,3),
C     &                                Jets(loop,2),Jets(loop,1)
         ENDDO
         NUMJETS=np
*     
C     ---   end JADE stuff
      ELSEIF (jetf.eq.6) THEN
C     ---   start stuff
*     do not use this space as it is referenced by JCLUST(photoprod.)
C     ---   end stuff
      ELSEIF (jetf.eq.7) THEN
c     --- start PUCELL stuff
         npi=0
         DO loop=1,NHEP
            IF (ISTHEP(loop).eq.ISTHEP_SEL(1).OR.ISTHEP(loop).eq
     &           .ISTHEP_SEL(2)) THEN

c     For running on parton showers: HERWIG generates a second set of
c     parton showers after a b-quark decay. Thus if we select all the
c     particles flagged as the final state of the parton shower stage,
c     we double count beauty quarks. So remove results of beauty quark
c     decays by hand here.

               if(JMOHEP(1,loop).ne.0) then  ! to prevent out of range crash
               IF (.NOT. (GEN(1:3).EQ.'HRW'.AND.JMOHEP(1,JMOHEP(1,loop))
     &              .GT.25.AND.parton_showers) ) THEN
                  npi=npi+1
                  IF (npi+100.LT.max_pucell_particles) THEN
                     PucellP(1,npi+100)=PHEP(1,loop)
                     PucellP(2,npi+100)=PHEP(2,loop)
                     PucellP(3,npi+100)=PHEP(3,loop)
                     PucellP(4,npi+100)=PHEP(4,loop)
                  ELSE
                     WRITE(*,*) 'HZJTFIND:Too many particles for PUCELL'
                  ENDIF
               ENDIF
               endif
            endif
         enddo
         Operation(1)=0.9       ! Et
         Operation(2)=REAL(CONER) ! cone
         Operation(3)=REAL(CONER) ! cone
         Operation(4)=20        ! Number of iterations
         Operation(5)=0.75      ! fraction energy to merge
         EnergyCuts(1)=0.05     !
         EnergyCuts(2)=0.10     !
         ENERGYCUTS(3)=0.50     ! (Seed)
         CALL Had_to_HCM(npi)
         call pacellgp(Operation,EnergyCuts,NrClus,
     +        IErr,npi)
         IF (Ierr.gt.0) PRINT*,'HZJTFIND : PUCELL error ',Ierr
         IF (NJET1.gt.MAXHZJETS) THEN
            NUMJETS=MAXHZJETS
         ELSE
            NUMJETS=NJET1
         ENDIF
         DO loop=1,NUMJETS
            Jets(loop,1)=dble(PGJET1(loop,6)) ! eta
            Jets(loop,2)=dble(PGJET1(loop,11)) ! phi
            Jets(loop,3)=dble(PGJET1(loop,10)) ! Et
            Jets(loop,4)=dble(PGJET1(loop,4)) ! E
            Jets(loop,5)=dble(PGJET1(loop,1)) ! px
            Jets(loop,6)=dble(PGJET1(loop,2)) ! py
            Jets(loop,7)=dble(PGJET1(loop,3)) ! pz
            Jets(loop,8)=dble(0.0) ! m
         ENDDO
c     --- end PUCELL stuff
      ELSEIF (jetf.eq.8) then
C     ---  KT algorithm mode optimised for resonance decays -> jets 
C     ---  as in M. H. Seymour LU TP 93-8 (particlualrly heavy higgs ->
c     WW ->lljj)
         ISUBJ=0
c     --- prepare KTCLUS call
         ktrecom=1
c     --- covariant E scheme in pp
         ktmode=4211
         ecut   = 1. 
         count=0
         DO loop=1,NHEP
*     skip unstable particles and all leptons 
*     note that this is very specific to 2 W analysis
            IF (ISTHEP(loop).eq.1.and.
     +           (abs(IDHEP(loop)).lt.11.or.
     +           abs(IDHEP(loop)).gt.16)) THEN
               IF (count.eq.IKTPMX) THEN
                  write(6,*) cxxxx,'Too many particles for KTCLUS'
     &                 ,count
                  GOTO 93
               ELSE
                  count=count+1
                  ppp(1,count) = PHEP(1,loop)
                  ppp(2,count) = PHEP(2,loop)
                  ppp(3,count) = PHEP(3,loop)
                  ppp(4,count) = PHEP(4,loop)
                  ref(count)=loop
               ENDIF
            ENDIF
         EndDo
 93      CONTINUE
*     
         NPART=count
         
c     --- do KTCLUS call
         
         call KTCLUS(ktmode,ppp,npart,ecut,yy,*91)
         call KTINCL(ktrecom,ppp,npart,ktpjet,parttojet,NJET,*91)
        
C     now look at subjets within the candidate W jets
         DO 800 loop=1,NJET,1
            px=ktPJET(1,loop)
            py=ktPJET(2,loop)
            pz=ktPJET(3,loop)
            ptsq=px**2+py**2
            eta=0.5*log((ktPJET(4,loop)-pz)/(ktPJET(4,loop)+pz))
            if (ptsq.gt.400.and.abs(eta).lt.4) then
               CALL KTIJOI(loop,YSUB,*91)
               isubj=isubj+1
               CALL KTIREC(ktrecom,ppp,npart,loop,YSUB(1),PSUB,NSUBJ,
     &              *91)
               do i=1,4,1
                  SUBJ(isubj,1,i)=REAL(PSUB(i,1))
                  SUBJ(isubj,2,i)=REAL(PSUB(i,2))
*     this entry carries the 4-vector of the parent jet
                  SUBJ(isubj,3,i)=ktpjet(i,loop)
               enddo
               ysubn(isubj)=YSUB(1)
            endif
 800     continue 
         if (NJET.gt.IKTPMX) goto 91 
c     --- sort jets in Et
         Do loop=1,NJET
            map(loop)=loop
         ENDDO
         
         do i = 1, NJET
            etjeti = sqrt( ktpjet(1,i)**2 + ktpjet(2,i)**2 )
            do j = (i+1), NJET
               etjetj = sqrt( ktpjet(1,j)**2 + ktpjet(2,j)**2 )
               if( etjetj.gt.etjeti )then
                  do k = 1, 4
                     help      = ktpjet(k,i)
                     ktpjet(k,i) = ktpjet(k,j)
                     ktpjet(k,j) = help
                  enddo
                  k=map(i)
                  map(i)=map(j)
                  map(j)=k
                  etjeti = etjetj
               endif
            enddo               ! j
         enddo                  ! i
         DO loop=1,NJET
            invmap(map(loop))=loop
         ENDDO
c     --- get output
         IF (NJET.gt.MAXHZJETS) THEN
            NUMJETS=MAXHZJETS
         ELSE
            NUMJETS=NJET
         ENDIF
*     
         DO loop=1,NUMJETS
            px=ktPJET(1,loop)
            py=ktPJET(2,loop)
            pz=ktPJET(3,loop)
            ptsq=px**2+py**2
            ppsq=(SQRT(ptsq+pz**2)+ABS(pz))**2
            IF (ptsq.le.4.25E-18*ppsq) THEN
               eta=20.
            ELSE
               eta=0.5*LOG(ppsq/ptsq)
            ENDIF
            eta=SIGN(eta,pz)
            IF (ptsq.EQ.0) THEN
               phi=0.
            ELSE
               phi=ATAN2(py,px)
            ENDIF
            Et=ktPJET(4,loop)*SQRT(ptsq/(ptsq+pz**2))
            Jets(loop,1)=eta    ! eta
            Jets(loop,2)=phi    ! phi
            Jets(loop,3)=Et     ! Et
            Jets(loop,4)=ktPJET(4,loop) ! E
            Jets(loop,5)=ktPJET(1,loop) ! px
            Jets(loop,6)=ktPJET(2,loop) ! py
            Jets(loop,7)=ktPJET(3,loop) ! pz
            Jets(loop,8)=0      ! m
         Enddo
         Do loop=1,npart
            if ((parttojet(loop).ge.1).and.(parttojet(loop).le.NJET))
     &           then
               iJetNo(ref(loop))=invmap(parttojet(loop))
            else
               iJetNo(ref(loop))=0
            endif
         ENDDO
         goto 92
 91      CONTINUE
         write(6,*) cxxxx,'Error in KTCLUS !'
 92      continue
***********************************************************************
***********************************************************************
      ELSEIF (jetf.eq.9) THEN
C     ---  start KTCLUS stuff - find jets
C     ---  KTCLUS ignores the radius parameter
C     ---  KTCLUS in massive mode with e-recombination scheme
         
c     --- prepare KTCLUS call
         ktrecom=1
         ktmode=3211
         ecut   = 1.
         count=0
         DO loop=1,NHEP
*     skip unstable particles and scattered lepton
            IF (isthep(loop).EQ.isthep_sel(1).OR.isthep(loop).EQ
     &           .isthep_sel(2)) THEN

c     For running on parton showers: HERWIG generates a second set of
c     parton showers after a b-quark decay. Thus if we select all the
c     particles flagged as the final state of the parton shower stage,
c     we double count beauty quarks. So remove results of beauty quark
c     decays by hand here.
*               WRITE(*,*) parton_showers
*               WRITE(*,*) JMOHEP(1,JMOHEP(1,loop)),JMOHEP(1,loop),loop
               if(JMOHEP(1,loop).ne.0) then  ! to prevent out of range crash
               IF (.NOT. (GEN(1:3).EQ.'HRW'.AND.JMOHEP(1,JMOHEP(1,loop))
     &              .GT.25.AND.parton_showers) ) THEN

                  drad = sqrt( PHEP(1,loop)**2+PHEP(2,loop)**2
     &                 +PHEP(3,loop)**2 )
                  if( drad.le.0.D0 ) then
                     if (lp) write(6,*) cxxxx, 'drad = ',drad
                  else
                     IF (count.eq.IKTPMX) THEN
                        write(6,*) cxxxx,'Too many particles for KTCLUS'
     &                       ,count
                        GOTO 859
                     ELSE
                        count=count+1
                        ppp(1,count) = PHEP(1,loop)
                        ppp(2,count) = PHEP(2,loop)
                        ppp(3,count) = PHEP(3,loop)
                        ppp(4,count) = PHEP(4,loop)
                        ref(count)=loop
                     ENDIF
                  ENDIF
               ENDIF
               endif
            ENDIF
         ENDDO

 859	CONTINUE
*     
         NPART=count

         if (lp) then
            write(*,*) 'NPART:',npart
            do j=1,npart
               write(*,*) (ppp(i,j),i=1,4)
            enddo
         endif
         
c     --- do KTCLUS call
         call KTCLUS(ktmode,ppp,npart,ecut,yy,*899)
         call KTINCL(ktrecom,ppp,npart,ktpjet,parttojet,NJET,*899)
         
         if (NJET.gt.IKTPMX) goto 899
         
         if (lp) then
            write(*,*) 'KTJETS:',njet
            do j=1,njet
               write(*,*) (ktpjet(i,j),i=1,4)
            enddo
         endif 

c     --- sort jets in pt (massive case) although variables denoted as et
         Do loop=1,NJET
            map(loop)=loop
         ENDDO
         
         do i = 1, NJET
            etjeti = sqrt( ktpjet(1,i)**2 + ktpjet(2,i)**2 )
            do j = (i+1), NJET
               etjetj = sqrt( ktpjet(1,j)**2 + ktpjet(2,j)**2 )
               if( etjetj.gt.etjeti )then
                  do k = 1, 4
                     help      = ktpjet(k,i)
                     ktpjet(k,i) = ktpjet(k,j)
                     ktpjet(k,j) = help
                  enddo
                  k=map(i)
                  map(i)=map(j)
                  map(j)=k
                  etjeti = etjetj
               endif
            enddo               ! j
         enddo                  ! i
         DO loop=1,NJET
            invmap(map(loop))=loop
         ENDDO
         
c     --- get output
         IF (NJET.gt.MAXHZJETS) THEN
            NUMJETS=MAXHZJETS
         ELSE
            NUMJETS=NJET
         ENDIF
*     
         DO loop=1,NUMJETS
            px=ktPJET(1,loop)
            py=ktPJET(2,loop)
            pz=ktPJET(3,loop)
            ptsq=px**2+py**2
            ppsq=(SQRT(ptsq+pz**2)+ABS(pz))**2
            IF (ptsq.le.4.25E-18*ppsq) THEN
               eta=20.
            ELSE
               eta=0.5*LOG(ppsq/ptsq)
            ENDIF
            eta=SIGN(eta,pz)
            IF (ptsq.EQ.0) THEN
               phi=0.
            ELSE
               phi=ATAN2(py,px)
            ENDIF
            Et=ktPJET(4,loop)*SQRT(ptsq/(ptsq+pz**2))
            Jets(loop,1)=eta    ! eta
            Jets(loop,2)=phi    ! phi
            Jets(loop,3)=Et     ! Et
            Jets(loop,4)=ktPJET(4,loop) ! E
            Jets(loop,5)=ktPJET(1,loop) ! px
            Jets(loop,6)=ktPJET(2,loop) ! py
            Jets(loop,7)=ktPJET(3,loop) ! pz
            Jets(loop,8)=0      ! m
         Enddo
         
         Do loop=1,npart
            if ((parttojet(loop).ge.1).and.(parttojet(loop).le.NJET))
     &           then
               iJetNo(ref(loop))=invmap(parttojet(loop))
            else
               iJetNo(ref(loop))=0
            endif
         ENDDO
         
         goto 909
 899     CONTINUE
         write(6,*) cxxxx,'Error in KTCLUS !'
         
 909     continue
C     --- end KTCLUS stuff - find jets
***********************************************************************
***********************************************************************
      ELSEIF (jetf.eq.10) THEN
C     ---  start KTCLUS stuff - find jets
C     ---  KTCLUS ignores the radius parameter
C     ---  KTCLUS in massive mode with e-recombination scheme
C     ---  run jtfinder on mesons before decay
c     --- prepare KTCLUS call
         ktrecom=1
         ktmode=3211
         ecut   = 1.
         count=0
C

*     
C     bmeson exclude array
         bmdaid = 0
         enesum = 0.0D0
*
         do 101  loop = 1, NHEP     
c     skip if from a cascade decay from beauty/charm
            if (JMOHEP(1,loop).gt.0) then
               do loop1=1,bmdaid
                  if (bmda(loop1).eq.JMOHEP(1,loop)) then
                     bmdaid = bmdaid + 1
                     if (bmdaid.gt.1024) bmdaid=1
                     bmda(bmdaid)=loop
                     goto 101
                  endif
               enddo
            endif
c     keep if beauty (and tag it to skip its daughters) 
            do loop1=1,nbrange
               if ( idhep(loop).eq.brange(loop1) ) then
                  bmdaid = bmdaid + 1
                  if (bmdaid.gt.1024) bmdaid=1
                  bmda(bmdaid)=loop
                  goto 98
               endif
            enddo
c     keep if charm and tag it 
            do loop1=1,ncrange
               if (idhep(loop).eq.crange(loop1) ) then
                  bmdaid = bmdaid + 1
                  if (bmdaid.gt.1024) bmdaid=1
                  bmda(bmdaid)=loop
                  goto 98
               endif
            enddo
            
C     check if particle final
            if (isthep(loop).EQ.isthep_sel(1).OR.isthep(loop).EQ
     &           .isthep_sel(2) ) THEN
               goto 98
            else
               goto 101
            endif
         
         
 98         continue
            
            count=count+1
            ppp(1,count) = PHEP(1,loop)
            ppp(2,count) = PHEP(2,loop)
            ppp(3,count) = PHEP(3,loop)
            ppp(4,count) = PHEP(4,loop)
            ref(count)=loop
            enesum = enesum + PHEP(4,loop)
            
 101     continue
*     
C         print*,'enesum: ',enesum

         NPART=count
         
         if (lp) then
            write(*,*) 'NPART:',npart
            do j=1,npart
               write(*,*) (ppp(i,j),i=1,4)
            enddo
         endif

c     --- do KTCLUS call
         call KTCLUS(ktmode,ppp,npart,ecut,yy,*898)
         call KTINCL(ktrecom,ppp,npart,ktpjet,parttojet,NJET,*898)
         
         if (NJET.gt.IKTPMX) goto 898
         
         if (lp) then
            write(*,*) 'KTJETS:',njet
            do j=1,njet
               write(*,*) (ktpjet(i,j),i=1,4)
            enddo
         endif 

c     --- sort jets in pt (massive case) although variable denoted
c     as et
         Do loop=1,NJET
            map(loop)=loop
         ENDDO
         
         do i = 1, NJET
            etjeti = sqrt( ktpjet(1,i)**2 + ktpjet(2,i)**2 )
            do j = (i+1), NJET
               etjetj = sqrt( ktpjet(1,j)**2 + ktpjet(2,j)**2 )
               if( etjetj.gt.etjeti )then
                  do k = 1, 4
                     help      = ktpjet(k,i)
                     ktpjet(k,i) = ktpjet(k,j)
                     ktpjet(k,j) = help
                  enddo
                  k=map(i)
                  map(i)=map(j)
                  map(j)=k
                  etjeti = etjetj
               endif
            enddo               ! j
         enddo                  ! i
         DO loop=1,NJET
            invmap(map(loop))=loop
         ENDDO
         
c     --- get output
         IF (NJET.gt.MAXHZJETS) THEN
            NUMJETS=MAXHZJETS
         ELSE
            NUMJETS=NJET
         ENDIF
*     
         DO loop=1,NUMJETS
            px=ktPJET(1,loop)
            py=ktPJET(2,loop)
            pz=ktPJET(3,loop)
            ptsq=px**2+py**2
            ppsq=(SQRT(ptsq+pz**2)+ABS(pz))**2
            IF (ptsq.le.4.25E-18*ppsq) THEN
               eta=20.
            ELSE
               eta=0.5*LOG(ppsq/ptsq)
            ENDIF
            eta=SIGN(eta,pz)
            IF (ptsq.EQ.0) THEN
               phi=0.
            ELSE
               phi=ATAN2(py,px)
            ENDIF
            Et=ktPJET(4,loop)*SQRT(ptsq/(ptsq+pz**2))
            Jets(loop,1)=eta    ! eta
            Jets(loop,2)=phi    ! phi
            Jets(loop,3)=Et     ! Et
            Jets(loop,4)=ktPJET(4,loop) ! E
            Jets(loop,5)=ktPJET(1,loop) ! px
            Jets(loop,6)=ktPJET(2,loop) ! py
            Jets(loop,7)=ktPJET(3,loop) ! pz
            Jets(loop,8)=0      ! m
         Enddo
         
         Do loop=1,npart
            if ((parttojet(loop).ge.1).and.(parttojet(loop).le.NJET))
     &           then
               iJetNo(ref(loop))=invmap(parttojet(loop))
            else
               iJetNo(ref(loop))=0
            endif
         ENDDO
         
         goto 908
 898     CONTINUE
         write(6,*) cxxxx,'Error in KTCLUS !'
         
 908     continue
C     --- end KTCLUS stuff - find jets
***********************************************************************
      ELSEIF (jetf.eq.11) THEN
C     ---   start stuff
*     do not use this space as it is referenced by KTCLUS in 'pure' pt mode
C     ---   end stuff
      ELSEIF (jetf.eq.12.or.jetf.eq.13) THEN
C     ---  run KTCLUS in exclusive mode
C     ---  KTCLUS in massive mode with e-recombination scheme
C     ---  angular distance scheme used
C     ---  mode 12: the "radius" parameter is used to set ycut
C     ---  mode 13: iArgNJts is used to set the required number of jets

c     --- prepare KTCLUS call
         ktrecom=1
c     ktmode = 3111 : pe, angular kt, E recom scheme
         ktmode=3111
c     ecut = 0 : use Etot as denominator for ycut
         ecut   = 0.
         count=0
         DO loop=1,NHEP
*     skip unstable particles and scattered lepton
            IF (isthep(loop).EQ.isthep_sel(1).OR.isthep(loop).EQ
     &           .isthep_sel(2)) THEN

c     For running on parton showers: HERWIG generates a second set of
c     parton showers after a b-quark decay. Thus if we select all the
c     particles flagged as the final state of the parton shower stage,
c     we double count beauty quarks. So remove results of beauty quark
c     decays by hand here.
*               WRITE(*,*) parton_showers
*               WRITE(*,*) JMOHEP(1,JMOHEP(1,loop)),JMOHEP(1,loop),loop
               if(JMOHEP(1,loop).ne.0) then  ! to prevent out of range crash
               IF (.NOT. (GEN(1:3).EQ.'HRW'.AND.JMOHEP(1,JMOHEP(1,loop))
     &              .GT.25.AND.parton_showers) ) THEN

                  drad = sqrt( PHEP(1,loop)**2+PHEP(2,loop)**2
     &                 +PHEP(3,loop)**2 )
                  if( drad.le.0.D0 ) then
                     if (lp) write(6,*) cxxxx, 'drad = ',drad
                  else
                     IF (count.eq.IKTPMX) THEN
                        write(6,*) cxxxx,'Too many particles for KTCLUS'
     &                       ,count
                        GOTO 123
                     ELSE
                        count=count+1
                        ppp(1,count) = PHEP(1,loop)
                        ppp(2,count) = PHEP(2,loop)
                        ppp(3,count) = PHEP(3,loop)
                        ppp(4,count) = PHEP(4,loop)
                        ref(count)=loop
                     ENDIF
                  ENDIF
               ENDIF
               endif
            ENDIF
         ENDDO

 123     CONTINUE
*     
         NPART=count

         if (lp) then
            write(*,*) 'NPART:',npart
            do j=1,npart
               write(*,*) (ppp(i,j),i=1,4)
            enddo
         endif
         
c     --- do KTCLUS call
         call KTCLUS(ktmode,ppp,npart,ecut,yy,*899)

c     --- set ycut from "radius" (mode 12) or to get required no. of jets (mode 13)
         ycut = radius
         if (jetf.eq.13) then
            njetreq = iArgNJts
            if (njetreq.gt.maxhzjets) njetreq = maxhzjets
            if (njetreq.gt.npart) njetreq = npart
            ycut = yy(njetreq)
         endif

c     --- set ymac = ycut since don't care about subjets right now
         call KTRECO(ktrecom,ppp,npart,ecut,ycut,ycut,
     &        ktpjet,parttojet,NJET,NSUB,*120)

         if (NJET.gt.IKTPMX) goto 120
         
         if (lp) then
            write(*,*) 'KTJETS:',njet
            do j=1,njet
               write(*,*) (ktpjet(i,j),i=1,4)
            enddo
         endif 

c     --- sort jets in pt (massive case) although variables denoted as et
         Do loop=1,NJET
            map(loop)=loop
         ENDDO
         
         do i = 1, NJET
            etjeti = sqrt( ktpjet(1,i)**2 + ktpjet(2,i)**2 )
            do j = (i+1), NJET
               etjetj = sqrt( ktpjet(1,j)**2 + ktpjet(2,j)**2 )
               if( etjetj.gt.etjeti )then
                  do k = 1, 4
                     help      = ktpjet(k,i)
                     ktpjet(k,i) = ktpjet(k,j)
                     ktpjet(k,j) = help
                  enddo
                  k=map(i)
                  map(i)=map(j)
                  map(j)=k
                  etjeti = etjetj
               endif
            enddo               ! j
         enddo                  ! i
         DO loop=1,NJET
            invmap(map(loop))=loop
         ENDDO
         
c     --- get output
         IF (NJET.gt.MAXHZJETS) THEN
            NUMJETS=MAXHZJETS
         ELSE
            NUMJETS=NJET
         ENDIF
*     
         DO loop=1,NUMJETS
            px=ktPJET(1,loop)
            py=ktPJET(2,loop)
            pz=ktPJET(3,loop)
            ptsq=px**2+py**2
            ppsq=(SQRT(ptsq+pz**2)+ABS(pz))**2
            IF (ptsq.le.4.25E-18*ppsq) THEN
               eta=20.
            ELSE
               eta=0.5*LOG(ppsq/ptsq)
            ENDIF
            eta=SIGN(eta,pz)
            IF (ptsq.EQ.0) THEN
               phi=0.
            ELSE
               phi=ATAN2(py,px)
            ENDIF
            Et=ktPJET(4,loop)*SQRT(ptsq/(ptsq+pz**2))
            Jets(loop,1)=eta    ! eta
            Jets(loop,2)=phi    ! phi
            Jets(loop,3)=Et     ! Et
            Jets(loop,4)=ktPJET(4,loop) ! E
            Jets(loop,5)=ktPJET(1,loop) ! px
            Jets(loop,6)=ktPJET(2,loop) ! py
            Jets(loop,7)=ktPJET(3,loop) ! pz
            Jets(loop,8)=0      ! m
         Enddo
         
         Do loop=1,npart
            if ((parttojet(loop).ge.1).and.(parttojet(loop).le.NJET))
     &           then
               iJetNo(ref(loop))=invmap(parttojet(loop))
            else
               iJetNo(ref(loop))=0
            endif
         ENDDO
         
         goto 121
 120     CONTINUE
         write(6,*) cxxxx,'Error in KTCLUS !'
         
 121     continue
C     --- end KTCLUS stuff - find jets

***********************************************************************
      ELSEIF (jetf.eq.14) THEN
C     ---  start KTCLUS stuff - find jets
C     ---  KTCLUS ignores the radius parameter
C     ---  KTCLUS in massive mode with e-recombination scheme
C     ---  run jtfinder on mesons before decay
c     --- prepare KTCLUS call
         ktrecom=1
         ktmode=3211
         ecut   = 1.
         count=0

         bmdaid = 0
         enesum = 0.0D0
*
         do 142  loop = 1, NHEP   

c     skip if decay product of the D*
            if (JMOHEP(1,loop).gt.0) then
               do loop1=1,bmdaid
                  if (bmda(loop1).eq.JMOHEP(1,loop)) then
                     bmdaid = bmdaid + 1
                     if (bmdaid.gt.1024) bmdaid=1
                     bmda(bmdaid)=loop
                     goto 142
                  endif
               enddo
            endif

C     Find the D* meson in an event
            if (abs(idhep(loop)).eq.413) then
               bmdaid = bmdaid + 1
               if (bmdaid.gt.1024) bmdaid=1
               bmda(bmdaid)=loop
               goto 141
            endif

C     check if particle final
            if (isthep(loop).EQ.isthep_sel(1).OR.isthep(loop).EQ
     &           .isthep_sel(2) ) THEN
               goto 141
            else
               goto 142
            endif
                  
 141        continue
            
            count=count+1
            ppp(1,count) = PHEP(1,loop)
            ppp(2,count) = PHEP(2,loop)
            ppp(3,count) = PHEP(3,loop)
            ppp(4,count) = PHEP(4,loop)
            ref(count)=loop
            enesum = enesum + PHEP(4,loop)
            
 142     continue

         NPART=count
         
         if (lp) then
            write(*,*) 'NPART:',npart
            do j=1,npart
               write(*,*) (ppp(i,j),i=1,4)
            enddo
         endif

c     --- do KTCLUS call
         call KTCLUS(ktmode,ppp,npart,ecut,yy,*898)
         call KTINCL(ktrecom,ppp,npart,ktpjet,parttojet,NJET,*898)
         
         if (NJET.gt.IKTPMX) goto 143
         
         if (lp) then
            write(*,*) 'KTJETS:',njet
            do j=1,njet
               write(*,*) (ktpjet(i,j),i=1,4)
            enddo
         endif 

c     --- sort jets in pt (massive case) although variable denoted
c     as et
         Do loop=1,NJET
            map(loop)=loop
         ENDDO
         
         do i = 1, NJET
            etjeti = sqrt( ktpjet(1,i)**2 + ktpjet(2,i)**2 )
            do j = (i+1), NJET
               etjetj = sqrt( ktpjet(1,j)**2 + ktpjet(2,j)**2 )
               if( etjetj.gt.etjeti )then
                  do k = 1, 4
                     help      = ktpjet(k,i)
                     ktpjet(k,i) = ktpjet(k,j)
                     ktpjet(k,j) = help
                  enddo
                  k=map(i)
                  map(i)=map(j)
                  map(j)=k
                  etjeti = etjetj
               endif
            enddo               ! j
         enddo                  ! i
         DO loop=1,NJET
            invmap(map(loop))=loop
         ENDDO
         
c     --- get output
         IF (NJET.gt.MAXHZJETS) THEN
            NUMJETS=MAXHZJETS
         ELSE
            NUMJETS=NJET
         ENDIF
*     
         DO loop=1,NUMJETS
            px=ktPJET(1,loop)
            py=ktPJET(2,loop)
            pz=ktPJET(3,loop)
            ptsq=px**2+py**2
            ppsq=(SQRT(ptsq+pz**2)+ABS(pz))**2
            IF (ptsq.le.4.25E-18*ppsq) THEN
               eta=20.
            ELSE
               eta=0.5*LOG(ppsq/ptsq)
            ENDIF
            eta=SIGN(eta,pz)
            IF (ptsq.EQ.0) THEN
               phi=0.
            ELSE
               phi=ATAN2(py,px)
            ENDIF
            Et=ktPJET(4,loop)*SQRT(ptsq/(ptsq+pz**2))
            Jets(loop,1)=eta    ! eta
            Jets(loop,2)=phi    ! phi
            Jets(loop,3)=Et     ! Et
            Jets(loop,4)=ktPJET(4,loop) ! E
            Jets(loop,5)=ktPJET(1,loop) ! px
            Jets(loop,6)=ktPJET(2,loop) ! py
            Jets(loop,7)=ktPJET(3,loop) ! pz
            Jets(loop,8)=0      ! m
         Enddo
         
         Do loop=1,npart
            if ((parttojet(loop).ge.1).and.(parttojet(loop).le.NJET))
     &           then
               iJetNo(ref(loop))=invmap(parttojet(loop))
            else
               iJetNo(ref(loop))=0
            endif
         ENDDO
         
         goto 144
 143     CONTINUE
         write(6,*) cxxxx,'Error in KTCLUS !'
         
 144     continue
C     --- end KTCLUS stuff - find jets
***********************************************************************
***********************************************************************
      ELSE
C     ---   user jet finders
C     ---   comments for user additions:
C     ---   1. use tracks from HEPEVTP
C     ---   2. return jets in highest Energy first order.
C     ---   3. update HZJTNAME
         PRINT*,'HZJTFIND : User jet finder called but not defined'
C     ---   end user jet finders
      ENDIF
      
c     --- copy common variables to argument variables
      iArgNJts=NUMJETS
      do loop=1,NUMJETS
         do loop2=1,8
            ArgJets(loop,loop2)=JETS(loop,loop2)
         enddo
         
      enddo
C     --- finish off HZJETCMN
      
C     --- turn the electron on again so that we haven't modified
c     anything
*     
      if (lp) write(6,*) '**hzjtfind: end myil= ',myil
*     
*     This is a bug that needs fixing !! B.C. 21/06/00
*     myil can be uninitialised for some reason, and > NMXHEP      
      IF (myil.ge.0.and.myil.lt.NMXHEP) Then
*     IF (myil.ge.0) then
         ISTHEP(myil)=1
      ENDIF
      Return
      End












