      SUBROUTINE HZNLC1(iflag)
*****************************************************************************
* This photoproduction routine has to be run twice with the following
* code additions:
* 	+1000 for the DIRECT component run.
*	+2000 for the SINGLY-RESOLVED component run.
*       +3000 for the DOUBLY-RESOLVED component run
* for all three phases (iflag=1,2,3)
*
* iflag = 1,2 or 3 depending on whether it is the initialization,
*		processing or termination phase (respectively)
*
* The default jetfinder is KTCLUS
* To change jetfinder use these values to call the initialisation phase.
* 1 + 10 * (jetfinder number)
*
* Cuts : 	Q2 < 4
*
* Recommended value for Ptmin is 2.2 GeV (to be set in MC set up)
*
* Author : Russell Taylor
*    JMB : SYmmetrized for HERWIG 19/5/99 JMB
*****************************************************************************
*
* HERA Tuning defined commons

	IMPLICIT NONE

#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
#include "hzjetcmn.inc" 

	CHARACTER*6 jetf

	INTEGER njet
c	DOUBLE PRECISION jets(50,8)
	DOUBLE PRECISION coner
        SAVE CONER
	
	INTEGER loop,loop2,nloop
	DOUBLE PRECISION rap,Et
	DOUBLE PRECISION ybj,q2
	DOUBLE PRECISION ybjcut(2),q2cut(2)
	DOUBLE PRECISION Etcut(6)
        SAVE etcut, ybjcut, q2cut
	DOUBLE PRECISION mhxsec(3)
	DOUBLE PRECISION mhntot(3)
        REAL norm, srwgt

	INTEGER intiflag, chjet, iproc, iflag
	SAVE chjet
C --- Default jet finder -- KTCLUS
	DATA chjet/3/

C --- choose direct/resolved
C --- recommend 1 - DIRECT, 2 - 1-RESOLVED, 3 - 2-RESOLVED
	IF (iflag.gt.1000) THEN
	  iproc=INT(iflag/1000)
	  intiflag=MOD(iflag,1000)
	ENDIF
	IF ((iproc.gt.3).or.(iproc.lt.1)) THEN
	  PRINT*,'HZNLC : Badly called routine'
	  PRINT*,'        set intiflag +1000, +2000 or +3000'
	ENDIF

C --- choose jet finder
	IF ((intiflag.gt.10).and.(MOD(intiflag,10).eq.1)) THEN
	  chjet=INT(intiflag/10)
	  intiflag=MOD(intiflag,10)
	ENDIF

	IF (intiflag.eq.1) THEN
C *** Initialisation run ***
C *** Create directories ***
	  IF (iproc.eq.1) THEN
	    CALL HCDIR('//HISTO',' ')
	    CALL HMDIR('NLC','S')
	    CALL HCDIR('//PAWC',' ')
	    CALL HMDIR('NLC','S')

            CALL HBOOK1(1,'X-Sec (nb), Barrel ECAL',20,0.,80.,0.)
            CALL HBOOK1(2,'X-Sec (nb), Forward/Backward ECAL'
     &                          ,20,0.,80.,0.)
            CALL HBOOK1(3,'Barrel ECAL',15,0.,15.,0.)
            CALL HBOOK1(4,'Forward/Backward ECAL',15,0.,15.,0.)

            CALL HBOOK1(10,'X-Sec (nb), Et>2 GeV',16,0.,3.2,0.)
            CALL HBOOK1(11,'X-Sec (nb), Et>4 GeV',16,0.,3.2,0.)
            CALL HBOOK1(12,'X-Sec (nb), Et>6 GeV',16,0.,3.2,0.)
            CALL HBOOK1(13,'X-Sec (nb), Et>10 GeV',16,0.,3.2,0.)
            CALL HBOOK1(14,'X-Sec (nb), Et>15 GeV',16,0.,3.2,0.)
            CALL HBOOK1(15,'X-Sec (nb), Et>20 GeV',16,0.,3.2,0.)
          ENDIF

	  CALL HCDIR('//HISTO/NLC',' ')
	  CALL HCDIR('//PAWC/NLC',' ')

          CALL HBOOK1(1+iproc*100,'X-Sec (nb), Barrel ECAL'
     &                          ,20,0.,80.,0.)
          CALL HBOOK1(2+iproc*100,'X-Sec (nb), Fwd/Back ECAL'
     &                          ,20,0.,80.,0.)
          CALL HBOOK1(3+iproc*100,'Barrel ECAL',15,0.,15.,0.)
          CALL HBOOK1(4+iproc*100,'Forward/Backward ECAL',15,0.,15.,0.)

          CALL HBOOK1(10+iproc*100,'X-Sec (nb), Et>2 GeV',16,0.,3.2,0.)
          CALL HBOOK1(11+iproc*100,'X-Sec (nb), Et>4 GeV',16,0.,3.2,0.)
          CALL HBOOK1(12+iproc*100,'X-Sec (nb), Et>6 GeV',16,0.,3.2,0.)
          CALL HBOOK1(13+iproc*100,'X-Sec (nb), Et>10 GeV',16,0.,3.2,0.)
          CALL HBOOK1(14+iproc*100,'X-Sec (nb), Et>15 GeV',16,0.,3.2,0.)
          CALL HBOOK1(15+iproc*100,'X-Sec (nb), Et>20 GeV',16,0.,3.2,0.)

          CALL HBARX(0)

	  ybjcut(1)=0.0
	  ybjcut(2)=1.0
	  q2cut(1)=0.
	  q2cut(2)=4.

C --- Et cuts for rapidity plots
          DATA Etcut/2.,4.,6.,10.,15.,20./

C --- standard cone radius is 1.0
	  CALL HZJETRAD(2,CONER)
	  IF (CONER.lt.0.0) THEN
	    CONER=1.0
	  ENDIF
	  CALL HZJTNAME(chjet,jetf)
	  WRITE(6,*)'********************************************'
	  WRITE(6,*)'* HZNLC called, histograms will be output. *'
	  WRITE(6,*)'*     Jet finder selected : ',jetf,'         *'
	  WRITE(6,1001)'*        with cone radius : ',CONER,' *'
	  WRITE(6,*)'********************************************'

 1001	  FORMAT(A29,F5.2,A11)

	ELSE IF (intiflag.eq.2) THEN
C *** Filling runs ***
	
C *** Change directory to our graphs ***
	  CALL HCDIR('//PAWC/NLC',' ')

	  q2=hzeekin(1)
	  IF (q2.eq.-1) THEN
	    PRINT*,'HZNLC: HZEEKIN returns error for Q2'
	  ELSEIF (q2.lt.0) THEN
	    q2=abs(q2)
	  ENDIF
C	  ybj=hzeekin(3)
C	  IF (ybj.eq.-1) THEN
C	    PRINT*,'HZNLC: HZEEKIN returns error for ybj'
C	  ENDIF

C *** Fill our histogram ***

	  IF (q2.lt.q2cut(2)) Then

      	    CALL hzjtfind(chjet,CONER,NJET,Jets)

	    DO loop=1,NJET
	      Et=Jets(loop,3)
              IF (iproc.eq.2) THEN
                 rap=ABS(Jets(loop,1))
                 srwgt=2.0
              ELSE
                 rap=Jets(loop,1)
                 srwgt=1.0
              ENDIF

              IF ((rap.ge.-1.3).and.(rap.le.1.3)) THEN
                 CALL HFILL(iproc*100+1,REAL(Et),0.,wtx*srwgt)
                 CALL HFILL(iproc*100+3,REAL(Et),0.,wtx*srwgt)
              ENDIF
              IF ((rap.ge.1.3).and.(rap.le.3.2)) THEN
                 CALL HFILL(iproc*100+2,REAL(Et),0.,wtx*srwgt)
                 CALL HFILL(iproc*100+4,REAL(Et),0.,wtx*srwgt)
              ENDIF

              DO loop2=1,6
                 IF ((rap.le.3.2)
     &                      .and.(Et.gt.Etcut(loop2))) THEN
                    CALL HFILL(iproc*100+9+loop2,REAL(rap),0.,wtx)
                 ENDIF
              ENDDO

      	    ENDDO ! loop
	 ENDIF

	ELSE IF (intiflag.eq.3) THEN
C *** Termination run ***
C *** Change directory ***
	  CALL HCDIR('//PAWC/NLC',' ')
C *** Finish off histograms ***
	  IF (Xsec.eq.0) THEN
	    PRINT*,'HZNLC: termination called with zero cross section'
	    PRINT*,'       cross section graph meaningless'
	    PRINT*,'	   in process :',iproc*1000
	    Xsec=1
	  ENDIF
	  IF (Ntot.eq.0) THEN
	    PRINT*,'HZNLC: termination called with no total events'
	    PRINT*,'       cross section graph meaningless'
	    PRINT*,'       in process :',iproc*1000
	    Ntot=1
	  ENDIF

C --- store Xsec and Ntot for both direct and resolved events.
	  mhXsec(iproc)=Xsec
	  mhntot(iproc)=Ntot

C --- normalise direct/resolved histograms
          norm=0.
          norm=REAL(mhntot(iproc)/mhXsec(iproc))
          CALL HZHINRM(100*iproc+1,0,norm,1)
          CALL HZHINRM(100*iproc+2,0,norm,1)
          CALL HZHINRM(100*iproc+3,0,norm,1)
          CALL HZHINRM(100*iproc+4,0,norm,1)
          DO nloop=1,6
             CALL HZHINRM(100*iproc+9+nloop,0,norm,1)
          ENDDO
             
          IF (iproc.eq.2) THEN
             CALL HOPERA(101,'+e',201,1,1.,1.)
             CALL HOPERA(102,'+e',202,2,1.,1.)
             CALL HOPERA(103,'+e',203,3,1.,1.)
             CALL HOPERA(104,'+e',204,4,1.,1.)

             CALL HOPERA(110,'+e',210,10,1.,1.)
             CALL HOPERA(111,'+e',211,11,1.,1.)
             CALL HOPERA(112,'+e',212,12,1.,1.)
             CALL HOPERA(113,'+e',213,13,1.,1.)
             CALL HOPERA(114,'+e',214,14,1.,1.)
             CALL HOPERA(115,'+e',215,15,1.,1.)
          ENDIF

          IF (iproc.eq.3) THEN
             CALL HOPERA(301,'+e',1,1,1.,1.)
             CALL HOPERA(302,'+e',2,2,1.,1.)
             CALL HOPERA(303,'+e',3,3,1.,1.)
             CALL HOPERA(304,'+e',4,4,1.,1.)

             CALL HOPERA(310,'+e',10,10,1.,1.)
             CALL HOPERA(311,'+e',11,11,1.,1.)
             CALL HOPERA(312,'+e',12,12,1.,1.)
             CALL HOPERA(313,'+e',13,13,1.,1.)
             CALL HOPERA(314,'+e',14,14,1.,1.)
             CALL HOPERA(315,'+e',15,15,1.,1.)
          ENDIF

        ELSE
C *** End ***
          PRINT*,'HZNLC:Please run routine with iflag set to 1,2 or 3'
          ENDIF

        RETURN
        END









