      SUBROUTINE HZNLC2(iflag)
*****************************************************************************
* This photoproduction routine has to be run twice with the following
* code additions:
* 	+1000 for the DIRECT component run.
*	+2000 for the SINGLY-RESOLVED component run.
*       +3000 for the DOUBLY-RESOLVED component run.
* for all three phases (iflag=1,2,3)
*
* iflag = 1,2 or 3 depending on wether it is the initialization,
*		processing or termination phase (respectively)
*
* Cuts : 	Q2 < 4
*
* Recommended value for Ptmin is 2.2 GeV (to be set in MC set up)
*
* Author : Russell Taylor
*
*****************************************************************************

      IMPLICIT NONE
#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"

      INTEGER intiflag,iproc,iflag,IHEP
      DOUBLE PRECISION X,Y,ANG,P,PRODX,PRODY,PRODV,D,DX,DY
      LOGICAL CHARGED,NEUTRINO
      DOUBLE PRECISION mhXsec(3)
      DOUBLE PRECISION mhntot(3),factor
      REAL norm,E
      DOUBLE PRECISION ybj,q2
      DOUBLE PRECISION ybjcut(2),q2cut(2)

      
C --- Choose direct/resolved
C --- recommend 1 - DIRECT, 2 - 1-RESOLVED, 3 - 2-RESOLVED
	IF (iflag.gt.1000) THEN
	  iproc=INT(iflag/1000)
	  intiflag=MOD(iflag,1000)
	ENDIF
	IF ((iproc.gt.3).or.(iproc.lt.1)) THEN
	  PRINT*,'HZNLC2 : Badly called routine'
	  PRINT*,'         set intiflag +1000, +2000 or +3000'
	ENDIF

      IF (intiflag.EQ.1) THEN
C --- Initialisation run
C --- Create directories
        IF (iproc.EQ.1) THEN
          CALL HCDIR('//HISTO',' ')
          CALL HMDIR('NLC2','S')
          CALL HCDIR('//PAWC',' ')
          CALL HMDIR('NLC2','S')

          CALL HBOOK1(1,'Momentum spectrum for particles inside mask'
     &                          ,20,0.,80.,0.)
          CALL HBOOK1(2,'Momentum spectrum for particles into mask'
     &                          ,20,0.,80.,0.)
          CALL HBOOK1(3,'Momentum spectrum for particles outside mask'
     &                          ,20,0.,80.,0.)
          CALL HBOOK1(4,'Momentum spectrum for particles into VDet'
     &                          ,20,0.,80.,0.)

          CALL HBOOK1(11,'Charged particles - inside mask'
     &                          ,20,0.,80.,0.)
          CALL HBOOK1(12,'Charged particles - into mask'
     &                          ,20,0.,80.,0.)
          CALL HBOOK1(13,'Charged particles - outside mask'
     &                          ,20,0.,80.,0.)
          CALL HBOOK1(14,'Charged particles - into VDet'
     &                          ,20,0.,80.,0.)

          CALL HBOOK1(21,'E/event inside mask'
     &                          ,1,0.,0.0555,0.)
          CALL HBOOK1(22,'E/event into mask'
     &                          ,1,0.0555,0.0831,0.)
          CALL HBOOK1(23,'E/event outside mask'
     &                          ,10,0.0831,1.5708,0.)

          CALL HBOOK1(31,'Charged E/event inside mask'
     &                          ,1,0.,0.0555,0.)
          CALL HBOOK1(32,'Charged E/event into mask'
     &                          ,1,0.0555,0.0831,0.)
          CALL HBOOK1(33,'Charged E/event outside mask'
     &                          ,10,0.0831,1.5708,0.)
        ENDIF

        CALL HCDIR('//HISTO/NLC2',' ')
        CALL HCDIR('//PAWC/NLC2',' ')

        CALL HBOOK1(1+iproc*100,'Particles - inside mask'
     &                          ,20,0.,80.,0.)
        CALL HBOOK1(2+iproc*100,'Particles - into mask'
     &                          ,20,0.,80.,0.)
        CALL HBOOK1(3+iproc*100,'Particles - outside mask'
     &                          ,20,0.,80.,0.)
        CALL HBOOK1(4+iproc*100,'Particles into VDet'
     &                          ,20,0.,80.,0.)

        CALL HBOOK1(11+iproc*100,'Charged particles - inside mask'
     &                          ,20,0.,80.,0.)
        CALL HBOOK1(12+iproc*100,'Charged particles - into mask'
     &                          ,20,0.,80.,0.)
        CALL HBOOK1(13+iproc*100,'Charged particles - outside mask'
     &                          ,20,0.,80.,0.)
        CALL HBOOK1(14+iproc*100,'Charged particles - into VDet'
     &                          ,20,0.,80.,0.)

        CALL HBOOK1(21+iproc*100,'E/event inside mask'
     &                          ,1,0.,0.0555,0.)
        CALL HBOOK1(22+iproc*100,'E/event into mask'
     &                          ,1,0.0555,0.0831,0.)
        CALL HBOOK1(23+iproc*100,'E/event outside mask'
     &                          ,10,0.0831,1.5708,0.)

        CALL HBOOK1(31+iproc*100,'Charged E/event inside mask'
     &                          ,1,0.,0.0555,0.)
        CALL HBOOK1(32+iproc*100,'Charged E/event into mask'
     &                          ,1,0.0555,0.0831,0.)
        CALL HBOOK1(33+iproc*100,'Charged E/event outside mask'
     &                          ,10,0.0831,1.5708,0.)

        CALL HBARX(0)

	  ybjcut(1)=0.0
	  ybjcut(2)=1.0
	  q2cut(1)=0.
	  q2cut(2)=4.

	WRITE(6,*)'*********************************************'
        WRITE(6,*)'* HZNLC2 called, histograms will be output. *'
	WRITE(6,*)'*********************************************'


      ELSE IF (intiflag.EQ.2) THEN
C --- Filling runs

C --- Change directory to our graphs
        CALL HCDIR('//PAWC/NLC2',' ')

	q2=hzeekin(1)
	IF (q2.eq.-1) THEN
	  PRINT*,'HZNLC: HZEEKIN returns error for Q2'
	ELSEIF (q2.lt.0) THEN
	  q2=abs(q2)
	ENDIF
C	ybj=hzeekin(3)
C	IF (ybj.eq.-1) THEN
C	  PRINT*,'HZNLC: HZEEKIN returns error for ybj'
C	ENDIF

C --- Fill our histograms
	IF (q2.lt.q2cut(2)) Then

C --- Loop over particles produced in current event
        DO IHEP=1,NHEP

C --- Flag if neutrino
C --- (TRUE corresponds to NOT a neutrino - just to be confusing!)
          NEUTRINO=.TRUE.
          IF (ABS(IDHEP(IHEP)).EQ.12) NEUTRINO=.FALSE.
          IF (ABS(IDHEP(IHEP)).EQ.14) NEUTRINO=.FALSE.

C --- Select whether NOT a neutrino
          IF (NEUTRINO) THEN

            X=PHEP(3,IHEP)
            Y=SQRT(PHEP(1,IHEP)**2+PHEP(2,IHEP)**2)
            ANG=HZPHMANG(X,Y)
            P=SQRT(X**2+Y**2)
            E=REAL(PHEP(4,IHEP))

C --- Flag whether charged particle
              CHARGED=.FALSE.
              IF (ABS(IDHEP(IHEP)).EQ.11) CHARGED=.TRUE.
              IF (ABS(IDHEP(IHEP)).EQ.13) CHARGED=.TRUE.
              IF (ABS(IDHEP(IHEP)).EQ.211) CHARGED=.TRUE.
              IF (ABS(IDHEP(IHEP)).EQ.321) CHARGED=.TRUE.
              IF (ABS(IDHEP(IHEP)).EQ.2212) CHARGED=.TRUE.

C --- Select final state particles (not incl. neutrinos)
          IF (ISTHEP(IHEP).EQ.1) THEN
C --- Currently assuming all particles originate at IP

C --- Inside mask
              IF (ANG.LT.0.0555) THEN
                CALL HFILL(iproc*100+1,REAL(P),0.,wtx)
                CALL HFILL(iproc*100+21,REAL(ANG),0.,wtx*E)
                IF (CHARGED) THEN
                  CALL HFILL(iproc*100+11,REAL(P),0.,wtx)
                  CALL HFILL(iproc*100+31,REAL(ANG),0.,wtx*E)
                ENDIF
              ENDIF

C --- Into mask
              IF ((ANG.GE.0.0555).AND.(ANG.LE.0.0831)) THEN
                CALL HFILL(iproc*100+2,REAL(P),0.,wtx)
                CALL HFILL(iproc*100+22,REAL(ANG),0.,wtx*E)
                IF (CHARGED) THEN
                  CALL HFILL(iproc*100+12,REAL(P),0.,wtx)
                  CALL HFILL(iproc*100+32,REAL(ANG),0.,wtx*E)
                ENDIF
              ENDIF

C --- Outside mask (everywhere else)
              IF ((ANG.GT.0.0831).AND.(ANG.LT.1.5708)) THEN
                CALL HFILL(iproc*100+3,REAL(P),0.,wtx)
                CALL HFILL(iproc*100+23,REAL(ANG),0.,wtx*E)
                IF (CHARGED) THEN
                  CALL HFILL(iproc*100+13,REAL(P),0.,wtx)
                  CALL HFILL(iproc*100+33,REAL(ANG),0.,wtx*E)
                ENDIF
              ENDIF

          ENDIF         !loop over final state particles

C --- Vertex detector histograms
          PRODX=VHEP(3,IHEP)
          PRODY=SQRT(VHEP(1,IHEP)**2+VHEP(2,IHEP)**2)
C --- Select if produced within inner edge of VDET
          IF ((PRODY.LE.25).AND.(ABS(PRODX).LE.300)) THEN
C --- See if final state particle, if not see whether decays within
C     inside edge of VDET
            IF ((ISTHEP(IHEP).EQ.1).OR.(ISTHEP(IHEP).EQ.2)) THEN
               IF (ISTHEP(IHEP).EQ.2) THEN
                  D=JDAHEP(1,IHEP)
                  DX=VHEP(3,D)
                  DY=SQRT(VHEP(1,D)**2+VHEP(2,D)**2)
                  IF ((DY.LE.25).AND.(ABS(DX).LE.300)) GOTO 55
               ENDIF
C --- Need to deal with particles produced away from IP
C               PRODV=SQRT(PRODX**2+PRODY**2)
C               IF (PRODV.GE.0.1) THEN
C                  PRINT*,'NOT VERY NEAR IP!'
C                  PRINT*,PRODV
C               ENDIF
C --- Assumes all particles produced at IP, sees if trajectories take them
C     into VDET
               IF ((ANG.GE.0.0835).AND.(ANG.LE.3.058)) THEN
                  CALL HFILL(iproc*100+4,REAL(P),0.,wtx)
                  IF (CHARGED) THEN
                     CALL HFILL(iproc*100+14,REAL(P),0.,wtx)
                  ENDIF
               ENDIF
            ENDIF
 55       ENDIF            !produced within VDET

          ENDIF        !neutrinos
        ENDDO        !loop over particles
        ENDIF        !y_bj & Q2 cuts


      ELSE IF (intiflag.EQ.3) THEN
C *** Termination run ***
C *** Change directory ***
	CALL HCDIR('//PAWC/NLC2',' ')
C *** Finish off histograms ***
	IF (Xsec.eq.0) THEN
	  PRINT*,'HZNLC2: termination called with zero cross section'
	  PRINT*,'        cross section graph meaningless'
	  PRINT*,'	    in process :',iproc*1000
	  Xsec=1
	ENDIF
	IF (Ntot.eq.0) THEN
          PRINT*,'HZNLC2: termination called with no total events'
	  PRINT*,'        cross section graph meaningless'
	  PRINT*,'        in process :',iproc*1000
	  Ntot=1
	ENDIF

C --- store Xsec and Ntot for both direct and resolved events.
	mhXsec(iproc)=Xsec
	mhntot(iproc)=Ntot

C --- normalise direct/resolved histograms
        norm=0.
        norm=REAL(mhntot(iproc)/mhXsec(iproc))
        CALL HZHINRM(100*iproc+1,0,norm,1)
        CALL HZHINRM(100*iproc+2,0,norm,1)
        CALL HZHINRM(100*iproc+3,0,norm,1)
        CALL HZHINRM(100*iproc+4,0,norm,1)
 
        CALL HZHINRM(100*iproc+11,0,norm,1)
        CALL HZHINRM(100*iproc+12,0,norm,1)
        CALL HZHINRM(100*iproc+13,0,norm,1)
        CALL HZHINRM(100*iproc+14,0,norm,1)

        IF (iproc.eq.2) THEN
          CALL HOPERA(101,'+e',201,1,1.,1.)
          CALL HOPERA(102,'+e',202,2,1.,1.)
          CALL HOPERA(103,'+e',203,3,1.,1.)
          CALL HOPERA(104,'+e',204,4,1.,1.)

          CALL HOPERA(111,'+e',211,11,1.,1.)
          CALL HOPERA(112,'+e',212,12,1.,1.)
          CALL HOPERA(113,'+e',213,13,1.,1.)
          CALL HOPERA(114,'+e',214,14,1.,1.)

          CALL HOPERA(121,'+e',221,21,1.,1.)
          CALL HOPERA(122,'+e',222,22,1.,1.)
          CALL HOPERA(123,'+e',223,23,1.,1.)

          CALL HOPERA(131,'+e',231,31,1.,1.)
          CALL HOPERA(132,'+e',232,32,1.,1.)
          CALL HOPERA(133,'+e',233,33,1.,1.)
        ENDIF

        IF (iproc.eq.3) THEN
          CALL HOPERA(301,'+e',1,1,1.,1.)
          CALL HOPERA(302,'+e',2,2,1.,1.)
          CALL HOPERA(303,'+e',3,3,1.,1.)
          CALL HOPERA(304,'+e',4,4,1.,1.)

          CALL HOPERA(311,'+e',11,11,1.,1.)
          CALL HOPERA(312,'+e',12,12,1.,1.)
          CALL HOPERA(313,'+e',13,13,1.,1.)
          CALL HOPERA(314,'+e',14,14,1.,1.)

          CALL HOPERA(321,'+e',21,21,1.,1.)
          CALL HOPERA(322,'+e',22,22,1.,1.)
          CALL HOPERA(323,'+e',23,23,1.,1.)

          CALL HOPERA(331,'+e',31,31,1.,1.)
          CALL HOPERA(332,'+e',32,32,1.,1.)
          CALL HOPERA(333,'+e',33,33,1.,1.)

          factor=1/(mhntot(1)+mhntot(2)+mhntot(3))
          CALL HOPERA(21,'+e',21,21,factor,0.)
          CALL HOPERA(22,'+e',22,22,factor,0.)
          CALL HOPERA(23,'+e',23,23,factor,0.)

          CALL HOPERA(31,'+e',31,31,factor,0.)
          CALL HOPERA(32,'+e',32,32,factor,0.)
          CALL HOPERA(33,'+e',33,33,factor,0.)
        ENDIF

      ELSE
C *** End ***
          PRINT*,'HZNLC:Please run routine with iflag set to 1,2 or 3'
      ENDIF

        RETURN
        END
