      SUBROUTINE HZBURGIN2(iflag)
*****************************************************************************
* This photoproduction routine has to be run thrice with the following
* code additions:
* 	+1000 for the DIRECT component run.
*	+2000 for the SINGLY-RESOLVED component run.
*       +3000 for the DOUBLY-RESOLVED component run
* for all three phases (iflag=1,2,3)
*
* iflag = 1,2 or 3 depending on whether it is the initialization,
*		processing or termination phase (respectively)
*
* The default jetfinder is PXCONE
* To change jetfinder use these values to call the initialisation phase.
* 1 + 10 * (jetfinder number)
*
* This routine produces the following graphs:
*     11   Dijets, 2-Resolved (pp. 103 & 113)
*     12   As above, but direct (pp. 103 & 113)
*     20   Inclusive Jets Et cross-section (pp. 92 & 111)
*     21   Inclusive Jets eta X-sec (Et>3) (pp. 95 & 112)
*     22   Inclusive Jets eta X-sec (Et>5) (pp. 96 & 112)
*     
* Cuts : 	Anti-tag on scattered beam particles (< 33 mrad)
*               Et > 3 GeV
*               |jet rapidity| < 2.0
*
* Recommended value for Ptmin is 2.2 GeV (to be set in MC set up)
*
* Author : Russell Taylor (rjt@hep.ucl.ac.uk)
*          
* Reference: Roland Burgin, Freiburg thesis, May 1998.
*            
*
*****************************************************************************
*
* HERA Tuning defined commons

	IMPLICIT NONE

#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
#include "hzjetcmn.inc" 

	CHARACTER*6 jetf

	INTEGER njet,njet2
c	DOUBLE PRECISION jets(50,8)
	DOUBLE PRECISION coner
        SAVE CONER
	
	INTEGER loop,loop2,nloop,sum,veto1,veto2
        LOGICAL hadron
	DOUBLE PRECISION rap(2),Et(2)
        DOUBLE PRECISION xplus,xminus,xpdenom,xmdenom
        DOUBLE PRECISION costh,mjj,etabar
	DOUBLE PRECISION ybj,q2,angle
	DOUBLE PRECISION ybjcut(2),q2cut(2)
	DOUBLE PRECISION Etcut(6)
	DOUBLE PRECISION mhxsec(3)
	INTEGER mhntot(3)
        REAL norm

        INTEGER ivec
        REAL temp(8)

        REAL xbins(8)
        DATA xbins/3.0,4.0,5.0,6.5,8.5,11.0,15.0,20.0/
C --- 2-Resolved component
        REAL det2(7),det2_e(7),det2_esys(7)
        DATA det2/131.0,57.4,15.2,6.43,2.11,0.46,0.12/
        DATA det2_e/3.0,1.9,0.7,0.43,0.24,0.09,0.04/
        DATA det2_esys/15.0,8.8,3.1,1.4,0.51,0.29,0.1/
C --- Direct component
        REAL det3(7),det3_e(7),det3_esys(7)
        DATA det3/11.8,4.92,3.75,2.34,0.98,0.55,0.01/
        DATA det3_e/0.9,0.42,0.3,0.24,0.13,0.09,0.01/
        DATA det3_esys/3.0,1.63,0.95,0.51,0.34,0.2,0.02/

C --- Data for Et inclusive jets graph (-2<eta<2)
        REAL det4(7),det4_e(7),det4_esys(7)
        DATA det4/495.0,136.0,40.7,15.0,5.51,1.65,0.22/
        DATA det4_e/6.0,3.0,1.0,0.6,0.32,0.15,0.03/
        DATA det4_esys/54.0,16.0,5.9,1.7,0.75,0.36,0.1/
C --- Inclusive jets as function of eta (Et>3)
        REAL drap(5),drap_e(5),drap_esys(5)
        DATA drap/430.8,429.3,363.3,371.0,405.9/
        DATA drap_e/7.4,7.0,6.0,7.0,9.3/
        DATA drap_esys/49.4,49.7,48.3,43.7,46.5/
C --- (Et>5)
        REAL drap2(5),drap2_e(5),drap2_esys(5)
        DATA drap2/63.2,65.2,48.0,42.3,47.3/
        DATA drap2_e/2.5,2.7,2.0,2.0,2.8/
        DATA drap2_esys/8.6,8.4,9.0,6.4,6.4/
        

	INTEGER intiflag, chjet, iproc, iflag
	SAVE chjet
C --- Default jet finder (as used in paper) -- PXCONE
	DATA chjet/2/

C --- choose direct/resolved
C --- recommend 1 - DIRECT, 2 - 1-RESOLVED, 3 - 2-RESOLVED
	IF (iflag.gt.1000) THEN
	  iproc=INT(iflag/1000)
	  intiflag=MOD(iflag,1000)
	ENDIF
	IF ((iproc.gt.3).or.(iproc.lt.1)) THEN
	  PRINT*,'HZBURGIN2 : Badly called routine'
	  PRINT*,'            set intiflag +1000, +2000 or +3000'
	ENDIF

C --- choose jet finder
	IF ((intiflag.gt.10).and.(MOD(intiflag,10).eq.1)) THEN
	  chjet=INT(intiflag/10)
	  intiflag=MOD(intiflag,10)
	ENDIF

	IF (intiflag.eq.1) THEN
C *** Initialisation run ***
C *** Create directories ***
	  IF (iproc.eq.1) THEN
	    CALL HCDIR('//HISTO',' ')
	    CALL HMDIR('BURGIN2','S')
	    CALL HCDIR('//PAWC',' ')
	    CALL HMDIR('BURGIN2','S')

            CALL HBOOKB(-11,'Data X-Sec(pb), 2-res',7,xbins,0.)
            CALL HBOOKB(-111,'Data X-Sec(pb), 2-res',7,xbins,0.)
            CALL HPAK(-11,det2)
            CALL HPAKE(-11,det2_e)

            DO ivec=1,7
               temp(ivec)=sqrt(det2_e(ivec)**2+det2_esys(ivec)**2)
            ENDDO
            CALL HPAK(-111,det2)
            CALL HPAKE(-111,temp)

            CALL HBOOKB(-12,'Data X-Sec(pb), direct',7,xbins,0.)
            CALL HBOOKB(-112,'Data X-Sec(pb), direct',7,xbins,0.)
            CALL HPAK(-12,det3)
            CALL HPAKE(-12,det3_e)

            DO ivec=1,7
               temp(ivec)=sqrt(det3_e(ivec)**2+det3_esys(ivec)**2)
            ENDDO
            CALL HPAK(-112,det3)
            CALL HPAKE(-112,temp)

            CALL HBOOKB(-20,'Data X-Sec(pb), inclusive',7,xbins,0.)
            CALL HBOOKB(-120,'Data X-Sec(pb), inclusive',7,xbins,0.)
            CALL HPAK(-20,det4)
            CALL HPAKE(-20,det4_e)

            DO ivec=1,7
               temp(ivec)=sqrt(det4_e(ivec)**2+det4_esys(ivec)**2)
            ENDDO
            CALL HPAK(-120,det4)
            CALL HPAKE(-120,temp)

            CALL HBOOK1(-21,'Data X-Sec(pb), Et>3',5,0.0,2.0,0.)
            CALL HBOOK1(-121,'Data X-Sec(pb), Et>3',5,0.0,2.0,0.)
            CALL HPAK(-21,drap)
            CALL HPAKE(-21,drap_e)

            DO ivec=1,5
               temp(ivec)=sqrt(drap_e(ivec)**2+drap_esys(ivec)**2)
            ENDDO
            CALL HPAK(-121,drap)
            CALL HPAKE(-121,temp)

            CALL HBOOK1(-22,'Data X-Sec(pb), Et>5',5,0.0,2.0,0.)
            CALL HBOOK1(-122,'Data X-Sec(pb), Et>5',5,0.0,2.0,0.)
            CALL HPAK(-22,drap2)
            CALL HPAKE(-22,drap2_e)

            DO ivec=1,5
               temp(ivec)=sqrt(drap2_e(ivec)**2+drap2_esys(ivec)**2)
            ENDDO
            CALL HPAK(-122,drap2)
            CALL HPAKE(-122,temp)


            CALL HBOOKB(11,'MC X-Sec(pb), |eta|<2',7,xbins,0.)
            CALL HBOOKB(12,'MC X-Sec(pb), |eta|<2',7,xbins,0.)
            CALL HBOOKB(20,'MC X-Sec(pb), |eta|<2',7,xbins,0.)
            CALL HBOOK1(21,'MC X-Sec(pb), Et>3',5,0.0,2.0,0.)
            CALL HBOOK1(22,'MC X-Sec(pb), Et>3',5,0.0,2.0,0.)

          ENDIF

          CALL HCDIR('//HISTO/BURGIN2',' ')
          CALL HCDIR('//PAWC/BURGIN2',' ')

          CALL HBOOKB(11+iproc*100,'MC component',7,xbins,0.)
          CALL HBOOKB(12+iproc*100,'MC component',7,xbins,0.)
          CALL HBOOKB(20+iproc*100,'MC component',7,xbins,0.)
          CALL HBOOK1(21+iproc*100,'MC component',5,0.0,2.0,0.)
          CALL HBOOK1(22+iproc*100,'MC component',5,0.0,2.0,0.)
          CALL HBARX(0)

C	  q2cut(1)=0.
C	  q2cut(2)=1.8      !rough estimate - will do for now

C --- standard cone radius is 1.0
	  CALL HZJETRAD(2,CONER)
	  IF (CONER.lt.0.0) THEN
	    CONER=1.0
	  ENDIF
	  CALL HZJTNAME(chjet,jetf)
	  WRITE(6,*)'***********************************************'
	  WRITE(6,*)'* HZBURGIN2 called, histograms will be output.*'
	  WRITE(6,*)'*     Jet finder selected : ',jetf,'           *'
	  WRITE(6,1001)'*        with cone radius : ',CONER,'         *'
	  WRITE(6,*)'***********************************************'

 1001	  FORMAT(A29,F5.2,A14)

	ELSE IF (intiflag.eq.2) THEN

C *** Filling runs ***
	
C *** Change directory to our graphs ***
	  CALL HCDIR('//PAWC/BURGIN2',' ')

C	  q2=hzeekin(1)
C	  IF (q2.eq.-1) THEN
C	    PRINT*,'HZBURGIN1: HZEEKIN returns error for Q2'
C	  ELSEIF (q2.lt.0) THEN
C	    q2=abs(q2)
C	  ENDIF

          angle=hzeekin(2)

C *** Fill our histogram ***

	  IF (angle.lt.0.033) Then

      	    CALL hzjtfind(chjet,CONER,NJET,Jets)

            IF (NJET.ge.1) THEN
             njet2=1
             xplus=0.
             xminus=0.
 	     DO loop=1,NJET
               IF ((Jets(loop,3).ge.3.0).and.(ABS(Jets(loop,1))
     &                .le.2.0).and.njet2.le.2) THEN
                 Et(njet2)=Jets(loop,3)
	         rap(njet2)=Jets(loop,1)
                 CALL HFILL(iproc*100+20,REAL(Et(njet2)),0.,wtx)
                 CALL HFILL(iproc*100+21,REAL(ABS(rap(njet2))),0.,wtx)
                 IF (Et(njet2).ge.5.0) THEN
                   CALL HFILL(iproc*100+22,REAL(ABS(rap(njet2))),0.,wtx)
                 ENDIF
                 xplus=xplus+(Jets(loop,4)+Jets(loop,7))
                 xminus=xminus+(Jets(loop,4)-Jets(loop,7))
                 njet2=njet2+1
               ENDIF
             ENDDO
             IF (njet2.eq.3) THEN

C --- Stuff to separate out resolved & direct events
               IF (GEN(1:3).eq.'HRW') THEN
                  veto1=1
                  veto2=2
               ELSEIF (GEN(1:3).eq.'PYT') THEN
                  veto1=3
                  veto2=4
               ELSE
                  write(6,*)'Unknown generator'
               ENDIF

               xpdenom=0.
               xmdenom=0.
               DO sum=1,NHEP
                  IF ((ISTHEP(sum).eq.1).and.(JMOHEP(1,sum).ne.veto1)
     &                 .and.(JMOHEP(1,sum).ne.veto2)) THEN
                     xpdenom=xpdenom+(PHEP(4,sum)+PHEP(3,sum))
                     xmdenom=xmdenom+(PHEP(4,sum)-PHEP(3,sum))
                  ENDIF
               ENDDO
               xplus=xplus/xpdenom
               xminus=xminus/xmdenom

               IF ((xplus.lt.0.8).and.(xminus.lt.0.8)) THEN
                 CALL HFILL(iproc*100+11,REAL(Et(1)),0.,wtx)
                 CALL HFILL(iproc*100+11,REAL(Et(2)),0.,wtx)
               ENDIF
               IF ((xplus.gt.0.8).and.(xminus.gt.0.8)) THEN
                 CALL HFILL(iproc*100+12,REAL(Et(1)),0.,wtx)
                 CALL HFILL(iproc*100+12,REAL(Et(2)),0.,wtx)
               ENDIF

             ENDIF          !njet2=3
            ENDIF           !NJET.ge.2

         ENDIF             !anti-tag cut

	ELSE IF (intiflag.eq.3) THEN
C *** Termination run ***
C *** Change directory ***
	  CALL HCDIR('//PAWC/BURGIN2',' ')
C *** Finish off histograms ***
	  IF (Xsec.eq.0) THEN
	    PRINT*,
     &        'HZBURGIN2: termination called with zero cross section'
	    PRINT*,'           cross section graph meaningless'
	    PRINT*,'	       in process :',iproc*1000
	    Xsec=1
	  ENDIF
	  IF (Ntot.eq.0) THEN
	    PRINT*,'HZBURGIN2: termination called with no total events'
	    PRINT*,'           cross section graph meaningless'
	    PRINT*,'           in process :',iproc*1000
	    Ntot=1
	  ENDIF

C --- store Xsec and Ntot for both direct and resolved events.
	  mhXsec(iproc)=1000*Xsec
	  mhntot(iproc)=Ntot

C --- normalise direct/resolved histograms
          norm=0.
          norm=REAL(mhntot(iproc)/mhXsec(iproc))
          CALL HZHINRM(100*iproc+11,0,norm,1)
          CALL HZHINRM(100*iproc+12,0,norm,1)
          CALL HZHINRM(100*iproc+20,0,norm,1)
          CALL HZHINRM(100*iproc+21,0,norm,1)
          CALL HZHINRM(100*iproc+22,0,norm,1)
             
          IF (iproc.eq.2) THEN
             CALL HOPERA(111,'+e',211,11,1.,1.)
             CALL HOPERA(112,'+e',212,12,1.,1.)
             CALL HOPERA(120,'+e',220,20,1.,1.)
             CALL HOPERA(121,'+e',221,21,1.,1.)
             CALL HOPERA(122,'+e',222,22,1.,1.)
          ENDIF

          IF (iproc.eq.3) THEN
             CALL HOPERA(311,'+e',11,11,1.,1.)
             CALL HOPERA(312,'+e',12,12,1.,1.)
             CALL HOPERA(320,'+e',20,20,1.,1.)
             CALL HOPERA(321,'+e',21,21,1.,1.)
             CALL HOPERA(322,'+e',22,22,1.,1.)

             CALL HZCHISQ(-111,11)
             CALL HZCHISQ(-112,12)
             CALL HZCHISQ(-120,20)
             CALL HZCHISQ(-121,21)
             CALL HZCHISQ(-122,22)
          ENDIF

        ELSE
C *** End ***
          PRINT*,'HZBURGIN2:Please run with iflag set to 1,2 or 3'
          ENDIF

        RETURN
        END









