      SUBROUTINE HZC00109(iflag)
*****************************************************************************
* This e-gamma DIS routine has to be run just the once.
* for all three phases (iflag=1,2,3)
*
* iflag = 1,2 or 3 depending on wether it is the initialization,
*		processing or termination phase (respectively)
*
* This routine contains the following data distributions:
*     -10 to -14  Herwig corrected, low Q^2
*     -20 to -24    "        "    , high Q^2
*     -30 to -34  Phojet corrected, low Q^2
*     -40 to -44    "        "    , high Q^2
*
* and produces the following graphs:
*
*     10     W_res
*     11     E_(t,out)
*     12     N_trk
*     13     P_(t,trk)
*     14     Energy Flow
*     20     W_res
*     21     E_(t,out)
*     22     N_trk
*     23     P_(t,trk)
*     24     Energy Flow
*
* optionally by defining the DEBUG directive the following plots are produced
*
*       -110 -> -144 the Monte Carlo plots shown in the paper
*
*          30 all tag energies
*          31 Wvis for tagged events
*          32 all anti tag energies
*          33 all anti tag angles
*          34 all tag angles
*          35 Ntrk for tagged events
*          36 MC Energy Flow, low Q^2 ( same as 14 but range shown in paper)
*          136 Eflow errors, low Q^2
*          37 MC Energy Flow, high Q^2  ( same as 24 but range shown in paper)
*          137 Eflow errors, high Q^2
*
*
*
*
*
* Cuts : 	Energy of tagged electron > 35 GeV
*		Theta(tag) either 27-55 mrad or 60-120 mrad
*		Anti-tag opposite tag: < 35 GeV @ theta < 25 mrad
*		No. charged particles with ...blah.... > 2
*		W > 3 GeV
*
* Author : Russell Taylor (rjt@hep.ucl.ac.uk)
*
* Reference: CERN-EP/2000-109, Submitted to Eur. Phys. J. C
*
*****************************************************************************
*
* HZTOOL commons
*
#include "hepevtp.inc"
#include "heracmn.inc"

      logical use

      INTEGER MX
      PARAMETER (MX=500)
      INTEGER i,loop,loop2,loop3,iloop,dummy,hzeebeam,ibase
      INTEGER i1,i2,el,po,tag,atag
      INTEGER acut,thcut,wcut,ncut,both,ecuts,tot
      REAL ang1,ang2,thtag,thatag
      REAL hzphmang
      REAL TH(MX),ETA(MX),EN(MX)
      INTEGER NTRK
      REAL PTOT,PIN,POUT,PTTAG,PT(MX),PTRK(MX),ETOUT
      REAL XVIS,YVIS,ZVIS,EVIS,WVIS
      DOUBLE PRECISION mhxsec(3)
      INTEGER mhntot(3),nev,nev2
      REAL norm,errlo(24),errhi(24)

      INTEGER ivec
      REAL temp(7)

      REAL xsecpb,angle

      integer event
      save event
c
c here is the data from the paper
C     --- Data from table 3
      REAL mwhl(19),dwhl(19),dwhl_e(19),mwhh(19),dwhh(19),dwhh_e(19)
      DATA mwhl/19.97,14.1,10.53,7.732,5.718,4.106,3.222,2.323,
     +     1.707,1.224,0.926,0.653,0.469,0.345,0.259,0.168,
     +     0.117,0.087,0.057/
      DATA dwhl/20.25,13.41,8.622,5.669,3.832,2.485,2.051,1.292,
     +     1.053,0.499,0.433,0.27,0.247,0.129,0.098,0.127,
     +     0.039,0.062,0.011/
      DATA dwhl_e/1.064,1.097,0.792,0.875,0.626,0.385,0.172,
     +     0.195,0.127,0.129,0.071,0.069,0.087,0.040,0.032,
     +     0.051,0.029,0.02,0.009/
      DATA mwhh/6.651,5.304,3.961,2.979,2.411,1.682,1.243,0.915,
     +     0.733,0.459,0.345,0.267,0.180,0.139,0.086,0.064,
     +     0.061,0.029,0.02/
      DATA dwhh/6.557,4.787,3.510,2.575,1.764,1.444,0.854,0.658,
     +     0.595,0.318,0.231,0.193,0.136,0.114,0.078,0.047,
     +     0.039,0.013,0.009/
      DATA dwhh_e/0.646,0.37,0.302,0.219,0.175,0.165,0.152,0.14,
     +     0.154,0.125,0.065,0.059,0.053,0.045,0.075,0.028,
     +     0.024,0.01,0.007/

C     --- Data from table 4
      REAL mwpl(19),dwpl(19),dwpl_e(19),mwph(19),dwph(19),dwph_e(19)
      DATA mwpl/16.47,11.55,7.704,5.078,3.399,2.115,1.405,0.858,
     +     0.581,0.366,0.266,0.215,0.119,0.087,0.086,0.054,
     +     0.045,0.028,0.03/
      DATA dwpl/19.06,13.12,8.447,5.601,3.532,2.124,1.531,0.875,
     +     0.623,0.323,0.254,0.16,0.102,0.084,0.065,0.042,
     +     0.044,0.037,0.016/
      DATA dwpl_e/0.545,0.341,0.432,0.394,0.29,0.142,0.114,0.087,
     +     0.072,0.104,0.039,0.03,0.023,0.022,0.025,0.024,
     +     0.027,0.016,0.008/
      DATA mwph/4.145,3.571,3.058,2.606,2.028,1.49,0.968,0.68,
     +		0.509,0.314,0.22,0.152,0.101,0.079,0.061,0.039,
     +     0.027,0.038,0.031/
      DATA dwph/6.266,4.58,3.244,2.819,1.74,1.339,0.95,0.598,
     +     0.363,0.288,0.175,0.143,0.098,0.061,0.051,0.021,
     +     0.023,0.023,0.0/
      DATA dwph_e/0.594,0.306,0.225,0.261,0.163,0.139,0.182,0.09,
     +     0.073,0.066,0.041,0.044,0.055,0.023,0.022,0.011,
     +     0.018,0.017,0.0/

C     --- Data from table 5
      REAL xbins(23)
      DATA xbins/0.0,0.5,1.0,1.5,2.0,2.5,3.0,3.5,4.0,4.5,5.0,5.5,
     +     6.0,6.5,7.0,8.0,9.0,10.0,11.0,12.0,13.0,14.0,15.0/
	REAL mehl(22),dehl(22),dehl_e(22),mehh(22),dehh(22),dehh_e(22)
	DATA mehl/0.521,7.221,20.73,27.75,24.38,18.34,13.12,9.595,
     +       7.048,4.939,3.68,2.855,2.131,1.446,0.937,0.488,
     +       0.234,0.095,0.066,0.037,0.025,0.022/
	DATA dehl/0.266,6.201,18.97,28.04,23.54,15.91,9.316,6.182,
     +       4.481,2.286,2.020,1.341,1.035,0.545,0.376,0.246,
     +       0.093,0.067,0.058,0.005,0.012,0.001/
	DATA dehl_e/0.064,0.413,0.875,1.259,2.567,2.566,1.17,1.275,
     +       0.473,0.249,0.431,0.289,0.225,0.106,0.062,0.052,
     +       0.027,0.015,0.031,0.006,0.014,0.002/
	DATA mehh/0.213,2.726,8.155,10.56,9.519,6.723,4.831,3.405,
     +       2.36,1.894,1.142,0.998,0.6,0.503,0.321,0.211,0.079,
     +       0.064,0.034,0.02,0.012,0.012/
	DATA dehh/0.330,1.955,6.647,9.095,9.33,6.075,4.739,2.196,
     +       1.782,1.357,0.83,0.75,0.513,0.385,0.517,0.288,
     +       0.149,0.104,0.045,0.0,0.004,0.0/
	DATA dehh_e/0.227,0.262,0.907,1.070,0.712,0.393,0.369,0.213,
     +       0.202,0.174,0.132,0.148,0.108,0.112,0.068,0.045,
     +       0.034,0.042,0.016,0.0,0.003,0.0/

C     --- Data from table 6
	REAL mepl(22),depl(22),depl_e(22),meph(22),deph(22),deph_e(22)
	DATA mepl/0.674,7.089,20.92,25.34,18.55,10.25,5.931,
     +       3.614,2.343,1.589,1.16,0.797,0.585,0.423,0.33,
     +       0.181,0.114,0.084,0.051,0.048,0.024,0.03/
	DATA depl/0.421,6.401,18.87,27.8,24.99,14.93,8.639,5.654,
     +       3.968,2.063,1.644,1.081,0.9,0.373,0.376,0.147,
     +       0.083,0.072,0.018,0.013,0.036,0.012/
	DATA depl_e/0.064,0.251,0.602,0.822,1.028,0.93,0.791,
     +       0.482,0.331,0.205,0.222,0.175,0.216,0.073,0.063,
     +       0.045,0.015,0.022,0.009,0.013,0.018,0.017/
	DATA meph/0.216,2.465,6.629,8.616,6.576,4.438,3.031,
     +       1.996,1.49,1.127,0.874,0.594,0.493,0.375,0.25,
     +       0.151,0.088,0.073,0.057,0.034,0.019,0.019/
	DATA deph/0.451,2.09,6.298,9.388,8.496,5.73,4.147,2.105,
     +       1.522,1.316,0.825,0.789,0.419,0.371,0.235,0.138,
     +       0.134,0.018,0.055,0.0,0.0,0.0/
	DATA deph_e/0.172,0.313,0.749,1.092,0.735,0.433,0.434,
     +       0.238,0.192,0.205,0.179,0.2,0.095,0.122,0.055,
     +       0.031,0.028,0.012,0.016,0.0,0.0,0.0/

C     --- Data from table 7
	REAL mnhl(13),dnhl(13),dnhl_e(13),mnhh(13),dnhh(13),dnhh_e(13)
	DATA mnhl/16.1,20.26,13.96,10.16,5.546,3.396,1.904,1.121,
     +       0.642,0.346,0.18,0.083,0.058/
	DATA dnhl/13.48,15.78,12.8,8.732,4.729,2.821,1.54,0.979,
     +       0.431,0.195,0.15,0.023,0.037/
	DATA dnhl_e/0.854,1.522,0.934,0.664,0.638,0.414,0.237,
     +       0.199,0.097,0.065,0.032,0.013,0.016/
	DATA mnhh/5.035,7.111,5.462,4.212,2.38,1.494,0.844,0.477,
     +       0.248,0.14,0.076,0.045,0.033/
	DATA dnhh/4.592,5.292,4.773,3.822,2.333,1.318,0.845,0.512,
     +       0.156,0.101,0.047,0.073,0.003/
	DATA dnhh_e/0.542,0.3,0.492,0.329,0.271,0.16,0.204,0.171,
     +       0.046,0.05,0.031,0.085,0.004/

C     --- Data from table 8
	REAL mnpl(13),dnpl(13),dnpl_e(13),mnph(13),dnph(13),dnph_e(13)
	DATA mnpl/11.02,13.76,10.38,7.034,3.891,2.249,1.099,0.558,
     +       0.279,0.136,0.072,0.036,0.018/
	DATA dnpl/13.44,15.73,12.76,7.931,4.405,2.412,1.286,0.684,
     +       0.318,0.127,0.053,0.03,0.029/
	DATA dnpl_e/0.893,1.022,0.536,0.713,0.443,0.321,0.172,0.136,
     +       0.054,0.035,0.042,0.017,0.014/
	DATA mnph/3.514,4.8,3.878,3.202,2.025,1.319,0.735,0.34,
     +       0.179,0.091,0.056,0.024,0.013/
	DATA dnph/4.594,5.392,4.207,3.537,2.031,1.385,0.805,0.387,
     +       0.158,0.113,0.017,0.018,0.003/
	DATA dnph_e/0.447,0.519,0.343,0.246,0.292,0.203,0.173,0.076,
     +       0.049,0.037,0.008,0.011,0.003/

C     --- Data from table 9
	REAL xbins2(29)
	DATA xbins2/0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9,1.0,1.1,1.2,1.3,
     +       1.4,1.5,1.6,1.7,1.8,1.9,2.0,2.2,2.4,2.6,2.8,3.0,3.4,
     +       3.8,4.2,4.6,5.0/
	REAL mphl(28),dphl(28),dphl_e(28),mphh(28),dphh(28),dphh_e(28)
	DATA mphl/757.2,656.1,509.7,388.7,292.4,223.5,173.9,132.8,
     +       106.3,80.99,68.34,53.41,44.89,36.04,30.36,24.71,
     +       20.61,16.67,13.29,9.261,6.303,4.399,3.155,1.962,
     +       1.087,0.651,0.344,0.146/
	DATA dphl/728.1,600.9,473.4,355.3,259.5,181.6,140.3,98.77,
     +       71.11,53.99,41.7,27.85,22.39,18.35,13.68,10.28,9.879,
     +       7.824,5.296,3.743,2.671,1.865,1.248,0.778,0.617,
     +       0.516,0.234,0.101/
	DATA dphl_e/42.83,61.32,36.16,26.63,19.57,17.54,9.73,10.44,
     +       5.713,4.323,2.616,2.93,4.662,1.716,1.789,1.309,1.162,
     +       1.246,0.712,0.395,0.351,0.342,0.348,0.212,0.152,
     +       0.077,0.21,0.032/
	DATA mphh/265.5,230.7,186.8,146.1,114.2,87.71,69.09,56.38,
     +       44.87,36.74,30.71,24.91,21.39,17.05,14.64,13.07,
     +       10.92,9.516,7.548,5.498,3.843,3.082,2.365,1.471,
     +       0.868,0.524,0.245,0.128/
	DATA dphh/224.7,201.3,172.9,125.0,104.1,80.57,64.64,57.42,
     +       40.83,32.82,30.03,20.61,21.69,12.94,11.68,9.734,9.056,
     +       7.013,6.173,5.039,2.861,2.25,1.273,1.62,0.832,0.295,
     +       0.199,0.107/
	DATA dphh_e/17.29,6.29,11.53,13.71,4.512,8.489,4.254,3.638,
     +       2.89,2.939,4.872,2.116,2.352,1.441,1.488,1.403,1.368,
     +       1.385,0.787,0.973,0.624,0.396,0.253,0.163,0.129,
     +       0.078,0.09,0.072/

C     --- Data from table 10
	REAL mppl(28),dppl(28),dppl_e(28),mpph(28),dpph(28),dpph_e(28)
	DATA mppl/557.9,489.3,382.5,287.8,208.2,150.8,106.8,75.99,
     +       55.68,41.27,30.74,22.78,17.09,12.58,10.34,7.549,
     +       6.152,5.55,3.853,2.499,1.698,1.221,0.831,0.545,
     +       0.337,0.211,0.132,0.084/
	DATA dppl/667.9,564.2,443.9,332.1,234.7,171.6,129.5,90.52,
     +       63.55,51.43,40.34,27.83,19.12,16.07,11.44,8.146,6.864,
     +       6.588,4.25,3.178,1.95,1.371,0.752,0.58,0.28,0.189,
     +       0.123,0.079/
	DATA dppl_e/49.08,47.42,35.9,27.61,11.73,10.26,6.614,6.095,
     +       3.961,2.551,2.822,2.839,2.86,1.555,1.734,1.307,0.971,
     +       0.919,0.693,0.569,0.246,0.211,0.137,0.179,0.095,0.036,
     +       0.075,0.05/
	DATA mpph/197.6,173.6,141.2,112.3,87.5,68.66,54.16,40.57,
     +       32.59,26.29,21.22,18.44,15.41,12.82,11.45,8.789,
     +       7.67,6.538,4.949,3.859,2.703,1.818,1.536,0.99,0.545,
     +       0.274,0.187,0.09/
	DATA dpph/213.8,201.1,160.3,118.8,95.93,76.02,61.77,54.56,
     +       38.27,32.04,26.86,18.6,16.47,15.19,10.29,8.366,8.055,
     +       6.632,4.814,3.298,2.664,1.784,1.41,1.239,0.779,0.649,
     +       0.097,0.009/
	DATA dpph_e/6.298,6.132,9.387,11.76,3.904,5.77,3.295,4.233,
     +       2.528,2.386,2.176,2.266,1.612,1.65,1.27,1.044,1.137,
     +       1.074,0.574,0.455,0.715,0.321,0.602,0.428,0.141,0.093,
     +       0.235,0.065/

C     --- Data from table 11
	REAL mrhl(15),drhl(15),drhl_e(15),mrhh(15),drhh(15),drhh_e(15)
	DATA mrhl/0.264,0.605,1.477,2.29,2.164,1.879,1.891,2.252,
     +       3.028,3.379,3.1,2.808,1.915,1.122,0.579/
	DATA drhl/0.281,1.010,1.715,2.225,2.077,1.794,1.707,1.993,
     +       2.559,2.754,1.78,1.063,1.267,0.992,0.634/
	DATA drhl_e/0.054,0.054,0.078,0.056,0.043,0.033,0.032,0.045,
     +       0.12,0.226,0.256,0.319,0.254,0.249,0.144/
	DATA mrhh/0.147,0.432,1.322,2.516,2.616,2.362,2.304,2.533,
     +       3.169,3.337,3.08,2.735,1.918,1.089,0.548/
	DATA drhh/0.076,0.651,1.449,2.221,2.366,2.365,2.383,2.602,
     +       3.054,2.879,1.888,1.059,1.496,0.528,0.292/
	DATA drhh_e/0.066,0.108,0.096,0.088,0.076,0.1,0.135,0.202,
     +       0.372,0.369,0.321,0.15,0.209,0.133,0.1/

C     --- Data from table 12
	REAL mrpl(15),drpl(15),drpl_e(15),mrph(15),drph(15),drph_e(15)
	DATA mrpl/0.511,1.064,1.836,2.17,1.835,1.601,1.604,1.941,
     +       2.503,2.77,2.718,2.772,2.565,1.929,1.193/
	DATA drpl/0.248,1.218,1.879,2.165,1.974,1.771,1.73,1.983,
     +       2.34,2.108,1.707,1.046,1.536,1.467,1.065/
	DATA drpl_e/0.03,0.046,0.058,0.046,0.043,0.033,0.047,0.063,
     +       0.139,0.275,0.284,0.337,0.187,0.164,0.09/
	DATA mrph/0.176,0.578,1.739,2.683,2.58,2.312,2.26,2.529,
     +       3.08,3.277,3.109,3.061,2.762,2.15,1.217/
	DATA drph/0.055,0.606,1.507,2.353,2.456,2.419,2.348,2.429,
     +       2.523,2.349,1.707,0.934,1.446,0.821,0.411/
	DATA drph_e/0.043,0.098,0.117,0.109,0.107,0.131,0.156,0.234,
     +       0.348,0.371,0.312,0.175,0.196,0.207,0.223/

	data event/0/
	INTEGER myiproc, iflag

        write(*,*)'iflag = ',iflag



	IF (iflag.eq.1) THEN
C     *** Initialisation run ***
C     *** Create directories ***
           CALL HCDIR('//HISTO',' ')
           CALL HMDIR('C00109','S')
           CALL HCDIR('//PAWC',' ')
           CALL HMDIR('C00109','S')

           CALL HBOOK1(-10,'Data W?res!, Herwig corrected, low Q^2',
     &          19,3.0,22.0,0.)
           CALL HPAK(-10,dwhl)
           CALL HPAKE(-10,dwhl_e)

           CALL HBOOKB(-11,'Data E?t,out!, Herwig corr, low Q^2',
     &          22,xbins,0.)
           CALL HPAK(-11,dehl)
           CALL HPAKE(-11,dehl_e)

           CALL HBOOK1(-12,'Data N?trk!, Herwig corr, low Q^2',
     &          13,3.0,16.0,0.)
           CALL HPAK(-12,dnhl)
           CALL HPAKE(-12,dnhl_e)

           CALL HBOOKB(-13,'Data p?t,trk!, Herwig corr, low Q^2',
     &          28,xbins2,0.)
           CALL HPAK(-13,dphl)
           CALL HPAKE(-13,dphl_e)

           CALL HBOOK1(-14,'Data Energy Flow, Herwig corr, low Q^2',
     &          15,-3.0,4.5,0.)
           CALL HPAK(-14,drhl)
           CALL HPAKE(-14,drhl_e)

           CALL HBOOK1(-20,'Data W?res!, Herwig corrected, high Q^2',
     &          19,3.0,22.0,0.)
           CALL HPAK(-20,dwhh)
           CALL HPAKE(-20,dwhh_e)

           CALL HBOOKB(-21,'Data E?t,out!, Herwig corr, high Q^2',
     &          22,xbins,0.)
           CALL HPAK(-21,dehh)
           CALL HPAKE(-21,dehh_e)

           CALL HBOOK1(-22,'Data N?trk!, Herwig corr, high Q^2',
     &          13,3.,16.,0.)
           CALL HPAK(-22,dnhh)
           CALL HPAKE(-22,dnhh_e)

           CALL HBOOKB(-23,'Data p?t,trk!, Herwig corr, high Q^2',
     &          28,xbins2,0.)
           CALL HPAK(-23,dphh)
           CALL HPAKE(-23,dphh_e)

           CALL HBOOK1(-24,'Data Energy Flow, Herwig corr, high Q^2',
     &          15,-3.0,4.5,0.)
           CALL HPAK(-24,drhh)
           CALL HPAKE(-24,drhh_e)

           CALL HBOOK1(-30,'Data W?res!, Phojet corrected, low Q^2',
     &          19,3.0,22.0,0.)
           CALL HPAK(-30,dwpl)
           CALL HPAKE(-30,dwpl_e)

           CALL HBOOKB(-31,'Data E?t,out!, Phojet corr, low Q^2',
     &          22,xbins,0.)
           CALL HPAK(-31,depl)
           CALL HPAKE(-31,depl_e)

           CALL HBOOK1(-32,'Data N?trk!, Phojet corr, low Q^2',
     &          13,3.0,16.0,0.)
           CALL HPAK(-32,dnpl)
           CALL HPAKE(-32,dnpl_e)

           CALL HBOOKB(-33,'Data p?t,trk!, Phojet corr, low Q^2',
     &          28,xbins2,0.)
           CALL HPAK(-33,dppl)
           CALL HPAKE(-33,dppl_e)

           CALL HBOOK1(-34,'Data Energy Flow, Phojet corr, low Q^2',
     &          15,-3.0,4.5,0.)
           CALL HPAK(-34,drpl)
           CALL HPAKE(-34,drpl_e)

           CALL HBOOK1(-40,'Data W?res!, Phojet corrected, high Q^2',
     &          19,3.0,22.0,0.)
           CALL HPAK(-40,dwph)
           CALL HPAKE(-40,dwph_e)

           CALL HBOOKB(-41,'Data E?t,out!, Phojet corr, high Q^2',
     &          22,xbins,0.)
           CALL HPAK(-41,deph)
           CALL HPAKE(-41,deph_e)

           CALL HBOOK1(-42,'Data N?trk!, Phojet corr, high Q^2',
     &          13,3.0,16.0,0.)
           CALL HPAK(-42,dnph)
           CALL HPAKE(-42,dnph_e)

           CALL HBOOKB(-43,'Data p?t,trk!, Phojet corr, high Q^2',
     &          28,xbins2,0.)
           CALL HPAK(-43,dpph)
           CALL HPAKE(-43,dpph_e)

           CALL HBOOK1(-44,'Data Energy Flow, Phojet corr, high Q^2',
     &          15,-3.0,4.5,0.)
           CALL HPAK(-44,drph)
           CALL HPAKE(-44,drph_e)

c#define DEBUG
#ifdef DEBUG

           CALL HBOOK1(-110,'W?res!, Herwig, low Q^2',19,3.0,22.0,0.)
           CALL HPAK(-110,mwhl)

           CALL HBOOKB(-111,'E?t,out!, Herwig, low Q^2',22,xbins,0.)
           CALL HPAK(-111,mehl)

           CALL HBOOK1(-112,'N?trk!, Herwig, low Q^2',13,3.0,16.0,0.)
           CALL HPAK(-112,mnhl)

           CALL HBOOKB(-113,'p?t,trk!, Herwig, low Q^2',28,xbins2,0.)
           CALL HPAK(-113,mphl)

           CALL HBOOK1(-114,'E.Flow, Herwig, low Q^2',15,-3.0,4.5,0.)
           CALL HPAK(-114,mrhl)

           CALL HBOOK1(-120,'W?res!, Herwig, high Q^2',19,3.0,22.0,0.)
           CALL HPAK(-120,mwhh)

           CALL HBOOKB(-121,'E?t,out!, Herwig, high Q^2',22,xbins,0.)
           CALL HPAK(-121,mehh)

           CALL HBOOK1(-122,'N?trk!, Herwig, high Q^2',13,3.0,16.0,0.)
           CALL HPAK(-122,mnhh)

           CALL HBOOKB(-123,'p?t,trk!, Herwig, high Q^2',28,xbins2,0.)
           CALL HPAK(-123,mphh)

           CALL HBOOK1(-124,'E.Flow, Herwig, high Q^2',15,-3.0,4.5,0.)
           CALL HPAK(-124,mrhh)

           CALL HBOOK1(-130,'W?res!, Phojet, low Q^2',19,3.0,22.0,0.)
           CALL HPAK(-130,mwpl)

           CALL HBOOKB(-131,'E?t,out!, Phojet, low Q^2',22,xbins,0.)
           CALL HPAK(-131,mepl)

           CALL HBOOK1(-132,'N?trk!, Phojet, low Q^2',13,3.0,16.0,0.)
           CALL HPAK(-132,mnpl)

           CALL HBOOKB(-133,'p?t,trk!, Phojet, low Q^2',28,xbins2,0.)
           CALL HPAK(-133,mppl)

           CALL HBOOK1(-134,'E.Flow, Phojet, low Q^2',15,-3.0,4.5,0.)
           CALL HPAK(-134,mrpl)

           CALL HBOOK1(-140,'W?res!, Phojet, high Q^2',19,3.0,22.0,0.)
           CALL HPAK(-140,mwph)

           CALL HBOOKB(-141,'E?t,out!, Phojet, high Q^2',22,xbins,0.)
           CALL HPAK(-141,meph)

           CALL HBOOK1(-142,'N?trk!, Phojet, high Q^2',13,3.0,16.0,0.)
           CALL HPAK(-142,mnph)

           CALL HBOOKB(-143,'p?t,trk!, Phojet, high Q^2',28,xbins2,0.)
           CALL HPAK(-143,mpph)

           CALL HBOOK1(-144,'E.Flow, Phojet, high Q^2',15,-3.0,4.5,0.)
           CALL HPAK(-144,mrph)

           call hbook1(30,' all tag energies ',100,0.0,50.0,0.0)
           call hbook1(31,' Wvis for tagged events',100,0.0,50.0,0.0)
           call hbook1(32,' all anti tag energies ',100,0.0,50.0,0.0)
           call hbook1(33,' all anti tag angles ',100,0.0,3.15,0.0)
           call hbook1(34,' all tag angles ',100,0.0,3.15,0.0)
           call hbook1(35,' Ntrk for tagged events',100,0.0,50.0,0.0)
           CALL HBOOK1(36,'MC Energy Flow, low Q^2',24,-6.0,6.0,0.)
           CALL HBOOK1(136,'Eflow errors, low Q^2',24,-6.0,6.0,0.)
           CALL HBOOK1(37,'MC Energy Flow, high Q^2',24,-6.0,6.0,0.)
           CALL HBOOK1(137,'Eflow errors, high Q^2',24,-6.0,6.0,0.)
#endif

           CALL HBOOK1(10,'MC W?res!, low Q^2',19,3.0,22.0,0.)
           CALL HBOOKB(11,'MC E?t,out!, low Q^2',22,xbins,0.)
           CALL HBOOK1(12,'MC N?trk!, low Q^2',13,3.0,16.0,0.)
           CALL HBOOKB(13,'MC p?t,trk!, low Q^2',28,xbins2,0.)
           CALL HBOOK1(14,'MC Energy Flow, low Q^2',15,-3.0,4.5,0.)
           CALL HBOOK1(114,'Eflow errors, low Q^2',15,-3.0,4.5,0.)
           CALL HBOOK1(20,'MC W?res!, high Q^2',19,3.0,22.0,0.)
           CALL HBOOKB(21,'MC E?t,out!, high Q^2',22,xbins,0.)
           CALL HBOOK1(22,'MC N?trk!, high Q^2',13,3.0,16.0,0.)
           CALL HBOOKB(23,'MC p?t,trk!, high Q^2',28,xbins2,0.)
           CALL HBOOK1(24,'MC Energy Flow, high Q^2',15,-3.0,4.5,0.)
           CALL HBOOK1(124,'Eflow errors, high Q^2',15,-3.0,4.5,0.)

           CALL HBARX(0)

           WRITE(6,*)'***********************************************'
           WRITE(6,*)'* HZC00109 called, histograms will be output. *'
           WRITE(6,*)'***********************************************'

        ELSE IF (iflag.eq.2) THEN
C     *** Filling runs ***

           event=event+1
C     *** Change directory to our graphs ***
           CALL HCDIR('//PAWC/C00109',' ')

           po=0
           el=0
           thtag=0.0
           thatag=0.0
           tag=0
           atag=0
           ntrk=0
           etout=0.0
           wvis=0.0
           evis=0.0
           xvis=0.0
           yvis=0.0
           zvis=0.0
           do i=1,MX
              TH(i)=0.0
              ETA(i)=0.0
              EN(i)=0.0
              PT(i)=0.0
           enddo
C     scattered.

           dummy = hzeebeam(i1,i2)
           If(dummy.ne.1) Then
              Write(6,*) 'HZC00109: No record of beam particles',
     >             dummy,i1,i2
           Endif

           IF (gen(1:3).eq.'HRW') THEN
              Do iloop=NHEP,1,-1
                 If (IDHEP(iloop).eq.11.and.ISTHEP(iloop).eq.1) Then
                    po=iloop
                 ENDIF
                 If (IDHEP(iloop).eq.-11.and.ISTHEP(iloop).eq.1) Then
                    el=iloop
                 ENDIF
              Enddo
c     c
           ELSEIF (gen(1:3).eq.'PYT') THEN
              If ((i1.eq.1).and.(i2.eq.2)) Then
                 el=3
                 po=4
              ElseIf ((i1.eq.2).and.(i2.eq.1)) Then
                 el=4
                 po=3
              EndIf
           ELSEIF (gen(1:3).eq.'PHO') THEN
              el=3
              po=5
           ELSE
              Write(6,*)'HZC00109: Unknown generator ',gen(1:3)
           ENDIF
c
           ang1=acos(abs(phep(3,el))/phep(4,el))

           ang2=acos(abs(phep(3,po))/phep(4,po))

c           write(6,*)' electron 4 vec :',(phep(i,el),i=1,4)
c           write(6,*)' positron 4 vec :',(phep(i,po),i=1,4)
           thtag=ang2
           thatag=ang1
           tag=po
           atag=el
c           write(6,*)' tag is positron, scattering angle is ',thtag
c           write(6,*)' anti tag is electron , scattering angle is ',
c     >          thatag

#ifdef DEBUG
           call hfill(30,sngl(phep(4,tag)),0.0,1.0)
           call hfill(32,sngl(phep(4,atag)),0.0,1.0)
           call hfill(33,thatag,0.0,1.0)
           call hfill(34,thtag,0.0,1.0)
#endif
c     first check the anti tag is NOT observed...
c
           IF (thatag.le.0.025.or.PHEP(4,atag).le.35.0) THEN
c
c     then check that the tag IS observed
c
              IF (PHEP(4,tag).gt.35.0.and.thtag.ge.0.027.and.
     +             thtag.le.0.12) THEN
c
c     loop over the final state particles
c
c
c                 write(6,*)' event is tagged and has ',nhep,' particles'
                 DO loop=1,NHEP
                    IF (ISTHEP(loop).eq.1.and.loop.ne.tag)THEN
c  Pick stable final state particles,
c which are not the tag particle.

                       EN(loop)=PHEP(4,loop)
                       PTOT=SQRT(PHEP(1,loop)**2+
     +                      PHEP(2,loop)**2+PHEP(3,loop)**2)
                       if (PTOT.gt.0) then
                          TH(loop)=ACOS(PHEP(3,loop)/PTOT)
                       else
                          TH(loop)=1.5708
                       endif
                       PT(loop)=sqrt(PHEP(1,loop)**2+PHEP(2,loop)**2)
C     ... calculate the momentum perpendicular to the tag-plane:
                       PTTAG = sqrt(PHEP(1,tag)**2 + PHEP(2,tag)**2)
                       if (PTTAG.GT.0) then
                          PIN = (PHEP(1,loop)*PHEP(1,tag)+PHEP(2,loop)
     +                         *PHEP(2,tag))/PTTAG
                       else
                          PIN =  0.
                       endif
                       POUT = PHEP(1,loop)**2+PHEP(2,loop)**2 - PIN*PIN
                       if (POUT.GT.0) then
                          POUT=sqrt(POUT)
                       else
                          POUT=0.
                       endif
C
C     ... count charged and neutrals with theta>0.2 and pt>200MeV
c
                       if (sin(TH(loop)).gt.0.342.and.PT(loop).gt.0.2)
     >                      then
                          use=.false.
                          if (abs(IDHEP(loop)).eq.11 .or.
     &                         abs(IDHEP(loop)).eq.13 .or.
     &                         abs(IDHEP(loop)).eq.211 .or.
     &                         abs(IDHEP(loop)).eq.321 .or.
     &                         abs(IDHEP(loop)).eq.2212) then
c NTRK is the charged track sum...
                             NTRK=NTRK+1
c                             write(6,*)' there are ',ntrk,
c     >                            ' charged tracks '
                             IF(NTRK.GT.MX)THEN
                                WRITE(6,*)' nTRK.gt.MX !!! ',
     >                               loop,nTRK,MX
                                goto 1
                                NTRK=MX
                             ENDIF
                             PTRK(ntrk)=PT(loop)
                             use=.true.
                          elseif (abs(IDHEP(loop)).eq.22 .or.
     &                            abs(IDHEP(loop)).eq.130) then
c
c this is a  neutral track used for w_res calculation
c
                             use=.true.
                          endif
                          if(use)then
                             ETOUT=ETOUT+POUT
C     ... and sum up the "visible" hadronic energy and momentum
                             EVIS=EVIS+PHEP(4,loop)
                             XVIS=XVIS+PHEP(1,loop)
                             YVIS=YVIS+PHEP(2,loop)
                             ZVIS=ZVIS+PHEP(3,loop)
                    endif
                 endif          ! PT and theta requirement
c
c remember energy and eta for later energy flow  histogram filling...
c
                 if (TH(loop).gt.0.00001.and.TH(loop).lt.3.1415826) then
c
c the tag electron defines the sign ...
c
                    if (PHEP(3,tag).gt.0) then
                       ETA(loop)=LOG(TAN(TH(loop)/2.))
                    else
                       ETA(loop)=-1.0*LOG(TAN(TH(loop)/2.))
                    endif
                 else
                    EN(loop)=0.
                 endif


              ENDIF             ! had. final state particle
           ENDDO                ! loop 1 -> NHEP

c
c having finished looping over the particles, apply cuts on final state
c and fill the histograms
c
 1         continue
           WVIS=EVIS**2-XVIS**2-YVIS**2-ZVIS**2
           WVIS=sqrt(WVIS)
c
#ifdef DEBUG
           call hfill(31,wvis,0.0,1.0)
           call hfill(35,float(ntrk),0.0,1.0)
#endif
c
c
           IF (WVIS.ge.3.0.and.NTRK.ge.3) THEN
c
c choose whether low or high q^2 hists...
c
              ibase=-1
              IF (thtag.le.0.055) THEN
                 ibase=10
              ELSE IF (thtag.ge.0.06) THEN
                 ibase=20
              endif
c              if(ibase.lt.0)write(6,*)' in neither tag range ',thtag
c
c and fill them
c
              if(ibase.gt.0)then
                 CALL HFILL(ibase,WVIS,0.,wtx)
                 CALL HFILL(ibase+1,ETOUT,0.,wtx)
                 CALL HFILL(ibase+2,REAL(NTRK),0.,wtx)
                 DO loop2=1,NTRK
                    CALL HFILL(ibase+3,PTRK(loop2),0.,wtx)
                 ENDDO
                 DO loop3=1,NHEP
                    if((ISTHEP(loop3).eq.1).and.(loop3.ne.tag))then
                          if (abs(IDHEP(loop3)).eq.11 .or.
     &                         abs(IDHEP(loop3)).eq.13 .or.
     &                         abs(IDHEP(loop3)).eq.211 .or.
     &                         abs(IDHEP(loop3)).eq.321 .or.
     &                         abs(IDHEP(loop3)).eq.2212.or.
     &                         abs(IDHEP(loop3)).eq.22 .or.
     &                         abs(IDHEP(loop3)).eq.130
     &                     ) then

                          if(en(loop3).gt.35.0.and.
     >                         (eta(loop3).lt.-2.5.or.eta(loop3).gt.4.5)
     >                         .and.abs(idhep(loop3)).eq.11)
     >                            en(loop3)=0.0

                       IF (EN(loop3).gt.0.0) THEN
                          CALL HFILL(ibase+4,ETA(loop3),0.,EN(loop3))
                          CALL HFILL(ibase+104,ETA(loop3),0.,
     +                         EN(loop3)*EN(loop3))
#ifdef DEBUG
                          if(ibase.eq.10)then
                          call hfill(36,ETA(loop3),0.,EN(loop3))
                          call hfill(136,ETA(loop3),0.,
     +                         EN(loop3)*EN(loop3))
                          else
                          call hfill(37,ETA(loop3),0.,EN(loop3))
                          call hfill(137,ETA(loop3),0.,
     +                         EN(loop3)*EN(loop3))
                          endif
#endif
                       ENDIF
                    ENDIF
                    ENDIF
                 ENDDO
              ENDIF             ! in tag angle
           ENDIF	        ! W & Ntrk cuts
        ENDIF                   ! tag
      ENDIF             	! anti-tag cut


      ELSE IF (iflag.eq.3) THEN
C     *** Termination run ***
C     *** Change directory ***
         CALL HCDIR('//PAWC/C00109',' ')
c
C     *** Finish off by normalising histograms ***
c
c
         IF (Xsec.eq.0) THEN
            PRINT*,
     +           'HZC00109: termination called with zero cross section'
            PRINT*,'          cross section graph meaningless'
            Xsec=1
         ENDIF
         IF (Ntot.eq.0) THEN
            PRINT*,'HZC00109: termination called with no total events'
            PRINT*,'          cross section graph meaningless'
            Ntot=1
         ENDIF

C     --- normalise histograms to units of picobarn
C         factor of 2 reflects the fact that
C         pythia and herwig generate tags on just one
c         side but data has them on both sides
c         may need fixing for smarter MC models
C
         norm=0.
c
c we need the cross section in pb.
c unfortunately the hztool utilities do not take care of
c this so we must do it ourselves
c
         xsecpb=xsec*2
         if(GEN(1:3).eq.'HRW')xsecpb=xsecpb*1000
         norm=REAL(Ntot/Xsecpb)


         write(6,*)' there are ',Ntot,' events and ',
     >        ' the cross section is ',Xsecpb ,' pb ,',
     >        ' so normalisation is ', norm
         CALL HNOENT(10,nev)
         write(6,*)' There are ',nev,' events selected at low Q^2'
         CALL HZHINRM(10,0,norm,1)
         CALL HZHINRM(11,0,norm,1)
         CALL HZHINRM(12,0,norm,1)
         CALL HZHINRM(13,0,norm,1)
         CALL HUNPAK(114,errlo,' ',1)
         do loop=1,15
            errlo(loop)=sqrt(errlo(loop))
         enddo
         CALL HPAKE(14,errlo)
         CALL HZHINRM(14,0,REAL(nev),1)
         CALL HDELET(114)

#ifdef DEBUG
         CALL HUNPAK(136,errlo,' ',1)
         do loop=1,24
            errlo(loop)=sqrt(errlo(loop))
         enddo
         CALL HPAKE(36,errlo)
         CALL HZHINRM(36,0,REAL(nev),1)
         CALL HDELET(136)
#endif
         CALL HNOENT(20,nev2)
         write(6,*)' There are ',nev2,' events selected at high Q^2'
         CALL HZHINRM(20,0,norm,1)
         CALL HZHINRM(21,0,norm,1)
         CALL HZHINRM(22,0,norm,1)
         CALL HZHINRM(23,0,norm,1)
         CALL HUNPAK(124,errhi,' ',1)
         do loop=1,15
            errhi(loop)=sqrt(errhi(loop))
         enddo
         CALL HPAKE(24,errhi)
         CALL HZHINRM(24,0,REAL(nev2),1)
         CALL HDELET(124)

#ifdef DEBUG
         CALL HUNPAK(137,errhi,' ',1)
         do loop=1,24
            errhi(loop)=sqrt(errhi(loop))
         enddo
         CALL HPAKE(37,errhi)
         CALL HZHINRM(37,0,REAL(nev2),1)
         CALL HDELET(137)
#endif
         CALL HZCHISQ(-10,10)
         CALL HZCHISQ(-11,11)
         CALL HZCHISQ(-12,12)
         CALL HZCHISQ(-13,13)
         CALL HZCHISQ(-14,14)
         CALL HZCHISQ(-20,20)
         CALL HZCHISQ(-21,21)
         CALL HZCHISQ(-22,22)
         CALL HZCHISQ(-23,23)
         CALL HZCHISQ(-24,24)
         CALL HZCHISQ(-30,10)
         CALL HZCHISQ(-31,11)
         CALL HZCHISQ(-32,12)
         CALL HZCHISQ(-33,13)
         CALL HZCHISQ(-34,14)
         CALL HZCHISQ(-40,20)
         CALL HZCHISQ(-41,21)
         CALL HZCHISQ(-42,22)
         CALL HZCHISQ(-43,23)
         CALL HZCHISQ(-44,24)

      ELSE
C     *** End ***
         PRINT*,'HZC00109:Please run with iflag set to 1,2 or 3'
      ENDIF

      RETURN
      END


