      subroutine HZC93153(iflag)
*****************************************************************************
*       
* Make plots for UA1 paper on the measurement of bbbar correlations.
* at sqrt(s) = 630 GeV
* ( Z. Phys. C 61, 41-52 (1994) , CERN PPE 93-153) 
*       
* A. cross-section for dimuon production from semileptonic beauty decays
*
*    sigma(ppbar->b(bbar)->mu mu X) [pb]
*    
*    from different b quarks
*
*    in 4 bins of transverse momentum from 3 to 20 GeV/c
*
*    cuts:
*    muons: p_t > 3 GeV/c
*           |eta| < 1.5
*           6 GeV/c^2 < m_{mu mu} < 35 GeV/c^2 for muons from different quarks
*           bin cuts refer to highest-transverse-momentum-muon
*
* B. cross-section for dimuon production from semileptonic beauty decays
*
*    sigma(ppbar->b(bbar)->mu mu X) [pb]
*    
*    dimuons b quark chains ( b->c mu, c-> mu )
*
*    in 2 bins of transverse momentum from 7 to 20 GeV/c
*
*    cuts:
*    muons: p_t > 3 GeV/c
*           |eta| < 1.5
*           m_{mu mu) > 4 GeV/c^2: bin cuts refer to 
*                                  highest-transverse-momentum-muon
*           m_{mu mu} < 4 GeV/c^2: bin cuts refer to transverse momentum of
*                                  dimuon system
*
* C. inclusive single b quark cross section
*
*    sigma(ppbar->b X) [nb]
*
*    extrapolated from events with dimuons from different b quarks
*
*    for b transverse momentum > (6,8,11,15) GeV/c (4 bins)
*
*    cuts:
*    b:     |eta| < 1.5
*
*    ATTENTION:
*    errors from paper are added in quadrature
*
* D. inclusive single b quark cross section
*
*    sigma(ppbar->b X) [nb]
*
*    extrapolated from events with dimuons b quark chains
*    ( b->c mu, c-> mu )
*
*    for b transverse momentum > (11,15) GeV/c (2 bins)
*
*    cuts:
*    b:     |eta| < 1.5
*
*    ATTENTION:
*    errors from paper are added in quadrature
*
* E. inclusive single b quark cross section
*
*    sigma(ppbar->b X) [nb]
*
*    extrapolated from single-muon events
*    ( b->mu X )
*
*    for b transverse momentum > (15,23,30,39,46,54) GeV/c (6 bins)
*
*    cuts:
*    b:     |eta| < 1.5
*
*    ATTENTION:
*    data points included in original paper, but taken from quoted paper:
*    Phys. Lett. B256 (1991) 121
*    "Beauty production at the CERN ppabr collider"
*
* F. inclusive single B hadron (incl. b-baryon) cross section
*
*    sigma(ppbar->B X) [nb]
*
*    extrapolated from events with dimuons from different b quarks
*
*    for B transverse momentum > (5,7,10,13) GeV/c (4 bins)
*
*    cuts:
*    B:     |eta| < 1.5
*
*    ATTENTION:
*    errors from paper are added in quadrature
*
* G. inclusive single B hadron (incl. b-baryon) cross section
*
*    sigma(ppbar->B X) [nb]
*
*    extrapolated from events with dimuons b quark chains
*    ( B->c mu, c-> mu )
*
*    for B transverse momentum > (10,13) GeV/c (2 bins)
*
*    cuts:
*    B:     |eta| < 1.5
*
*    ATTENTION:
*    errors from paper are added in quadrature
*
* H. inclusive single B hadron (incl. b-baryon) cross section
*
*    sigma(ppbar->B X) [nb]
*
*    extrapolated from single-muon-events
*    ( B->mu X )
*
*    for B transverse momentum > (15,23,30,39,46,54) GeV/c (6 bins)
*
*    cuts:
*    B:     |eta| < 1.5
*
*    ATTENTION:
*    data points included in original paper, but taken from quoted paper:
*    Phys. Lett. B256 (1991) 121
*    "Beauty production at the CERN ppabr collider"
*
* I. bbbar angular phi distribution for b quark pairs
*
*    sigma(bbbar) [mub]
*
*    for transverse momentum of the 
*    highest-transverse-momentum-b-quark > 6 GeV/c
*
*    in 6 bins of 30 degree between 0 and 180 degree
*
* J. bbbar angular phi distribution for b quark pairs
*
*    sigma(bbbar) [mub]
*
*    for transverse momentum of the 
*    highest-transverse-momentum-b-quark > 11 GeV/c
*
*    in 6 bins of 30 degree between 0 and 180 degree
*
* K. bbbar angular deltaR distribution for b quark pairs
*
*    sigma(bbbar) [mub]
*
*    for transverse momentum of the 
*    highest-transverse-momentum-b-quark > 6 GeV/c
* 
*    cuts:
*    bbbar-pair:  deltaPhi < 120 degree
*
*    in 5 bins of 0.8 rad between 0 and 4 rad
*
*    with deltaR = sqrt(deltaEta^2 + deltaPhi^2)
*
* L. bbbar angular deltaR distribution for b quark pairs
*
*    sigma(bbbar) [mub]
*
*    for transverse momentum of the 
*    highest-transverse-momentum-b-quark > 11 GeV/c
* 
*    cuts:
*    bbbar-pair:  deltaPhi < 120 degree
*
*    in 5 bins of 0.8 rad between 0 and 4 rad
*
*    with deltaR = sqrt(deltaEta^2 + deltaPhi^2)
*
* M. inclusive single b quark cross section
*
*    sigma(ppbar->b X) [mub]
*
*    for b transverse momentum > (6,8,11,15) GeV/c (4 bins)
*
*
* N. inclusive single b quark cross section
*
*    sigma(ppbar->b X) [mub]
*
*    for b transverse momentum > (6,8,11,15) GeV/c (4 bins)
*
*    for events with a bbbar-pair and 
*    deltaPhi(bbbar) < 150 degree
*
* O. inclusive single b quark cross section
*
*    sigma(ppbar->b X) [mub]
*
*    for b transverse momentum > (6,8,11,15) GeV/c (4 bins)
*
*    for events with a bbbar-pair and 
*    deltaPhi(bbbar) > 150 degree
*
* Arguments:  iflag=1 initialisation/booking of histograms
*             iflag=2 event processing/filling of MC histograms
*             iflag=3 termination/final normalisation
*                     (the variables Xsec (total cross section) and
*                      Ntot (total number of events) have to be set before
*                      calling the termination routine in order to get a
*                      correct normalisation for the MC histograms)
*
* written by: Oliver Gutsche (oliver.gutsche@desy.de) on 04.06.2003
*       
*****************************************************************************
*
* Output graphs:
*---------------
* DATA
*
* histograms -1 .. -12 contain the measured cross section data,
* -1        sigma(ppbar->b(bbar)->mu mu X) for cross section A.
* -2        sigma(ppbar->b(bbar)->mu mu X) for cross section B.
* -3        sigma(ppbar->b X) for cross section C.
* -4        sigma(ppbar->b X) for cross section D.
* -5        sigma(ppbar->b X) for cross section E.
* -6        sigma(ppbar->B X) for cross section F.
* -7        sigma(ppbar->B X) for cross section G.
* -8        sigma(ppbar->B X) for cross section H.
* -9        sigma(bbbar) for cross section I.
* -10       sigma(bbbar) for cross section J.
* -11       sigma(bbbar) for cross section K.
* -12       sigma(bbbar) for cross section L.
* -13       sigma(ppbar->b X) for cross section M.
* -14       sigma(ppbar->b X) for cross section N.
* -15       sigma(ppbar->b X) for cross section O.
*
* Monte-Carlo
*  1        sigma(ppbar->b(bbar)->mu mu X) for cross section A.
*  2        sigma(ppbar->b(bbar)->mu mu X) for cross section B.
*  3        sigma(ppbar->b X) for cross section C.
*  4        sigma(ppbar->b X) for cross section D.
*  5        sigma(ppbar->b X) for cross section E.
*  6        sigma(ppbar->B X) for cross section F.
*  7        sigma(ppbar->B X) for cross section G.
*  8        sigma(ppbar->B X) for cross section H.
*  9        sigma(bbbar) for cross section I.
*  10       sigma(bbbar) for cross section J.
*  11       sigma(bbbar) for cross section K.
*  12       sigma(bbbar) for cross section L.
*  13       sigma(ppbar->b X) for cross section M.
*  14       sigma(ppbar->b X) for cross section N.
*  15       sigma(ppbar->b X) for cross section O.
*               These graphs will be
*               meaningless unless Xsec and Ntot are set before
*               calling the termination routine.
*               (Xsec - total cross section returned by MC)
*               (Ntot - number of events passed to this routine)
*
*****************************************************************************	
      IMPLICIT NONE
        
#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
      
*     *********************************************************     
*     Variable Declaration     
*     *********************************************************      
      
      Character xxxx*8
      Data xxxx/'c93153'/

      DOUBLE PRECISION pi
      PARAMETER(pi=3.141592653589793238)

      
      INTEGER iflag,ID
      REAL phi

*     bins, data and errors for cross section A.
      INTEGER NABIN
      PARAMETER (NABIN=4)
      REAL ABIN(NABIN+1)
      REAL ADATA(NABIN),AERROR(NABIN)
      
      DATA ABIN /3.,5.,7.,10.,20./
      SAVE ABIN
      DATA ADATA /879.,551.,190.,58./
      SAVE ADATA
      DATA AERROR /146.,88.,36.,15./
      SAVE AERROR
      
*     bins, data and errors for cross section B.
      INTEGER NBBIN
      PARAMETER (NBBIN=2)
      REAL BBIN(NBBIN+1)
      REAL BDATA(NBBIN),BERROR(NBBIN)
      
      DATA BBIN /7.,10.,20./
      SAVE BBIN
      DATA BDATA /93.,89./
      SAVE BDATA
      DATA BERROR /39.,28./
      SAVE BERROR
      
*     bins, data and errors for cross section C.
      INTEGER NCBIN
      PARAMETER (NCBIN=4)
      REAL CBIN(NCBIN+1)
      REAL CDATA(NCBIN),CERROR(NCBIN)
      
      DATA CBIN /5.,7.,9.,13.,17./
      SAVE CBIN
      DATA CDATA /2310.,1300.,385.,109./
      SAVE CDATA
      DATA CERROR /687.1,376.4,119.,46./
      SAVE CERROR
      
*     bins, data and errors for cross section D.
      INTEGER NDBIN
      PARAMETER (NDBIN=2)
      REAL DBIN(NDBIN+1)
      REAL DDATA(NDBIN),DERROR(NDBIN)
      
      DATA DBIN /8.5,13.5,17.5/
      SAVE DBIN
      DATA DDATA /220.,90./
      SAVE DDATA
      DATA DERROR /114.,48.8/
      SAVE DERROR
      
*     bins, data and errors for cross section E.
      INTEGER NEBIN
      PARAMETER (NEBIN=6)
      REAL EBIN(NEBIN+1)
      REAL EDATA(NEBIN),EERROR(NEBIN)
      
      DATA EBIN /11,19,27,33,45,47,61/
      SAVE EBIN
      DATA EDATA /120.,24.,4.1,0.55,0.32,0.04/
      SAVE EDATA
      DATA EERROR /38.,8.,1.4,0.3,0.2,0.03/
      SAVE EERROR
      
*     bins, data and errors for cross section F.
      INTEGER NFBIN
      PARAMETER (NFBIN=4)
      REAL FBIN(NFBIN+1)
      REAL FDATA(NFBIN),FERROR(NFBIN)
      
      DATA FBIN /4.,6.,8.,12.,14./
      SAVE FBIN
      DATA FDATA /2570.,1360.,315.,103./
      SAVE FDATA
      DATA FERROR /698.6,357.4,90.4,41.2/
      SAVE FERROR
      
*     bins, data and errors for cross section G.
      INTEGER NGBIN
      PARAMETER (NGBIN=2)
      REAL GBIN(NGBIN+1)
      REAL GDATA(NGBIN),GERROR(NGBIN)
      
      DATA GBIN /8,12,14/
      SAVE GBIN
      DATA GDATA /180.,111./
      SAVE GDATA
      DATA GERROR /86.,45.3/
      SAVE GERROR
      
*     bins, data and errors for cross section H.
      INTEGER NHBIN
      PARAMETER (NHBIN=6)
      REAL HBIN(NHBIN+1)
      REAL HDATA(NHBIN),HERROR(NHBIN)
      
      DATA HBIN /11,17,23,29,37,45,51/
      SAVE HBIN
      DATA HDATA /80.,19.,3.5,0.5,0.21,0.025/
      SAVE HDATA
      DATA HERROR /24.,6.,1.2,0.2,0.12,0.015/
      SAVE HERROR
      
*     bins, data and errors for cross section I.
      INTEGER NIBIN
      PARAMETER (NIBIN=6)
      REAL IBIN(NIBIN+1)
      REAL IDATA(NIBIN),IERROR(NIBIN)
      
      DATA IBIN /0.,30.,60.,90.,120.,150.,180./
      SAVE IBIN
      DATA IDATA /0.44,0.12,0.32,0.52,0.74,2.48/
      SAVE IDATA
      DATA IERROR /0.16,0.06,0.10,0.09,0.11,0.12/
      SAVE IERROR
      
*     bins, data and errors for cross section J.
      INTEGER NJBIN
      PARAMETER (NJBIN=6)
      REAL JBIN(NJBIN+1)
      REAL JDATA(NJBIN),JERROR(NJBIN)
      
      DATA JBIN /0.,30.,60.,90.,120.,150.,180./
      SAVE JBIN
      DATA JDATA /0.114,0.057,0.071,0.050,0.107,0.336/
      SAVE JDATA
      DATA JERROR /0.071,0.038,0.029,0.025,0.025,0.048/
      SAVE JERROR
      
*     bins, data and errors for cross section K.
      INTEGER NKBIN
      PARAMETER (NKBIN=5)
      REAL KBIN(NKBIN+1)
      REAL KDATA(NKBIN),KERROR(NKBIN)
      
      DATA KBIN /0.,0.8,1.6,2.4,3.2,4./
      SAVE KBIN
      DATA KDATA /0.086,0.295,0.461,0.225,0.096/
      SAVE KDATA
      DATA KERROR /0.139,0.067,0.064,0.064,0.083/
      SAVE KERROR
      
*     bins, data and errors for cross section L.
      INTEGER NLBIN
      PARAMETER (NLBIN=5)
      REAL LBIN(NLBIN+1)
      REAL LDATA(NLBIN),LERROR(NLBIN)
      
      DATA LBIN /0.,0.8,1.6,2.4,3.2,4./
      SAVE LBIN
      DATA LDATA /0.010,0.071,0.094,0.046,0.066/
      SAVE LDATA
      DATA LERROR /0.021,0.031,0.039,0.031,0.056/
      SAVE LERROR
      
*     bins, data and errors for cross section M.
      INTEGER NMBIN
      PARAMETER (NMBIN=4)
      REAL MBIN(NMBIN+1)
      REAL MDATA(NMBIN),MERROR(NMBIN)
      
      DATA MBIN /5.,7.,9.,13.,17./
      SAVE MBIN
      DATA MDATA /3.2,1.69,0.463,0.108/
      SAVE MDATA
      DATA MERROR /0.35,0.13,0.065,0.033/
      SAVE MERROR
      
*     bins, data and errors for cross section N.
      INTEGER NNBIN
      PARAMETER (NNBIN=4)
      REAL NBIN(NNBIN+1)
      REAL NDATA(NNBIN),NERROR(NNBIN)
      
      DATA NBIN /5.,7.,9.,13.,17./
      SAVE NBIN
      DATA NDATA /2.02,1.09,0.245,0.064/
      SAVE NDATA
      DATA NERROR /0.46,0.13,0.041,0.018/
      SAVE NERROR
      
*     bins, data and errors for cross section O.
      INTEGER NOBIN
      PARAMETER (NOBIN=4)
      REAL OBIN(NOBIN+1)
      REAL ODATA(NOBIN),OERROR(NOBIN)
      
      DATA OBIN /5.,7.,9.,13.,17./
      SAVE OBIN
      DATA ODATA /1.18,0.6,0.218,0.044/
      SAVE ODATA
      DATA OERROR /0.4,0.11,0.041,0.025/
      SAVE OERROR

*     loop variables
      INTEGER loop_1,loop_2,loop_3

*     b quark variables
      INTEGER NB,NMAXB
      INTEGER NBBAR,NMAXBBAR
      INTEGER NBBBAR,NMAXBBBAR
      PARAMETER (NMAXB=3,NMAXBBAR=3,NMAXBBBAR=6)
      INTEGER B(NMAXB),BBAR(NMAXBBAR),BBBAR(NMAXBBBAR)
      LOGICAL replace

*     B-hadron (incl. b-baryon) variables with only a b quark
      INTEGER nbrange
      PARAMETER (nbrange = 50)
      INTEGER brange(nbrange)
*     hep particle id's
      data brange /-511,-521,-10511,-10521,-513,-523,-10513,-10523,
     +     -20513,-20523,-515,-525,-531,-10531,-533,-10533,-20533,-535,
     +     -541,-10541,-543,-10543,-20543,-545,5122,5112,5212,5222,
     +     5114,5214,5224,5132,5232,5312,5322,5314,5324,5332,5334,5142,
     +     5242,5412,5422,5414,5424,5342,5432,5434,5442,5444 /
      INTEGER NMAXBMES
      PARAMETER (NMAXBMES=6)
      INTEGER NBMES,BMES(NMAXBMES)
      LOGICAL inlist

*     muon variables
      INTEGER NMAXMU,NMUIND
      PARAMETER (NMAXMU=6,NMUIND=2)
      INTEGER NMU,MU(NMAXMU,NMUIND)
      INTEGER MUMOID
      REAL MUPT,MUETA
      INTEGER MAXARRAY
      PARAMETER(MAXARRAY=1024)
      INTEGER STEPINDEX,TOTINDEX,ARRAY(MAXARRAY)
      LOGICAL matched

*     bbbar array sorting in pt
      INTEGER exchange

*     bin calculation
      REAL bin

*     histo filling
      LOGICAL filled
      REAL dphi,dr
      REAL fillpt,mmumu

*     histo normalisation
      REAL norm

*     printout variable
      LOGICAL printout

*     *********************************************************     
*     running condition
*     iflag: 
*     1: initialisation
*     2: execution
*     3: finalisation
*     *********************************************************      
      
      IF (iflag.gt.3) THEN
         PRINT*,xxxx,' Badly called routine'
         PRINT*,'          please set iflag to 1,2 or 3'
      ENDIF
      
*     *********************************************************     
*     initialisation run
*     *********************************************************     
      IF (iflag.eq.1) THEN
         
         CALL HCDIR('//PAWC',' ')
         CALL HMDIR(xxxx,'S')
         CALL HCDIR('//HISTO',' ')
         CALL HMDIR(xxxx,'S')
         
         ID=1
         CALL HBOOKB(ID,' [s](ppbar->b(bbar)->[m][m] X) ',NABIN,ABIN,0.)
         CALL HBARX(ID)
         CALL HBOOKB(-ID,' [s](ppbar->b(bbar)->[m][m] X) ',
     +        NABIN,ABIN,0.)
         CALL HPAK(-ID,ADATA)
         CALL HPAKE(-ID,AERROR)

         ID=2
         CALL HBOOKB(ID,' [s](ppbar->b(bbar)->[m] [m] X) ',
     +        NBBIN,BBIN,0.)
         CALL HBARX(ID)
         CALL HBOOKB(-ID,' [s](ppbar->b(bbar)->[m] [m] X) ',
     +        NBBIN,BBIN,0.)
         CALL HPAK(-ID,BDATA)
         CALL HPAKE(-ID,BERROR)

         ID=3
         CALL HBOOKB(ID,' [s](ppbar->b X) ',NCBIN,CBIN,0.)
         CALL HBARX(ID)
         CALL HBOOKB(-ID,' [s](ppbar->b X) ',NCBIN,CBIN,0.)
         CALL HPAK(-ID,CDATA)
         CALL HPAKE(-ID,CERROR)

         ID=4
         CALL HBOOKB(ID,' [s](ppbar->b X) ',NDBIN,DBIN,0.)
         CALL HBARX(ID)
         CALL HBOOKB(-ID,' [s](ppbar->b X) ',NDBIN,DBIN,0.)
         CALL HPAK(-ID,DDATA)
         CALL HPAKE(-ID,DERROR)

         ID=5
         CALL HBOOKB(ID,' [s](ppbar->b X) ',NEBIN,EBIN,0.)
         CALL HBARX(ID)
         CALL HBOOKB(-ID,' [s](ppbar->b X) ',NEBIN,EBIN,0.)
         CALL HPAK(-ID,EDATA)
         CALL HPAKE(-ID,EERROR)

         ID=6
         CALL HBOOKB(ID,' [s](ppbar->B X) ',NFBIN,FBIN,0.)
         CALL HBARX(ID)
         CALL HBOOKB(-ID,' [s](ppbar->B X) ',NFBIN,FBIN,0.)
         CALL HPAK(-ID,FDATA)
         CALL HPAKE(-ID,FERROR)

         ID=7
         CALL HBOOKB(ID,' [s](ppbar->B X) ',NGBIN,GBIN,0.)
         CALL HBARX(ID)
         CALL HBOOKB(-ID,' [s](ppbar->B X) ',NGBIN,GBIN,0.)
         CALL HPAK(-ID,GDATA)
         CALL HPAKE(-ID,GERROR)

         ID=8
         CALL HBOOKB(ID,' [s](ppbar->B X) ',NHBIN,HBIN,0.)
         CALL HBARX(ID)
         CALL HBOOKB(-ID,' [s](ppbar->B X) ',NHBIN,HBIN,0.)
         CALL HPAK(-ID,HDATA)
         CALL HPAKE(-ID,HERROR)

         ID=9
         CALL HBOOKB(ID,'[s](bbbar) for p?T^b!>6GeV/c ',NIBIN,IBIN,0.)
         CALL HBARX(ID)
         CALL HBOOKB(-ID,'[s](bbbar) for p?T^b!>6GeV/c ',NIBIN,IBIN,0.)
         CALL HPAK(-ID,IDATA)
         CALL HPAKE(-ID,IERROR)

         ID=10
         CALL HBOOKB(ID,' [s](bbbar) for p?T^b!>11GeV/c ',NJBIN,JBIN,0.)
         CALL HBARX(ID)
         CALL HBOOKB(-ID,' [s](bbbar) for p?T^b!>11GeV/c ',
     +        NJBIN,JBIN,0.)
         CALL HPAK(-ID,JDATA)
         CALL HPAKE(-ID,JERROR)

         ID=11
         CALL HBOOKB(ID,' [s](bbbar) for p?T^b!>6GeV/c ',NKBIN,KBIN,0.)
         CALL HBARX(ID)
         CALL HBOOKB(-ID,' [s](bbbar) for p?T^b!>6GeV/c ',NKBIN,KBIN,0.)
         CALL HPAK(-ID,KDATA)
         CALL HPAKE(-ID,KERROR)

         ID=12
         CALL HBOOKB(ID,' [s](bbbar) for p?T^b!>11GeV/c ',NLBIN,LBIN,0.)
         CALL HBARX(ID)
         CALL HBOOKB(-ID,' [s](bbbar) for p?T^b!>11GeV/c ',
     +        NLBIN,LBIN,0.)
         CALL HPAK(-ID,LDATA)
         CALL HPAKE(-ID,LERROR)

         ID=13
         CALL HBOOKB(ID,' [s](ppbar->b X) ',NMBIN,MBIN,0.)
         CALL HBARX(ID)
         CALL HBOOKB(-ID,' [s](ppbar->b X) ',NMBIN,MBIN,0.)
         CALL HPAK(-ID,MDATA)
         CALL HPAKE(-ID,MERROR)

         ID=14
         CALL HBOOKB(ID,' [s](ppbar->b X) ',NNBIN,NBIN,0.)
         CALL HBARX(ID)
         CALL HBOOKB(-ID,' [s](ppbar->b X) ',NNBIN,NBIN,0.)
         CALL HPAK(-ID,NDATA)
         CALL HPAKE(-ID,NERROR)

         ID=15
         CALL HBOOKB(ID,' [s](ppbar->b X) ',NOBIN,OBIN,0.)
         CALL HBARX(ID)
         CALL HBOOKB(-ID,' [s](ppbar->b X) ',NOBIN,OBIN,0.)
         CALL HPAK(-ID,ODATA)
         CALL HPAKE(-ID,OERROR)

      ELSE IF (iflag.eq.2) THEN
         CALL HCDIR('//PAWC/'//xxxx,' ')
         CALL HCDIR('//HISTO/UA1630',' ')

*     initialise variables
         NB=0
         DO loop_1=1,NMAXB
            B(loop_1)=0
         ENDDO
         NBBAR=0
         DO loop_1=1,NMAXBBAR
            BBAR(loop_1)=0
         ENDDO
         NBBBAR=0
         DO loop_1=1,NMAXBBBAR
            BBBAR(loop_1)=0
         ENDDO
         NBMES=0
         DO loop_1=1,NMAXBMES
            BMES(loop_1)=0
         ENDDO
         NMU=0
         DO loop_1=1,NMAXMU
            DO loop_2=1,NMUIND
               MU(loop_1,loop_2)=0
            ENDDO
         ENDDO
         MUMOID=0
         MUPT=0.0
         MUETA=0.0
         inlist=.false.
         exchange=0
         bin=0.0
         dphi=0.0
         dr=0.0
         replace=.false.
         printout=.false.
         
         
*     search for specific particles in nhep
         DO loop_1=1,NHEP
            
*     search for b, pythia style, ISTHEP<21
            IF ( GEN.eq.'PYT' ) THEN
*     first found b quark is kept, chain b quarks are discarded
               IF ( IDHEP(loop_1).eq.5.AND.ISTHEP(loop_1).ge.10 ) THEN
                  IF (NB.ge.NMAXB) THEN
                     print*,xxxx,' ERROR: More than ',NMAXB,' b quarks'
                     RETURN
                  ELSE
*     check 3 consecutive mother if they are already in list 
                     inlist=.false.
                     DO loop_2=1,NB
                        IF ( B(loop_2).eq.JMOHEP(1,loop_1) ) THEN
                           inlist=.true.
                        ELSE IF ( JMOHEP(1,JMOHEP(1,loop_1)).gt.0.AND.
     +                          JMOHEP(1,JMOHEP(1,loop_1)).eq.B(loop_2)
     +                          ) THEN
                           inlist=.true.
                        ELSE IF ( JMOHEP(1,JMOHEP(1,JMOHEP(1,loop_1))).
     +                          gt.0.AND.
     +                          JMOHEP(1,JMOHEP(1,JMOHEP(1,loop_1))).eq.
     +                          B(loop_2)
     +                          ) THEN
                           inlist=.true.
                        ENDIF
                     ENDDO
                     IF ( .NOT.inlist ) THEN
                        NB=NB+1
                        B(NB)=loop_1
                        IF (NBBBAR.ge.NMAXBBBAR) THEN
                           print*,xxxx,' ERROR: More than ',NMAXBBBAR,
     +                          ' b or anti bar quarks'
                           RETURN
                        ELSE
                           NBBBAR=NBBBAR+1
                           BBBAR(NBBBAR)=loop_1
                        ENDIF
                     ENDIF
                  ENDIF
               ELSE IF ( IDHEP(loop_1).eq.-5.AND.ISTHEP(loop_1).ge.10 ) 
     +                 THEN
                  IF (NBBAR.ge.NMAXBBAR) THEN
                     print*,xxxx,' ERROR: More than ',NMAXB,
     +                    ' anti b quarks'
                     RETURN
                  ELSE
*     check 3 consecutive mother if they are already in list 
                     inlist=.false.
                     DO loop_2=1,NBBAR
                        IF ( BBAR(loop_2).eq.JMOHEP(1,loop_1) ) THEN
                           inlist=.true.
                        ELSE IF ( JMOHEP(1,JMOHEP(1,loop_1)).gt.0.AND.
     +                          JMOHEP(1,JMOHEP(1,loop_1)).eq.
     +                          BBAR(loop_2) ) THEN
                           inlist=.true.
                        ELSE IF ( JMOHEP(1,JMOHEP(1,JMOHEP(1,loop_1))).
     +                          gt.0.AND.
     +                          JMOHEP(1,JMOHEP(1,JMOHEP(1,loop_1))).eq.
     +                          BBAR(loop_2)
     +                          ) THEN
                           inlist=.true.
                        ENDIF
                     ENDDO
                     IF ( .NOT.inlist ) THEN
                        NBBAR=NBBAR+1
                        BBAR(NBBAR)=loop_1
                        IF (NBBBAR.ge.NMAXBBBAR) THEN
                           print*,xxxx,' ERROR: More than ',NMAXBBBAR,
     +                          ' b or anti bar quarks'
                           RETURN
                        ELSE
                           NBBBAR=NBBBAR+1
                           BBBAR(NBBBAR)=loop_1
                        ENDIF
                     ENDIF
                  ENDIF
               ENDIF
            ELSE IF ( GEN.eq.'HRW' ) THEN
*     take all b quarks which have ISTHEP=2 (parton before hadronisation
*     quarks have as second mother color information, so only consider first mother
               IF ( IDHEP(loop_1).eq.5.AND.ISTHEP(loop_1).eq.2 ) THEN
                  IF (NB.ge.NMAXB) THEN
                     print*,xxxx,' ERROR: More than ',NMAXB,' b quarks'
                     RETURN
                  ELSE
*     check 3 consecutive mother if they are already in list 
                     inlist=.false.
                     DO loop_2=1,NB
                        IF ( B(loop_2).eq.JMOHEP(1,loop_1) ) THEN
                           inlist=.true.
                        ELSE IF ( JMOHEP(1,JMOHEP(1,loop_1)).gt.0.AND.
     +                          JMOHEP(1,JMOHEP(1,loop_1)).eq.B(loop_2)
     +                          ) THEN
                           inlist=.true.
                        ELSE IF ( JMOHEP(1,JMOHEP(1,JMOHEP(1,loop_1))).
     +                          gt.0.AND.
     +                          JMOHEP(1,JMOHEP(1,JMOHEP(1,loop_1))).eq.
     +                          B(loop_2)
     +                          ) THEN
                           inlist=.true.
                        ENDIF
                     ENDDO
                     IF ( .NOT.inlist ) THEN
                        NB=NB+1
                        B(NB)=loop_1
                        IF (NBBBAR.ge.NMAXBBBAR) THEN
                           print*,xxxx,' ERROR: More than ',NMAXBBBAR,
     +                          ' b or anti bar quarks'
                           RETURN
                        ELSE
                           NBBBAR=NBBBAR+1
                           BBBAR(NBBBAR)=loop_1
                        ENDIF
                     ENDIF
                  ENDIF
               ELSE IF ( IDHEP(loop_1).eq.-5.AND.ISTHEP(loop_1).eq.2 ) 
     +                 THEN
                  IF (NBBAR.ge.NMAXBBAR) THEN
                     print*,xxxx,' ERROR: More than ',NMAXB,
     +                    ' anti b quarks'
                     RETURN
                  ELSE
*     check 3 consecutive mother if they are already in list 
                     inlist=.false.
                     DO loop_2=1,NBBAR
                        IF ( BBAR(loop_2).eq.JMOHEP(1,loop_1) ) THEN
                           inlist=.true.
                        ELSE IF ( JMOHEP(1,JMOHEP(1,loop_1)).gt.0.AND.
     +                          JMOHEP(1,JMOHEP(1,loop_1)).eq.
     +                          BBAR(loop_2) ) THEN
                           inlist=.true.
                        ELSE IF ( JMOHEP(1,JMOHEP(1,JMOHEP(1,loop_1))).
     +                          gt.0.AND.
     +                          JMOHEP(1,JMOHEP(1,JMOHEP(1,loop_1))).eq.
     +                          BBAR(loop_2)
     +                          ) THEN
                           inlist=.true.
                        ENDIF
                     ENDDO
                     IF ( .NOT.inlist ) THEN
                        NBBAR=NBBAR+1
                        BBAR(NBBAR)=loop_1
                        IF (NBBBAR.ge.NMAXBBBAR) THEN
                           print*,xxxx,' ERROR: More than ',NMAXBBBAR,
     +                          ' b or anti bar quarks'
                           RETURN
                        ELSE
                           NBBBAR=NBBBAR+1
                           BBBAR(NBBBAR)=loop_1
                        ENDIF
                     ENDIF
                  ENDIF
               ENDIF
            ELSE
               print*,xxxx,' generator not implemented for this routine'
               RETURN
            ENDIF
*     look for B hadrons, select first in decay chain
            DO loop_2=1,nbrange
               IF ( IDHEP(loop_1).eq.brange(loop_2) ) THEN
*     check if 3 consecutive mothers of loop_1 are included in BMES
                  inlist=.false.
                  DO loop_3=1,NBMES
                     IF ( BMES(loop_3).eq.JMOHEP(1,loop_1) ) THEN
                        inlist=.true.
                     ELSE IF ( JMOHEP(1,JMOHEP(1,loop_1)).gt.0.AND.
     +                       JMOHEP(1,JMOHEP(1,loop_1)).eq.BMES(loop_3)
     +                       ) THEN
                        inlist=.true.
                     ELSE IF ( JMOHEP(1,JMOHEP(1,JMOHEP(1,loop_1))).gt.
     +                       0.AND.
     +                       JMOHEP(1,JMOHEP(1,JMOHEP(1,loop_1))).eq.
     +                       BMES(loop_3)
     +                       ) THEN
                        inlist=.true.
                     ENDIF
                  ENDDO
                  IF ( .NOT.inlist ) THEN   
                     IF ( NBMES.ge.NMAXBMES ) THEN
                        print*,xxxx,' ERROR: More than ',
     +                       NMAXBMES,' B hadrons'
                        RETURN
                     ELSE
                        NBMES=NBMES+1
                        BMES(NBMES)=loop_1
                     ENDIF
                  ENDIF
               ENDIF
            ENDDO
*     look for muons
            IF ( ISTHEP(loop_1).eq.1 ) THEN
               IF ( IDHEP(loop_1).eq.13.OR.IDHEP(loop_1).eq.-13 ) THEN
                  MUPT=REAL(HZPT(loop_1))
                  MUETA=REAL(HZETA(loop_1))
                  IF ( ABS(MUETA).lt.1.5.AND.MUPT.gt.3.0 ) THEN
                     IF ( NMU.ge.NMAXMU ) THEN
                        PRINT*,xxxx,'ERROR: More than ',NMAXMU,
     +                       ' muons'
                        RETURN
                     ELSE 
                        NMU=NMU+1
                        MU(NMU,1)=loop_1
                     ENDIF
                  ENDIF
C     ENDIF
               ENDIF
            ENDIF
         ENDDO

*     sort b quark and B hadron arrays for pt
         DO loop_1=1,NBBBAR
            DO loop_2=(loop_1+1),NBBBAR
               IF ( HZPT(BBBAR(loop_2)).gt.HZPT(BBBAR(loop_1)) ) THEN
                  exchange = BBBAR(loop_2)
                  BBBAR(loop_2) = BBBAR(loop_1)
                  BBBAR(loop_1) = exchange
               ENDIF
            ENDDO
         ENDDO
         DO loop_1=1,NB
            DO loop_2=(loop_1+1),NB
               IF ( HZPT(B(loop_2)).gt.HZPT(B(loop_1)) ) THEN
                  exchange = B(loop_2)
                  B(loop_2) = B(loop_1)
                  B(loop_1) = exchange
               ENDIF
            ENDDO
         ENDDO
         DO loop_1=1,NBBAR
            DO loop_2=(loop_1+1),NBBAR
               IF ( HZPT(BBAR(loop_2)).gt.HZPT(BBAR(loop_1)) ) THEN
                  exchange = BBAR(loop_2)
                  BBAR(loop_2) = BBAR(loop_1)
                  BBAR(loop_1) = exchange
               ENDIF
            ENDDO
         ENDDO
         DO loop_1=1,NBMES
            DO loop_2=(loop_1+1),NBMES
               IF ( HZPT(BMES(loop_2)).gt.HZPT(BMES(loop_1)) ) THEN
                  exchange = BMES(loop_2)
                  BMES(loop_2) = BMES(loop_1)
                  BMES(loop_1) = exchange
               ENDIF
            ENDDO
         ENDDO

*     if two or more muons, check for muons from which b-quark in list bbbar they come from
         matched=.false.
         stepindex=0
         totindex=0
         IF ( NMU.ge.1 ) THEN
            DO loop_1=1,NMU
               IF ( NBBBAR.ge.1 ) THEN
                  DO loop_2=1,NBBBAR
                     matched=.false.
                     stepindex=0
                     totindex=0
                     IF ( JDAHEP(1,BBBAR(loop_2)).gt.0 ) THEN
                        IF ( JDAHEP(1,BBBAR(loop_2)).le.
     +                       JDAHEP(2,BBBAR(loop_2)) ) THEN
                           DO loop_3=JDAHEP(1,BBBAR(loop_2)),
     +                          JDAHEP(2,BBBAR(loop_2))
                              IF ( loop_3.eq.MU(loop_1,1) ) THEN
                                 MU(loop_1,2)=BBBAR(loop_2)
                                 matched=.true.
                                 stepindex=totindex
                              ELSE
                                 IF ( totindex.ge.maxarray ) THEN
                                    print*,xxxx,' ERROR: More than ',
     +                                   MAXARRAY,' entries in array'
                                    RETURN
                                 ELSE
                                    totindex=totindex+1
                                    array(totindex)=loop_3
                                 ENDIF
                              ENDIF
                           ENDDO
                        ELSE
                           IF ( JDAHEP(1,BBBAR(loop_2)).eq.MU(loop_1,1)
     +                          ) THEN
                              MU(loop_1,2)=BBBAR(loop_2)
                              matched=.true.
                              stepindex=totindex
                           ELSE
                              IF ( totindex.ge.maxarray ) THEN
                                 print*,xxxx,' ERROR: More than ',
     +                                MAXARRAY,' entries in array'
                                 RETURN
                              ELSE
                                 totindex=totindex+1
                                 array(totindex)=JDAHEP(1,BBBAR(loop_2))
                              ENDIF
                           ENDIF
                        ENDIF
                     ENDIF
                     IF ( .NOT.matched ) THEN
                        DO WHILE ( stepindex.lt.totindex )
                           IF ( .NOT.matched ) THEN
                              stepindex=stepindex+1
                              IF ( JDAHEP(1,array(stepindex)).gt.0) THEN
                                 IF ( JDAHEP(1,array(stepindex)).le.
     +                                JDAHEP(2,array(stepindex)) ) THEN
                                    DO loop_3=JDAHEP(1,
     +                                   array(stepindex)),
     +                                JDAHEP(2,array(stepindex))
                                       IF ( .NOT.Matched ) THEN
                                          IF ( loop_3.eq.MU(loop_1,1) 
     +                                         ) THEN
                                             MU(loop_1,2)=
     +                                            BBBAR(loop_2)
                                             matched=.true.
                                             stepindex=totindex
                                          ELSE
                                             IF ( totindex.ge.
     +                                            maxarray ) 
     +                                            THEN
                                                print*,xxxx,
     +                                               ' ERROR:', 
     +                                               ' More than ',
     +                                               MAXARRAY,
     +                                               ' entries in ',
     +                                               'array'
                                                RETURN
                                             ELSE
                                                totindex=totindex+1
                                                array(totindex)=
     +                                               loop_3
                                             ENDIF
                                          ENDIF
                                       ENDIF
                                    ENDDO
                                 ELSE
                                    IF ( JDAHEP(1,array(stepindex))
     +                                   .eq.MU(loop_1,1) ) THEN
                                       MU(loop_1,2)=BBBAR(loop_2)
                                       matched=.true.
                                       stepindex=totindex
                                    ELSE
                                       IF ( totindex.ge.maxarray ) 
     +                                      THEN
                                          print*,xxxx,
     +                                         ' ERROR: More than ',
     +                                         MAXARRAY,
     +                                         ' entries in array'
                                          RETURN
                                       ELSE
                                          totindex=totindex+1
                                          array(totindex)=
     +                                         JDAHEP(1,
     +                                         array(stepindex))
                                       ENDIF
                                    ENDIF
                                 ENDIF
                              ENDIF
                           ENDIF
                        ENDDO
                     ENDIF
                  ENDDO
               ENDIF
            ENDDO
         ENDIF

*     fill histogram, always remember to reset filled flag
         
         
         
*     fill histo A
         filled=.false.
         FILLPT=9999.99
         IF ( NB.gt.0.AND.NBBAR.gt.0.AND.NMU.ge.2 ) THEN
*     check if dimuon pair passes cuts.
            DO loop_1=1,NMU
               DO loop_2=(loop_1+1),NMU
                  IF ( FILLPT.eq.9999.99 ) THEN
                     if ( MU(loop_1,2).gt.0.AND.MU(loop_2,2).gt.0.AND.
     +                    MU(loop_1,2).ne.MU(loop_2,2) ) THEN
                        MMUMU=REAL(HZINVMAS(MU(loop_1,1),MU(loop_2,1)))
                        IF ( MMUMU.gt.6.0.AND.MMUMU.lt.35.0 ) THEN
                           IF ( REAL(HZPT(MU(loop_1,1))).ge.
     +                          REAL(HZPT(MU(loop_2,1))) ) THEN
                              FILLPT=REAL(HZPT(MU(loop_1,1)))
                              CALL HFILL (1,FILLPT,0.,1.)
                           ELSE
                              FILLPT=REAL(HZPT(MU(loop_1,2)))
                              CALL HFILL (1,FILLPT,0.,1.)
                           ENDIF
                        ENDIF
                     ENDIF
                  ENDIF
               ENDDO
            ENDDO
         ENDIF

*     fill histo B
         filled=.false.
         FILLPT=9999.99
         IF ( NB.gt.0.AND.NBBAR.gt.0.AND.NMU.ge.2 ) THEN
*     check if dimuon pair passes cuts.
            DO loop_1=1,NMU
               DO loop_2=(loop_1+1),NMU
                  IF ( FILLPT.eq.9999.99 ) THEN
                     if ( MU(loop_1,2).gt.0.AND.MU(loop_2,2).gt.0.AND.
     +                    MU(loop_1,2).eq.MU(loop_2,2) ) THEN
                        MMUMU=REAL(HZINVMAS(MU(loop_1,1),MU(loop_2,1)))
                        IF ( MMUMU.gt.4.0.AND.MMUMU.lt.35.0) THEN
                           IF ( REAL(HZPT(MU(loop_1,1))).ge.
     +                          REAL(HZPT(MU(loop_2,1))) ) THEN
                              FILLPT=REAL(HZPT(MU(loop_1,1)))
                              CALL HFILL (2,FILLPT,0.,1.)
                           ELSE
                              FILLPT=REAL(HZPT(MU(loop_1,2)))
                              CALL HFILL (2,FILLPT,0.,1.)
                           ENDIF
                        ELSE IF ( MMUMU.le.4.0 ) THEN
                           FILLPT=sqrt((PHEP(1,MU(loop_1,1))+
     +                          PHEP(1,MU(loop_2,1)))**2+
     +                          (PHEP(2,MU(loop_1,1))+
     +                          PHEP(2,MU(loop_2,1)))**2)
                           CALL HFILL (2,FILLPT,0.,1.)
                        ENDIF
                     ENDIF
                  ENDIF
               ENDDO
            ENDDO
         ENDIF

*     fill histo C
*     fill only b or anti b quark with hightest pt satisfying cuts
         filled=.false.
         IF ( NBBBAR.gt.0 ) THEN
            DO loop_1=1,NBBBAR
               IF ( .NOT.filled ) THEN
                  IF ( ABS(REAL(HZRAP(BBBAR(loop_1)))).lt.1.5 ) THEN
                     DO loop_2=1,NCBIN
                        bin=REAL(CBIN(loop_2)+
     +                       ((CBIN(loop_2+1)-CBIN(loop_2))/2))
                        IF ( REAL(HZPT(BBBAR(loop_1))).ge.bin ) THEN
                           CALL HFILL (3,bin,0.,1.)
                           filled=.true.
                        ENDIF
                     ENDDO
                  ENDIF
               ENDIF
            ENDDO
         ENDIF
         
*     fill histo D
*     fill only b or anti b quark with hightest pt satisfying cuts
         filled=.false.
         IF ( NBBBAR.gt.0 ) THEN
            DO loop_1=1,NBBBAR
               IF ( .NOT.filled ) THEN
                  IF ( ABS(REAL(HZRAP(BBBAR(loop_1)))).lt.1.5 ) THEN
                     DO loop_2=1,NDBIN
                        bin=REAL(DBIN(loop_2)+
     +                       ((DBIN(loop_2+1)-DBIN(loop_2))/2))
                        IF ( REAL(HZPT(BBBAR(loop_1))).ge.bin ) THEN
                           CALL HFILL (4,bin,0.,1.)
                           filled=.true.
                        ENDIF
                     ENDDO
                  ENDIF
               ENDIF
            ENDDO
         ENDIF

*     fill histo E
*     fill only b or anti b quark with hightest pt satisfying cuts
         filled=.false.
         IF ( NBBBAR.gt.0 ) THEN
            DO loop_1=1,NBBBAR
               IF ( .NOT.filled ) THEN
                  IF ( ABS(REAL(HZRAP(BBBAR(loop_1)))).lt.1.5 ) THEN
                     DO loop_2=1,NEBIN
                        bin=REAL(EBIN(loop_2)+
     +                       ((EBIN(loop_2+1)-EBIN(loop_2))/2))
                        IF ( REAL(HZPT(BBBAR(loop_1))).ge.bin ) THEN
                           CALL HFILL (5,bin,0.,1.)
                           filled=.true.
                        ENDIF
                     ENDDO
                  ENDIF
               ENDIF
            ENDDO
         ENDIF
         
*     fill histo F
*     fill only B hadron with hightest pt satisfying cuts
         filled=.false.
         IF ( NBMES.gt.0 ) THEN
            DO loop_1=1,NBMES
               IF ( .NOT.filled ) THEN
                  IF ( ABS(REAL(HZRAP(BMES(loop_1)))).lt.1.5 ) THEN
                     DO loop_2=1,NFBIN
                        bin=REAL(FBIN(loop_2)+
     +                       ((FBIN(loop_2+1)-FBIN(loop_2))/2))
                        IF ( REAL(HZPT(BMES(loop_1))).ge.bin ) THEN
                           CALL HFILL (6,bin,0.,1.)
                           filled=.true.
                        ENDIF
                     ENDDO
                  ENDIF
               ENDIF
            ENDDO
         ENDIF
         
*     fill histo G
*     fill only B hadron with hightest pt satisfying cuts
         filled=.false.
         IF ( NBMES.gt.0 ) THEN
            DO loop_1=1,NBMES
               IF ( .NOT.filled ) THEN
                  IF ( ABS(REAL(HZRAP(BMES(loop_1)))).lt.1.5 ) THEN
                     DO loop_2=1,NGBIN
                        bin=REAL(GBIN(loop_2)+
     +                       ((GBIN(loop_2+1)-GBIN(loop_2))/2))
                        IF ( REAL(HZPT(BMES(loop_1))).ge.bin ) THEN
                           CALL HFILL (7,bin,0.,1.)
                           filled=.true.
                        ENDIF
                     ENDDO
                  ENDIF
               ENDIF
            ENDDO
         ENDIF
         
*     fill histo H
*     fill only B hadron with hightest pt satisfying cuts
         filled=.false.
         IF ( NBMES.gt.0 ) THEN
            DO loop_1=1,NBMES
               IF ( .NOT.filled ) THEN
                  IF ( ABS(REAL(HZRAP(BMES(loop_1)))).lt.1.5 ) THEN
                     DO loop_2=1,NHBIN
                        bin=REAL(HBIN(loop_2)+
     +                       ((HBIN(loop_2+1)-HBIN(loop_2))/2))
                        IF ( REAL(HZPT(BMES(loop_1))).ge.bin ) THEN
                           CALL HFILL (8,bin,0.,1.)
                           filled=.true.
                        ENDIF
                     ENDDO
                  ENDIF
               ENDIF
            ENDDO
         ENDIF
         
*     fill histo I
         filled=.false.
         IF ( NB.gt.0.AND.NBBAR.gt.0 ) THEN
            DO loop_1=1,NB
               DO loop_2=1,NBBAR
                  IF ( .NOT.filled ) THEN
                     IF ( HZPT(B(loop_1)).ge.HZPT(BBAR(loop_2)) ) 
     +                    THEN
                        IF ( REAL(HZPT(B(loop_1))).gt.6.0 ) THEN
                           dphi=REAL(HZDPHI(B(loop_1),
     +                          BBAR(loop_2)))/pi*180
                           CALL HFILL (9,dphi,0.,1.)
                           filled=.true.
                        ENDIF
                     ELSE
                        IF ( REAL(HZPT(BBAR(loop_2))).gt.6.0 ) THEN
                           dphi=REAL(HZDPHI(B(loop_1),
     +                          BBAR(loop_2)))/pi*180
                           CALL HFILL (9,dphi,0.,1.)
                           filled=.true.
                        ENDIF
                     ENDIF
                  ENDIF
               ENDDO
            ENDDO
         ENDIF

*     fill histo J
         filled=.false.
         IF ( NB.gt.0.AND.NBBAR.gt.0 ) THEN
            DO loop_1=1,NB
               DO loop_2=1,NBBAR
                  IF ( .NOT.filled ) THEN
                     IF ( HZPT(B(loop_1)).ge.HZPT(BBAR(loop_2)) ) 
     +                    THEN
                        IF ( REAL(HZPT(B(loop_1))).gt.11.0 ) THEN
                           dphi=REAL(HZDPHI(B(loop_1),
     +                          BBAR(loop_2)))/pi*180
                           CALL HFILL (10,dphi,0.,1.)
                           filled=.true.
                        ENDIF
                     ELSE
                        IF ( REAL(HZPT(BBAR(loop_2))).gt.11.0 ) THEN
                           dphi=REAL(HZDPHI(B(loop_1),
     +                          BBAR(loop_2)))/pi*180
                           CALL HFILL (10,dphi,0.,1.)
                           filled=.true.
                        ENDIF
                     ENDIF
                  ENDIF
               ENDDO
            ENDDO
         ENDIF

*     fill histo K
         filled=.false.
         IF ( NB.gt.0.AND.NBBAR.gt.0 ) THEN
            DO loop_1=1,NB
               DO loop_2=1,NBBAR
                  IF ( .NOT.filled ) THEN
                     dphi=REAL(HZDPHI(B(loop_1),
     +                    BBAR(loop_2)))/pi*180
                     IF ( HZPT(B(loop_1)).ge.HZPT(BBAR(loop_2)) ) 
     +                    THEN
                        IF ( REAL(HZPT(B(loop_1))).gt.6.0
     +                       .AND.dphi.lt.120.0 ) THEN
                           dr=REAL(HZDR(B(loop_1),
     +                          BBAR(loop_2)))
                           CALL HFILL (11,dr,0.,1.)
                           filled=.true.
                        ENDIF
                     ELSE
                        dphi=REAL(HZDPHI(B(loop_1),
     +                       BBAR(loop_2)))/pi*180
                        IF ( REAL(HZPT(BBAR(loop_2))).gt.6.0
     +                       .AND.dphi.lt.120.0 ) THEN
                           dr=REAL(HZDR(B(loop_1),
     +                          BBAR(loop_2)))
                           CALL HFILL (11,dr,0.,1.)
                           filled=.true.
                        ENDIF
                     ENDIF
                  ENDIF
               ENDDO
            ENDDO
         ENDIF

*     fill histo L
         filled=.false.
         IF ( NB.gt.0.AND.NBBAR.gt.0 ) THEN
            DO loop_1=1,NB
               DO loop_2=1,NBBAR
                  IF ( .NOT.filled ) THEN
                     IF ( HZPT(B(loop_1)).ge.HZPT(BBAR(loop_2)) ) 
     +                    THEN
                        dphi=REAL(HZDPHI(B(loop_1),
     +                       BBAR(loop_2)))/pi*180
                        IF ( REAL(HZPT(B(loop_1))).gt.11.0
     +                       .AND.dphi.lt.120.0 ) THEN
                           dr=REAL(HZDR(B(loop_1),
     +                          BBAR(loop_2)))
                           CALL HFILL (12,dr,0.,1.)
                           filled=.true.
                        ENDIF
                     ELSE
                        dphi=REAL(HZDPHI(B(loop_1),
     +                       BBAR(loop_2)))/pi*180
                        IF ( REAL(HZPT(BBAR(loop_2))).gt.11.0
     +                       .AND.dphi.lt.120.0 ) THEN
                           dr=REAL(HZDR(B(loop_1),
     +                          BBAR(loop_2)))
                           CALL HFILL (12,dr,0.,1.)
                           filled=.true.
                        ENDIF
                     ENDIF
                  ENDIF
               ENDDO
            ENDDO
         ENDIF

*     fill histo M
*     fill only b or anti b quark with hightest pt
         filled=.false.
         IF ( NB.gt.0.AND.NBBAR.gt.0  ) THEN
            DO loop_1=1,NMBIN
               bin=REAL(MBIN(loop_1)+
     +              ((MBIN(loop_1+1)-MBIN(loop_1))/2))
               IF ( REAL(HZPT(BBBAR(1))).ge.bin ) THEN
                  CALL HFILL (13,bin,0.,1.)
               ENDIF
            ENDDO
         ENDIF

*     fill histo N
*     fill only b or anti b quark with hightest pt corresponding to the bbbar-pair
*     taken for cut
         filled=.false.
         IF ( NB.gt.0.AND.NBBAR.gt.0 ) THEN
            DO loop_1=1,NB
               DO loop_2=1,NBBAR
                  IF ( .NOT.filled ) THEN
                     dphi=REAL(HZDPHI(B(loop_1),
     +                    BBAR(loop_2)))/pi*180
                     IF ( dphi.ge.150) THEN
                        DO loop_3=1,NNBIN
                           bin=REAL(NBIN(loop_3)+
     +                          ((NBIN(loop_3+1)-NBIN(loop_3))/2))
                           IF ( HZPT(B(loop_1)).ge.HZPT(BBAR(loop_2)) ) 
     +                          THEN
                              IF ( REAL(HZPT(B(loop_1))).ge.bin ) THEN
                                 CALL HFILL (14,bin,0.,1.)
                                 filled=.true.
                              ENDIF
                           ELSE
                              IF ( REAL(HZPT(BBAR(loop_2))).ge.
     +                             bin ) THEN
                                 CALL HFILL (14,bin,0.,1.)
                                 filled=.true.
                              ENDIF
                           ENDIF
                        ENDDO
                     ENDIF
                  ENDIF
               ENDDO
            ENDDO
         ENDIF

*     fill histo O
*     fill only b or anti b quark with hightest pt corresponding to the bbbar-pair
*     taken for cut
         filled=.false.
         IF ( NB.gt.0.AND.NBBAR.gt.0 ) THEN
            DO loop_1=1,NB
               DO loop_2=1,NBBAR
                  IF ( .NOT.filled ) THEN
                     dphi=REAL(HZDPHI(B(loop_1),
     +                    BBAR(loop_2)))/pi*180
                     IF ( dphi.LT.150 ) THEN
                        DO loop_3=1,NOBIN
                           bin=REAL(OBIN(loop_3)+
     +                          ((OBIN(loop_3+1)-OBIN(loop_3))/2))
                           IF ( HZPT(B(loop_1)).ge.HZPT(BBAR(loop_2)) ) 
     +                          THEN
                              IF ( REAL(HZPT(B(loop_1))).ge.bin ) THEN
                                 CALL HFILL (15,bin,0.,1.)
                                 filled=.true.
                              ENDIF
                           ELSE
                              IF ( REAL(HZPT(BBAR(loop_2))).ge.
     +                             bin ) THEN
                                 CALL HFILL (15,bin,0.,1.)
                                 filled=.true.
                              ENDIF
                           ENDIF
                        ENDDO
                     ENDIF
                  ENDIF
               ENDDO
            ENDDO
         ENDIF
         
*     printout
         IF ( printout ) THEN
            IF ( NB.gt.0.OR.NBBAR.gt.0 ) THEN
               DO loop_1=1,NB
                  print*,'NB: ',loop_1,' id: ',B(loop_1),
     +                 ' mo1: ',JMOHEP(1,B(loop_1)),
     +                 ' mo2: ',JMOHEP(2,B(loop_1)),
     +                 ' da1: ',JDAHEP(1,B(loop_1)),
     +                 ' da2: ',JDAHEP(2,B(loop_1)),
     +                 ' isthep: ',ISTHEP(B(loop_1)),
     +                 ' phi: ',HZPHI(B(loop_1))+pi
               ENDDO
               DO loop_1=1,NBBAR
                  print*,'NBBAR: ',loop_1,' id: ',BBAR(loop_1),
     +                 ' mo1: ',JMOHEP(1,BBAR(loop_1)),
     +                 ' mo2: ',JMOHEP(2,BBAR(loop_1)),
     +                 ' da1: ',JDAHEP(1,BBAR(loop_1)),
     +                 ' da2: ',JDAHEP(2,BBAR(loop_1)),
     +                 ' isthep: ',ISTHEP(BBAR(loop_1)),
     +                 ' phi: ',HZPHI(BBAR(loop_1))+pi
               ENDDO
            ENDIF
            IF ( NBBBAR.gt.0 ) THEN
               DO loop_1=1,NBBBAR
                  print*,'NBBBAR: ',loop_1,' id: ',BBBAR(loop_1),
     +                 ' mo1: ',JMOHEP(1,BBBAR(loop_1)),
     +                 ' mo2: ',JMOHEP(2,BBBAR(loop_1)),
     +                 ' da1: ',JDAHEP(1,BBBAR(loop_1)),
     +                 ' da2: ',JDAHEP(2,BBBAR(loop_1)),
     +                 ' pt: ',HZPT(BBBAR(loop_1)),
     +                 ' phi: ',HZPHI(BBBAR(loop_1))+pi
               ENDDO
            ENDIF
            IF ( NBMES.gt.0 ) THEN
               DO loop_1=1,NBMES
                  print*,'NBMES: ',loop_1,' id: ',BMES(loop_1),
     +                 ' mo1: ',JMOHEP(1,BMES(loop_1)),
     +                 ' mo2: ',JMOHEP(2,BMES(loop_1)),
     +                 ' da1: ',JDAHEP(1,BMES(loop_1)),
     +                 ' da2: ',JDAHEP(2,BMES(loop_1)),
     +                 ' pt: ',HZPT(BMES(loop_1))
               ENDDO
            ENDIF
            IF ( NMU.GT.0 ) THEN
               DO loop_1=1,NMU
                  print*,'NMU: ',loop_1,' id: ',MU(loop_1,1),
     +                 ' QUARK: ',MU(loop_1,2),' pt: ',
     +                 REAL(HZPT(MU(loop_1,1))),' eta: ',
     +                 REAL(HZETA(MU(loop_1,1)))
               ENDDO
            ENDIF
            IF ( NB.gt.0.OR.NBBAR.gt.0.OR.NBBBAR.gt.0.OR.
     +           NBMES.gt.0.OR.NMU.GT.0 ) THEN

               DO loop_1=1,NHEP
                  print*,'id: ',loop_1,
     +                 ' idhep: ',IDHEP(loop_1),
     +                 ' isthep: ',ISTHEP(loop_1),
     +                 ' mo1: ',JMOHEP(1,loop_1),
     +                 ' mo2: ',JMOHEP(2,loop_1),
     +                 ' da1: ',JDAHEP(1,loop_1),
     +                 ' da2: ',JDAHEP(2,loop_1)
               ENDDO

            ENDIF

         ENDIF

      ELSE IF (iflag.eq.3) THEN
         CALL HCDIR('//PAWC/'//xxxx,' ')
         CALL HCDIR('//HISTO/UA1630',' ')

*     initialisation
         norm = 0.

         IF (Xsec.eq.0) THEN
            PRINT*,xxxx,': termination called with zero cross 
     +           section'
            PRINT*,'          cross section graph meaningless'
            Xsec=1
         ENDIF
         IF (Ntot.eq.0) THEN
            PRINT*,xxxx,' no total events'
            PRINT*,'          cross section graph meaningless'
            Ntot=1
         ENDIF
         norm=XSEC/Ntot

         write(6,*) xxxx,': ntot= ', ntot,' xsec (nb)= ',xsec
         print*,'norm: ',norm

*     normalize to pb
         CALL HOPERA(1,'+e',1,1,norm*1000,0.)
         CALL HOPERA(2,'+e',2,2,norm*1000,0.)

*     normalize to nb
*     in selection, b and bbar are taken, cross section only b, to benefit from statistics,
*     divide by two
         CALL HOPERA(3,'+e',3,3,(norm/2),0.)
         CALL HOPERA(4,'+e',4,4,(norm/2),0.)
         CALL HOPERA(5,'+e',5,5,(norm/2),0.)

*     normalize to nb
         CALL HOPERA(6,'+e',6,6,norm,0.)
         CALL HOPERA(7,'+e',7,7,norm,0.)
         CALL HOPERA(8,'+e',8,8,norm,0.)

*     normalize to mub
         CALL HOPERA(9,'+e',9,9,(norm/1000),0.)
         CALL HOPERA(10,'+e',10,10,(norm/1000),0.)
         CALL HOPERA(11,'+e',11,11,(norm/1000),0.)
         CALL HOPERA(12,'+e',12,12,(norm/1000),0.)

*     normalize to mub
*     in selection, b and bbar are taken, cross section only b, to benefit from statistics,
*     divide by two
         CALL HOPERA(13,'+e',13,13,(norm/1000/2),0.)
         CALL HOPERA(14,'+e',14,14,(norm/1000/2),0.)
         CALL HOPERA(15,'+e',15,15,(norm/1000/2),0.)
         
      ELSE
         WRITE(6,*) xxxx,' Unknown flag',iflag
      ENDIF
      RETURN
      END        

















