      subroutine hzh0001021(iflag)
*****************************************************************************
*     THE TRANSVERSE MOMENTUM AND TOTAL CROSS-SECTION OF E+ E- PAIRS IN
*     THE Z-BOSON REGION FROM P ANTI-P COLLISIONS AT S**(1/2) = 1.8-TEV.
*     (CDF)
*     Phys.Rev.Lett.84:845-850,2000, hep-ex/0001021.
*
*     This routine produces one Monte Carlo graph, the pT distribution
*     for Z->e+e- events at the Tevatron.  
*     Histogram -1 ---> the data from Table 1 and Fig.1 
*               +1 ---> the simulated results to compare with it.
*
*     The normalisation of the graph will be meaningless unless Xsec and
*     Ntot are set before calling the termination routine.  (Xsec -
*     total cross section returned by MC) (Ntot - number of events
*     passed to this routine)
*                    
* Author : Claire O'Dea, Jon Butterworth, Ben Waugh, Emily Nurse
* Contact: hztool@hep.ucl.ac.uk 
*
*****************************************************************************
*
* HERA Tuning defined commons

      IMPLICIT NONE

#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"

      REAL norme(50)
      REAL zpt
      INTEGER m, ihep, loop2
      
C     --- cuts arrays
      REAL masscut(2)
      SAVE masscut
      
C     --- private copies of Xsec and Ntot from HERACMN
      DOUBLE PRECISION mhxsec(2),mhntot(2)
      
      INTEGER intiflag,iflag,MAXEV
      
      Real pi
      
      REAL norm(50),binPt(51),mass(50),error(50)
      
c     Storing the data for the values of the bin edge with 
c     values from table 1*******
      
      Data binPt/0.5,1,1.5,2,2.5,3,3.5,4,4.5,5,5.5,6,6.5,7,7.5,
     1    8,8.5,9,9.5,10,10.5,11,11.5,12,13,14,15,16,17,
     2     18,19,20,22,24,26,28,30,34,36,40,44,46,50,60,
     3       70,80,90,100,125,150,200/
           
c     storing real data for the differential cross section given 
c     in table I ******
           
      Data mass/3.35d0,1.01d1,1.48d1,1.94d1,2.02d1,2.36d1,2.36d1,
     1         2.3d1,1.99d1,1.93d1,1.79d1,1.8d1,1.46d1,1.45d1,
     2         1.35d1,1.37d1,1.19d1,1.04d1,1.11d1,9.56d0,8.35d0,
     3         7.82d0,8.18d0,7.48d0,7.21d0,6.05d0,4.73d0,5.21d0,
     4         4.46d0,4.28d0,3.51d0,3.01d0,2.63d0,1.82d0,1.85d0,
     5         1.58d0,1.41d0,1.02d0,6.78d-1,6.44d-1,4.34d-1,
     6         3.94d-1,2.1d-1,1.07d-1,9.10d-2,4.50d-2,3.51d-2,
     7         1.81d-2,7.11d-3,9.74d-4/
        

c     storing real data for the total error ******

      Data error/0.54d0,0.10d1,0.12d1,0.14d1,0.14d1,0.15d1,
     1          0.14d1,0.14d1,0.13d1,0.12d1,0.12d1,0.12d1,0.10d1,
     2          0.10d1,0.10d1,0.10d1,0.09d1,0.09d1,0.09d1,
     3          0.82d0,0.76d0,0.74d0,0.76d0,0.72d0,0.53d0,
     4          0.47d0,0.41d0,0.44d0,0.40d0,0.39d0,0.35d0, 
     5          0.33d0,0.22d0,0.18d0,0.18d0,0.17d0,0.16d0, 
     6          0.10d0,0.79d-1,0.76d-1,0.62d-1,0.59d-1,
     7          0.27d-1,0.19d-1,1.75d-2,1.22d-2,1.07d-2,
     8          0.48d-2,2.97d-3,7.56d-4/

      intiflag=iflag

      IF (intiflag.eq.1) THEN
C     *** Initialisation run ***
C     *** Create directories ***
         
         CALL HCDIR('//HISTO',' ')
         CALL HMDIR('hzh0001021','S')
         CALL HCDIR('//PAWC',' ')
         CALL HMDIR('hzh0001021','S')
         
         
c$$$  c open all the hbook's for generating the final histograms for
c$$$  the c paper number hep-ex/hzh0001021
         
         CALL HBOOKB(1,'Diff Z Pt Cross Section',50
     $        ,binPt,0.)
         
         CALL HBOOKB(3,'Unnorm. Diff. Z Pt Cross Section',50
     $        ,binPt,0.)
         
         CALL HBOOKB(2,'Normalisation values',50,binPt,0.0)
         
         CALL HBOOKB(-1,'Measured data for Diff Pt cross' //' section'
     $        ,50,binPt,0.0)
         
         CALL HBOOKB(10,'Z mass',50,binPt,0.0)
	    
         CALL HIDOPT(0,'stat')
         CALL HBARX(0)
         
         CALL HCDIR('//HISTO/hzh0001021',' ')
         CALL HCDIR('//PAWC/hzh0001021',' ')
                  
C     ---basic cuts for mass plots
         masscut(1)=66
         masscut(2)=116
                  
         WRITE(6,*)'**********************************************'
         WRITE(6,*)'*  hzh0001021 called, histos will be output. *'
         WRITE(6,*)'**********************************************'
         
 1001    FORMAT(A29,F5.2,A13)
         
      ELSE IF (intiflag.eq.2) THEN
         
C     *** Filling runs ***
         
C     *** Change directory to our graphs ***
         
         CALL HCDIR('//PAWC/hzh0001021',' ')
                 
c     ******loop over the particles in the event to find electron
c     and positron
c    ******loop through particles in event  and fill histo
        do ihep=1,nhep
******  pick out Z/gamma*'s AFTER the parton shower.
******  the Z/gamma* will decay to an e+e-, egamma or gammagamma
             if((IDHEP(ihep).eq.23.or.IDHEP(ihep).eq.22).and.
     +           JDAHEP(1,ihep).ne.0) then
               if( (abs(IDHEP(JDAHEP(1,ihep))).eq.11.or.
     +              IDHEP(JDAHEP(1,ihep)).eq.22).and.
     +             (abs(IDHEP(JDAHEP(2,ihep))).eq.11.or.
     +              IDHEP(JDAHEP(2,ihep)).eq.22) ) then
                 CALL hfill(10,real(phep(5,ihep)),0.0,wtx)
******  make invariant mass cut (not clear whevether to use e+e- 
*****   or Z/gamma* mass due to possible collinear emission of gamma)
                 if(phep(5,ihep).gt.66.and.phep(5,ihep).lt.116) then     
                 zpt = sqrt(phep(1,ihep)**2+phep(2,ihep)**2)
                 call hfill(3,real(zpt),0.0,wtx)
                 endif
               endif
             endif
         end do
c     ***********************************
         
      ELSE IF (intiflag.eq.3) THEN
         
         write(*,*) 'finishing up cdf z.'
C     *** Termination run ***
C     *** Change directory ***
         CALL HCDIR('//PAWC/hzh0001021',' ')
         
c     Calculating the normalisation values
         
         Do loop2=1,50
            norm(loop2)=xsec*1.d3/((binPt(loop2+1)-binPt(loop2))*Ntot)
            norme(loop2)=0
         end do
         
         
c     this will store the above numbers in the histogram 
         
         CALL HPAK(2,norm)
         CALL HPAKE(2,norme)
         
c     this will plot the right cross section, taking into account the
c     normalised values ***********
         
         CALL HOPERA(3,'*E',2,1,1.0,1.0)           
         
         
C     fills the histo with real data *****
	    
         CALL HPAK(-1,mass)
         CALL HPAKE(-1,error)        
         
C     *** Finish off histograms ***
         
         IF (Xsec.eq.0) THEN
            PRINT*,'HZH0001021:termination called with zero xsec'
            PRINT*,'       cross section graph meaningless'
            Xsec=1
         ENDIF
         IF (Ntot.eq.0) THEN
            PRINT*,'HZH0001021:termination called with no events'
            PRINT*,'       cross section graph meaningless'
            Ntot=1
         ENDIF
         
         
      ELSE
         
c     *** End ****
         
         print*
     $        ,'CDF Inc: Please run routine with iflag set to 1,2 or 3'
         
      ENDIF
      
      RETURN
      
      END 
