*****************************************************************************
      Subroutine hzh0404004(iflag)
*****************************************************************************
*     Purpose: This routine reproduces the data from Fig. 2/3, 4, 5, 7, 8 and 9 
*     from  hep-ex/0404004, and produces histograms of simulated data
*     which can be compared to them.
*
*     Initial state: 900 GeV protons vs 900 GeV anti-protons for Fig. 2-7
*                    315 GeV protons vs 315 GeV anti-protons for Fig. 8-9
*
*     Reference: "The Underlying event in hard interactions at the Tevatron anti-p p collider (CDF)" 
*     Phys. Rev. D70, 072002 (2004), hep-ex/0404004
*
*     Arguments: iflag=10*[D]+1 initialisation
*                iflag=10*[D]+2 filling
*                iflag=10*[D]+3 termination
*
*     [D]= JETF  decides jet finder type. Ignored if IRUN=1. If set to 0 the 
*                finder set by chjet below is used. The jetfinder used in the paper is PUCELL
*     Needs to be run with merging/splitting set to 75%    

*     This routine produces 17 Monte Carlo and 17 data plots. The details of each plot are
*     in the comments below

*     (Xsec - total cross section returned by MC)
*     (Ntot - number of events passed to this routine)
*     
*     CROSS SECTIONS ARE IN PB. 
*     
*     Authors: Rob Snihur and Emily Nurse
*     Date:    4th September 2006
*     Contact: nurse@fnal.gov
*     
*****************************************************************************
*     
*     HERA Tuning defined commons

      IMPLICIT NONE
#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
#include "data/hzh0404004.inc"
#include "hzjetcmn.inc"
     
      CHARACTER*6 jetf
      INTEGER NJET
c      DOUBLE PRECISION Jets(50,8)
      DOUBLE PRECISION CONER
      SAVE CONER
      INTEGER loop, ierr, ip, ipb
      integer iloop, jloop
      REAL beamE
C     --- private copies of Xsec and Ntot from HERACMN
      DOUBLE PRECISION mhxsec(2),mhntot(2)
      INTEGER intiflag, chjet, iproc, iflag,MAXEV
      SAVE chjet, beamE
C     --- define default jet algorithm (PUCELL)
      
      DATA chjet/7/
      Real pi 
      INTEGER m 
      INTEGER id1
      double precision norm

c     write(*,*) Ntot
c     write(*,*) Xsec

      intiflag=iflag
      iproc=2

C     --- choose jet finder
      
      IF ((intiflag.gt.10).and.(MOD(intiflag,10).eq.1)) THEN
         chjet=INT(intiflag/10)
         intiflag=MOD(intiflag,10)
      ENDIF

      IF (intiflag.eq.1) THEN
C    --- Initialisation run 
C    --- Create directories 

         CALL HCDIR('//HISTO',' ')
         CALL HMDIR('hzh0404004','S')
         CALL HCDIR('//PAWC',' ')
         CALL HMDIR('hzh0404004','S')

C    ---  open all the hbook's for generating the final histo for the
C    ---  paper number hep-ex/0404004

         ierr = HZPPBBEAM(ip,ipb)

      if(ierr.gt.-1) then
          beamE=PHEP(4,ip)
      else 
          write(*,*) 'Could not find p / pbar beams!'     
          return
      end if
      if(beamE.gt.899.99) then

C     ---   Figure 2/3, top plot (1800 GeV) {d2,y1} 
         id1 = 81
c         nbins = nbins_d2_y1 -1
         CALL HBOOKB(-id1,'Sum pT of Max Cone versus Et of lead jet'
     &        ,nbins_d2_y1,binedges_d2_y1,0.) 
         CALL HBPROF(id1,'MC Sum pT of Max Cone versus Et of lead jet'
     &        ,nbins_d2_y1, binedges_d2_y1(1),
     &        binedges_d2_y1(nbins_d2_y1+1)
     &        ,0.,150.,' ')

C     ---   Figure 2/3, middle plot (1800 GeV) {d2,y2}
         id1 = 82
c         nbins = nbins_d2_y2-1
         CALL HBOOKB(-id1,'Sum pT of Min Cone versus Et of lead jet'
     &        ,nbins_d2_y2,binedges_d2_y2,0.) 
         CALL HBPROF(id1,'MC Sum pT of Min Cone versus Et of lead jet'
     &        ,nbins_d2_y2,binedges_d2_y2(1),
     &        binedges_d2_y2(nbins_d2_y2+1)
c     &        ,nbins,binedges_d2_y2(1),binedges_d2_y2(nbins_d2_y2) 
     &        ,0.,150.,' ')


C    ---   Figure 2/3, bottom plot (1800 GeV) {d2,y3}
         id1 = 83
c         nbins = nbins_d2_y3 - 1
         CALL HBOOKB(-id1,'Max - Min Cone versus Et of lead jet'
     &        ,nbins_d2_y3,binedges_d2_y3,0.) 
         CALL HBPROF(id1,'MC Max - Min Cone versus Et of lead jet'
     &        ,nbins_d2_y3, binedges_d2_y3(1),
     &        binedges_d2_y3(nbins_d2_y3+1)
     &        ,0.,150.,' ')

C    ---   Figure 4, plot1 (1800 GeV) {d3,y1}
         id1 = 84
         CALL HBOOKB(-id1,'Max + Min Cone, 40 < Et < 80'
     &        ,nbins_d3_y1,binedges_d3_y1,0.) 
         CALL HBOOKB(id1,'MC Max + Min Cone, 40 < Et < 80'
     &        ,nbins_d3_y1,binedges_d3_y1,0.)

C    ---   Figure 4, plot2 (1800 GeV) {d3,y2}
         id1 = 85
         CALL HBOOKB(-id1,'Max + Min Cone, 80 < Et < 120'
     &        ,nbins_d3_y2,binedges_d3_y2,0.) 
         CALL HBOOKB(id1,'MC Max + Min Cone, 80 < Et < 120'
     &        ,nbins_d3_y2,binedges_d3_y2,0.)

C    ---   Figure 4, plot3 (1800 GeV) {d3,y3}
         id1 = 86
         CALL HBOOKB(-id1,'Max + Min Cone, 120 < Et < 160'
     &        ,nbins_d3_y3,binedges_d3_y3,0.) 
         CALL HBOOKB(id1,'MC Max + Min Cone, 120 < Et < 160'
     &        ,nbins_d3_y3,binedges_d3_y3,0.)
         
C    ---   Figure 4, plot4 (1800 GeV) {d3,y4}
         id1 = 87
         CALL HBOOKB(-id1,'Max + Min Cone, 160 < Et < 200'
     &        ,nbins_d3_y4,binedges_d3_y4,0.) 
         CALL HBOOKB(id1,'MC Max + Min Cone, 160 < Et < 200'
     &        ,nbins_d3_y4,binedges_d3_y4,0.)

C    ---   Figure 4, plot5 (1800 GeV) {d3,y5}
         id1 = 88
         CALL HBOOKB(-id1,'Max + Min Cone, 200 < Et < 270'
     &        ,nbins_d3_y2,binedges_d3_y1,0.) 
         CALL HBOOKB(id1,'MC Max + Min Cone, 200 < Et < 270'
     &        ,nbins_d3_y5,binedges_d3_y5,0.)

C    ---   Figure 5 (1800 GeV) {d4,y1}
         id1 = 89
c         nbins = nbins_d4_y1 - 1
         CALL HBOOKB(-id1,'Ntracks of Max Cone versus Et of lead jet'
     &        ,nbins_d4_y1,binedges_d4_y1,0.) 
         CALL HBPROF(id1,
     &        'MC Ntracks of Max Cone versus Et of lead jet'
     &        ,nbins_d4_y1, binedges_d4_y1(1),
     &        binedges_d4_y1(nbins_d4_y1+1)
     &        ,-0.5,49.5,' ')


C    ---   Figure 5 (1800 GeV)  {d4,y2}
         id1 = 90
c         nbins = nbins_d4_y2 - 1
         CALL HBOOKB(-id1,'Ntracks of Min Cone versus Et of lead jet'
     &        ,nbins_d4_y2,binedges_d4_y2,0.) 
         CALL HBPROF(id1,
     &        'MC Ntracks of Min Cone versus Et of lead jet'
     &        ,nbins_d4_y2, binedges_d4_y2(1),
     &        binedges_d4_y2(nbins_d4_y2+1)
     &        ,-0.5,49.5,' ')

c     Figure 6A (1800 GeV)  {d5,y1}
C??????????????????????????????????????????????????????
c     Figure 6B (1800 GeV)  {d6,y1}
C??????????????????????????????????????????????????????

C    ---   Figure 7 (1800 GeV) {d7,y1}
         id1 = 91
c         nbins = nbins_d7_y1 - 1
         CALL HBOOKB(-id1,'swiss cheese (2 jets)'
     &        ,nbins_d7_y1,binedges_d7_y1,0.) 
         CALL HBPROF(id1,'MC Swiss cheese (2 jets)'
     &        ,nbins_d7_y1, binedges_d7_y1(1),
     &        binedges_d7_y1(nbins_d7_y1+1)
     &        ,0.,150.,' ')

C    ---    Figure 7 (1800 GeV)  {d7,y2}
         id1 = 92
c         nbins = nbins_d7_y2 - 1
         CALL HBOOKB(-id1,'swiss cheese (3 jets)'
     &        ,nbins_d7_y2,binedges_d7_y2,0.) 
         CALL HBPROF(id1,'MC Swiss cheese (3 jets)'
     &        ,nbins_d7_y2, binedges_d7_y2(1),
     &        binedges_d7_y2(nbins_d7_y2+1)
     &        ,0.,150.,' ')

C     ---  Book 630 GeV histos  
      else 
C     ---   Figure 8, top plot (630 GeV) {d8,y1} 
         id1 = 93
c         nbins = nbins_d8_y1 - 1
         CALL HBOOKB(-id1,
     &        'Sum pT of Max Cone versus Et of lead jet, 630 GeV'
     &        ,nbins_d8_y1,binedges_d8_y1,0.) 
         CALL HBPROF(id1,
     &        'MC Sum pT of Max Cone versus Et of lead jet, 630 GeV'
     &        ,nbins_d8_y1, binedges_d8_y1(1),
     &        binedges_d8_y1(nbins_d8_y1+1)
     &        ,0.,150.,' ')

C     ---   Figure 8, middle plot (630 GeV) {d8,y2}
         id1 = 94
c         nbins = nbins_d8_y2 - 1
         CALL HBOOKB(-id1,
     &        'Sum pT of Min Cone versus Et of lead jet, 630 GeV'
     &        ,nbins_d8_y2,binedges_d8_y2,0.) 
         CALL HBPROF(id1,
     &        'MC Sum pT of Min Cone versus Et of lead jet, 630 GeV'
     &        ,nbins_d8_y2,binedges_d8_y2(1),
     &        binedges_d8_y2(nbins_d8_y2+1)
     &        ,0.,150.,' ')


C    ---   Figure 8, bottom plot (630 GeV) {d8,y3}
C         id1 = 95
C         CALL HBOOKB(-id1,
C     &        'Max - Min Cone versus Et of lead jet, 630 GeV'
C     &        ,nbins_d8_y3,binedges_d8_y3,0.) 
C         CALL HBPROF(id1,
C     &        'MC Max - Min Cone versus Et of lead jet, 630 GeV'
C     &        ,nbins_d8_y3,binedges_d8_y3(1),binedges_d8_y3(nbins_d8_y3)
C     &        ,0.,150.,' ')


C    ---  Figure 9, top plot (630 GeV) {d9,y1} 
        id1 = 96
c        nbins = nbins_d9_y1 - 1
         CALL HBOOKB(-id1,'swiss cheese (2 jets), 630 GeV'
     &        ,nbins_d9_y1,binedges_d9_y1,0.) 
         CALL HBPROF(id1,'MC Swiss cheese (2 jets), 630 GeV'
     &        ,nbins_d9_y1, binedges_d9_y1(1),
     &        binedges_d9_y1(nbins_d9_y1+1)
     &        ,0.,150.,' ')

C    ---  Figure 9, bottom plot (630 GeV) {d9,y2} 
        id1 = 97
c        nbins = nbins_d9_y2 - 1
         CALL HBOOKB(-id1,'swiss cheese (3 jets), 630 GeV'
     &        ,nbins_d9_y2,binedges_d9_y2,0.) 
         CALL HBPROF(id1,'MC Swiss cheese (3 jets), 630 GeV'
     &        ,nbins_d9_y2,binedges_d9_y2(1),
     &        binedges_d9_y2(nbins_d9_y2+1)
     &        ,0.,150.,' ')
C    ---- Toggle between beam energies
         endif
         CALL HIDOPT(0,'stat')
         CALL HBARX(0)
         CALL HCDIR('//HISTO/hzh0404004',' ')
         CALL HCDIR('//PAWC/hzh0404004',' ')

C    --- standard cone radius is 1.0** change cone radius to 0.7
         CONER=0.7
         CALL HZJETRAD(1,CONER) 
         IF (CONER.lt.0.0) THEN
            CONER=0.7
         ENDIF
         CALL HZJTNAME(chjet,jetf)
         
         WRITE(6,*)'**********************************************'
         WRITE(6,*)'*   HZEVT called, histograms will be output. *'
         WRITE(6,*)'*     Jet finder selected : ',jetf,'           *'
         WRITE(6,1001)'*        with cone radius : ',CONER,'   *'
         WRITE(6,*)'**********************************************'
         
 1001    FORMAT(A29,F5.2,A13)
         
      ELSE IF (intiflag.eq.2) THEN
C    --- Filling runs ---
         
C    --- Change directory to our graphs ---
         CALL HCDIR('//PAWC/hzh0404004',' ')
         CALL hzjtfind(chjet,CONER,NJET,Jets)

C     --- Fill MC histograms           
         call mc2nt(NJET,Jets)
         call track_05(NJET)

      ELSE IF (intiflag.eq.3) THEN
         
         write(*,*) 'finishing up cdf UE.'
C     --- Termination run ---
C     --- Change directory ---
         CALL HCDIR('//PAWC/hzh0404004',' ')

c     write(*,*) Ntot
c     write(*,*) Xsec
C     --- Fill data histos ---  
C     --- 900 GeV histos   
         write(*,*) beamE
         if(beamE.gt.899.99) then 
         CALL HPAK(-81,xsec_d2_y1)
         CALL HPAKE(-81,staterr_d2_y1)        
         CALL HPAK(-82,xsec_d2_y2)
         CALL HPAKE(-82,staterr_d2_y2)    
         CALL HPAK(-83,xsec_d2_y3)
         CALL HPAKE(-83,staterr_d2_y3)  
         CALL HPAK(-84,xsec_d3_y1)
         CALL HPAKE(-84,staterr_d3_y1)        
         CALL HPAK(-85,xsec_d3_y2)
         CALL HPAKE(-85,staterr_d3_y2) 
         CALL HPAK(-86,xsec_d3_y3)
         CALL HPAKE(-86,staterr_d3_y3)  
         CALL HPAK(-87,xsec_d3_y4)
         CALL HPAKE(-87,staterr_d3_y4)
         CALL HPAK(-88,xsec_d3_y5)
         CALL HPAKE(-88,staterr_d3_y5)  
         CALL HPAK(-89,xsec_d4_y1)
         CALL HPAKE(-89,staterr_d4_y1)  
         CALL HPAK(-90,xsec_d4_y2)
         CALL HPAKE(-90,staterr_d4_y2)  
         CALL HPAK(-91,xsec_d7_y1)
         CALL HPAKE(-91,staterr_d7_y1) 
         CALL HPAK(-92,xsec_d7_y2)
         CALL HPAKE(-92,staterr_d7_y2)
C    ---  630 GeV histos
         else
         CALL HPAK(-93,xsec_d8_y1)
         CALL HPAKE(-93,staterr_d8_y1)  
         CALL HPAK(-94,xsec_d8_y2)
         CALL HPAKE(-94,staterr_d8_y2)
C         CALL HPAK(-95,xsec_d8_y3)
C         CALL HPAKE(-95,staterr_d8_y3)
         CALL HPAK(-96,xsec_d9_y1)
         CALL HPAKE(-96,staterr_d9_y1)
         CALL HPAK(-97,xsec_d9_y2)
         CALL HPAKE(-97,staterr_d9_y2)
C    --- toggle between beam energies
         end if
C    --- Finish off histograms ---
         IF (Xsec.eq.0) THEN
            PRINT*,'CDF UE: termination called with zero cross '
            PRINT*,'section          cross section graph meaningless'
            Xsec=1
         ENDIF
         IF (Ntot.eq.0) THEN
            PRINT*,'CDF UE: termination called with no total '
            PRINT*,'events          cross section graph meaningless'
            Ntot=1
         ENDIF

C    ---  Normalize histograms (but not profiles) to get absolute values
         if(beamE.gt.899.99) then

            loop = 1
            norm = 0.0
            do while (loop.le.nbins_d3_y1)
               norm = norm + 
     &              xsec_d3_y1(loop) * 
     &              (binedges_d3_y1(loop +1)-binedges_d3_y1(loop))
               loop = loop +1
            end do
            CALL hzhinrm(84,0,real(ntot/(norm)),1)

            loop = 1
            norm = 0.0
            do while (loop.le.nbins_d3_y2)
               norm = norm + 
     &              xsec_d3_y2(loop) * 
     &              (binedges_d3_y2(loop +1)-binedges_d3_y2(loop))
               loop = loop +1
            end do
            CALL hzhinrm(85,0,real(ntot/(norm)),1)

            loop = 1
            norm = 0.0
            do while (loop.le.nbins_d3_y3)
               norm = norm + 
     &              xsec_d3_y3(loop) * 
     &              (binedges_d3_y3(loop +1)-binedges_d3_y3(loop))
               loop = loop +1
            end do
            CALL hzhinrm(86,0,real(ntot/(norm)),1)

            loop = 1
            norm = 0.0
            do while (loop.le.nbins_d3_y4)
               norm = norm + 
     &              xsec_d3_y4(loop) * 
     &              (binedges_d3_y4(loop +1)-binedges_d3_y4(loop))
               loop = loop +1
            end do
            CALL hzhinrm(87,0,real(ntot/(norm)),1)

            loop = 1
            norm = 0.0
            do while (loop.le.nbins_d3_y5)
               norm = norm + 
     &              xsec_d3_y5(loop) * 
     &              (binedges_d3_y5(loop +1)-binedges_d3_y5(loop))
               loop = loop +1
            end do
            CALL hzhinrm(88,0,real(ntot/(norm)),1)

c         CALL hzhinrm(84,0,real(ntot/xsec),1)
c         CALL hzhinrm(85,0,real(ntot/xsec),1)
c         CALL hzhinrm(86,0,real(ntot/xsec),1)
c         CALL hzhinrm(87,0,real(ntot/xsec),1)
c        CALL hzhinrm(88,0,real(ntot/xsec),1)
         end if
C     --- store Xsec and Ntot
         mhXsec(iproc)=Xsec
         mhntot(iproc)=Ntot

      ELSE

C     --- End ---

         print*
     &        ,'CDF Inc: Please run routine with iflag set to 1,2 or 3'
         
      ENDIF

      RETURN

      END 

*****************************************************************************
      Subroutine mc2nt(NJET,Jets)
*****************************************************************************
      IMPLICIT NONE

#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
#include "et.inc"
                   
      INTEGER NJET
      DOUBLE PRECISION Jets(50,8)
      integer loop, maxjet, maxtrack
      real theta
      REAL pi,TWOPI
      maxjet=20
      maxtrack=250
      Pi=3.141592654
      TWOPI=2.0*Pi  
      
      DO loop=1,maxjet
      etajet(loop) = 0.
      phijet(loop) = 0.
      etjet(loop)  = 0.
      enddo

      DO loop=1,min(njet,maxjet)
         etajet(loop) = Jets(loop,1)
         phijet(loop) = Jets(loop,2)
         etjet(loop)  = Jets(loop,3)
      enddo

      DO loop=1, maxtrack
         ptTrack(loop)= -1.
         etaTrack(loop)= 1000.
         phitrack(loop)= -1.
      ENDDO                     ! loop=1,NHEP 

      ntrack = 0
      DO loop=1,NHEP 
c     select  only charged stable particles             
         IF((ISTHEP(loop).eq.1).and.(IHCHRG(loop).ne.0)) THEN
C     -------- Calculate PT,THETA,ETA for each particle -------
C            if ( ntrack .le. maxtrack ) then
               ntrack = ntrack + 1
               ptTrack(ntrack)=SQRT(PHEP(1,loop)**2+PHEP(2,loop)**2)
               THETA=ACOS(PHEP(3,loop)/SQRT(PTTrack(ntrack)**2
     &              +PHEP(3,loop)**2))
               etaTrack(ntrack)=-ALOG(TAN(THETA/2.))
               phitrack(ntrack)=atan2(PHEP(2,loop),PHEP(1,loop))
               IF (phitrack(ntrack).LT.(0.)) then 
                  phitrack(ntrack) = phitrack(ntrack) + TWOPI 
               END IF
               DO WHILE(phitrack(ntrack).gt.TWOPI)
                  phitrack(ntrack)=phitrack(ntrack)-TWOPI
               ENDDO                  
C           else
C               print *,'too many particles'
C            endif               ! if ( ntrack .le. maxtrack ) then
         ENDIF                  ! ((ISTHEP(loop).eq.1).and.(IHCHRG(loop).ne.0)) THEN
      ENDDO                     ! loop=1,NHEP 
      
      return
      end
*****************************************************************************
      Subroutine track_05(NJET)
*****************************************************************************
      IMPLICIT NONE
#include "et.inc"
      INTEGER NJET

      real eta, phi
      real cone 
      parameter (cone=0.7)

      REAL PI
      PARAMETER (PI=3.141592653)  
      REAL HALFPI
      PARAMETER (HALFPI=0.5*PI) 

      real pttrack_p90, pttrack_m90, pttrack_max, pttrack_min
      integer ntrack_p90, ntrack_m90, ntrack_max, ntrack_min
      integer ievtok

      ievtok = 0

C    ---    Cut on central region
      if(abs(etajet(1)).gt.0.5) ievtok=1
      if(ievtok.eq.1)  goto 110

C    ---    Swiss Cheese Plots
      call swiss_cheese(njet)

C    ---    Track Plots

      if(ntrack.gt.0) then
         eta = etajet(1)
         phi=phijet(1)+HALFPI      
         call get_track_cone90(eta,phi,cone,ntrack_p90,
     &        pttrack_p90,0.4)
         
         eta = etajet(1)
         phi=phijet(1)-HALFPI      
         call get_track_cone90(eta,phi,cone,ntrack_m90,
     &        pttrack_m90,0.4)
         
         call plot_track(ntrack_p90,pttrack_p90,ntrack_m90,
     &        pttrack_m90)
         
      end if                    ! cut on ntrack
 110  continue
      END

*-----------------------------------------------------------------
      subroutine plot_track(ntrack_p90,pttrack_p90,ntrack_m90,
     &     pttrack_m90)
*-----------------------------------------------------------------
*     INPUT : 
*     
*     
*     OUTPUT: 
*     
*     
*-----------------------------------------------------------------
      implicit none
#include "hzfunc.inc"
#include "hepevtp.inc"
#include "heracmn.inc"
#include "et.inc"

      integer id1
      REAL PI
      PARAMETER (PI=3.141592653)  
      REAL HALFPI
      PARAMETER (HALFPI=0.5*PI) 

      real pttrack_max, pttrack_min
      integer ntrack_max, ntrack_min
      real pttrack_p90, pttrack_m90
      integer ntrack_p90, ntrack_m90
      integer ip, ipb, ierr
      real beamE
C    ---  Max and min cone
      if(pttrack_p90.gt.pttrack_m90) then
         pttrack_max = pttrack_p90
         ntrack_max = ntrack_p90
         pttrack_min = pttrack_m90
         ntrack_min = ntrack_m90
      else
         pttrack_max = pttrack_m90
         ntrack_max = ntrack_m90
         pttrack_min = pttrack_p90
         ntrack_min = ntrack_p90
      end if
      ierr = HZPPBBEAM(ip,ipb)

      if(ierr.gt.-1) then
          beamE=PHEP(4,ip)
      else 
          write(*,*) 'Could not find p / pbar beams!'     
          return
      end if
C    ---  Fill histos
      if(beamE.gt.899.99) then
        call hfill(81,etjet(1),pttrack_max,wtx)
        call hfill(82,etjet(1),pttrack_min,wtx)
        call hfill(83,etjet(1),(pttrack_max - pttrack_min),wtx)
        if(etjet(1).gt.40.and.etjet(1).lt.80) then
          call hfill(84,(pttrack_min + pttrack_max),0.,wtx)
        elseif(etjet(1).gt.80.and.etjet(1).lt.120) then
          call hfill(85,(pttrack_min + pttrack_max),0.,wtx)
        elseif(etjet(1).gt.120.and.etjet(1).lt.160) then
          call hfill(86,(pttrack_min + pttrack_max),0.,wtx)
         elseif(etjet(1).gt.160.and.etjet(1).lt.200) then
          call hfill(87,(pttrack_min + pttrack_max),0.,wtx)
         elseif(etjet(1).gt.200.and.etjet(1).lt.270) then
          call hfill(88,(pttrack_min + pttrack_max),0.,wtx)
         endif

      call hfill(89,etjet(1),real(ntrack_max),wtx)
      call hfill(90,etjet(1),real(ntrack_min),wtx)
C ---- for 630 GeV
      else
        call hfill(93,etjet(1),pttrack_max,wtx)
        call hfill(94,etjet(1),pttrack_min,wtx)
C        call hfill(95,etjet(1),(pttrack_max - pttrack_min),wtx)
        call hfill(96,etjet(1),real(ntrack_max),wtx)
        call hfill(97,etjet(1),real(ntrack_min),wtx)
C  --- end toggle between beam energies
      end if
      return 
      end
*-----------------------------------------------------------------
      subroutine get_track_cone90(eta,phi,cone,ntrack90,pttrack90,
     &     pttrmin)
*-----------------------------------------------------------------
*     INPUT : eta   
*     phi
*     cone
*     pttrmin
*     OUTPUT: ntrack90
*     pttrack90
*     
*     Return the number and pt of tracks in a cone(cone) in eta,phi
*-----------------------------------------------------------------
      implicit none
#include "et.inc"

      integer i,id

      real cone

      real eta, phi
      real phidf, radf

      integer ntrack90
      real pttrack90
      real pttrmin
*     
      ntrack90 = 0
      pttrack90 = 0.0

*     track inside a cone (eta,phi)

      if(ntrack.gt.0) then

         do i=1,ntrack
            if(pttrack(i).gt.pttrmin) then

               phidf= min(abs(phitrack(i)-phi),
     *              6.28318531-abs(phitrack(i)-phi))
               radf= sqrt((etatrack(i)-eta)**2+phidf**2)
               
               if(radf.le.cone) then
                  ntrack90 = ntrack90 + 1
                  pttrack90 = pttrack90 + pttrack(i)
               end if
            end if
         end do
      end if


      return
      end

*-----------------------------------------------------------------
      subroutine swiss_cheese(njet)
*-----------------------------------------------------------------
*     
      implicit none

#include "hepevtp.inc"
#include "heracmn.inc"
#include "et.inc"

      integer njet
*     
*--   Enter user code here
*     
      real L, areaseg, area, areatot2, areatot3
      real  sum_2, sum_3 
      integer ievtok,id
      real pt_central
      real track1,track2,track3
      real pt_central_jet2,pt_central_jet3
      real sumpt_5_2, sumpt_5_3 
      integer i, j
      integer njet_5
      real etjet_5(20), etadet_5(20), phijet_5(20)

      REAL PI
      PARAMETER (PI=3.141592653)  

*     DETECTOR LEVEL
      pt_central = 0.
      pt_central_jet2 = 0.
      pt_central_jet3 = 0.
c     sumpt_5_2 =0.
c     sumpt_5_3 =0.

      if(ntrack.ne.0)then

C     Only jets with Et> 5 GeV
         njet_5 = 0
         do i = 1, njet
            if(etjet(i).gt.5) then
               njet_5 = njet_5 + 1
               etjet_5(njet_5) = etjet(i)
               etadet_5(njet_5) = etajet(i)
               phijet_5(njet_5) = phijet(i)
            end if
         end do
C     leading jet in central region
         if(abs(etadet_5(1)).lt.1.) then            
C     

            do j=1,ntrack

               if(pttrack(j).gt.0.4) then
                  
                  if(abs(etatrack(j)).lt.1) then
                     pt_central=pt_central+pttrack(j)
                  end if
               end if
            end do

*     
C     Sum of pt on all towers in the central region (eta<1)
            do j = 1, ntrack
               if(pttrack(j).gt.0.4) then
                  if(abs(etatrack(j)).lt.1.) then
C     Sum of energy inside jets in central region
                     do i=1, 2
                        track1 = (etatrack(j)-etadet_5(i))**2 + 
     &                       (phitrack(j)-phijet_5(i))**2
                        track2 = (etatrack(j)-etadet_5(i))**2 + 
     &                       (phitrack(j)-6.28-phijet_5(i))**2
                        track3 = (etatrack(j)-etadet_5(i))**2 + 
     &                       (phitrack(j)+6.28-phijet_5(i))**2
                        if(sqrt(track1).le.0.7.or.
     &                       sqrt(track2).le.0.7.or.
     &                       sqrt(track3).le.0.7) then
                           pt_central_jet2=pt_central_jet2+pttrack(j)
                           goto 10
                        end if
                     end do
 10                  continue
                     
                     do i=1, 3
                        track1 = (etatrack(j)-etadet_5(i))**2 + 
     &                       (phitrack(j)-phijet_5(i))**2
                        track2 = (etatrack(j)-etadet_5(i))**2 + 
     &                       (phitrack(j)-6.28-phijet_5(i))**2
                        track3 = (etatrack(j)-etadet_5(i))**2 + 
     &                       (phitrack(j)+6.28-phijet_5(i))**2
                        if(sqrt(track1).le.0.7.or.
     &                       sqrt(track2).le.0.7.or.
     &                       sqrt(track3).le.0.7) then
                           pt_central_jet3=pt_central_jet3+pttrack(j)
                           goto 11
                        end if
                     end do
 11                  continue
                  end if
               end if 
            end do

            sumpt_5_2 = pt_central - pt_central_jet2 
            sumpt_5_3 = pt_central - pt_central_jet3 
c     print*,sumpt_5_2,pt_central,pt_central_jet2

            if(etjet_5(1).ne.etjet(1)) goto 110
            call hfill(91,etjet_5(1),sumpt_5_2,wtx)
            call hfill(92,etjet_5(1),sumpt_5_3,wtx)

 110        continue
         end if
      end if
 100  continue
*     
      END


