      subroutine hzh9907009(IFLAG)
****************************************************************************
*   Measurement of the inclusive differential cross section for Z bosons as  a
*   function of transverse momentum in anti-p p collisions at  s**(1/2) =
*   1.8-TeV (Dzero).
*
*   Reference:
*             Published in PHYS.REV.,D61,032004,1999
*             e-Print Archive: hep-ex/9907009
*     This routine produces the Z/gamma* pT distribution for Z/gamma*->e+e- 
*     events with 75 < Mee < 105 at the Tevatron.  
*     Histogram -1 ---> the data from Table IV and Fig.25
*                1 ---> the simulated results to compare with it.
*                2 ---> the normalisation values for histo 1
*     The normalisation of the graph will be meaningless unless Xsec and
*     Ntot are set before calling the termination routine.  (Xsec -
*     total cross section returned by MC) (Ntot - number of events
*     passed to this routine)
*                    
* Arguments: iflag=1 initialisation
*            iflag=1 filling
*            iflag=3 termination
*
* written by: Emily Nurse (UCL) on 25/03/2005
****************************************************************************
      IMPLICIT NONE

#include "hzfunc.inc"
#include "hzhbook.inc"
#include "hepevtp.inc"
#include "heracmn.inc"

******* declare variables
      Integer ihep,iflag,count
      Integer nbins/26/
      Real ptbin(27),pt(26),dsigdpt(26)
      Real error(26),norm(26),norme(26)
      Real zpt

******* private copies of Xsec and Ntot from HERACMN
      DOUBLE PRECISION mhxsec(2),mhntot(2)
  
****** bins used for Monte Carlo and data histograms
      DATA ptbin/0.,1.,2.,3.,4.,5.,6.,7.,8.,9.,10.,12.,14.,16.,18.,20.,
     + 25.,30.,35.,40.,50.,60.,70.,85.,100.,200.,300./

****** data points from Table I
******   dsigdpt
      DATA dsigdpt/6.04,16.2,20.4,19.7,16.2,15.0,14.1,11.1,10.0,9.29,
     + 7.25,5.45,4.45,3.54,3.21,2.06,1.29,0.962,0.840,0.373,0.251,0.163,  
     + 0.053,0.034,0.0050,0.0004/
******   stat. error on dsigdpt
      DATA error/0.53,0.96,1.1,1.1,0.92,0.87,0.84,0.73,0.69,0.67,0.54,
     + 0.44,0.39,0.33,0.31,0.18,0.13,0.11,0.10,0.045,0.036,0.027,0.012, 
     + 0.009,0.0013,0.0004/ 

****** INITIALISATION ******
      IF (iflag.eq.1) THEN
****** make subdirectory in PAWC
       call hcdir('//PAWC',' ')
       call hmdir('hzh9907009','S')
****** make subdirectory in o/p file
       call hcdir('//HISTO',' ')
       call hmdir('hzh9907009','S')

****** book histograms:
******  Monte Carlo
      call hbookb(1,'1/sigma dsigma/d-pt(MC)',nbins,ptbin,0.)
******  normalisation values
      call hbookb(2,'normalisation values',nbins,ptbin,0.)
******  data
      call hbookb(-1,'1/sigma dsigma/d-pt(data)',nbins,ptbin,0.0)

      call hcdir('//HISTO/hzh9907009',' ')

      write(6,*)'**********************************************'
      write(6,*)'*  hzh9907009 called, histos will be output. *'
      write(6,*)'**********************************************'

******  LOOP OVER EVENTS ******
      ELSE IF(iflag.eq.2) THEN

******  move to the correct sub-directory in PAWC
       call hcdir('//PAWC/hzh9907009',' ')
******  loop through particles in event  and fill histo
        do ihep=1,nhep
******  pick out Z/gamma*'s AFTER the parton shower.
******  the Z/gamma* will decay to an e+e-, egamma or gammagamma
             if((IDHEP(ihep).eq.23.or.IDHEP(ihep).eq.22).and.
     +           JDAHEP(1,ihep).ne.0) then
               if( (abs(IDHEP(JDAHEP(1,ihep))).eq.11.or.
     +              IDHEP(JDAHEP(1,ihep)).eq.22).and.
     +             (abs(IDHEP(JDAHEP(2,ihep))).eq.11.or.
     +              IDHEP(JDAHEP(2,ihep)).eq.22) ) then
******  make invariant mass cut (not clear whether to use e+e- 
*****   or Z/gamma* mass due to possible collinear emission of gamma)
                 if(phep(5,ihep).gt.75.and.phep(5,ihep).lt.105) then     
                 zpt = sqrt(phep(1,ihep)**2+phep(2,ihep)**2)
                 call hfill(1,real(zpt),0.,wtx)
                 endif
               endif
             endif
         end do
******  TERMINATION ******
      ELSE IF(iflag.eq.3) THEN
******  Move to the correct PAW subdirectory
       call hcdir('//PAWC/hzh9907009',' ')

****** loop through histo bins 
      do  count=1,nbins
****** get normalisation values per bin, convert to pb from nb
        norm(count)=1000.*xsec/((ptbin(count+1)-ptbin(count))*Ntot)
        norme(count)=0.
      end do     

****** fill normalisation histogram
      call hpak(2,norm)
      call hpake(2,norme)
****** multiply Monte Carlo histo by normalisation histo
      call hopera(1,'*E',2,1,1.0,1.0) 
***** fill data histogram       
      call hpak(-1,dsigdpt)
      call hpake(-1,error)
      write(6,*) 'hzh9907009 X-section is ',xsec

      ENDIF

      RETURN
      END

