      FUNCTION HzDPhi (ipart1,ipart2)
************************************************************************
*     input:  ipart1:        index of particle 1 in HEP common
*     input:  ipart2:        index of particle 2 in HEP common
*
*     return value: azimuthal angle (phi) between particles 
*     
*     written by: Oliver Gutsche
************************************************************************
      IMPLICIT NONE
 
#include "hepevtp.inc"
#include "hzfunc.inc"
 
      Integer ipart1,ipart2

      DOUBLE PRECISION phi1,phi2,dphi,pi
      DOUBLE PRECISION dpx1,dpy1,dpx2,dpy2
      PARAMETER(pi=3.141592653589793238)

*     check particle index
      if ((ipart1.lt.1).or.(ipart1.gt.NHEP).or.
     &     (ipart2.lt.1).or.(ipart2.gt.NHEP)) then
         dphi = 0.D0
      else

*     check pt non-zero for both particles
         dpx1 = PHEP(1,ipart1)
         dpy1 = PHEP(2,ipart1)
         dpx2 = PHEP(1,ipart2)
         dpy2 = PHEP(2,ipart2)

         if ((dpx1.eq.0.D0 .and. dpy1.eq.0.D0).or.
     &       (dpx2.eq.0.D0 .and. dpy2.eq.0.D0)) then
            dphi = 0.D0
         else
 
*     get phi for particles
            phi1 = HZPHI(ipart1)+pi
            phi2 = HZPHI(ipart2)+pi

*     check if all particles returned valid phi
            IF ((phi1.eq.0.D0).OR.(phi2.eq.0.D0)) THEN
               HzDPhi = 0.D0
               RETURN
            ENDIF
 
*     calculate difference 1 -> 2
            
            dphi = phi1-phi2
            IF ( dphi.gt.(2*pi) ) THEN
               dphi = dphi - (2*pi)
            ELSE IF ( dphi.lt.0.D0 ) THEN
               dphi = dphi + (2*pi)
            ENDIF

*     check if 2*pi-dphi is smaller than dphi
            IF ( dphi.gt.pi ) THEN
               dphi = (2*pi) - dphi
            ENDIF

         ENDIF
      ENDIF

      HzDPhi = dphi
      
      RETURN
      END
