*CMZ :  2.00/00 13/08/99  15.27.38  by  Tancredi Carli
*-- Author :
      Function Hzeekin(Itype)
**********************************************************
* Purpose:
* Return kinem. variables
* Input:
* If Itype = 1  -> Q2 of highest Q2 photon
*    Itype = 11 -> Q2 of lowest Q2 photon
*    itype = 2 -> Largest electron (positron) scattering angle
*    itype = 3 -> ybj of highest Q2 photon
*    itype = 13 -> ybj of lowest Q2 photon
*    itype = 4 -> W2
* Photoproduction Author : M. Hayes
* Original Author: N. Brook
* Adapted for e+e-: J. Butterworth
* Electron scattering angle added: R. Taylor
* PYT,PHO,W2 by: J.Elmsheuser
*
* last change: 02 Aug 99
**********************************************************

      IMPLICIT NONE

#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"

      INTEGER itype,jgam, kbeam,loop
      DOUBLE PRECISION q21,ybj1,q22,ybj2,pgam1(5),pgam2(5),temp
      DOUBLE PRECISION el,po,ang1,ang2,angle,w2
      INTEGER i1,i2

      DOUBLE PRECISION EHAD(4),WHAD

      hzeekin = -1.0d0

      Jgam = hzeegamn(pgam1,pgam2)
      if(jgam.ne.1) Then
         Write(6,*) 'HZEEKIN: No record of virt. photon found'
         Return
      endif

      kbeam = hzeebeam(i1,i2)
      If(kbeam.ne.1) Then
        Write(6,*) 'HZEEKIN: No record of beam particles'
        Return
      Endif

      q21 = -(pgam1(4)**2-pgam1(3)**2-pgam1(2)**2-pgam1(1)**2)
      ybj1=(phep(4,i2)*pgam1(4) - phep(3,i2)*pgam1(3)
     +        -phep(2,i2)*pgam1(2) - phep(1,i2)*pgam1(1))/
     1    (phep(4,i1)*phep(4,i2) - phep(3,i1)*phep(3,i2)
     2        -phep(2,i1)*phep(2,i2) - phep(1,i1)*phep(1,i2))

      q22 = -(pgam2(4)**2-pgam2(3)**2-pgam2(2)**2-pgam2(1)**2)
      ybj2=(phep(4,i1)*pgam2(4) - phep(3,i1)*pgam2(3)
     +        -phep(2,i1)*pgam2(2) - phep(1,i1)*pgam2(1))/
     1    (phep(4,i2)*phep(4,i1) - phep(3,i2)*phep(3,i1)
     2        -phep(2,i2)*phep(2,i1) - phep(1,i2)*phep(1,i1))


      if (q21.lt.q22) then
         temp=q21
         q21=q22
         q22=temp
         temp=ybj1
         ybj1=ybj2
 1       ybj2=temp
      endif

C The following calls HZPHMANG to determine angle through which e+/e- is
C scattered.

      IF (gen(1:3).eq.'HRW') THEN
         el=7            !actually positron
         po=5            !actually electron
      ELSEIF (gen(1:3).eq.'PYT') THEN
        If ((i1.eq.1).and.(i2.eq.2)) Then
          el=3
          po=4
        ElseIf ((i1.eq.2).and.(i2.eq.1)) Then
          el=4
          po=3
        EndIf
      ELSEIF (gen(1:3).eq.'PHO') THEN
        el=3
        po=5
      ELSE
         Write(6,*)'Hzeekin: Unknown generator ',gen(1:3)
      ENDIF
      ang1=HZPHMANG(phep(3,el),sqrt(phep(1,el)**2+phep(2,el)**2))
      ang2=HZPHMANG(-1.*phep(3,po),sqrt(phep(1,po)**2+phep(2,po)**2))
      IF (ang1.gt.ang2) THEN
         angle=ang1
      ELSE
         angle=ang2
      ENDIF

C ---- Determination of W2 ----
      w2 = (pgam1(4)+pgam2(4))**2-(pgam1(3)+pgam2(3)**2)-
     +     (pgam1(2)+pgam2(2))**2-(pgam1(1)+pgam2(1)**2)

      If(itype.eq.1) Then
         hzeekin = q21
      elseif(itype.eq.11) then
         hzeekin = q22
      elseif(itype.eq.2) then
         hzeekin = angle
      elseif(itype.eq.3) then
         hzeekin = ybj1
      elseif(itype.eq.13) then
         hzeekin = ybj2
      elseif(itype.eq.4) then
         hzeekin = w2
      else
        write(6,*) ' HZEEKIN: itype should be in range 1-4'
      endif
*
      return
      end
