            subroutine hzhinrm( iid, iidnew, nevt, iifl )
****************************************************************************
* Purpose:
* Normalize a given 1-dim histogram with factors 1/nevt and 1/binwidth
* works also for non-equidistant bins
*
* Arguments:
* iid      histogram id to be normalized
* iidnew   new histo id (if 0, old histo is modified)
* nevt     normalization factor (usually # events)
* iifl     flag=1: normalize errors too, otherwise they are zeroed
*
* written by: Michel Kuhlen on 12/02/96
* Changed: T. Carli nevt is now real to allow for weighted histogram
****************************************************************************
      IMPLICIT NONE
*
      Integer   id, idnew,  ifl, iifl, iid, iidnew,nmax
      Real  nevt
      Parameter(nmax=200)               ! max # of bins
      Real      cont(nmax), err(nmax), zeroes(nmax), revt
      Integer nx,ny,ntw,iad,i,idfill
      Real xmin,xmax,ymin,ymax,xleft,xright,binw,hie,hi
      Character*80  title,rout*10
      Logical hexist
      Data zeroes / nmax*0. /,rout/'**hzhinrm '/
*
*                                       PAW gets confused with types
      id    = iid
      idnew = iidnew
      revt  = nevt
      ifl   = iifl
*                                       check normalization
      if( revt.le.0. ) then
        write(6,*) rout,' normalization is 0! or <0 id,revt: ',id,revt
        goto 9999
      endif
*                                       check existence of histo
      if( .not.hexist(id) ) then
        write(6,*) rout,' non-existent histo id ',id
        goto 9999
      endif
*                                       get title, number of bins
*
      call hgive( id, title, nx, xmin, xmax, ny, ymin, ymax, ntw, iad )
*
      if( nx .eq. 0 ) then
        write(6,*) rout,' histo with 0 bins, id= ',id
        goto 9999
      elseif( nx .gt. nmax ) then
        write(6,*) rout,' histo with #bins>max, id= ',id,nx,nmax
        goto 9999
      endif
*                                           loop over histogram channels
*                                           careful with last bin!
      do i=1,nx
*                                           lower edge of bin
       call hix(id,i,xleft)
*                                           upper edge of bin
       xright=xmax
       if(i.lt.nx) call hix(id,i+1,xright)
*                                           bin width
       binw = xright-xleft
*                                           bin content
       cont(i) = hi(id,i)
*      write(6,*) '  bin #, xl,xr, cont ', i,xleft,xright,cont(i)
*                                           normalized contents
       cont(i) = cont(i)  /(revt*binw)
*                                           corresponding error
       err(i)  = hie(id,i)/(revt*binw)
      enddo
*
*                                           determine where to put the contents
      if(idnew.gt.0) then
       title = 'norm - '//title
       call hcopy(id,idnew,title)
       idfill = idnew
      else
       idfill = id
      endif
*                                           reset target histo
      call hreset(idfill,title)
*
*                                            pack normalized contents into histo
*
      call hpak(idfill,cont)
      if(ifl.eq.1) then
       call hpake(idfill,err)
      else
       call hpake(idfill,zeroes)
      endif
*
 9999 continue
      return
      end
